/*!
 * Copyright (c) 2009-2011 Zenoss, Inc.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * For complete information please visit http://www.zenoss.com/oss/
 */
/* global moment: true, swfobject:true */
(function(){


/**
 * Natural Sort algorithm for Javascript - Version 0.7 - Released under MIT license
 * Author: Jim Palmer (based on chunking idea from Dave Koelle)
 *
 * Imported from https://github.com/overset/javascript-natural-sort.  Thanks!
 */

Ext.override(Ext.util.Sorter, {
    defaultSorterFn: function (o1, o2) {
        var a = this.getRoot(o1)[this.property],
            b = this.getRoot(o2)[this.property],
            re = /(^-?[0-9]+(\.?[0-9]*)[df]?e?[0-9]?$|^0x[0-9a-f]+$|[0-9]+)/gi,
            sre = /(^[ ]*|[ ]*$)/g,
            dre = /(^([\w ]+,?[\w ]+)?[\w ]+,?[\w ]+\d+:\d+(:\d+)?[\w ]?|^\d{1,4}[\/\-]\d{1,4}[\/\-]\d{1,4}|^\w+, \w+ \d+, \d{4})/,
            hre = /^0x[0-9a-f]+$/i,
            ore = /^0/,
            //force case insensitivity - Joseph and Seth
            //i = function(s) { return naturalSort.insensitive && (''+s).toLowerCase() || ''+s },
            i = function(s) { return (''+s).toLowerCase() || ''+s; },

            // convert all to strings strip whitespace
            x = i(a).replace(sre, '') || '',
            y = i(b).replace(sre, '') || '',
            // chunk/tokenize
            xN = x.replace(re, '\0$1\0').replace(/\0$/,'').replace(/^\0/,'').split('\0'),
            yN = y.replace(re, '\0$1\0').replace(/\0$/,'').replace(/^\0/,'').split('\0'),
            // numeric, hex or date detection
            xD = parseInt(x.match(hre)) || (xN.length !== 1 && x.match(dre) && Date.parse(x)),
            yD = parseInt(y.match(hre)) || xD && y.match(dre) && Date.parse(y) || null,
            oFxNcL,
            oFyNcL;
        // first try and sort Hex codes or Dates
        if (yD) {
            if ( xD < yD ) {
                return -1;
            } else if ( xD > yD ) {
                return 1;
            }
        }
        // natural sorting through split numeric strings and default strings
        for(var cLoc=0, numS=Math.max(xN.length, yN.length); cLoc < numS; cLoc++) {
            // find floats not starting with '0', string or 0 if not defined (Clint Priest)
            oFxNcL = !(xN[cLoc] || '').match(ore) && parseFloat(xN[cLoc]) || xN[cLoc] || 0;
            oFyNcL = !(yN[cLoc] || '').match(ore) && parseFloat(yN[cLoc]) || yN[cLoc] || 0;
            // handle numeric vs string comparison - number < string - (Kyle Adams)
            if (isNaN(oFxNcL) !== isNaN(oFyNcL)) { return (isNaN(oFxNcL)) ? 1 : -1; }
            // rely on string comparison if different types - i.e. '02' < 2 != '02' < '2'
            else if (typeof oFxNcL !== typeof oFyNcL) {
                oFxNcL += '';
                oFyNcL += '';
            }
            if (oFxNcL < oFyNcL) {
                return -1;
            }
            if (oFxNcL > oFyNcL) {
                return 1;
            }
        }
        return 0;
    }
});
    /**
     * This file contains overrides we are appyling to the Ext framework. These are default values we are setting
     * and convenience methods on the main Ext classes.
     **/

    /**
     * This makes the default value for checkboxes getSubmitValue (called by getFieldValues on the form)
     * return true/false if it is checked or unchecked. The normal default is "on" or nothing which means the
     * key isn't even sent to the server.
     **/
    Ext.override(Ext.form.field.Checkbox, {
        inputValue: true,
        uncheckedValue: false
    });

    /**
    * Splitter needs to be resized thinner based on the older UI. The default is 5
    **/
   Ext.override(Ext.resizer.Splitter, {
       width: 2
   });

    /**
     * In every one of our panels we want border and frame to be false so override it on the base class.
     **/
    Ext.override(Ext.panel.Panel, {
        frame: false,
        border: false
    });

    /**
     * Refs were removed when going from Ext3 to 4, we rely heavily on this feature and it is much more
     * concise way of accessing children so we are patching it back in.
     **/
    Ext.override(Ext.AbstractComponent, {
        initRef: function() {
            if(this.ref && !this.refOwner){
                var levels = this.ref.split('/'),
                last = levels.length,
                i = 0,
                t = this;
                while(t && i < last){
                    t = t.ownerCt;
                    ++i;
                }
                if(t){
                    t[this.refName = levels[--i]] = this;
                    this.refOwner = t;
                }
            }
        },
        recursiveInitRef: function() {
            this.initRef();
            if (Ext.isDefined(this.items)) {
                Ext.each(this.items.items, function(item){
                    item.recursiveInitRef();
                }, this);
            }
            if (Ext.isFunction(this.child)) {
                var tbar = this.child('*[dock="top"]');
                if (tbar) {
                    tbar.recursiveInitRef();
                }
                var bbar = this.child('*[dock="bottom"]');
                if (bbar) {
                    bbar.recursiveInitRef();
                }
            }
        },
        removeRef: function() {
            if (this.refOwner && this.refName) {
                delete this.refOwner[this.refName];
                delete this.refOwner;
            }
        },
        onAdded: function(container, pos) {
            this.ownerCt = container;
            this.recursiveInitRef();
            this.fireEvent('added', this, container, pos);
        },
        onRemoved: function() {
            this.removeRef();
            var me = this;
            me.fireEvent('removed', me, me.ownerCt);
            delete me.ownerCt;
        },
        removeCls : function() {
          try{
            var me = this,
            el = me.rendered ? me.el : me.protoEl;
            el.removeCls.apply(el, arguments);
            return me;
            }catch(e){}
        }
    });


    /**
     * Back compat for Ext3 Component grid definitions.
     * NOTE: This only works if you follow the convention of having the xtype be the same
     * as the last part of the namespace defitions. (e.g. "Zenoss.component.foo" having an xtype "foo")
     * @param xtype
     * @param cls
     */
    Ext.reg = function(xtype, cls){
        if (Ext.isString(cls)) {
            Ext.ClassManager.setAlias(cls, 'widget.'+xtype);
        } else {
            // try to register the component
            var clsName ="Zenoss.component." + xtype;
            if (Ext.ClassManager.get(clsName)) {
                Ext.ClassManager.setAlias(clsName, 'widget.'+xtype);
            }else {
                throw Ext.String.format("Unable to to register the xtype {0}: change the Ext.reg definition from the object to a string", xtype);
            }
        }
    };

    /**
     * The Ext.grid.Panel component row selection has a flaw in it:

     Steps to recreate:
     1. Create a standard Ext.grid.Panel with multiple records in it and turn "multiSelect: true"
     Note that you can just go to the documentation page
     http://docs.sencha.com/ext-js/4-0/#!/api/Ext.grid.Panel and insert the multiSelect:
     true line right into there and flip to live preview.

     2. Select the top row, then press and hold shift and click on the second row, then the third row,
     then the fourth. You would expect to see all 4 rows selected but instead you just get the last two.

     3. For reference, release the shift and select the bottom row (4th row). Now press and hold shift
     and select the 3rd row, then the 2nd row, then the 1st row. You now see all four rows selected.

     To Fix this I have to override the Ext.selection.Model to handle the top down versus bottom up selection.
     *
     */
    Ext.override(Ext.selection.Model, {
        /**
         * Selects a range of rows if the selection model {@link #isLocked is not locked}.
         * All rows in between startRow and endRow are also selected.
         * @param {Ext.data.Model/Number} startRow The record or index of the first row in the range
         * @param {Ext.data.Model/Number} endRow The record or index of the last row in the range
         * @param {Boolean} keepExisting (optional) True to retain existing selections
         */
        selectRange : function(startRow, endRow, keepExisting, dir){
            var me = this,
                store = me.store,
                selectedCount = 0,
                i,
                tmp,
                dontDeselect,
                records = [];

            if (me.isLocked()){
                return;
            }

            if (!keepExisting) {
                me.deselectAll(true);
            }

            if (!Ext.isNumber(startRow)) {
                startRow = store.indexOf(startRow);
            }
            if (!Ext.isNumber(endRow)) {
                endRow = store.indexOf(endRow);
            }

            // WG: create a flag to see if we are swapping
            var swapped = false;
            // ---

            // swap values
            if (startRow > endRow){
                // WG:  set value to true for my flag
                swapped = true;
                // ----
                tmp = endRow;
                endRow = startRow;
                startRow = tmp;
            }

            for (i = startRow; i <= endRow; i++) {
                if (me.isSelected(store.getAt(i))) {
                    selectedCount++;
                }
            }

            if (!dir) {
                dontDeselect = -1;
            } else {
                dontDeselect = (dir === 'up') ? startRow : endRow;
            }

            for (i = startRow; i <= endRow; i++){
                if (selectedCount === (endRow - startRow + 1)) {
                    if (i !== dontDeselect) {
                        me.doDeselect(i, true);
                    }
                } else {
                    records.push(store.getAt(i));
                }
            }

            //WG:  START  CHANGE
            // This is my fix, we need to flip the order
            // for it to correctly track what was selected first.
            if(!swapped){
                records.reverse();
            }
            //WG:  END CHANGE



            me.doMultiSelect(records, true);
        }
    });

    /**
     * This is a workaround to make sure the node isn't null as it has happened
     * to be on occasion. These only affect the UI class switches.
     * See Trac Ticket #29912
     **/
    Ext.override(Ext.view.AbstractView, {
        // invoked by the selection model to maintain visual UI cues
        onItemDeselect: function(record) {
            var node = this.getNode(record);
            if(node) {
                Ext.fly(node).removeCls(this.selectedItemCls);
            }
        },
        // invoked by the selection model to maintain visual UI cues
        onItemSelect: function(record) {
            var node = this.getNode(record);
            if(node) {
                Ext.fly(node).addCls(this.selectedItemCls);
            }
        }
    });


   /**
    * workaround for scrollbars missing in IE. IE ignores the parent size between parent and child
    * so we end up with the part that should have scrollbars the same size as the child, thus
    * no scrollbars. This normalizes the sizes between elements in IE only.
    **/
    if(Ext.isIE){
        Ext.override(Ext.form.ComboBox, {
            onExpand: function() {
                var me = this, picker = this.getPicker();
                var child = Ext.DomQuery.selectNode('#'+picker.id+' .x-boundlist-list-ct');
                Ext.defer(function(){ // defer a bit so the grandpaw will have a height
                    try{
                        Ext.DomQuery.selectNode('#'+picker.id);
                        child.style.cssText = 'width: 100%; height: 100%; overflow: auto;';
                    }catch(e){
                        // couldn't traverse, so just swallow it.
                    }
                }, 100, me);
                this.callOverridden();
            }
        }
                    );
    }

    Ext.override(Ext.form.ComboBox, {
        getSelectedIndex: function() {
            var combo = this,
                v = combo.getValue(),
            record = combo.findRecord(combo.valueField || combo.displayField, v),
            index = combo.store.indexOf(record);
            return index;
    }});

    /**
     * The Event console filters are not rendering correctly in our application. This override is a temporary workaround
     * until we can figure out exactly why it is not rendering. Instead of aborting on an failed layout, just keep
     * running (flush) and ignore the failed layout.
     **/
    Ext.override(Ext.layout.Context, {
        runComplete: function () {
            var me = this;

            me.state = 2;

            if (me.remainingLayouts) {
                me.handleFailure();
                // return false;
            }

            me.flush();

            // Call finishedLayout on all layouts, but do not clear the queue.
            me.flushLayouts('finishQueue', 'finishedLayout', true);

            // Call notifyOwner on all layouts and then clear the queue.
            me.flushLayouts('finishQueue', 'notifyOwner');

            me.flush(); // in case any setProp calls were made

            me.flushAnimations();

            return true;
        }

    });

    /**
     * The multiselect doesn't test to see if it has a valid return value.
     *
     **/
    Ext.override(Ext.ux.form.MultiSelect, {
        getSubmitValue: function() {
            var me = this,
                delimiter = me.delimiter,
                val = me.getValue();
            if (Ext.isString(val)) {
                return Ext.isString(delimiter) ? val.join(delimiter) : val;
            }
            return "";
        }
    });

    /**
     *  Fixes a bug in Ext where when the store is canceling
     *  requests and there are not any outstanding requests.
     **/
    Ext.override(Ext.data.Store, {
        onPageMapClear: function() {
            var me = this,
            reqs = me.pageRequests,
            req,
            page;

            // If any requests return, we no longer respond to them.
            if (me.pageMap.events.pageadded) {
                me.pageMap.events.pageadded.clearListeners();
            }

            // If the page cache gets cleared it's because a full reload is in porogress
            this.totalCount = 0;

            // Cancel all outstanding requests
            for (page in reqs) {
                if (reqs.hasOwnProperty(page)) {
                    req = reqs[page];
                    delete reqs[page];
                    if (req) {
                        delete req.callback;
                    }
                }
            }
        }

    });

    /**
     *  Fixes a bug in Ext: they forgot to make the flashParams actually work
     *  Added wmode: 'transparent' here so that IE would allow us to overlay a div
     **/
    Ext.override(Ext.flash.Component, {
        afterRender: function() {
            var me = this,
                flashParams = Ext.apply({wmode: 'transparent'}, me.flashParams),
                flashVars = Ext.apply({}, me.flashVars);

            me.callParent();

            new swfobject.embedSWF(
                me.url,
                me.getSwfId(),
                me.swfWidth,
                me.swfHeight,
                me.flashVersion,
                me.expressInstall ? me.statics.EXPRESS_INSTALL_URL : undefined,
                flashVars,
                flashParams,
                me.flashAttributes,
                Ext.bind(me.swfCallback, me)
            );
        }

    });



    /*
        Fixes a known issue in Ext where sometimes the target is null and so getTarget cannot connect the
        event to the target at that moment. One suggestion on the forums is to update ExtJs. We're not
        going to do that yet, so here's a work-around. Not going to bother with browser sniffing since this
         == null should only happen happen in IE anyway.
    */
    Ext.EventObjectImpl.prototype.getTarget = function (selector, maxDepth, returnEl) {
        if (this.target === null) {
            return null;
        }
        if (selector) {
            return Ext.fly(this.target).findParent(selector, maxDepth, returnEl);
        }
        return returnEl ? Ext.get(this.target) : this.target;
    };

    Ext.override(Ext.grid.column.Column, {
        defaultRenderer: Ext.htmlEncode
    });

    Ext.define('Ext.data.TreeStoreOverride',{
        override: 'Ext.data.TreeStore',

        /**
         * @private
         * @param {Object[]} filters The filters array
         */
        applyFilters: function(filters){
            var me = this,
                decoded = me.decodeFilters(filters),
            i = 0,
            length = decoded.length,
            node,
            visibleNodes = [],
            resultNodes = [],
            root = me.getRootNode(),
            flattened = me.tree.flatten(),
            items,
            item,
            fn;


            /**
             * @property {Ext.util.MixedCollection} snapshot
             * A pristine (unfiltered) collection of the records in this store. This is used to reinstate
             * records when a filter is removed or changed
             */
            me.snapshot = me.snapshot || me.getRootNode().copy(null, true);

            for (i = 0; i < length; i++) {
                me.filters.replace(decoded[i]);
            }


            //collect all the nodes that match the filter
            items = me.filters.items;
            length = items.length;
            for (i = 0; i < length; i++){
                item = items[i];
                fn = item.filterFn || function(item){ return item.get(item.property) === item.value; };
                visibleNodes = Ext.Array.merge(visibleNodes, Ext.Array.filter(flattened, fn));
            }

            //collect the parents of the visible nodes so the tree has the corresponding branches
            length = visibleNodes.length;
            for (i = 0; i < length; i++){
                node = visibleNodes[i];
                node.bubble(function(n){
                    if (n.parentNode){
                        resultNodes.push(n.parentNode);
                    } else {
                        return false;
                    }
                });
            }
            visibleNodes = Ext.Array.merge(visibleNodes, resultNodes);

            //identify all the other nodes that should be removed (either they are not visible or are not a parent of a visible node)
            resultNodes = [];
            root.cascadeBy(function(n){
                if (!Ext.Array.contains(visibleNodes,n)){
                    resultNodes.push(n);
                }
            });
            //we can't remove them during the cascade - pulling rug out ...
            length = resultNodes.length;
            for (i = 0; i < length; i++){
                resultNodes[i].remove();
            }

            //necessary for async-loaded trees
            root.getOwnerTree().getView().refresh();
            root.getOwnerTree().expandAll();
            if (Ext.isFunction(root.getOwnerTree().postFilter)) {
                root.getOwnerTree().postFilter();
            }
        },
        //@inheritdoc
        filter: function(filters, value) {
            var nodes, nodeLength, i, filterFn;

                if (Ext.isString(filters)) {
                    filters = {
                        property: filters,
                        value: value
                    };
                }

            //find branch nodes that have not been loaded yet - this approach is in contrast to expanding all nodes recursively, which is unnecessary if some nodes are already loaded.
            filterFn = function(item){ return !item.isLeaf() && !(item.isLoading() || item.isLoaded()); };
            nodes = Ext.Array.filter(this.tree.flatten(), filterFn);
            nodeLength = nodes.length;

            if (nodeLength === 0){
                this.applyFilters(filters);
            } else {
                for (i = 0; i < nodeLength; i++){
                    this.load({
                        node: nodes[i],
                        callback: function(){
                            nodeLength--;
                            if (nodeLength === 0){
                                //start again & re-test for newly loaded nodes in case more branches exist
                                this.filter(filters,value);
                            }
                        },
                        scope: this
                    });
                }
            }
        },
        clearFilter: function() {

            this.filters.clear();

            if (this.isFiltered()){
                this.setRootNode(this.snapshot);
                delete this.snapshot;
            }
        },
        isFiltered: function() {
            var snapshot = this.snapshot;
            return !! snapshot && snapshot !== this.getRootNode();
        }
    });

    function toMomentInTimezone(sourceMoment, timezone) {
        var result = moment.tz(timezone);
        result.year(sourceMoment.year());
        result.month(sourceMoment.month());
        result.date(sourceMoment.date());
        result.hour(sourceMoment.hour());
        result.minute(sourceMoment.minute());
        result.second(sourceMoment.second());
        result.millisecond(sourceMoment.millisecond());
        return result;
    }

    /**
     * Override the date selector to return dates in the current users
     * timezone.
     **/
    Ext.override(Ext.form.field.Date, {
        getUnixTimestamp: function() {
            var date = this.getValue();
            if (!date) {
                return 0;
            }
            return toMomentInTimezone(moment(date), Zenoss.USER_TIMEZONE).unix();
        }
    });

    /**
     * Override the default behavior of moment-timezone to
     * not translate the times if the timezone that is passed
     * is UTC.
     **/
    var oldMomentTz = moment.fn.tz;
    moment.fn.tz = function (name) {
        if (name === "UTC") {
            return this;
        }
        return oldMomentTz.apply(this, [name]);
    };


    var origGetDragData = Ext.dd.DragZone.prototype.getDragData;
    Ext.override(Ext.dd.DragZone, {
        getDragData: function(e) {
            var t = Ext.lib.Event.getTarget(e);
            // If it's a link, set the target to the ancestor cell so the browser
            // doesn't do the default anchor-drag behavior. Otherwise everything
            // works fine, so proceed as normal.
            if (t.tagName==='A') {
                e.target = e.getTarget('div.x-grid3-cell-inner');
            }
            return origGetDragData.call(this, e);
        }
    });

    /* The following classes, prefixed with 'EXTJSIV-6824', exist to address the bug
     * under that number, affecting multi-select in the tree selection model.  The
     * bug is fixed in 4.2.0, and this code should be remove when we upgrade to that release
     */

    Ext.define('EXTJSIV-6824.selection.RowModel', {
        override: 'Ext.selection.RowModel',
        bindComponent: function(view) {
            var me = this;

            me.views = me.views || [];
            me.views.push(view);
            me.bindStore(view.getStore(), true);

            view.on({
                itemmousedown: me.onRowMouseDown,
                itemclick: me.onRowClick,
                scope: me
            });

            if (me.enableKeyNav) {
                me.initKeyNav(view);
            }
        },
        onRowMouseDown: function(view, record, item, index, e) {

            // Record index will be -1 if the clicked record is a metadata record and not selectable
            if (index !== -1) {
                if (!this.allowRightMouseSelection(e)) {
                    return;
                }

                if (!this.isSelected(record)) {
                    this.mousedownAction = true;
                    this.selectWithEvent(record, e);
                }
            }
        },
        onRowClick: function(view, record, item, index, e) {
            if (this.mousedownAction) {
                this.mousedownAction = false;
            } else {
                this.selectWithEvent(record, e);
            }
        }
    });

    Ext.define('EXTJSIV-6824.selection.TreeModel', {
        override: 'Ext.selection.TreeModel',
        onKeySpace: function(e, t) {
            if (e.record.data.checked !== null) {
                this.toggleCheck(e);
            } else {
                this.callSuper(arguments);
            }
        },
        onKeyEnter: function(e, t) {
            if (e.record.data.checked !== null) {
                this.toggleCheck(e);
            } else {
                this.callSuper(arguments);
            }
        }
    });

    Ext.define('EXTJSIV-6824.tree.ViewDropZone', {
        override: 'Ext.tree.ViewDropZone',
        handleNodeDrop : function(data, targetNode, position) {
            var me = this,
                view = me.view,
                parentNode = targetNode ? targetNode.parentNode : view.panel.getRootNode(),
                Model = view.getStore().treeStore.model,
                records, i, len, record,
                insertionMethod, argList,
                needTargetExpand,
                transferData;

            // If the copy flag is set, create a copy of the models
            if (data.copy) {
                records = data.records;
                data.records = [];
                for (i = 0, len = records.length; i < len; i++) {
                    record = records[i];
                    if (record.isNode) {
                        data.records.push(record.copy(undefined, true));
                    } else {
                        // If it's not a node, make a node copy
                        data.records.push(new Model(record[record.persistenceProperty], record.getId()));
                    }
                }
            }

            // Cancel any pending expand operation
            me.cancelExpand();

            if (position === 'before') {
                insertionMethod = parentNode.insertBefore;
                argList = [null, targetNode];
                targetNode = parentNode;
            }
            else if (position === 'after') {
                if (targetNode.nextSibling) {
                    insertionMethod = parentNode.insertBefore;
                    argList = [null, targetNode.nextSibling];
                }
                else {
                    insertionMethod = parentNode.appendChild;
                    argList = [null];
                }
                targetNode = parentNode;
            }
            else {
                if (!(targetNode.isExpanded() || targetNode.isLoading())) {
                    needTargetExpand = true;
                }
                insertionMethod = targetNode.appendChild;
                argList = [null];
            }

            // A function to transfer the data into the destination tree
            transferData = function() {
                var color,
                    n;

                // Insert the records into the target node
                for (i = 0, len = data.records.length; i < len; i++) {
                    argList[0] = data.records[i];
                    insertionMethod.apply(targetNode, argList);
                }

                // If configured to sort on drop, do it according to the TreeStore's comparator
                if (me.sortOnDrop) {
                    targetNode.sort(targetNode.getOwnerTree().store.generateComparator());
                }

                // Kick off highlights after everything's been inserted, so they are
                // more in sync without insertion/render overhead.
                // Element.highlight can handle highlighting table nodes.
                if (Ext.enableFx && me.dropHighlight) {
                    color = me.dropHighlightColor;

                    for (i = 0; i < len; i++) {
                        n = view.getNode(data.records[i]);
                        if (n) {
                            Ext.fly(n).highlight(color);
                        }
                    }
                }
            };

            // Remove nodes from their current place in case there's a delay while the target node loads
            view.getSelectionModel().clearSelections();
            for (i = 0, len = data.records.length; i < len; i++) {
                record = data.records[i];
                record.parentNode.removeChild(record);
            }

            // If dropping right on an unexpanded node, transfer the data after it is expanded.
            if (needTargetExpand) {
                targetNode.expand(false, transferData);
            }
            else if (targetNode.isLoading()) {
                targetNode.on({
                    expand: transferData,
                    delay: 1,
                    single: true
                });
            }
            // Otherwise, call the data transfer function immediately
            else {
                transferData();
            }
        }
    });

    Ext.define('EXTJSIV-6824.view.DragZone', {
        override: 'Ext.view.DragZone',
        onItemMouseDown: function(view, record, item, index, e) {
            if (!this.isPreventDrag(e, record, item, index)) {
                this.view.focus();
                this.handleMouseDown(e);
            }
        }
    });

    Ext.define('EXTJSIV-6824.selection.Model', {
        override: 'Ext.selection.Model',
        selectWithEvent: function(record, e, keepExisting) {
            var me = this,
                isSelected = me.isSelected(record);

            //if it is a shift select and one row has been selected start the selection with that row.
            if (e.shiftKey && me.hasSelection() && me.getSelection().length == 1 && !me.selectionStart) {
                me.selectionStart = me.getSelection()[0];
            }

            switch (me.selectionMode) {
                case 'MULTI':
                    if (e.shiftKey && me.selectionStart) {
                        me.selectRange(me.selectionStart, record, e.ctrlKey);
                    } else if (e.ctrlKey && isSelected) {
                        me.doDeselect(record, false);
                    } else if (e.ctrlKey) {
                        me.doSelect(record, true, false);
                    } else if (isSelected && !e.shiftKey && !e.ctrlKey && me.selected.getCount() > 1) {
                        me.doSelect(record, keepExisting, false);
                    } else if (!isSelected) {
                        me.doSelect(record, false);
                    }
                    break;
                case 'SIMPLE':
                    if (isSelected) {
                        me.doDeselect(record);
                    } else {
                        me.doSelect(record, true);
                    }
                    break;
                case 'SINGLE':
                    // if allowDeselect is on and this record isSelected, deselect it
                    if (me.allowDeselect && isSelected) {
                        me.doDeselect(record);
                    // select the record and do NOT maintain existing selections
                    } else {
                        me.doSelect(record, false);
                    }
                    break;
            }

            // selectionStart is a start point for shift/mousedown to create a range from.
            // If the mousedowned record was not already selected, then it becomes the
            // start of any range created from now on.
            // If we drop to no records selected, then there is no range start any more.
            if (!e.shiftKey) {
                if (me.isSelected(record)) {
                    me.selectionStart = record;
                } else {
                    me.selectionStart = null;
                }
            }
        },
        afterKeyNavigate: function(e, record) {
            var me = this,
                recIdx,
                fromIdx,
                isSelected = me.isSelected(record),
                from = (me.selectionStart && me.isSelected(me.lastFocused)) ? me.selectionStart : (me.selectionStart = me.lastFocused),
                key = e.getCharCode(),
                isSpace = key === e.SPACE,
                direction = key === e.UP || key === e.PAGE_UP ? 'up' : (key === e.DOWN || key === e.DOWN ? 'down' : null);

            switch (me.selectionMode) {
                case 'MULTI':

                    if (isSpace) {
                        // SHIFT+SPACE, select range
                        if (e.shiftKey) {
                            me.selectRange(from, record, e.ctrlKey);
                        } else {
                            // SPACE pessed on a selected item: deselect but leave it focused.
                            // e.ctrlKey means "keep existing"
                            if (isSelected) {
                                me.doDeselect(record, e.ctrlKey);

                                // This record is already focused. To get the focus effect put on it (as opposed to selected)
                                // we have to focus null first.
                                me.setLastFocused(null);
                                me.setLastFocused(record);
                            }
                            // SPACE on an unselected item: select it
                            else {
                                me.doSelect(record, e.ctrlKey);
                            }
                        }
                    }

                    // SHIFT-navigate selects intervening rows from the last selected (or last focused) item and target item
                    else if (e.shiftKey && from) {

                        // If we are going back *into* the selected range, we deselect.
                        fromIdx = me.store.indexOf(from);
                        recIdx = me.store.indexOf(record);

                        // If we are heading back TOWARDS the start rec - deselect skipped range...
                        if (direction === 'up' && fromIdx <= recIdx && e.ctrlKey) {
                            me.deselectRange(me.lastFocused, recIdx + 1);
                        }
                        else if (direction === 'down' && fromIdx >= recIdx && e.ctrlKey) {
                            me.deselectRange(me.lastFocused, recIdx - 1);
                        }

                        // If we are heading AWAY from start point, or no CTRL key, so just select the range and let the CTRL control "keepExisting"...
                        else if (from !== record) {
                            me.selectRange(from, record, e.ctrlKey);
                        }
                        me.lastSelected = record;
                        me.setLastFocused(record);
                    }

                    // CTRL-navigate onto a selected item just focuses it
                    else if (e.ctrlKey && isSelected) {
                        me.setLastFocused(record);
                    }

                    // CTRL-navigate, just move focus
                    else if (e.ctrlKey) {
                        me.setLastFocused(record);
                    }

                    // Just navigation - select the target
                    else {
                        me.doSelect(record, false);
                    }
                    break;
                case 'SIMPLE':
                    if (isSelected) {
                        me.doDeselect(record);
                    } else {
                        me.doSelect(record, true);
                    }
                    break;
                case 'SINGLE':
                    // Space hit
                    if (isSpace) {
                        if (isSelected) {
                            me.doDeselect(record);
                            me.setLastFocused(record);
                        } else {
                            me.doSelect(record);
                        }
                    }

                    // CTRL-navigation: just move focus
                    else if (e.ctrlKey) {
                        me.setLastFocused(record);
                    }

                    // if allowDeselect is on and this record isSelected, deselect it
                    else if (me.allowDeselect && isSelected) {
                        me.doDeselect(record);
                    }

                    // select the record and do NOT maintain existing selections
                    else {
                        me.doSelect(record, false);
                    }
                    break;
            }

            // selectionStart is a start point for shift/mousedown to create a range from.
            // If the mousedowned record was not already selected, then it becomes the
            // start of any range created from now on.
            // If we drop to no records selected, then there is no range start any more.
            if (!e.shiftKey) {
                if (me.isSelected(record)) {
                    me.selectionStart = record;
                }
            }
        },
        selectRange : function(startRow, endRow, keepExisting) {
            var me = this,
                store = me.store,
                i,
                toSelect = [];

            if (me.isLocked()){
                return;
            }

            if (!keepExisting) {
                me.deselectAll(true);
            }

            if (!Ext.isNumber(startRow)) {
                startRow = store.indexOf(startRow);
            }
            if (!Ext.isNumber(endRow)) {
                endRow = store.indexOf(endRow);
            }

            // swap values
            if (startRow > endRow){
                i = endRow;
                endRow = startRow;
                startRow = i;
            }

            for (i = startRow; i <= endRow; i++){
                if (!me.isSelected(store.getAt(i))) {
                    toSelect.push(store.getAt(i));
                }
            }
            me.doMultiSelect(toSelect, true);
        }
    });

}());
(function() {
    Ext.namespace('Zenoss.flares');

    /**
     * An invisible layer that contains all the Flares.
     * Must be the highest layer in order for Flares to show up.
     */
    Ext.define('Zenoss.flares.Container', {
        extend: 'Ext.Container',
        _layoutDelay: null,
        constructor: function(config) {
            config = Ext.applyIf(config || {}, {
                id: 'flare-container',
                baseCls: 'x-flare-container',
                alignment: 't-t',
                width: '100%',
                zindex: 400000,
                items: []
            });

            this.callParent([config]);
        },
        onRender: function(ct) {
            this.el = ct.createChild({
                cls: this.baseCls + '-layer',
                children: [{
                    cls: this.baseCls + '-body'
                }]
            });
            this.body = this.el.child('.' + this.baseCls + '-body');
            this.el = new Ext.Layer({ zindex: this.zindex }, this.el);
            Zenoss.flares.Container.superclass.onRender.apply(this, arguments);
            this.el.alignTo(document, this.alignment);
        },
        getLayoutTarget: function() {
            return this.body;
        },
        onShow: function() {
            this.el.show();
            Zenoss.flares.Container.superclass.onShow.apply(this, arguments);
        },
        onRemove: function() {
            // Move the sticky items back to the top. Wait a few microseconds to do it in case more items are
            // being removed at the same time.
            if ( !this._layoutDelay ) {
                this._layoutDelay = new Ext.util.DelayedTask(this.doLayout, this);
            }
            this._layoutDelay.delay(500);
        },
        show: function() {
            if ( !this.rendered ) {
                this.render(Ext.getBody());
            }
            Zenoss.flares.Container.superclass.show.apply(this, arguments);
        },
        onLayout: function() {
            Ext.each(this.items.items, function(item, index, items) {
                if ( item.canLayout() ) {
                    if ( index === 0 ) {
                        item.anchorTo(this.el, this.alignment);
                    }
                    else {
                        item.anchorTo(items[index - 1].el, 'tl');
                    }
                }
            }, this);
            Zenoss.flares.Container.superclass.onLayout.apply(this, arguments);
        }
    });

    /**
     * The UI manager for flares.
     */
    Zenoss.flares.Manager = {
        container: Ext.create('Zenoss.flares.Container', {}),
        INFO: 'x-flare-info',
        ERROR: 'x-flare-error',
        WARNING: 'x-flare-warning',
        DEBUG: 'x-flare-debug',
        SUCCESS: 'x-flare-success',
        CRITICAL: 'x-flare-critical',
        _visibleFlares: new Ext.util.MixedCollection(false, function(flare) {
            return flare._bodyHtml;
        }),

        /**
         * Add the flare to the container and show it.
         *
         * @param flare Zenoss.flares.Flare
         */
        flare: function(flare) {
            var otherFlare = Zenoss.flares.Manager._visibleFlares.last();
            Zenoss.flares.Manager._visibleFlares.add(flare._bodyHtml, flare);

            flare.on('afterrender', function() {
                /**
                 * Checks whether a message is longer than a flare.
                 * If so, sets new styles for a message to set it properly in a flare.
                 * (reduces width and line-height of a message)
                 **/
                 var dom_message = Ext.dom.Query.select('.x-flare-message');
                 var message = Ext.get(dom_message[dom_message.length-1]);
                 var documentWidth = Ext.getBody().getViewSize().width;

                 if (message.getWidth() >= documentWidth-15) {
                     var dom_flare = Ext.dom.Query.select('.x-flare');
                     var dom_close = Ext.dom.Query.select('.x-tool-close');
                     var flare = Ext.get(dom_flare[dom_flare.length-1]);
                     var close =  Ext.get(dom_close[dom_close.length-1]);

                     flare.setStyle('line-height', '16px');
                     message.setStyle('width', '92%');
                     close.setStyle('border-top', '2px solid transparent');
                 }
             });

            // if we have other flares, make sure this one renders to the bottom of the
            // previous flares
            if (otherFlare) {
                flare.on('afterrender', function(fl){
                    fl.alignTo(otherFlare.getEl(), 'bl-bl', [0, 32]);
                });
            }else {
                flare.on('afterrender', function(fl){
                    fl.alignTo(Ext.getBody(), 'tl-tl');
                });
            }
            if(Ext.isIE){
                flare.on('afterrender', function(fl){
                // ie isn't calc width correct. must set after render to known width
                       fl.setWidth(fl.getWidth());
                });
            }
            if(Zenoss.SELENIUM){
                flare.on('afterrender', function(fl){
                    fl.getEl().query('.x-flare-message')[0].id = "flare-message-span";
                });
            }
            Zenoss.flares.Manager.container.add(flare);
            Zenoss.flares.Manager.container.doLayout();
            flare.show();
        },
        removeFlare: function(flare) {
            Zenoss.flares.Manager._visibleFlares.removeAtKey(flare._bodyHtml);
            Zenoss.flares.Manager.adjustFlares();
        },
        /**
         * Adjusts the locations of the flares, when one
         * is deleted it iterates through all the existing flares
         * and realigns their position
         **/
        adjustFlares: function() {
            var flares = Zenoss.flares.Manager._visibleFlares;
            var container = Ext.getBody(), useOffset = false;
            flares.each(function(flare){
                if (!useOffset) {
                    flare.alignTo(container, 'tl-tl');
                    useOffset = true;
                } else {
                    flare.alignTo(container, 'bl-bl', [0, 32]);
                }
                container = flare.getEl();
            });
        },
        /**
         * Format a message and create a Flare.
         *
         * @param message string A message template
         * @param type string One of the status types assigned to this class (ex: INFO, ERROR)
         * @param args array Optional orguments to fill in the message template
         */
        _formatFlare: function(message, type, args) {
            args = Array.prototype.slice.call(args, 1);
            var flare = new Zenoss.flares.Flare(message, args, {
                iconCls: type,
                animateTarget: Zenoss.flares.Manager.container.el
            });

            var existingFlare = Zenoss.flares.Manager._visibleFlares.get(flare._bodyHtml);
            if (!Ext.isDefined(existingFlare)) {
                Zenoss.flares.Manager.flare(flare);
                return flare;
            }
            flare.destroy();
            return existingFlare;
        },
        /**
         * Show a Flare with the info status.
         *
         * @param message string A message template
         * @param args mixed Optional orguments to fill in the message template
         */
        info: function(message) {
            return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.INFO, arguments);
        },
        error: function(message) {
            return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.ERROR, arguments);
        },
        warning: function(message) {
            return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.WARNING, arguments);
        },
        debug: function(message) {
            return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.DEBUG, arguments);
        },
        critical: function(message) {
            return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.CRITICAL, arguments);
        },
        success: function(message) {
            return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.SUCCESS, arguments);
        }
    };

    /**
     * Flares are growl like flash messages. Used for transient notifications. Flares are
     * managed by Zenoss.flares.Manager.
     *
     * Example:
     *
     * Zenoss.flares.Manager.info('{0} was saved as {1}.', itemName, newItemName);
     */
    Ext.define('Zenoss.flares.Flare', {
        extend: 'Ext.Window',
        _bodyHtml: null,
        _task: null,
        _closing: false,
        focus: Ext.emptyFn,
        constructor: function(message, params, config) {
            if ( Ext.isArray(message) ) {
                var children = [];
                Ext.each(message, function(m) {
                    children.push({ tag: 'li', html: m });
                });

                message = Ext.DomHelper.markup({
                    tag: 'ul',
                    children: children
                });
            }

            var template =  new Ext.Template(message, { compiled: true } );
            this._bodyHtml = template.apply(params);

            Ext.applyIf(config, {
                headerAsText: false,
                bodyCls: config.iconCls || 'x-flare-info',
                baseCls: 'x-flare',
                plain: false,
                draggable: false,
                shadow: false,
                closable: true,
                resizable: false,
                delay: 5000, // How long to show the message for
                alignment: 't-t',
                duration: 0.2, // How long to do the opening slide in
                hideDuration: 1, // How long to do the closing fade out
                template: template,
                cls: 'x-flare-body',
                height: '32',
                dismissOnClick: true,
                listeners: {
                    show: function() {
                        if ( this._task ) {
                            this._task.delay(this.delay);
                        }
                    },
                    hide: function() {
                        Zenoss.flares.Manager.removeFlare(this);
                    },
                    scope: this
                },
                items: {
                    html: "<div class='x-flare-icon'></div><span class='x-flare-message'>" + this._bodyHtml + "</span>"
                }
            });
            this.callParent([config]);

        },
        initEvents: function() {
            this.callParent(arguments);
            this.mon(this.el, 'mouseover', function(){
                this.sticky();
            }, this);
            this.mon(this.el, 'mouseout', function(){
                Ext.defer(function(){
                    if ( !this.sticky ) {
                        this.hide();
                    }
                }, 1000, this);
            }, this);
            if ( this.dismissOnClick ) {
                this.mon(this.el, 'click', function() {
                    this.hide();
                }, this);
            }
        },
        initComponent: function() {
            if ( this.delay ) {
                this._task = new Ext.util.DelayedTask(this.hide, this);
            }
            this.callParent(arguments);
        },
        /**
         * Make this Flare "stick". It will not fade away and must manually be dismissed by the user.
         */
        sticky: function() {
            if ( this._task ) {
                this._task.cancel();
                delete this._task;
            }
        },
        animShow: function() {
            this.el.slideIn('t', {
                duration: this.duration,
                callback: this._afterShow,
                scope: this
            });
        },
        close: function() {
            Zenoss.flares.Manager._visibleFlares.removeAtKey(this._bodyHtml);
            this.hide();
        },
        animHide: function() {
            Zenoss.flares.Manager._visibleFlares.removeAtKey(this._bodyHtml);
            this._closing = true;
            this.el.ghost("t", {
                duration: this.hideDuration,
                remove: false,
                callback : Ext.bind(function () {
                    this.destroy();
                }, this)
            });
        },
        canLayout: function() {
            if ( !this._closing ) {
                // Only move if it's not already closing
                return Zenoss.flares.Flare.superclass.canLayout.apply(this, arguments);
            }
        }
    });

    Ext.namespace('Zenoss.messaging');
    /**
     * Message for Zenoss.messaging.Messenger. This is for back-compat and should not be used directly.
     */
    Ext.define('Zenoss.messaging.Message',  {
        INFO: 0,        // Same as in messaging.py
        WARNING: 1,     // Same as in messaging.py
        CRITICAL: 2,    // Same as in messaging.py
        constructor: function(config) {
            config = Ext.applyIf(config || {}, {
                body: '',
                priority: this.INFO,
                sticky: false
            });
            Ext.apply(this, config);
        }
    });

    /**
     * An interface to the old messaging API. This is for back-compat and should not be used directly.
     */
    Ext.define('Zenoss.messaging.Messenger', {
        extend: 'Ext.util.Observable',
        constructor: function(config) {
            config = Ext.applyIf(config || {}, {
                interval: 30000
            });
            Ext.apply(this, config);
            this.callParent([config]);
            this.addEvents('message');
        },
        init: function() {
            this._task = new Ext.util.DelayedTask(function(){
                this.checkMessages();
            }, this);
            this._task.delay(this.interval);
            this.checkMessages();
        },
        checkMessages: function() {
            Zenoss.remote.MessagingRouter.getUserMessages({}, function(results) {
                Ext.each(results.messages, function(m) {
                    this.send(m);
                }, this);
            }, this);
        },
        send: function(msgConfig) {
            var message = new Zenoss.messaging.Message(msgConfig);
            this.fireEvent('message', this, message);

            var flare;
            if ( message.priority  === message.WARNING ) {
                flare = Zenoss.flares.Manager.warning(message.body);
            }
            else if ( message.priority  === message.CRITICAL ) {
                flare = Zenoss.flares.Manager.critical(message.body);
            }
            else {
                flare = Zenoss.flares.Manager.info(message.body);
            }

            if ( message.sticky ) {
                flare.sticky();
            }
        }
    });

    Zenoss.messenger = Ext.create('Zenoss.messaging.Messenger', {});

    Ext.onReady(function() {
        Zenoss.flares.Manager.container.show();
        Zenoss.messenger.init();
    });

    /**
     * Inform the user with a message. This is usually represented with a Flare. Use
     * this interface to alert users.
     */
    Zenoss.message = {
        /**
         * Show a message with the info status.
         *
         * @param message string A message template
         * @param args mixed Optional orguments to fill in the message template
         */
        info: function() {
            return Zenoss.flares.Manager.info.apply(null, arguments);
        },
        error: function() {
            return Zenoss.flares.Manager.error.apply(null, arguments);
        },
        warning: function() {
            return Zenoss.flares.Manager.warning.apply(null, arguments);
        },
        debug: function() {
            return Zenoss.flares.Manager.debug.apply(null, arguments);
        },
        /**
         * These messages have a critical error icon and stay until dismissed by the user.
         */
        critical: function() {
            var flare = Zenoss.flares.Manager.critical.apply(null, arguments);
            flare.sticky();
            return flare;
        },
        success: function() {
            return Zenoss.flares.Manager.success.apply(null, arguments);
        }
    };

}());
/* global EventActionManager:true, moment:true */
/* jshint freeze: false, eqeqeq: false */
/*TODO: move overriding the prototype to a util funciton */


(function(){ // Local scope
/**
 * Base namespace to contain all Zenoss-specific JavaScript.
 */
Ext.namespace('Zenoss');

/**
 * Check compatibilty mode turned on
 */
if(navigator.userAgent.indexOf("Trident") > -1 && navigator.userAgent.indexOf("MSIE 7.0") > -1){
    Ext.onReady(function(){
        Ext.Msg.show({
            title: _t("Compatibility Mode Unsupported"),
            msg: _t("Zenoss does not support running in IE Compatibility Mode."),
            buttons: Ext.Msg.OK,
            cls: "compatibilityModeAlert"
        });
    });
}
/**
 * Global Ext settings.
 */
Ext.BLANK_IMAGE_URL = '/++resource++zenui/img/s.gif';

/**
 * Enable this setting to log the stack trace of all direct requests to the browser console
 **/
Zenoss.logDirectRequests = false;

Ext.apply(Ext.direct.RemotingProvider.prototype, {
    queueTransaction: Ext.Function.createInterceptor(Ext.direct.RemotingProvider.prototype.queueTransaction, function(transaction) {
        // will render a stack trace on firefox
        if (Zenoss.logDirectRequests) {
            console.log(Ext.String.format("Router: {0} Method: {1}", transaction.action, transaction.method));
            console.trace(Ext.String.format("Router: {0} Method: {1}", transaction.action, transaction.method));
        }
    })
});

/**
 * Constants
 */
Zenoss.SEVERITY_CLEAR = 0;
Zenoss.SEVERITY_DEBUG = 1;
Zenoss.SEVERITY_INFO = 2;
Zenoss.SEVERITY_WARNING = 3;
Zenoss.SEVERITY_ERROR = 4;
Zenoss.SEVERITY_CRITICAL = 5;
Zenoss.STATUS_NEW = 0;
Zenoss.STATUS_ACKNOWLEDGED = 1;
Zenoss.STATUS_SUPPRESSED = 2;
Zenoss.STATUS_CLOSED = 3; // Closed by the user.
Zenoss.STATUS_CLEARED = 4; // Closed by a matching clear event.
Zenoss.STATUS_DROPPED = 5; // Dropped via a transform.
Zenoss.STATUS_AGED = 6; // Closed via automatic aging.

Zenoss.SELENIUM = 0;

/**
 * Namespace for anonymous scripts to attach data to avoid dumping it into
 * the global namespace.
 */
Ext.namespace('Zenoss.env');

Ext.QuickTips.init();

/**
 * This workaround is the full copy of current ExtJs GC function in Zenoss except lines with deprecated function. 
 * Currently new version of ExtJs hasn't fixed deprecated calls in GC function
 */

Ext.enableGarbageCollector = false;
Zenoss.env.enableGarbageCollector = true;

Zenoss.env.garbageCollect = function() {
    if (!Zenoss.env.enableGarbageCollector) {
        clearInterval(Zenoss.collectorThreadId);
    } else {
        var EC = Ext.cache,
            eid,
            d,
            o,
            t;

        for (eid in EC) {
            if (!EC.hasOwnProperty(eid)) {
                continue;
            }

            o = EC[eid];

            // Skip document and window elements
            if (o.skipGarbageCollection) {
                continue;
            }

            d = o.dom;

            // Should always have a DOM node
            if (!d) {
                Ext.Error.raise('Missing DOM node in element garbage collection: ' + eid);
            }

            // Check that document and window elements haven't got through
            if (d && (d.getElementById || d.navigator)) {
                Ext.Error.raise('Unexpected document or window element in element garbage collection');
            }

            // -------------------------------------------------------
            // Determining what is garbage:
            // -------------------------------------------------------
            // !d.getBoundingClientRect()
            // no getBoundingClientRect() == direct orphan, definitely garbage
            // -------------------------------------------------------
            // !d.getBoundingClientRect() && !document.getElementById(eid)
            // display none elements have no getBoundingClientRect() so we will
            // also try to look it up by it's id. However, check
            // getBoundingClientRect() first so we don't do unneeded lookups.
            // This enables collection of elements that are not orphans
            // directly, but somewhere up the line they have an orphan
            // parent.
            // -------------------------------------------------------
            if (d && (!d.parentNode || (!d.getBoundingClientRect() && !Ext.getElementById(eid)))) {
                if (Ext.enableListenerCollection) {
                    Ext.EventManager.removeAll(d);
                }
                delete EC[eid];
            }
        }
        // Cleanup IE Object leaks
        if (Ext.isIE) {
            t = {};
            for (eid in EC) {
                if (!EC.hasOwnProperty(eid)) {
                    continue;
                }
                t[eid] = EC[eid];
            }
            EC = Ext.cache = t;
        }
    }
};

Zenoss.collectorThreadId = setInterval(Zenoss.env.garbageCollect, 30000);

/**
 *
 * @param except Return all columns except the ones
 * where id is in this array.
 */
Zenoss.env.getColumnDefinitions = function(except) {
    Ext.each(Zenoss.env.COLUMN_DEFINITIONS, function(col){
        if (Zenoss.events.customColumns[col.dataIndex]) {
            Ext.apply(col, Zenoss.events.customColumns[col.dataIndex]);
        }
    });
    if (except) {
        return Zenoss.util.filter(Zenoss.env.COLUMN_DEFINITIONS, function(d){
            return Ext.Array.indexOf(except, d.id)===-1;
        });
    }
    else {
        return Zenoss.env.COLUMN_DEFINITIONS;
    }
};

Zenoss.env.initProductionStates= function(){
    var d = Zenoss.env.productionStates;
    if (!Zenoss.env.PRODUCTION_STATES ) {
        Zenoss.env.PRODUCTION_STATES = [];
        Zenoss.env.PRODUCTION_STATES_MAP = {};
        Ext.each(d, function(item) {
            Zenoss.env.PRODUCTION_STATES.push(item);
            Zenoss.env.PRODUCTION_STATES_MAP[item.value] = item.name;
        });
    }
};

Zenoss.env.initPriorities = function(){
    var d = Zenoss.env.priorities;

    if (!Zenoss.env.PRIORITIES) {
        Zenoss.env.PRIORITIES = [];
        Zenoss.env.PRIORITIES_MAP = {};
        Ext.each(d, function(item) {
            Zenoss.env.PRIORITIES.push(item);
            Zenoss.env.PRIORITIES_MAP[item.value] = item.name;
        });
    }
};


Zenoss.env.textMasks = {
        allowedNameTextMask: /[\w\s]/i,
        allowedNameText: /^[\w\s]+$/,
        allowedNameTextFeedback: 'Only letters, numbers, underscores and spaces allowed',
        allowedNameTextMaskDash: /[\w\s\-]/i,
        allowedNameTextDash: /^[\w\s\-]+$/,
        allowedNameTextFeedbackDash: 'Only letters, numbers, underscores, dashes and spaces allowed',
        allowedNameTextDashDot: /^[\w\s\-\.]+$/,
        allowedNameTextFeedbackDashDot: 'Only letters, numbers, underscores, dashes, periods, and spaces allowed',
        allowedDescTextMask: /[\w\?,\s\.\-]/i,
        allowedDescText: /^[\w\?,\s\.\-]+$/,
        allowedDescTextFeedback: 'Allowed text: . - ? spaces, letters and numbers only'
};

Ext.define('Zenoss.state.PersistentProvider', {
    extend: 'Ext.state.Provider',
    directFn: Zenoss.remote.MessagingRouter.setBrowserState,
    constructor: function() {
        this.callParent(arguments);
        this.on('statechange', this.onStateChange, this);
        this.task = null;
        this.isDirty = false;
        this.prevStateString = null;
    },
    setState: function(stateString) {
        var state = Ext.decode(stateString);
        this.state = Ext.isObject(state) ? state : {};
    },

    onStateChange: function(me, key, val){
        var newStateString = Ext.JSON.encode(this.state);

        // if prev and new state are the same, we done
        if(this.prevStateString == newStateString){
            this.isDirty = false;
            return;
        }

        // if prevStateString is null, it is the first
        // time setting state, so skip marking dirty
        // and saving
        if(this.prevStateString !== null){
            this.isDirty = true;
            this.save();
        }

        this.prevStateString = newStateString;
    },

    // Private
    save: function() {
        // no point in savin if things aint changed
        if(!this.isDirty){
            return;
        }

        // in the case where we get multiple requests to
        // update the state just send one request
        if(!this.onSaveTask) {
            this.onSaveTask = new Ext.util.DelayedTask(function(){
                this.directFn(
                    {state: Ext.encode(this.state)}, 
                    function(){
                        this.isDirty = false;
                    });
            }, this);
        }
        // delay for half a second
        this.onSaveTask.delay(500);
    },
    saveStateNow: function(callback, scope) {
        this.directFn(
            {state: Ext.encode(this.state)},
            function() {
                Ext.callback(callback, scope);
                this.isDirty = false;
            }
        );
    }
});
Ext.state.Manager.setProvider(Ext.create('Zenoss.state.PersistentProvider'));

/*
 * Hook up all Ext.Direct requests to the connection error message box.
 */
Ext.Direct.on('event', function(e){
    if ( Ext.isDefined(e.result) && e.result && Ext.isDefined(e.result.asof) ) {
        Zenoss.env.asof = e.result.asof || null;
    }
});

Ext.Direct.on('event', function(e){

    if ( Ext.isDefined(e.result) && e.result && Ext.isDefined(e.result.msg) && e.result.msg.startswith("ObjectNotFoundException") ) {
         new Zenoss.dialog.SimpleMessageDialog({
                title: _t('Stale Data Warning'),
                message: _t('Another user has edited the information on this page since you loaded it. Please reload the page.'),
                buttons: [{
                    xtype: 'DialogButton',
                    text: _t('OK'),
                    handler: function() {
                        window.location.reload();
                    }
                }, {
                    xtype: 'DialogButton',
                    text: _t('Cancel')
                }]
            }).show();
        return false;
    }
});

/**
 * After every router request show a flare is the "msg" property is set
 **/
Ext.Direct.on('event', function(e){
    if (Ext.isDefined(e.result) && e.result && Ext.isDefined(e.result.msg)) {
        var success = e.result.success || false,
            sticky = e.result.sticky || false,
            flare;
        if (success) {
            flare = Zenoss.message.success(e.result.msg);
        } else {
            flare = Zenoss.message.error(e.result.msg);
        }
        if (sticky) {
            flare.sticky();
        }
    }
});


/**
 * Each time there is an ajax request we change the mouse
 * cursor to a "wait" style to signify that something is going on.
 *
 * This is entirely to provide feedback to a user that their
 * actions had an effect.
 */
function openAjaxRequests() {
    var i = 0, idx, request, currentTime = new Date().getTime();
    for (idx in Ext.Ajax.requests) {
        request = Ext.Ajax.requests[idx];
        // mark the first time we are checking on this request
        if (!request.firstCheckTime) {
            request.firstCheckTime = new Date().getTime();
        }
        // do not count requests that have been open for a long time (longer than 3 minutes)
        // those are most likely failures that ExtJs hasn't cleaned up.
        if ((currentTime - request.firstCheckTime > 180000)) {
            continue;
        }
        i++;
    }
    return i;
}

function setCursorStyle(style) {
    if (document.body) {
        document.body.style.cursor = style;
    }
}

Ext.Ajax.on('beforerequest', function(){
    setCursorStyle("wait");
});

function setToDefaultCursorStyle() {
    // the number of open ajax requests is
    // what the current number is including this one
    // that is ending
    if (openAjaxRequests() <= 1) {
        setCursorStyle("default");
    }
}
Ext.Ajax.on('requestcomplete', setToDefaultCursorStyle);
Ext.Ajax.on('requestexception', setToDefaultCursorStyle);


Zenoss.env.unloading=false;

Ext.EventManager.on(window, 'beforeunload', function() {
    Zenoss.env.unloading=true;
});


Ext.Direct.on('exception', function(e) {
    if (Zenoss.env.unloading === true){
        return;
    }

    if (e.message.startswith("Error parsing json response") &&
        e.message.endswith("null")) {
        window.location.reload();
        return;
    }
    var dialogId = "serverExceptionDialog", cmp;
    cmp = Ext.getCmp(dialogId);
    if(cmp) {
        cmp.destroy();
    }

    Ext.create('Zenoss.dialog.SimpleMessageDialog', {
        id: dialogId,
        title: _t('Server Exception'),
        message: '<p>' + _t('The server reported the following error:') + '</p>' +
            '<p class="exception-message">' + e.message + '</p>' +
            '<p>' + _t('The system has encountered an error.') + ' ' +
            _t('Please reload the page.') + '</p>' ,
                buttons: [{
                    xtype: 'DialogButton',
                    text: _t('RELOAD'),
                    handler: function() {
                        window.location.reload();
                    }
                }, {
                    xtype: 'DialogButton',
                    text: _t('DISMISS')
                }]
        }).show();
});

/*
 * Hide the server exception MessageBox if we get a good response. Primarily
 * used to have the event console starting functioning after a temporary
 * inability to reach the server.
 */
Ext.Direct.on('event', function(e){
    var serverExceptionDialog = Ext.getCmp("serverExceptionDialog");
    if (serverExceptionDialog && Ext.isDefined(e.result)){
        serverExceptionDialog.hide();
        serverExceptionDialog.destroy();
    }
});


/**
 * @class Zenoss.PlaceholderPanel
 * @extends Ext.Panel
 * A custom panel that displays text in its center. This panel is styled
 * with custom CSS to look temporary. It is used to show devs the names of
 * panels and should not be seen by users.
 * @constructor
 * @param {Object} config
 * @cfg {String} text The text to be displayed in the center of the panel.
 */
Zenoss.PlaceholderPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        Ext.apply(config, {
            cls: 'placeholder',
            layout: 'fit',
            border: false,
            items: [{
                baseCls: 'inner',
                border: false,
                html: config.text,
                listeners: {'resize':function(ob){
                        ob.getEl().setStyle({'line-height':
                            ob.getEl().getComputedHeight()+'px'});
                }}
            }]
        });
        Zenoss.PlaceholderPanel.superclass.constructor.apply(
            this, arguments);
    }
});

/**
 * @class Zenoss.LargeToolbar
 * @extends Ext.Toolbar
 * A toolbar with greater height and custom CSS class. Used at the top of
 * several screens, including the event console.
 * @constructor
 */

Ext.define('Zenoss.LargeToolbar',{
    alias: 'widget.largetoolbar',
    extend: 'Ext.toolbar.Toolbar',
    constructor: function(config) {
        Ext.applyIf(config, {
            ui: 'large',
            cls: 'largetoolbar',
            height: 45,
            border: false
        });
        Zenoss.LargeToolbar.superclass.constructor.apply(
            this, arguments);
    }
});

Ext.define('Zenoss.SingleRowSelectionModel', {
    extend: 'Ext.selection.RowModel',
    mode: 'SINGLE',
    getSelected: function() {
        var rows = this.getSelection();
        if (!rows.length) {
            return null;
        }
        return rows[0];
    }
});

/**
 * @class Zenoss.ExtraHooksSelectionModel
 * @extends Zenoss.SingleRowSelectionModel
 * A selection model that fires extra events.
 */
Ext.define("Zenoss.ExtraHooksSelectionModel", {
    extend: "Zenoss.SingleRowSelectionModel",
    suppressDeselectOnSelect: false,
    initEvents: function() {
        Zenoss.ExtraHooksSelectionModel.superclass.initEvents.call(this);
        this.addEvents('rangeselect');
        this.on('beforeselect', function(){
            if (this.suppressDeselectOnSelect) {
                this.selectingRow = true;
            }
        }, this);
    },
    clearSelections: function() {
        if (this.selectingRow) {
            this.suspendEvents();
        }
        Zenoss.ExtraHooksSelectionModel.superclass.clearSelections.apply(this, arguments);
        if (this.selectingRow) {
            this.resumeEvents();
            this.selectingRow = false;
        }
    },
    selectRange: function () {
        this.suspendEvents();
        Zenoss.ExtraHooksSelectionModel.superclass.selectRange.apply(
            this, arguments);
        this.resumeEvents();
        this.fireEvent('rangeselect', this);
    }
});


/**
 * @class Zenoss.MultiselectMenu
 * @extends Ext.Toolbar.Button
 * A combobox-like menu that allows one to toggle each option, and is able
 * to deliver its value like a form field.
 * @constructor
 */
Ext.define("Zenoss.MultiselectMenu", {
    extend: "Ext.button.Button",
    alias: ["widget.multiselectmenu"],
    makeItemConfig: function(text, value) {
        var config = {
            hideOnClick: false,
            handler: Ext.bind(function() {
                this.fireEvent('change');
            }, this),
            value: value,
            text: text
        };
        return config;
    },
    constructor: function(config) {
        config.menu = config.menu || [];
        Zenoss.MultiselectMenu.superclass.constructor.apply(this, arguments);
        this.initialSetValue(config);
    },
    initialSetValue: function(config) {
        var defaultValues = this.defaultValues || [];
        if (Ext.isDefined(config.store)) {
            this.hasLoaded = false;
            config.store.on('load', function(s, rows) {
                this.menu.removeAll();
                Ext.each(rows, function(row){
                    var cfg = this.makeItemConfig(row.data.name, row.data.value);
                    cfg.checked = (Ext.Array.indexOf(defaultValues, row.data.value)>-1);
                    this.menu.add(cfg);
                }, this);
                this.hasLoaded = true;
            }, this);
            config.store.load();
        } else {
            this.hasLoaded = true;
            Ext.each(config.source, function(o){
                var cfg = this.makeItemConfig(o.name, o.value);
                cfg.checked = (Ext.Array.indexOf(defaultValues, o.value)>-1);
                this.menu.add(cfg);
            }, this);
        }
    },
    reset: function() {
        this.setValue();
    },
    _initialValue: null,
    getValue: function() {
        if (!this.hasLoaded) {
            // Check state, otherwise return default
            return this._initialValue || this.defaultValues;
        }
        var result = [];
        Ext.each(this.menu.items.items, function(item){
            if (item.checked)  {
                result[result.length] = item.value;
            }
        });
        return result;
    },
    setValue: function(val) {
        var check = function(item) {
            var shouldCheck = false;
            try{
                shouldCheck = val.indexOf(item.value)!==-1;
            } catch(e) {}
            item.setChecked(shouldCheck);
        };
        if (!val) {
            this.initialSetValue(this.initialConfig);
        } else {
            if (!this.hasLoaded) {
                this._initialValue = val;
                this.menu.on('add', function(menu, item) {
                    check(item);
                });
            } else {
                Ext.each(this.menu.items.items, check);
            }
        }
    }

});

/**
 * @class Zenoss.StatefulRefreshMenu
 * @extends Ext.Menu
 * A refresh menu that is able to save and restore its state.
 * @constructor
 */
Ext.define("Zenoss.StatefulRefreshMenu", {
    extend: "Ext.menu.Menu",
    alias: ['widget.statefulrefreshmenu'],
    constructor: function(config) {
        config.stateful = true;
        config.stateEvents = ['click'];
        this.callParent([config]);
    },
    getState: function() {
        //returning raw value doesn't work anymore; need to wrap in object/array
        return [this.trigger.interval];
    },
    applyState: function(interval) {

        //old cookie value not being in an array and we can't get the value, so
        //default to 60
        var savedInterval = interval[0] || 60;
        // removing one second as an option
        // for performance reasons
        if (savedInterval === 1) {
            savedInterval = 5;
        }

        var items = this.items.items;
        Ext.each(items, function(item) {
            item.checked = false;
            if (item.value === savedInterval) {
                item.checked = true;
            }
        }, this);
        this.trigger.on('afterrender', function() {
            this.trigger.setInterval(savedInterval);
        }, this);
    }
});


/**
 * @class Zenoss.RefreshMenu
 * @extends Ext.SplitButton
 * A button that manages refreshing and allows the user to set a polling
 * interval.
 * @constructor
 */
Ext.define("Zenoss.RefreshMenuButton", {
    extend: "Ext.button.Split",
    alias: ['widget.refreshmenu'],
    constructor: function(config) {
        var menu = {
            xtype: 'statefulrefreshmenu',
            id: config.stateId || 'evc_refresh',
            refreshWhenHidden: false,
            trigger: this,
            width: 127,
            items: [{
                cls: 'refreshevery',
                text: 'Refresh every',
                canActivate: false
            },{
                xtype: 'menucheckitem',
                text: '5 seconds',
                value: 5,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: '10 seconds',
                value: 10,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: '30 seconds',
                value: 30,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: '1 minute',
                value: 60,
                checked: true,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: 'Manually',
                value: -1,
                group: 'refreshgroup'
            }]
        };
        Ext.applyIf(config, {
            menu: menu
        });
        this.callParent([config]);
        this.refreshTask = new Ext.util.DelayedTask(this.poll, this);
        this.menu.on('click', function(menu, item){
            this.setInterval(item.value);
        }, this);
        // 60 is the default interval; it matches the checked item above
        this.setInterval(60);
        this.on('beforedestroy', this.removeRefreshTask, this, {single: true});
    },
    removeRefreshTask: function() {
        if (this.refreshTask) {
            this.refreshTask.cancel();
            delete this.refreshTask;
        }
    },
    setInterval: function(interval) {
        var isValid = false;
        // make sure what they are setting is a valid option
        this.menu.items.each(function(item){
            if (item.value === interval) {
                isValid = true;
            }
        });
        if (isValid) {
            this.interval = interval;
            this.refreshTask.delay(this.interval*1000);
        }
    },
    poll: function(){
        // do not poll if the user is not looking at the tab, refreshWhenHidden will force the refresh
        if (Ext.isString(document.visibilityState) && document.visibilityState == "hidden" && ! this.refreshWhenHidden) {
            this.refreshTask.delay(this.interval*1000);
            return;
        }
        if (this.interval>0) {
            if ( !this.disabled ) {
                if (Ext.isDefined(this.pollHandler)){
                    this.pollHandler();
                }
                else{
                    this.handler(this);
                }
            }
            this.refreshTask.delay(this.interval*1000);
        }
    }
});


/*
 * This EventActionManager class will handle issuing a router request for
 * actions to be taken on events. When constructing this class you must
 * provide the findParams() method and may provide the onFinishAction() method.
 * Unless you mimic the existing params structure, you'll need to override
 * isLargeRequest() as well in order to determine whether or not a dialog and
 * progress bar should be shown.
 *
 * Example to configure the EventActionManager:
 *    Zenoss.EventActionManager.configure({
 *       findParams: function() { ... },
 *       onFinishAction: function() { ... }
 *    });
 *
 * Examples to execute a router request, once configured:
 *    Zenoss.EventActionManager.execute(Zenoss.remote.MyRouter.foo);
 *
 * Here's how it works. This runs execute() which stores the passed router
 * method in variable me.action, and stores findParams() result in me.params.
 * It then calls startAction() which opens a progress dialog for large requests
 * then calls run(), which actually calls the router method with the params.
 * When the router finishes it calls requestCallback() with three cases:
 * error, complete, or incomplete. While incomplete it'll loop by calling run(),
 * which this time calls Router.nextEventSummaryUpdate() instead of the original
 * router method. Once complete it calls finishAction() to hide the progress
 * dialog and call the configured onFinishAction().
 *
 * In summary:
 *    execute() --> startAction() --> open progress dialog, then run() -->
 *    remote router method on server (with params) --> requestCallback() -->
 *    [if incomplete then calls run() to keep looping] -->
 *    finishAction() --> hide progress dialog, then onFinishAction().
 */
Ext.define("EventActionManager", {
    extend: "Ext.util.Observable",
    constructor: function(config) {
        var me = this;
        config = config || {};
        Ext.applyIf(config, {
            cancelled: false,
            dialog: new Ext.Window({
                width: 300,
                modal: true,
                title: _t('Processing...'),
                layout: 'anchor',
                closable: false,
                bodyBorder: false,
                border: false,
                hideBorders: true,
                plain: true,
                buttonAlign: 'left',
                stateful:false,
                items: [{
                    xtype: 'panel',
                    ref: 'panel',
                    layout: 'anchor',
                    items: [{
                        xtype: 'box',
                        ref: '../status',
                        autoEl: {
                            tag: 'p',
                            html: _t('Processing...')
                        },
                        height: 20
                    },{
                        xtype: 'progressbar',
                        width: 270,
                        unstyled: true,
                        ref: '../progressBar'
                    }]
                }],
                buttons: [{
                    xtype: 'DialogButton',
                    text: _t('Cancel'),
                    ref: '../cancelButton',
                    handler: function() {
                        me.cancelled = true;
                        me.finishAction();
                    }
                }]
            }),
            events: {
                'updateRequestIncomplete': true,
                'updateRequestComplete': true
            },
            listeners: {
                updateRequestIncomplete: function() {
                    if (!me.cancelled) {
                        me.run();
                    }
                },
                updateRequestComplete: function() {
                    me.finishAction();
                }
            },
            isLargeRequest: function() {
                // determine if this request is going to require batch
                // requests. If you have selected more than 100 ids, show
                // a progress bar. also if you have not selected ANY and
                // are just executing on a filter, we don't have any idea
                // how many will be updated, so show a progress bar just
                // in case.
                return me.params.evids.length > 100 || me.params.evids.length === 0;
            },
            action: function() {
                throw('The EventActionManager action must be implemented before use.');
            },
            startAction: function() {
                me.cancelled = false;
                if (me.isLargeRequest()) {
                    me.dialog.show();
                    me.dialog.status.update(_t('Processing...'));
                }
                me.run();
            },
            run: function() {
                // First request
                if (me.next_request === null) {
                    Ext.apply(me.params, {limit: 100});
                    me.action(me.params, me.requestCallback);
                }
                else {
                    Zenoss.remote.EventsRouter.nextEventSummaryUpdate({next_request: me.next_request},
                            me.requestCallback);
                }
            },
            finishAction: function() {
                me.dialog.hide();
                if (me.onFinishAction) {
                    me.onFinishAction();
                }
            },
            requestCallback: function(provider, response) {
                var data = response.result.data;

                // no data due to an error. Handle it.
                if (!data) {
                    new Zenoss.dialog.ErrorDialog({message: _t('There was an error handling your request.')});
                    me.finishAction();
                    return;
                }

                me.eventsUpdated += data.updated;
                if (data.next_request) {
                    me.next_request = data.next_request;
                    // don't try to update the progress bar if it hasn't
                    // been created due to this being a small request.
                    if (me.isLargeRequest()) {
                        var progress = data.next_request.offset/data.total;
                        me.dialog.status.update(Ext.String.format(_t('Progress: {0}%'), Math.ceil(progress*100)));
                        me.dialog.progressBar.updateProgress(progress);
                    }
                    me.fireEvent('updateRequestIncomplete', {data:data});
                }else {
                    if(me.dialog.isVisible()){
                        // this is still flagged as being a large request so shows the dialog
                        // but there is no next_request, so just show a one pass progress
                        me.dialog.progressBar.wait({
                            interval: 120,
                            duration: 1200,
                            increment: 10,
                            text: '',
                            scope: this,
                            fn: function(){
                                me.next_request = null;
                                me.fireEvent('updateRequestComplete', {data:data});
                            }
                        });
                    }else{
                        me.next_request = null;
                        me.fireEvent('updateRequestComplete', {data:data});
                    }
                }
            },
            reset: function() {
                me.eventsUpdated = 0;
                me.next_request = null;
                me.dialog.progressBar.reset();
            },
            findParams: function() {
                throw('The EventActionManager findParams() method must be implemented before use.');
            },
            execute: function(actionFunction) {
                me.action = actionFunction;
                me.params = me.findParams();
                if (me.params) {
                    me.reset();
                    me.startAction();
                }
            },
            configure: function(config) {
                Ext.apply(this, config);
            }
        });
        Ext.apply(this, config);
        EventActionManager.superclass.constructor.call(this, config);
    }
});


Ext.onReady(function() {
    Zenoss.EventActionManager = new EventActionManager();
});

/**
 * General utilities
 */
Ext.namespace('Zenoss.util');

/*
* Wrap the Ext.Direct remote call passed in as func so that calls to the
* wrapped function won't be sent in a batch. If you have an expensive call that
* you really want to run in parallel with the rest of the page, wrap it in
* this.
*
* e.g. {directFn: isolatedRequest(Zenoss.remote.DeviceRouter.getTree)}
*/
Zenoss.util.isolatedRequest = function(func) {
    var provider = Ext.Direct.getProvider(func.directCfg.action),
        combineAndSend = Ext.bind(provider.combineAndSend, provider),
        newFn;
    newFn = Ext.Function.createSequence(Ext.Function.createInterceptor(func, combineAndSend),
                                combineAndSend);
    newFn.directCfg = Ext.clone(func.directCfg);
    return newFn;
};


Zenoss.util.isSuccessful = function(response) {
    // Check the results of an Ext.Direct response for success.
    return response.result && response.result.success;
};

Zenoss.env.ACTIONS = ['history', 'drop', 'status', 'heartbeat', 'log', 'alert_state', 'detail'];

Zenoss.env.SEVERITIES = [
    [5, 'Critical'],
    [4, 'Error'],
    [3, 'Warning'],
    [2, 'Info'],
    [1, 'Debug'],
    [0, 'Clear']
];

Zenoss.util.convertSeverity = function(severity){
    if (Ext.isString(severity)) {
        return severity;
    }
    var sevs = ['clear', 'debug', 'info', 'warning', 'error', 'critical'];
    return sevs[severity];
};

Zenoss.util.convertStatus = function(stat){
    var stati = ['New', 'Acknowledged', 'Suppressed'];
    return stati[stat];
};

Zenoss.util.render_severity = function(sev) {
    return Zenoss.render.severity(sev);
};

Zenoss.util.render_status = function(stat) {
    return "<div style='text-align: center;'>" +
        Zenoss.render.evstatus(stat) +
        "</div>";
};

Zenoss.util.render_status_text = function(stat) {
    return Zenoss.render.evstatus(stat) +
        "<span style='vertical-align: middle; padding-left:5px;'>"+ stat +"</span>";
};

Zenoss.util.render_linkable = function(name, col, record) {
    var url = record.data[col.id + '_url'];
    var title = record.data[col.id + '_title'] || name;
    if (url) {
        return '<a href="'+url+'">'+title+'</a>';
    } else {
        return title;
    }
};

Zenoss.util.render_device_group_link = function(name, col, record) {
    var links = record.data.DeviceGroups.split('|'),
        returnString = "",
        link;
    // return a pipe-deliminated set of links to the ITInfrastructure page
    for (var i = 0; i < links.length; i++) {
        link = links[i];
        if (link) {
            returnString +=  '&nbsp;|&nbsp;' + Zenoss.render.DeviceGroup(link, link) ;
        }
    }

    return returnString;
};

/* jshint bitwise: false */
Zenoss.util.base64 = {
    base64s : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_",
    encode: function(decStr){
        if (typeof btoa === 'function') {
             return btoa(decStr);
        }
        var base64s = this.base64s,
            i = 0,
            encOut = "",
            bits, dual, x, y, z;
        while(decStr.length >= i + 3){
            x = (decStr.charCodeAt(i) & 0xff) <<16;
            i++;
            y = (decStr.charCodeAt(i) & 0xff) <<8;
            i++;
            z = decStr.charCodeAt(i) & 0xff;
            i++;
            bits = x | y | z;
            encOut += base64s.charAt((bits & 0x00fc0000) >>18) +
                      base64s.charAt((bits & 0x0003f000) >>12) +
                      base64s.charAt((bits & 0x00000fc0) >> 6) +
                      base64s.charAt((bits & 0x0000003f));
        }
        if(decStr.length -i > 0 && decStr.length -i < 3){
            dual = Boolean(decStr.length -i -1);
            x = ((decStr.charCodeAt(i) & 0xff) <<16);
            i++;
            y = (dual ? (decStr.charCodeAt(i) & 0xff) <<8 : 0);
            bits = x | y;
            encOut += base64s.charAt((bits & 0x00fc0000) >>18) +
                      base64s.charAt((bits & 0x0003f000) >>12) +
                      (dual ? base64s.charAt((bits & 0x00000fc0) >>6) : '=') + '=';
        }
        return(encOut);
    },
    decode: function(encStr){
        if (typeof atob === 'function') {
            return atob(encStr);
        }
        var base64s = this.base64s;
        var bits;
        var decOut = "";
        var i = 0;
        for(; i<encStr.length; i += 4){
            bits = (base64s.indexOf(encStr.charAt(i)) & 0xff) <<18 |
                   (base64s.indexOf(encStr.charAt(i +1)) & 0xff) <<12 |
                   (base64s.indexOf(encStr.charAt(i +2)) & 0xff) << 6 |
                   base64s.indexOf(encStr.charAt(i +3)) & 0xff;
            decOut += String.fromCharCode((bits & 0xff0000) >>16,
                                          (bits & 0xff00) >>8, bits & 0xff);
        }
        if(encStr.charCodeAt(i -2) === 61){
            return(decOut.substring(0, decOut.length -2));
        }
        else if(encStr.charCodeAt(i -1) === 61){
            return(decOut.substring(0, decOut.length -1));
        }
        else {
            return(decOut);
        }
    }
};
/* jshint bitwise: true */
// two functions for converting IP addresses
Zenoss.util.dot2num = function(dot) {
    var d = dot.split('.');
    return ((((((+d[0])*256)+(+d[1]))*256)+(+d[2]))*256)+(+d[3]);
};

Zenoss.util.num2dot = function(num) {
    var d = num % 256;
    for (var i = 3; i > 0; i--) {
        num = Math.floor(num/256);
        d = num%256 + '.' + d;
    }
    return d;
};

Zenoss.util.setContext = function(uid) {
    var ids = Array.prototype.slice.call(arguments, 1);
    Ext.each(ids, function(id) {
        Ext.getCmp(id).setContext(uid);
    });
};

/**
 * Doing Ext.each's job for it. If it's an array, do Ext.each, if the obj has an each function, use it instead.
 * @param {Object} obj Array or MixedCollection works here
 * @param {Function} filterFn
 * @param {Object} scope
 */
Zenoss.util.each = function(obj, filterFn, scope) {
    if ( Ext.isFunction(obj.each) ) {
        obj.each(filterFn, scope);
    }
    else {
        Ext.each(obj, filterFn, scope);
    }
};

/**
 * Return an array filtered by function argument; Filter function should
 * return true if a value should be included in the filtered result
 * @param {Object} arr array to be filtered
 * @param {Object} filterFn function used to filter
 */
Zenoss.util.filter = function(arr, filterFn, scope) {
    var result = [];

    Zenoss.util.each(arr, function(val) {
        var include = filterFn.call(scope || this, val);
        if (include) {
            result.push(val);
        }
    });

    return result;
};

/**
 * Copies all the properties of values to orig only if they already exist.
 * @param {Object} orig The receiver of the properties
 * @param {Object} values The source of the properties
 * @return {Object} returns orig
 **/
Zenoss.util.applyNotIf = function(orig, values) {
    var k;
    if (orig) {
        for (k in values) {
            if (k in orig) {
                orig[k] = values[k];
            }
        }
    }
    return orig;
};

/**
 * Calls a function when a component is available. If it is already
 * available then the function is called immediately
 * @param {String} componentId The id of the component we want available
 * @param {Function} func Callable function, no arguments
 * @param {Object} scope (optional) the scope in which we want to call this func
 **/
Zenoss.util.callWhenReady = function(componentId, func, scope) {
    var cmp = Ext.getCmp(componentId);
    if (Ext.isDefined(cmp)){
        if (scope){
            Ext.bind(func, scope)();
        }else{
            func();
        }
    }else{
        Ext.ComponentMgr.onAvailable(componentId, func, scope);
    }

};

/**
 * @class Zenoss.DateRange
 * @extends Ext.form.field.Date
 * A DateRange
 */
Ext.define("Zenoss.DateRange", {
    extend: "Ext.form.field.Date",
    alias: ['widget.DateRange'],
    xtype: "daterange",
    getErrors: function(value) {
        var errors = new Array();
        if (value == "") {
            return errors;
        }
        //Look first for invalid characters, fail fast
        if (/[^0-9TO :-]/.test(value)) {
            errors.push("Date contains invalid characters - valid characters include digits, dashes, colons, and spaces");
            return errors;
        }
        if (value.indexOf("TO") === -1) {
            if (!Zenoss.date.regex.ISO8601Long.test(value)) {
                errors.push("Date is formatted incorrectly - format should be " + Zenoss.date.ISO8601Long);
                return errors;
            }
        }
        else {
            if (!Zenoss.date.regex.ISO8601LongRange.test(value)) {
                errors.push("Date range is formatted incorrectly - format should be " + Zenoss.date.ISO8601LongRange.replace(/\\/g, ''));
                return errors;
            }
        }
        return errors;
    },
    getValue: function() {
        // Counting on getErrors() to sanitize input before executing this code
        var value = this.getRawValue();
        if (!value) {
            return "";
        }
        var has_TO = (value.indexOf("TO") === -1);
        if (has_TO) {
            var retVal = value.replace(" TO ", "/");
            return retVal.replace(" ", "T");
        }
        else {
            return value.replace(" ", "T");
        }
    },
    parseDate: function(value) {
        var newVal = Ext.form.field.Date.prototype.parseDate.call(this, value);
        Ext.iterate(Zenoss.date.regex, function(key, regex) {
            if (regex.test(value)) {
                newVal = value;
                return false;
            }
        });
        return newVal;
    }
});

/**
* Used in BaseGrid.js by onFocus() and onResize() events.
 * Fixes misalignment between filter and header of a column in IE9.
 */
Zenoss.util.refreshScrollPosition = function(me) {
    if (me.grid.view.getEl().dom.children[1]) {
        if (me.grid.view.getHeight() < parseFloat(me.grid.view.getEl().dom.children[1].scrollHeight)){
            me.view.el.dom.scrollTop += 1;
            me.view.el.dom.scrollTop -= 1;
        } else if (me.grid.view.getWidth() < parseFloat(me.grid.view.getEl().dom.children[1].style.width)){
            me.view.el.dom.scrollLeft += 1;
            me.view.el.dom.scrollLeft -= 1;
        } else if (me.grid.columns[0].filterField) {
            me.grid.columns[0].filterField.focus();
        }
    }
};

/**
 * Used by classes to validate config options that
 * are required.
 * Called like this:
 *      Zenoss.util.validateConfig(config, 'store', 'view');
 **/
Zenoss.util.validateConfig = function() {
    var config = arguments[0];
    Ext.each(arguments, function(param){
        if (Ext.isString(param) && !Ext.isDefined(config[param])){
            var error =  Ext.String.format("Did not receive expected config options: {0}", param);
            if (Ext.global.console) {
                // will show a stacktrace in firebug
                console.error(error);
            }
        }
    });
};

/**
* Used in AddDeviceController.js by _AddJob(record) method.
 * Convert IPv6 addresses to a Zope-friendly format.
 * Original source: Products.ZenUtils.IpUtil.py -> ipwrap(ip).
 */
Zenoss.util.ipv6wrap = function(string) {
    if (Ext.form.VTypes.ipv6address(string)) {
        var wrapped = string.replace(/:/g, '..');
        return wrapped.replace(/%/g, '...');
    }
    return string;
};


/**
 * Zenoss date patterns and manipulations
 */
Ext.namespace('Zenoss.date');

/**
 * A set of useful date formats. All dates should come from the server as
 * ISO8601Long, but we may of course want to render dates in many different
 * ways.
 */
Ext.apply(Zenoss.date, {
    ISO8601Long:"Y-m-d H:i:s",
    ISO8601Short:"Y-m-d",
    ShortDate: "n/j/Y",
    LongDate: "l, F d, Y",
    FullDateTime: "l, F d, Y g:i:s A",
    MonthDay: "F d",
    ShortTime: "g:i A",
    LongTime: "g:i:s A",
    SortableDateTime: "Y-m-d\\TH:i:s",
    UniversalSortableDateTime: "Y-m-d H:i:sO",
    YearMonth: "F, Y",
    ISO8601LongRange: "Y-m-d H:i:s \\T\\O Y-m-d H:i:s",
    LongRangeAndDefault: Ext.form.field.Date.prototype.altFormats + '|' + Zenoss.date.ISO8601LongRange,
    regex: {
        ISO8601LongRange: /^(19|20)\d\d-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01]) ([01][0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9]) TO (19|20)\d\d-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01]) ([01][0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])$/,
        ISO8601Long: /^(19|20)\d\d-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01]) ([01][0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])$/
    }
});

/**
 * This takes a unix timestamp and renders it in the
 * logged in users's selected timezone.
 * Format is optional, if not passed in the default will be used.
 * NOTE: value here must be in seconds, not milliseconds
 **/
Zenoss.date.renderWithTimeZone = function (value, format) {
    if (Ext.isNumeric(value)) {
        if (!format) {
            format = "YYYY-MM-DD hh:mm:ss a z";
        }
        return moment.utc(value, "X").tz(Zenoss.USER_TIMEZONE).format(format);
    }
    return value;
};

Zenoss.date.renderDateColumn = function(format) {
    return function(v) {
        return Zenoss.date.renderWithTimeZone(v, format||Zenoss.USER_DATE_FORMAT + ' ' + Zenoss.USER_TIME_FORMAT);
    };
};

Ext.onReady(function(){
    // For compatibility use the same data format for Zenoss.date.timeZones
    // as before.
    zones = {}

    Ext.each(moment.tz.names(), function(zone) {
        zones[zone] = zone;
    });
    Zenoss.date.timeZones = {zones: zones};
});

// Force checkbox to fire valid
var oldcbsetvalue = Ext.form.Checkbox.prototype.setValue;
Ext.override(Ext.form.Checkbox, {
    setValue: function(v) {
        var result = oldcbsetvalue.call(this, v);
        this.fireEvent('valid', this);
        return result;
    }
});

String.prototype.startswith = function(str){
    return (this.match('^'+str)==str);
};

String.prototype.endswith = function(str){
    return (this.match(str+'$')==str);
};

/* Readable dates */

var _time_units = [
    ['year',   60*60*24*365],
    ['month',  60*60*24*30],
    ['week',   60*60*24*7],
    ['day',    60*60*24],
    ['hour',   60*60],
    ['minute', 60],
    ['second', 1]
];

Date.prototype.readable = function(precision) {
    var diff = (new Date().getTime() - this.getTime())/1000,
        remaining = Math.abs(diff),
        result = [], i;
    for (i=0;i<_time_units.length;i++) {
        var unit = _time_units[i],
            unit_name = unit[0],
            unit_mult = unit[1],
            num = Math.floor(remaining/unit_mult);
        remaining = remaining - num * unit_mult;
        if (num) {
            result.push(num + " " + unit_name + (num>1 ? 's' : ''));
        }
        if (result.length === precision) {
            break;
        }
    }
    var base = result.join(' ');
    return diff >= 0 ? base + " ago" : "in " + base;
};


/* Cross-Browser Split 1.0.1
(c) Steven Levithan <stevenlevithan.com>; MIT License
An ECMA-compliant, uniform cross-browser split method
http://blog.stevenlevithan.com/archives/cross-browser-split
*/

var cbSplit;

// avoid running twice, which would break `cbSplit._nativeSplit`'s reference to the native `split`
if (!cbSplit) {

cbSplit = function (str, separator, limit) {
    // if `separator` is not a regex, use the native `split`
    if (Object.prototype.toString.call(separator) !== "[object RegExp]") {
        return cbSplit._nativeSplit.call(str, separator, limit);
    }

    var output = [],
        lastLastIndex = 0,
        flags = (separator.ignoreCase ? "i" : "") +
                (separator.multiline  ? "m" : "") +
                (separator.sticky     ? "y" : ""),
        separator = new RegExp(separator.source, flags + "g"), // make `global` and avoid `lastIndex` issues by working with a copy
        separator2, match, lastIndex, lastLength;

    str = str + ""; // type conversion
    if (!cbSplit._compliantExecNpcg) {
        separator2 = new RegExp("^" + separator.source + "$(?!\\s)", flags); // doesn't need /g or /y, but they don't hurt
    }

    /* behavior for `limit`: if it's...
    - `undefined`: no limit.
    - `NaN` or zero: return an empty array.
    - a positive number: use `Math.floor(limit)`.
    - a negative number: no limit.
    - other: type-convert, then use the above rules. */
    if (limit === undefined || +limit < 0) {
        limit = Infinity;
    } else {
        limit = Math.floor(+limit);
        if (!limit) {
            return [];
        }
    }
    /* jshint boss: true */
    while (match = separator.exec(str)) {
        lastIndex = match.index + match[0].length; // `separator.lastIndex` is not reliable cross-browser

        if (lastIndex > lastLastIndex) {
            output.push(str.slice(lastLastIndex, match.index));

            // fix browsers whose `exec` methods don't consistently return `undefined` for nonparticipating capturing groups
            if (!cbSplit._compliantExecNpcg && match.length > 1) {
                match[0].replace(separator2, function () {
                    for (var i = 1; i < arguments.length - 2; i++) {
                        if (arguments[i] === undefined) {
                            match[i] = undefined;
                        }
                    }
                });
            }

            if (match.length > 1 && match.index < str.length) {
                Array.prototype.push.apply(output, match.slice(1));
            }

            lastLength = match[0].length;
            lastLastIndex = lastIndex;

            if (output.length >= limit) {
                break;
            }
        }

        if (separator.lastIndex === match.index) {
            separator.lastIndex++; // avoid an infinite loop
        }
    }

    if (lastLastIndex === str.length) {
        if (lastLength || !separator.test("")) {
            output.push("");
        }
    } else {
        output.push(str.slice(lastLastIndex));
    }

    return output.length > limit ? output.slice(0, limit) : output;
};

cbSplit._compliantExecNpcg = /()??/.exec("")[1] === undefined; // NPCG: nonparticipating capturing group
cbSplit._nativeSplit = String.prototype.split;

} // end `if (!cbSplit)`

// for convenience...
String.prototype.xsplit = function (separator, limit) {
    return cbSplit(this, separator, limit);
};

Ext.ns("Zenoss.settings");

Zenoss.settings.deviceMoveIsAsync = function(devices) {
    switch (Zenoss.settings.deviceMoveJobThreshold) {
        case 0:
            return true;
        case -1:
            return false;
        default:
            var len, threshold = Zenoss.settings.deviceMoveJobThreshold || 5;
            if (devices === null) {
                len = 0;
            } else if (!Ext.isArray(devices)) {
                len = 1;
            } else {
                len = devices.length;
            }
            return threshold <= len;
    }
};

// The data for all states
    Zenoss.util.states = [
        {"abbr":"AL","name":"Alabama","slogan":"The Heart of Dixie"},
        {"abbr":"AK","name":"Alaska","slogan":"The Land of the Midnight Sun"},
        {"abbr":"AZ","name":"Arizona","slogan":"The Grand Canyon State"},
        {"abbr":"AR","name":"Arkansas","slogan":"The Natural State"},
        {"abbr":"CA","name":"California","slogan":"The Golden State"},
        {"abbr":"CO","name":"Colorado","slogan":"The Mountain State"},
        {"abbr":"CT","name":"Connecticut","slogan":"The Constitution State"},
        {"abbr":"DE","name":"Delaware","slogan":"The First State"},
        {"abbr":"DC","name":"District of Columbia","slogan":"The Nation's Capital"},
        {"abbr":"FL","name":"Florida","slogan":"The Sunshine State"},
        {"abbr":"GA","name":"Georgia","slogan":"The Peach State"},
        {"abbr":"HI","name":"Hawaii","slogan":"The Aloha State"},
        {"abbr":"ID","name":"Idaho","slogan":"Famous Potatoes"},
        {"abbr":"IL","name":"Illinois","slogan":"The Prairie State"},
        {"abbr":"IN","name":"Indiana","slogan":"The Hospitality State"},
        {"abbr":"IA","name":"Iowa","slogan":"The Corn State"},
        {"abbr":"KS","name":"Kansas","slogan":"The Sunflower State"},
        {"abbr":"KY","name":"Kentucky","slogan":"The Bluegrass State"},
        {"abbr":"LA","name":"Louisiana","slogan":"The Bayou State"},
        {"abbr":"ME","name":"Maine","slogan":"The Pine Tree State"},
        {"abbr":"MD","name":"Maryland","slogan":"Chesapeake State"},
        {"abbr":"MA","name":"Massachusetts","slogan":"The Spirit of America"},
        {"abbr":"MI","name":"Michigan","slogan":"Great Lakes State"},
        {"abbr":"MN","name":"Minnesota","slogan":"North Star State"},
        {"abbr":"MS","name":"Mississippi","slogan":"Magnolia State"},
        {"abbr":"MO","name":"Missouri","slogan":"Show Me State"},
        {"abbr":"MT","name":"Montana","slogan":"Big Sky Country"},
        {"abbr":"NE","name":"Nebraska","slogan":"Beef State"},
        {"abbr":"NV","name":"Nevada","slogan":"Silver State"},
        {"abbr":"NH","name":"New Hampshire","slogan":"Granite State"},
        {"abbr":"NJ","name":"New Jersey","slogan":"Garden State"},
        {"abbr":"NM","name":"New Mexico","slogan":"Land of Enchantment"},
        {"abbr":"NY","name":"New York","slogan":"Empire State"},
        {"abbr":"NC","name":"North Carolina","slogan":"First in Freedom"},
        {"abbr":"ND","name":"North Dakota","slogan":"Peace Garden State"},
        {"abbr":"OH","name":"Ohio","slogan":"The Heart of it All"},
        {"abbr":"OK","name":"Oklahoma","slogan":"Oklahoma is OK"},
        {"abbr":"OR","name":"Oregon","slogan":"Pacific Wonderland"},
        {"abbr":"PA","name":"Pennsylvania","slogan":"Keystone State"},
        {"abbr":"RI","name":"Rhode Island","slogan":"Ocean State"},
        {"abbr":"SC","name":"South Carolina","slogan":"Nothing Could be Finer"},
        {"abbr":"SD","name":"South Dakota","slogan":"Great Faces, Great Places"},
        {"abbr":"TN","name":"Tennessee","slogan":"Volunteer State"},
        {"abbr":"TX","name":"Texas","slogan":"Lone Star State"},
        {"abbr":"UT","name":"Utah","slogan":"Salt Lake State"},
        {"abbr":"VT","name":"Vermont","slogan":"Green Mountain State"},
        {"abbr":"VA","name":"Virginia","slogan":"Mother of States"},
        {"abbr":"WA","name":"Washington","slogan":"Green Tree State"},
        {"abbr":"WV","name":"West Virginia","slogan":"Mountain State"},
        {"abbr":"WI","name":"Wisconsin","slogan":"America's Dairyland"},
        {"abbr":"WY","name":"Wyoming","slogan":"Like No Place on Earth"}
    ];

// Create a portlets object
//
Ext.define('PortletManager', {
    mixins: {
        observable: 'Ext.util.Observable'
    },
    constructor: function (config) {
        this.mixins.observable.constructor.call(this, config);
        this.addEvents('ready', 'beforeregister');
    }
});
Zenoss.PortletManager = Ext.create('PortletManager', {});

})(); // End local scope
(function () {


    /**
     * Base class for non paginated and paginated stores.
     * @class Zenoss.Store
     */
    Ext.define('Zenoss.Store', {
        extend:'Ext.data.Store',
        constructor:function (config) {
            this.callParent([config]);
            this.addEvents(
                /**
                 * @event afterguaranteedrange
                 * This fires after the callback from the guaranteerange method.
                 * @param {Zenoss.Store} this
                 */
                'afterguaranteedrange'
            );
        },
        setBaseParam:function (key, value) {
            this.proxy.extraParams[key] = value;
        },
        setParamsParam:function (key, value) {
            if (! this.proxy.extraParams.params) {
                this.proxy.extraParams.params = {};
            }
            this.proxy.extraParams.params[key] = value;
        },
        onGuaranteedRange: function(range, start, end, options) {
            this.callParent(arguments);
            this.fireEvent('afterguaranteedrange', this);
        }
    });



    /**
     * Base Configuration for direct stores
     * @class Zenoss.DirectStore
     */
    Ext.define('Zenoss.DirectStore', {
        extend:'Zenoss.Store',
        alias:'store.zendirectstore',
        constructor:function (config) {
            config = config || {};
            var sorters;
            if (config.initialSortColumn) {
                sorters =[
                    {
                        property:config.initialSortColumn,
                        direction:config.initialSortDirection || 'ASC'
                    }
                ];
            }
            Ext.applyIf(config, {
                remoteSort:true,
                pageSize:config.pageSize || 50,
                buffered: true,
                sorters: sorters,
                proxy:{
                    type:'direct',
                    simpleSortMode: true,
                    directFn:config.directFn,
                    extraParams: config.baseParams || {},
                    reader:{
                        root:config.root || 'data',
                        totalProperty:config.totalProperty || 'totalCount'
                    }
                }
            });
            this.callParent(arguments);
        }
    });

    /**
     * @class Zenoss.NonPaginatedStore
     * @extends Ext.data.Store
     * Direct Store for when you want all of the results on a single
     * grid.
     * Use this if the expected number of rows is always going to be less than
     * a hundred or so since every request will load the entire grid without pagination.
     **/
    Ext.define('Zenoss.NonPaginatedStore', {
        extend:'Zenoss.Store',
        alias:['store.directcombo'],
        constructor:function (config) {
            config = config || {};
            Ext.applyIf(config, {
                remoteSort:false,
                buffered:false,
                proxy:{
                    type:'direct',
                    limitParam:undefined,
                    startParam:undefined,
                    pageParam: undefined,
                    sortParam: undefined,
                    extraParams: config.baseParams || {},
                    directFn:config.directFn,
                    reader:{
                        type:'json',
                        root:config.root || 'data'
                    }
                }
            });
            if (config.initialSortColumn || config.initialSortDirection) {
                Ext.applyIf(config, {
                    sorters:[ {
                            property:config.initialSortColumn || undefined,
                            direction:config.initialSortDirection || 'ASC'
                        }
                    ]
                });
            }
            this.callParent(arguments);
        },
        setContext:function (context) {
            if (this.proxy.extraParams) {
                this.proxy.extraParams.uid = context;
            }
            this.load();
        }
    });

    Ext.define('Ext.ux.grid.FilterRow', {
        extend:'Ext.util.Observable',

        init:function (grid) {
            this.grid = grid;

            // when column width programmatically changed
            grid.headerCt.on('columnresize', this.resizeFilterField, this);
            grid.headerCt.on('columnshow', this.resetFilterRow, this);
            grid.headerCt.on('columnhide', this.resetFilterRow, this);

            grid.headerCt.on('columnmove', this.resetFilterRow, this);
            grid.on('columnmove', function(col, moved, movedIndex){
                this.gridColumnMoveWithFilter(col, moved, movedIndex);
            }, this);
            this.view = this.grid.getView();
            this.view.on('bodyscroll', this.onViewScroll, this);
        },
        /**
         * Make sure that when we scroll to the left on the grid that we adjust the
         * filters to scroll as well.
         **/
        onViewScroll: function(e) {
            if (e) {
                var viewScrollLeft = this.view.el.dom.scrollLeft,
                    // need to scroll the inner container for the changes to be noticed
                    innerEl = this.dockedFilter.el.dom.childNodes[0],
                    scrollLeft = innerEl.scrollLeft;
                if (viewScrollLeft !== scrollLeft) {
                    innerEl.scrollLeft = viewScrollLeft;
                }
            }
        },
        gridColumnMoveWithFilter: function(column, moved, movedIndex){
            var grid = this.grid;
            var me = this;
            var filterBar = me.dockedFilter;
            if (filterBar) {
                me.eachColumn(function (col) {
                    if (Ext.isDefined(col.filterField)) {
                        col.filterField.destroy();
                        delete col.filterField;
                    }
                });

                // remove all the filters
                filterBar.removeAll(true);

                Ext.each(grid.getDockedItems(), function (item) {
                    if (item.id === grid.id + 'docked-filter') {
                        grid.removeDocked(item, true);
                    }
                });

                me.applyTemplate();
                // force the columns to relayout themselves (work around
                // a bug in Chrome where this would be a blank row otherwise)
                me.eachColumn(function(col){
                    if (col.isVisible() && col.getEl()) {
                        col.setWidth(col.getWidth() + 1);
                        col.setWidth(col.getWidth() - 1);

                    }
                });
                this.view.el.dom.scrollLeft -= 1;
            }

        },
        applyTemplate:function () {
            var searchItems = [];
            // set the default params
            this.eachColumn(function (col) {
                // this is the value we are going to send to the server
                // for this filter
                if (!col.filterKey) {
                    col.filterKey = col.id;
                    if (!col.filterKey || col.filterKey.startswith("gridcolumn")) {
                        col.filterKey = col.dataIndex;
                    }
                }

                var filterDivId = this.getFilterDivId(col.filterKey);

                if (!col.filterField) {
                    if (Ext.isDefined(col.filter) && col.filter === false) {
                        col.filter = {};
                        col.filter.xtype = 'hidden';
                    }
                    if (col.nofilter || Ext.isDefined(col.isCheckerHd)) {
                        col.filter = { };
                    } else if (!col.filter) {
                        col.filter = { };
                        col.filter.xtype = 'textfield';
                    }

                    col.filter = Ext.apply({
                        id:filterDivId,
                        hidden:col.isHidden(),
                        xtype:'component',
                        baseCls:'x-grid-filter',
                        width:col.width - 2,
                        enableKeyEvents:true,
                        style:{
                            margin:'1px 1px 1px 1px'
                        },
                        hideLabel:true,
                        value:this.defaultFilters[col.filterKey]
                    }, col.filter);
                    col.filterField = Ext.ComponentManager.create(col.filter);

                } else {
                    if (col.hidden !== col.filterField.hidden) {
                        col.filterField.setVisible(!col.hidden);
                    }
                }
                if(Zenoss.SELENIUM){
                    col.filterField.on('afterrender', function(e){
                        if(Ext.getCmp(e.id).inputEl){
                            /*  add an id to the input element of filters if
                                it has one. If not, don't worry about it.
                                example: events_grid-filter-devices-input
                                -- for selenium automation --
                            */
                            var filterInput = Ext.getCmp(e.id).inputEl;
                            filterInput.dom.id = e.id+"-input";
                        }
                    }, this);
                }

                if (Zenoss.settings.enableLiveSearch) {
                    col.filterField.on('change', this.onChange, this);
                }
                col.filterField.on('keydown', this.onKeyDown, this);
                col.filterField.on('validitychange', this.onInvalidFilter, this);
                col.filterField.on('focus', this.onFocus, this);

                searchItems.push(col.filterField);
            });
            // make sure we send our default filters on the initial load
            if (!Ext.isEmpty(this.defaultFilters)) {
                if (!this.grid.store.proxy.extraParams) {
                    this.grid.store.proxy.extraParams = {};
                }
                Ext.applyIf(this.grid.store.proxy.extraParams.params, this.getSearchValues());
            }

            if (searchItems.length > 0) {
                this.grid.addDocked(this.dockedFilter = Ext.create('Ext.container.Container', {
                    id:this.grid.id + 'docked-filter',
                    weight:100,
                    dock:'top',
                    border:false,
                    baseCls:Ext.baseCSSPrefix + 'grid-header-ct',
                    items:searchItems,
                    layout:{
                        type:'hbox'
                    }
                }));

            }
            this.grid.on('afterrender', function(){
               /* using storeSearch here to force the filters to acknowledge the fact that there
                  may be new filters since the last time the page was visited. This is questionable
                  and probably should be considered temporary until a better solution to filters
                  can be found
               */
               this.storeSearch();
            }, this, {single: true});
        },
        clearFilters:function () {

          /* when using .reset(), it applies setValue(), which in turn applies the
            previous value. In the case of multiselect, it adds duplicates to the list instead of resetting it.
            Hardwiring a reset for each of the changed multiselections here to disallow dupes and firing the onChange manually.
            This only fires if any of the multiselects have been changed, otherwise it only resets the text fields.
          */
          this.eachColumn(function (col) {
                if (Ext.isDefined(col.filterField)) {
                    if(col.filterField.isXType("multiselectmenu") ){
                        var dirty = false;
                        col.filterField.menu.items.each(function(f){
                            if(f.checked !== f.initialConfig.checked){
                                f.setChecked(f.initialConfig.checked);
                                dirty = true;
                            }
                        });
                        if (dirty === true){
                            this.onChange();
                        }
                    }else{
                        col.filterField.reset();
                    }
                }
            });
        },
        getState:function () {
            return this.getSearchValues();
        },
        applyState:function (state) {
            if (Ext.isEmpty(state)) {
                return;
            }
            if (!this.dockedFilter) {
                this.applyTemplate();
            }
            this.eachColumn(function (col) {
                col.filterField.setVisible(!col.isHidden());
                // do not apply a filter to a hidden column (will be confusing for the user)
                if (!col.isHidden() &&
                    Ext.isDefined(state[col.filterKey]) &&
                    !Ext.isEmpty(state[col.filterKey])) {
                        if (col.filter.xtype == "daterange") {
                            state[col.filterKey] = state[col.filterKey].replace("T", " ");
                        }
                        col.filterField.setValue(state[col.filterKey]);
                }
            });
        },
        onChange:function (field, newValue, oldValue) {
            if (!this.onChangeTask) {
                this.onChangeTask = new Ext.util.DelayedTask(function () {
                    this.storeSearch();
                }, this);
            }

            this.onChangeTask.delay(1000);

        },
        onFocus: function(field) {
            /**
             * When search field is in focus, we check its right x coordinate.
             * If this coordinate is bigger than width of a document,
             * we perform horizontal scroll on delta pixels.
             * delta is a difference between right x
             * coordinate of a search field and a document width.
             */

            var documentWidth = Ext.getBody().getViewSize().width;
            var searchFieldRightXPosition = field.getEl().getX() + field.width;
            var delta = searchFieldRightXPosition - documentWidth;
            var shift = 20;

            if (delta > 0) {
                this.view.el.dom.scrollLeft += delta + shift;
            }
            /**
             * check the case when search field right x coordinate is on the
             * edge or very close (20px) to the right side of the window.
             */
            else if (delta > -shift && delta <= 0) {
                this.view.el.dom.scrollLeft += shift;
            }
        },
        onKeyDown:function (field, e) {
            // if they explicitly pressed enter then search now
            if (e.getKey() === e.ENTER) {
                this.onChange();
            }
        },
        /**
         * Determines if the current set of filters are all valid.
         * If any one filter is invalid the set of filters is invalid
         **/
        isValid: function() {
            var isValid = true;
            this.eachColumn(function(col){
                if (col.filterField.isValid) {
                    isValid = isValid && col.filterField.isValid();
                }
            }, this);

            return isValid;
        },
        /**
         * Flare the user when we have an invalid filter.
         **/
        onInvalidFilter: function(field, isValid) {
            if (!isValid) {
                var error = field.activeError;
                if ( error ) {
                    Zenoss.message.error(error);
                }
            }
        },
        getSearchValues:function () {
            var values = {},
                visibleColumns = [],
                globbing = (this.appendGlob && (Ext.isDefined(globbing) ? globbing : true));

            this.eachColumn(function (col) {
                visibleColumns.push(col.id);
                var filter = col.filterField, excludeGlobChars = ['*', '"', '?'], query;
                if (filter && filter.xtype !== 'component') {
                    if (!Ext.isEmpty(filter.getValue())) {
                        query = filter.getValue();
                        if (globbing && filter.xtype === 'textfield' && filter.vtype !== 'numcmp' &&
                            filter.vtype !== 'numrange' && filter.vtype !== 'floatrange' &&
                            Ext.Array.indexOf(excludeGlobChars,query.charAt(query.length - 1)) === -1) {
                            query += '*';
                        }
                        values[col.filterKey] = query;
                    }
                }
            });

            // apply default filters if the column is visible
            // and the filter has not already been set
            for(var key in this.defaultFilters){
                if(Ext.Array.contains(visibleColumns, key) && !values[key]){
                    values[key] = this.defaultFilters[key];
                }
            }

            // always add tags if it is not present
            if(!values["tags"]){
                values["tags"] = this.defaultFilters["tags"];
            }

            return values;
        },

        storeSearch:function () {
            // do not store the filters if they are not valid
            if (!this.isValid()) {
                return;
            }
            var values = this.getSearchValues();
            // ZEN-23418 Convert firstSeen and/or lastSeen date filter params to UTC before making query request
            var timeParams = ["firstTime", "lastTime"];
            timeParams.forEach(function(paramKey){
                if (values[paramKey]) {
                    var timeParam = values[paramKey];
                    // The param is a Date range, so convert each date to UTC
                    // and re-build Date range string
                    if (timeParam.indexOf(" TO ") != -1) {
                        var dateValues = timeParam.split(" TO ");
                        values[paramKey] = dateValues.map(function(rangeValue){
                            return moment(rangeValue).utc().format("YYYY-MM-DD HH:mm:ss")
                        }).join(" TO ");
                    }
                    else {
                        values[paramKey] = moment(timeParam).utc().format("YYYY-MM-DD HH:mm:ss")
                    }
                }
            });
            if (!this.grid.store.proxy.extraParams) {
                this.grid.store.proxy.extraParams = {};
            }
            // this will make sure that all subsequent buffer loads have the parameters
            this.grid.store.proxy.extraParams.params = values;

            // reset their scrolling when the filters change
            this.grid.scrollToTop();

            // only load the store if a context has been applied
            if (Ext.isDefined(this.grid.getContext()) || this.grid.getStore().autoLoad) {
                this.grid.getStore().load({
                    callback:function () {
                        this.grid.fireEvent('filterschanged', this.grid, values);
                    },
                    scope:this
                });
            }

            // save the state
            this.grid.saveState();
        },

        resetFilterRow:function () {
            this.eachColumn(function (col) {
                if (!col.filterField) {
                    return;
                }
                col.filterField.setVisible(!col.isHidden());
            });

            if (!this.dockedFilter) {
                this.applyTemplate();
            }
        },

        resizeFilterField:function (headerCt, column, newColumnWidth) {
            var editor;
            if (!column.filterField) {
                //This is because of the reconfigure
                this.resetFilterRow();
                editor = this.grid.headerCt.items.findBy(
                    function (item) {
                        return item.dataIndex === column.dataIndex;
                    }).filterField;
            } else {
                editor = column.filterField;
            }

            if (editor) {
                editor.setWidth(newColumnWidth - 2);
            }
             this.view.el.dom.scrollLeft -= 1;
        },

        scrollFilterField:function (e, target) {
            var width = this.grid.headerCt.el.dom.firstChild.style.width;
            this.dockedFilter.el.dom.firstChild.style.width = width;
            this.dockedFilter.el.dom.scrollLeft = target.scrollLeft;
        },

        // Returns HTML ID of element containing filter div
        getFilterDivId:function (columnId) {
            return this.grid.id + '-filter-' + columnId;
        },

        // Iterates over each column that has filter
        eachFilterColumn:function (func) {
            this.eachColumn(function (col, i) {
                if (col.filterField) {
                    func.call(this, col, i);
                }
            });
        },
        setFilter:function (colId, value) {
            this.eachFilterColumn(function (col) {
                if (col.filterKey === colId) {
                    col.filterField.setValue(value);
                }
            });
        },
        // Iterates over each column in column config array
        eachColumn:function (func) {
            Ext.each(this.grid.columns, func, this);
        }
    });

    /**
     * @class Zenoss.ContextGridPanel
     * @extends Ext.grid.GridPanel
     * Base class for all of our grids that have a context.
     * @constructor
     */
    Ext.define('Zenoss.ContextGridPanel', {
        extend:'Ext.grid.Panel',
        alias:['widget.contextgridpanel'],
        selectedNodes:[],
        constructor:function (config) {
            var viewConfig = config.viewConfig || {};

            Zenoss.util.validateConfig(config,
                'store',
                'columns');
            viewConfig = config.viewConfig || {};

            Ext.applyIf(viewConfig, {
                autoScroll:false,
                stripeRows:true,
                loadMask:false,
                preserveScrollOnRefresh: true
            });

            Ext.applyIf(config, {
                scroll:'both',
                viewConfig:viewConfig
            });
            this.callParent([config]);
            var after_request = function () {
                if (!this._disableSavedSelection) {
                    this.applySavedSelection();
                }
                // In case the new request returns less results that the previous
                // to avoid displaying a page that does not exist
                if(!this.getStore().buffered) {
                    var store = this.getStore();
                    var last_page = Math.floor(store.getTotalCount() / store.pageSize) + 1;
                    if (store.currentPage > last_page) {
                        store.loadPage(last_page);
                    }
                }
            };

            var before_request = function (store, operation) {
                if (!operation) {
                    return true;
                }
                this.start = operation.start;
                this.limit = operation.limit;
                if (!Ext.isDefined(operation.params)) {
                    operation.params = {};
                }
                // once a uid is set always send that uid
                if (this.uid) {
                    operation.params.uid = this.uid;
                }
                this.applyOptions(operation);
                return true;
            };

            if (this.getStore().buffered) {
                this.getStore().on('beforeprefetch', before_request, this);
                this.getStore().on("afterguaranteedrange", after_request, this);
            }
            else {
                this.getStore().on('beforeload', before_request, this);
                this.getStore().on("load", after_request, this);
            }

            this.addEvents(
                /**
                 * @event beforeactivate
                 * Fires when the context of the grid panel changes
                 * @param {Ext.Component} this
                 * @param {String} contextId
                 */
                'contextchange'
            );
        },
        saveSelection:function () {
            this.selectedNodes = this.getSelectionModel().getSelection();
        },
        disableSavedSelection: function(bool) {
            this._disableSavedSelection = bool;
        },
        applySavedSelection:function () {
            var curStore = this.getStore(),
                selModel = this.getSelectionModel(),
                i, node, rec,
                items = [];
            for (i = 0; i < this.selectedNodes.length; i ++) {
                node = this.selectedNodes[i];
                // idProperty is usually UID or EVID in the case of events, but it has to uniquely identify the record
                rec = curStore.findRecord(node.idProperty, node.get(node.idProperty));
                if (rec) {
                    items.push(rec);
                }
            }

            if (items.length > 0) {
                this.suspendEvents();
                this.getSelectionModel().select(items, false, true);
                this.resumeEvents();
                selModel.fireEvent('selectionchange', selModel, selModel.getSelection());
                this.selectedNodes = [];
            }
        },
        clearSavedSelections: function() {
            this.selectedNodes = [];
        },
        applyOptions:function (options) {
            // Do nothing in the base implementation
        },
        /**
         * This will add a parameter to be sent
         * back to the server on every request for this store.
         **/
        setStoreParameter:function (name, value) {
            var store = this.getStore();
            if (!store.proxy.extraParams) {
                store.proxy.extraParams = {};
            }
            store.proxy.extraParams[name] = value;
        },
        setContext:function (uid) {
            this.uid = uid;
            this.setStoreParameter('uid', uid);
            this.fireEvent('contextchange', this, uid);
            this.getStore().load();
        },
        getContext:function () {
            return this.uid;
        },
        refresh:function () {
            // only refresh if we have a context set
            if (!this.getContext()) {
                return;
            }
            this.saveSelection();
            this.getStore().load();
        }
    });


    /**
     * @class Zenoss.BaseGridPanel
     * @extends Zenoss.ContextGridPanel
     * Base class for all of our Live Grids.
     * @constructor
     */
    Ext.define('Zenoss.BaseGridPanel', {
        extend:'Zenoss.ContextGridPanel',
        alias:['widget.basegridpanel'],
        constructor:function (config) {
            Ext.applyIf(config, {
                verticalScrollerType:'paginggridscroller',
                invalidateScrollerOnRefresh:false,
                scroll:'both',
                verticalScroller: {
                    scrollToLoadBuffer: 100
                },
                bbar: { cls: 'commonlivegridinfopanel',
                        items: [ {xtype:'pagingtoolbar', cls: 'commonlivegridinfopanel'},
                                 '->',
                                 {xtype:'livegridinfopanel', grid:this}
                        ]
                }
            });
            this.callParent([config]);
        },
        initComponent: function() {
            this.callParent(arguments);
            this.headerCt.on('columnhide', this.onColumnChange, this);
            this.headerCt.on('columnshow', this.onColumnChange, this);

            this.refresh_in_progress = 0;

            if (this.getStore().buffered) {
                this.getStore().on('beforeprefetch', this.before_request, this);
                this.getStore().on("afterguaranteedrange", this.after_request, this);
            } else {
                this.getStore().on('beforeload', this.before_request, this);
                this.getStore().on("load", this.after_request, this);
            }

            var paging_tb = this.down('pagingtoolbar');
            if (paging_tb) {
                // If we have an infinite grid or NonPagination store we hide the paging toolbar
                if (this.getStore().buffered || this.getStore().alias[0] === "store.directcombo") {
                    paging_tb.hide();
                }
                else {
                    paging_tb.on('beforechange', this.scrollToTop, this);
                    paging_tb.bindStore(this.getStore());
                    paging_tb.down('#refresh').hide();
                }
            }
        },
        before_request: function() {
            if (this.getStore().buffered) {
                this.refresh_in_progress = 1;
            }
            else {
                this.refresh_in_progress += 1;
            }
        },
        after_request: function() {
            if (this.refresh_in_progress > 0) {
                this.refresh_in_progress -= 1;
            }
            if (this._previousScrollPosition && this.getView().getEl()) {
                this.getView().getEl().dom.scrollTop = this._previousScrollPosition;
                delete this._previousScrollPosition;
            }
        },
        /**
         * Listeners for when you hide/show a column, the data isn't fetched yet so
         * we need to refresh the grid to get it. Otherwise there will be a blank column
         * until the page is manually refreshed.
         **/
        onColumnChange:function () {
            if (!this.onColumnChangeTask) {
                this.onColumnChangeTask = new Ext.util.DelayedTask(function () {
                    this.refresh();
                }, this);
            }

            // give them one second to hide/show other columns
            this.onColumnChangeTask.delay(1000);
        },
        setContext: function(uid) {
            this.scrollToTop();
            if (this.getStore().pageMap) {
                this.getStore().pageMap.clear();
            }
            this.callParent([uid]);
        },
        refresh:function (callback, scope) {
            // only refresh if a context is set
            if (!this.getContext()) {
                return;
            }
            this.saveSelection();
            var store = this.getStore();

            if (! store.buffered || store.totalCount < store.pageSize) {
                this._previousScrollPosition = this.getViewScrollPosition();
                store.load({
                    callback: callback,
                    scope: scope || this
                });
            } else {
                // need to refresh the current rows, without changing the scroll position
                var start = Math.max(store.lastRequestStart, 0),
                    end = Math.min(start + store.pageSize - 1, store.totalCount),
                    page = store.pageMap.getPageFromRecordIndex(end);
                // make sure we do not have any records in cache
                store.pageMap.removeAtKey(page);

                // If a refresh kicks off before the initial store load store.totalCount is NaN
                if (isNaN(store.totalCount)) {
                    end = start + store.pageSize - 1;
                }

                if (store.getCount() >= store.getTotalCount() && this.verticalScroller) {
                    start = this.verticalScroller.getFirstVisibleRowIndex();
                }

                // this will fetch from the server and update the view since we removed it from cache
                if (Ext.isFunction(callback)) {
                    store.guaranteeRange(start, end, callback, scope);
                } else {
                    store.guaranteeRange(start, end);
                }
            }
        },
        scrollToTop:function () {
            var view = this.getView();
            if (view.getEl()) {
                view.getEl().dom.scrollTop = 0;
            }
        },
        getViewScrollPosition: function() {
            var view = this.getView();
            if (view.getEl()) {
                return view.getEl().dom.scrollTop;
            }
            return 0;
        }
    });


    /**
     * @class Zenoss.FilterGridPanel
     * @extends Zenoss.BaseGridPanel
     * Sub class of the base grid that allows adds filters to the columns.
     * @constructor
     */
    Ext.define('Zenoss.FilterGridPanel', {
        extend:'Zenoss.BaseGridPanel',
        alias:['widget.filtergridpanel'],
        constructor:function (config) {
            config = config || {};
            Ext.applyIf(config, {
                displayFilters:true,
                // only make it stateful if we have an id set
                stateful: Ext.isDefined(config.id),
                stateId: config.id
            });

            this.callParent(arguments);
        },
        initComponent:function () {
            /**
             * @event filterschanged
             * Fires after the filters are changed but after the store is reloaded
             * @param {Zenoss.FilterGridPanel} grid The grid panel.
             * @param {Object} filters Key/value pair of the new filters.
             */
            this.addEvents('filterschanged');
            this.callParent();
            // create the filter row
            var filters = Ext.create('Ext.ux.grid.FilterRow', {
                grid:this,
                appendGlob:this.appendGlob,
                defaultFilters:this.defaultFilters || {}
            });


            this.filterRow = filters;
        },
        getState:function () {
            var state = this.callParent();
            state.filters = this.filterRow.getState();
            return state;
        },
        applyState:function (state) {
            this.callParent([state]);
            if (this.displayFilters) {
                 // defer rendering the filter rows until after the column state has been applied so
                 // the filters have the correct column widths
                 if (this.displayFilters) {
                     this.filterRow.init(this);
                 }
                this.filterRow.applyState(state.filters);
            }
        },
        getFilters:function () {
            return this.filterRow.getSearchValues();
        },
        setFilter:function (colId, value) {
            this.filterRow.setFilter(colId, value);
        },
        afterRender:function() {
            this.callParent();
            this.applyState(this.getState());
        },
        refresh:function (callback) {
        if (!Zenoss.settings.enableLiveSearch) {
            var values = this.getFilters(),
                store = this.getStore();
            if (!store.proxy.extraParams) {
                store.proxy.extraParams = {};
                }
            store.proxy.extraParams.params = values;
            if (this.filterRow.isValid()) {
                this.saveState();
                }
            }
        this.callParent([callback]);
        }
    });

    /**
     * @class Zenoss.LiveGridInfoPanel
     * @extends Ext.toolbar.TextItem
     * Toolbar addition that displays, e.g., "Showing 1-10 of 100 Rows"
     * @constructor
     * @grid {Object} the GridPanel whose information should be displayed
     */
    Ext.define('Zenoss.LiveGridInfoPanel', {
        extend:'Ext.toolbar.TextItem',
        alias:['widget.livegridinfopanel'],
        cls:'livegridinfopanel',
        constructor: function(config){
            this.emptyMsg = _t('NO RESULTS');
            this.firstRender = true;
            config = config || {};
            Ext.applyIf(config,{
                text: this.emptyMsg
            });
            this.callParent([config]);
        },
        initComponent:function () {
            if (this.grid) {
                if (!Ext.isObject(this.grid)) {
                    this.grid = Ext.getCmp(this.grid);
                }
                this.view = this.grid.getView();
                // We need to refresh this when one of two events happen:
                //  1.  The data in the data store changes
                //  2.  The user scrolls.
                this.grid.getStore().on('datachanged', this.onDataChanged, this);
                /*  added this guaranteedrange hack to make up for the ext bug where-by
                    updating store doesn't fire the datachanged except on load.
                */
                var store = this.grid.getStore();
                if(store.buffered) {
                    store.on('guaranteedrange', this.onDataChanged, this);
                }
                else {
                    store.on('load', this.onDataChanged, this);
                }
                this.view.on('bodyscroll', this.onScroll, this);
                this.view.on('resize', this.onResize, this);
            }
            this.lastScrollLeft = 0;
            this.rowHeight = null;
            this.visibleRows = null;
            this.displayMsg = _t('DISPLAYING {0} - {1} of {2} ROWS');
            this.callParent(arguments);
        },
        onResize: function() {
            this.visibleRows = null;
            this.onScroll();
            Zenoss.util.refreshScrollPosition(this);
        },
        getNumberOfVisibleRows: function() {
            if (this.visibleRows) {
                return this.visibleRows;
            }

            var gridHeight;

            gridHeight = this.view.el.getHeight();

            // this assumes that all rows are uniform height
            // and can not change
            var node = this.view.getNode(0),
                el = Ext.fly(node);
            // make sure the first row is rendered
            if (el) {
                this.rowHeight = el.getHeight();
            }

            if (this.rowHeight && gridHeight) {
                this.visibleRows = Math.floor(gridHeight / this.rowHeight);
                return this.visibleRows;
            }
            return 0;
        },
        getEndCount: function(start) {
            var numrows = this.getNumberOfVisibleRows();
            if (numrows){
                return start + numrows + 1;
            }
            // we either aren't fully rendered yet or
            // there aren't any rows
            return 0;
        },
        getStartCount: function() {
            var scrollTop = this.view.el.dom.scrollTop;
            if (this.rowHeight && scrollTop) {
                return Math.ceil(scrollTop / this.rowHeight);
            }
            // ask the scroller, if the store is paginated
            if (this.grid.verticalScroller && this.grid.verticalScroller.getFirstVisibleRowIndex && this.grid.getStore().buffered){
                var start = this.grid.verticalScroller.getFirstVisibleRowIndex();
                if (start) {
                    return start;
                }
            }
            return 0;
        },
        onDataChanged:function () {
            var totalCount = this.grid.getStore().getTotalCount();
            this.totalCount = totalCount;
            if (totalCount && totalCount > 0) {
                this.onScroll();
            } else {
                this.setText(this.emptyMsg);
            }
        },
        onScroll: function(e, t) {
            // introduce a small delay so that we are
            // are not constantly updating the text when they are scrolling like crazy
            if (!this.onScrollTask || this.firstRender){
                this.onScrollTask = new Ext.util.DelayedTask(this._doOnScroll, this);
            }
            this.onScrollTask.delay(250);
        },
        setText: function(text) {
            if (text !== this.text) {
                this.callParent(arguments);
            }
        },
        _doOnScroll: function() {
            var pagingScroller = this.grid.verticalScroller, msg;
            // ext will fire the scroll event sometimes before the data is even set
            if (!this.totalCount) {
                return this.setText(this.emptyMsg);
            }
            if (this.firstRender) {
                this.firstRender = false;
            }
            if (pagingScroller) {
                var start = Math.max(this.getStartCount(), 0),
                    end = Math.min(this.getEndCount(start), this.totalCount),
                    currentScrollLeft = this.view.el.dom.scrollLeft;


                var store = this.grid.getStore();
                if (!store.buffered) {
                    var current_page = store.currentPage;
                    if (current_page > 0) {
                        var page_size = store.pageSize;
                        var offset = (current_page - 1) * page_size;
                        start = offset + start;
                        end = offset + end;
                        var real_page_end = current_page * page_size;
                        if (real_page_end > store.totalCount) {
                            real_page_end = store.totalCount;
                        }
                    }
                }
                msg = Ext.String.format(this.displayMsg, start + 1, end, store.totalCount);

                 if (this.scrollLeft !== currentScrollLeft) {
                     this.scrollLeft = currentScrollLeft;
                 } else {
                     // only redraw the text if we're scrolling vertically; the DOM scrollLeft increment/decrement is a HACK
                     // to work around a (suspected) ExtJS bug that causes the headers to become misaligned
                     this.view.el.dom.scrollLeft += 1;
                     this.setText(msg);
                     this.view.el.dom.scrollLeft -= 1;
                 }
            } else {
                // Drat, we didn't have the paging scroller, so assume we are showing all
                var showingAllMsg = _t('Found {0} records');
                msg = Ext.String.format(showingAllMsg, this.totalCount);
                this.setText(msg);
            }
        }
    });

}());
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {
     // NOTE: All permissions are handled on the server, this is just to
     // enhance the user experience
     Ext.ns('Zenoss.Security');

     // Defined in ZenUI3/security/security.py this is a dictionary of all the
     // permissions the current user has on the current context.
     var all_permissions = _global_permissions(),
         callbacks = [];

     /**
      * The main method for ACL on the front end. It asks
      * if the current user has this permission.
      * The permissions are defined in Zope
      *
      * NOTE: The permissions are not case-sensitive here
      * @returns True/False if the user has permission or not
      **/
     Zenoss.Security.hasPermission = function(permission) {
         // uses all_permissions as a closure
         return all_permissions[permission.toLowerCase()];
     };

     /**
      * Asks if the current user has this permission in the
      * global context.
      * @returns True/False if the user has the permission or not
      **/
     Zenoss.Security.hasGlobalPermission = function(permission) {
         return _global_permissions()[permission.toLowerCase()];
     };

     /**
      * Convenience method, it makes the Hide and Disable properties
      * easier to read.
      * For instance:
      *      config {
      *           hidden: Zenoss.Security.doesNotHavePermission('Manage DMD');
      *      };
      * @returns True if the user does NOT have permission
      **/
     Zenoss.Security.doesNotHavePermission = function(permission) {
         return (!this.hasPermission(permission));
     };

     /**
      * Add an callback to be executed every time the permissions
      * are changed, usually this is done by changing context.
      * Example Usage:
      *     Zenoss.Security.onPermissionChange(function() {
      *         Ext.getCmp('foo').setDisabled(Zenoss.Security.doesNotHavePermission('Manage DMD'));
      * });
      *@param callback: function to execute
      *@param scope[Optional]: scope of the callback
      **/
     Zenoss.Security.onPermissionsChange = function(callback, scope) {
         if (scope) {
             callbacks.push(Ext.bind(callback, scope));
         }else {
             callbacks.push(callback);
         }
     };

     /**
      * If the context you are working on changes call this
      * method to update the security permissions for that new context
      **/
     Zenoss.Security.setContext = function(uid) {
         var params = {
             uid:uid
         };
         function callback(response) {
             var i;
             if (response.success) {
                 all_permissions = response.data;
                 if (callbacks) {
                     for (i = 0; i < callbacks.length; i += 1) {
                         callbacks[i]();
                     }
                 }
             }
         }
         Zenoss.remote.DetailNavRouter.getSecurityPermissions(params, callback);

     };

}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function() {
    Ext.ns('Zenoss.Action');

    Ext.define('Zenoss.PermissionableAction', {
        externallyDisabled: false,
        permitted: false,
        filtered: false,
        updateDisabled: function() {
            this.setDisabled(this.checkDisabled());
        },
        checkDisabled: function() {
            if (!this.permitted) {
                return true;
            }
            if (this.externallyDisabled) {
                return true;
            }
            if (!this.permitted && !this.filtered) {
                return true;
            }
            return false;
        },
        checkPermitted: function() {
            if (this.permission) {
                if (this.permissionContext) {
                    if (Zenoss.env.PARENT_CONTEXT === this.permissionContext) {
                        return Zenoss.Security.hasPermission(this.permission);
                    } else {
                        return Zenoss.Security.hasGlobalPermission(this.permission);
                    }
                } else {
                    return !Zenoss.Security.doesNotHavePermission(this.permission);
                }
            } else {
                return true;
            }
        },
        setPermission: function(config) {
            var me = this;
            var recheck = function() {
                me.permitted = me.checkPermitted();
                me.updateDisabled();
            };
            // if they set the permissions config property
            // and the logged in user does not have permission, disable this element
            if (config.permission) {
                this.permission = config.permission;
                // register the control to be disabled or enabled based on the current context
                if (config.permissionContext) {
                    this.permissionContext = config.permissionContext;
                    Zenoss.Security.onPermissionsChange(recheck);
                } else {
                    // update when the context changes
                    Zenoss.Security.onPermissionsChange(recheck, this);
                }
            }
            this.permitted = this.checkPermitted();
        }
    });

    function setDisabled(disable) {
        this.externallyDisabled = disable;
        this.callParent([this.checkDisabled()]);
    }

    Ext.define("Zenoss.Action", {
        extend: "Ext.menu.Item",
        alias: ['widget.Action'],
        mixins: {
            permissions: 'Zenoss.PermissionableAction'
        },
        constructor: function(config){
            this.setPermission(config);
            this.filtered = config.filtered;
            config.disabled = this.checkDisabled();
            this.callParent([config]);
        },
        setDisabled: setDisabled
    });

    Ext.define("Zenoss.ActionButton", {
        extend: "Ext.button.Button",
        alias: ['widget.buttonaction'],
        mixins: {
            permissions: 'Zenoss.PermissionableAction'
        },
        constructor: function(config){
            this.setPermission(config);
            this.filtered = config.filtered;
            config.disabled = this.checkDisabled();
            this.callParent([config]);
        },
        setDisabled: setDisabled
    });
}());
(function(){

var _tm = {
    'IpAddress':        ["^/zport/dmd/.*/ipaddresses/[^/]*/?$"],
    'IpInterface':      ["^/zport/dmd/Devices/.*/devices/.*/os/interfaces/.*"],
    'Device':           ["^/zport/dmd/.*/devices/[^/]*/?$"],
    'DeviceLocation':   ["^/zport/dmd/Locations/"],
    'DeviceSystem':     ["^/zport/dmd/Systems/"],
    'DeviceGroup':      ["^/zport/dmd/Groups/"],
    'DeviceClass':      ["^/zport/dmd/Devices(/(?!devices)[^/]+)*/?$"],
    'EventClass':       ["^/zport/dmd/Events(/[A-Za-z][^/]*)*/?$"],
    'Network':          ["^/zport/dmd/Networks(/(?!ipaddresses)[^/]+)*/?$"],
    'Process':          ["^/zport/dmd/Processes(.*)$"],
    'HyperlinkTag':     ["href=[\"\'](.*?)[\"\']"]
};

var T = Ext.ns('Zenoss.types');

Ext.apply(T, {

    TYPES: {},

    getAllTypes: function() {
        var result = [];
        for (var k in T.TYPES) {
            if (true) {
                result.push(k);
            }
        }
        return result;
    }, // getAllTypes

    type: function(uid) {
        var _f;
        for (var type in T.TYPES) {
            if (T.TYPES[type]) {
                _f = true;
                Ext.each(T.TYPES[type], function(test) {
                    if (!_f) {
                        return;
                    }
                    _f = test.test(uid);
                });
                if (_f) {
                    return type;
                }
            }
        }
        return null;
    }, // getType

    register: function(config) {
        function addRegex(k, t) {
            var types = T.TYPES[k] = T.TYPES[k] || [];
            if (!(t instanceof RegExp)) {
                t = new RegExp(t);
            }
            if (!(t in types)) {
                types.push(t);
            }
        }
        for (var k in config) {
            var t = config[k];
            if (Ext.isString(t)) {
                addRegex(k, t);
            } else if (Ext.isArray(t)) {
                Ext.each(t, function(r) {
                    addRegex(k, r);
                });
            }

        }
    } // register

}); // Ext.apply

T.register(_tm);

})(); // End local scope
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, 2012, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss.render');

// templates for the events renderer
var iconTemplate = new Ext.Template(
    '<td class="severity-icon-small {severity} {cssclass}" title="{severity:uppercase()}: {acked} out of {total} events acknowledged">{total}</td>');
iconTemplate.compile();

var rainbowTemplate = new Ext.Template(
    '<table class="eventrainbow eventrainbow_cols_{count}"><tr>{cells}</tr></table>');
rainbowTemplate.compile();

var upDownTemplate = new Ext.Template(
    '<span class="status-{0}{2}">{1}</span>');
upDownTemplate.compile();

var ipInterfaceStatusTemplate = new Ext.Template(
    '<span title="Administrative / Operational">{adminStatus} / {operStatus}</span>');
ipInterfaceStatusTemplate.compile();

function convertToUnits(num, divby, unitstr, places){
    unitstr = unitstr || "B";
    places = Ext.isDefined(places) ? places : 2;
    divby = divby || 1024.0;
    var units = [];
    Ext.each(['', 'K', 'M', 'G', 'T', 'P'], function(p){
        units.push(p+unitstr);
    });
    var sign = 1;
    if (num < 0) {
        num = Math.abs(num);
        sign = -1;
    }
    var i;
    for (i=0;i<units.length;i++) {
        if (num<divby) {
            break;
        }
        num = num/divby;
    }
    return (num*sign).toFixed(places) + units[i];
}

function pingStatusBase(bool) {
    /*
     * The "bool" variable can be null, undefined, a string, or a boolean.
     * We need to handle all cases and also make sure they are
     * handled in proper order.
     */
    if (bool === null || !Ext.isDefined(bool)) {
        return 'Unknown';
    }

    if(Ext.isString(bool)){
        if(bool.toLowerCase() === "none"){
            return 'Unknown';
        }else{
            bool = bool.toLowerCase() === 'up';
        }
    }

    var str = bool ? 'Up' : 'Down';
    return str;
}

Ext.apply(Zenoss.render, {

    conditionalEscaping: function(data) {
        if (!Zenoss.settings.enableHtmlInEventFields)
            return Ext.htmlEncode(data);
        else
            return data;
    },

    bytesString: function(num) {
        return num===0 ? '0' :convertToUnits(num, 1024.0, 'B');
    },

    memory: function(mb) {
        return (mb === 0) ? '0' : convertToUnits(mb, 1024.0, 'B', 2);
    },

    link_speed: function(bps) {
      return (bps===0) ? '0': convertToUnits(bps, 1000.0, 'bps', 0);
    },

    cpu_speed: function(speed) {
        if (speed) {
            var n = parseFloat(speed);
            if (isNaN(n)) {
                return speed;
            } else {
                return convertToUnits(n, 1000, 'Hz', 2);
            }
        } else {
            return speed;
        }
    },

    checkbox: function(bool) {
        if (bool) {
            return '<input type="checkbox" checked="true" disabled="true">';
        } else {
            return '<input type="checkbox" disabled="true">';
        }
    },

    pingStatus: function(bool) {
        var str = pingStatusBase(bool);
        return upDownTemplate.apply([str.toLowerCase(), str]);
    },

    pingStatusLarge: function(bool) {
        var str = pingStatusBase(bool);
        return upDownTemplate.apply([str.toLowerCase(), str, '-large']);
    },

    upDownUnknown: function(status,displayString){
        return upDownTemplate.apply([status.toLowerCase(),displayString]);
    },

    upDownUnknownLarge: function(status,displayString){
        return upDownTemplate.apply([status.toLowerCase(),displayString,'-large']);
    },

    ipInterfaceStatus: function(ifStatus) {
        return ipInterfaceStatusTemplate.apply(ifStatus);
    },

    ipAddress: function(ip) {
        if (Ext.isObject(ip)) {
            ip = ip.name;
        }
        if (!ip||ip==='0.0.0.0') {
            return '';
        }
        return Ext.isString(ip) ? ip : Zenoss.util.num2dot(ip);
    },

    severity: function(sev) {
        return '<div class="severity-icon-small '+
            Zenoss.util.convertSeverity(sev) +
            '"'+'><'+'/div>';
    },

    // renders availability as a percentage with 3 digits after decimal point
    availability: function(value) {
        return Ext.util.Format.number(value*100, '0.000%');
    },

    evstatus: function(evstatus) {
        if (!evstatus){
            return '';
        }
        return '<span class="status-icon-small status-icon-small-'+evstatus.toLowerCase()+'"><'+'/span>';
    },

    events: function(value, count) {
        var result = '',
            sevs = ['critical', 'error', 'warning', 'info', 'debug', 'clear'],
            cssclass = '',
            total,
            acked;
        count = count || 3;
        Ext.each(sevs.slice(0, count), function(severity) {
            total = value[severity].count;
            acked = value[severity].acknowledged_count;
            cssclass = (total===0) ? 'no-events' : (total===acked) ? 'acked-events' : '';
            result += iconTemplate.apply({
                severity: severity,
                total: total,
                acked: acked,
                cssclass: cssclass
            });
        });
        return rainbowTemplate.apply({cells: result, count: count});
    },
    worstevents: function(value) {
        var result = '',
            sevs = ['critical', 'error', 'warning', 'info', 'debug', 'clear'],
            cssclass = '',
            total,
            acked;
        Ext.each(sevs, function(severity) {
            if (value[severity] && value[severity].count && !result) {
                total = value[severity].count;
                acked = value[severity].acknowledged_count;
                cssclass = (total===acked) ? 'acked-events' : '';
                result = iconTemplate.apply({
                    severity: severity,
                    total: total,
                    acked: acked,
                    cssclass: cssclass
                });
            }
        });
        return rainbowTemplate.apply({cells: result});
    },

    locking: function(obj) {
        /*
        * Expects an object with keys updates, deletion, events, all boolean
        */
        var from = [];
        if (obj.updates) {
            from.push('updates');
        }
        if (obj.deletion) {
            from.push('deletion');
        }
        if (!Ext.isEmpty(from)) {
            var l = _t('Locked from ') + from.join(_t(' and '));
            if (obj.events) {
                l += "<br/>"+_t("Send event when blocked");
            } else {
                l += "<br/>"+_t("Do not send event when blocked");
            }
            return l;
        } else {
            return _t('Unlocked');
        }
    },

    locking_icons: function(obj) {
        /*
        * Expects an object with keys updates, deletion, events, all boolean
        * Returns images representing locking status.
        */
        var tpl = new Ext.Template(
            '<img border="0" src="locked-{str}-icon.png"',
            'style="vertical-align:middle"/>'),
            result = '';
        tpl.compile();
        if (obj.updates) {
            result += tpl.apply({str:'update'});
        }
        if (obj.deletion) {
            result += tpl.apply({str:'delete'});
        }
        if (obj.events) {
            result += tpl.apply({str:'sendevent'});
        }
        return result;
    },

    /*
     * Given a uid, determines the type of the object and passes rendering
     * off to the appropriate rendering function.
     * e.g. Zenoss.render.link('/zport/dmd/Devices/Server') =>
     * <a href="/zport/dmd/itinfrastructure#devices:/Devices/Server/Linux">...
     *
     * Can also just accept a url and name for wrapping in an anchor tag, by
     * passing in null for the first argument.
     */
    link: function(uid, url, name) {
        if (!url) {
            var dflt = 'default_uid_renderer',
                type = Zenoss.types.type(uid) || dflt,
                renderer = Zenoss.render[type];
            if (renderer) {
                return renderer(uid, name);
            }
        }
        if (url && name) {
            return '<a class="z-entity" href="'+url+'">'+Ext.htmlEncode(name)+'</a>';
        }
        if (url) {
           return url;
        }
    },

    default_uid_renderer: function(uid, name) {
        // Just straight up links to the object.
        var parts;
        if (!uid) {
            return uid;
        }
        if (Ext.isObject(uid)) {
            name = uid.name;
            uid = uid.uid;
        }
        if (!name) {
            parts = uid.split('/');
            name = parts[parts.length-1];
        }
        return Zenoss.render.link(null, uid, name);
    },

    linkFromGrid: function(value) {
        var item;
        if (Ext.isObject(value)) {
            item = value;
            if(item === null){
                return value;
            }else if(item.url) {
                return Zenoss.render.link(null, item.url, item.text);
            }else if(item.uid) {
                return Zenoss.render.link(item.uid, null, item.text);
            }
            return Ext.htmlEncode(item.text);
        }
        return Ext.htmlEncode(value);
    },

    LinkFromGridGuidGroup: function(name) {
        if (!name) {
            return name;
        }

        var url, results = [];
        Ext.each(name, function(item){
            url = "/zport/dmd/goto?guid=" + item.uuid;
            results.push(Zenoss.render.link(null, url, item.name));
        });

        return results.join(" | ");
    },

    LinkFromGridUidGroup: function(name) {
        if (!name) {
            return name;
        }

        var results = [];
        Ext.each(name, function(item) {
            results.push(Zenoss.render.default_uid_renderer(item.uid, item.name));
        });

        return results.join(" | ");
    },

    componentLinkFromGrid: function(obj, col, record) {
        if (!obj) {
            return;
        }

        if (typeof(obj) === 'string') {
            obj = record.data;
        }

        if (!obj.title && obj.name) {
            obj.title = obj.name;
        }

        if (this.subComponentGridPanel || this.componentType !== obj.meta_type) {
            return '<a href="javascript:Ext.getCmp(\'component_card\').componentgrid.jumpToEntity(\''+obj.uid+'\', \''+obj.meta_type+'\');">'+obj.title+'</a>';
        }
        return obj.title;
    },

    Device: function(uid, name, fragment) {
        if (!Ext.isString(fragment)) {
            fragment = 'deviceDetailNav:device_overview';
        }
        // For now, link to the old device page
        return Zenoss.render.link(null, uid+'/devicedetail#'+fragment, name);
    },

    DeviceClass: function(uid, name) {
        var value = uid.replace(/^\/zport\/dmd\/Devices/, '');
        value = value.replace(/\/devices\/.*$/, '');
        var url = '/zport/dmd/itinfrastructure#devices:.zport.dmd.Devices' + value.replace(/\//g,'.');
        if (!Ext.isString(name)) {
            name = value;
        }
        return Zenoss.render.link(null, url, name);
    },

    DeviceLocation: function(uid, name) {
        var value = uid.replace(/^\/zport\/dmd\/Locations/, '');
        value = value.replace(/\/devices\/.*$/, '');
        var url = '/zport/dmd/itinfrastructure#locs:.zport.dmd.Locations' + value.replace(/\//g,'.');
        if (!Ext.isString(name)) {
            name = value;
        }
        return Zenoss.render.link(null, url, name);
    },

    DeviceGroup: function(uid, name) {
        var value = uid.replace(/^\/zport\/dmd\/Groups/, '');
        value = value.replace(/\/devices\/.*$/, '');
        var url = '/zport/dmd/itinfrastructure#groups:.zport.dmd.Groups' + value.replace(/\//g,'.');
        if (!Ext.isString(name)) {
            name = value;
        }
        return Zenoss.render.link(null, url, name);
    },
    DeviceSystem: function(uid, name) {
        var value = uid.replace(/^\/zport\/dmd\/Systems/, '');
        value = value.replace(/\/devices\/.*$/, '');
        var url = '/zport/dmd/itinfrastructure#systemsTree:.zport.dmd.Systems' + value.replace(/\//g,'.');
        if (!Ext.isString(name)) {
            name = value;
        }
        return Zenoss.render.link(null, url, name);
    },

    DeviceComponent: function(name, col, record) {
        var item = record.data[col.id];
        if (item.uid){
            // TODO once these pages are built fix the link
            return Zenoss.render.default_uid_renderer(item.uid, item.text);
        }
        return item.text;
    },

    EventClass: function(uid, name) {
        return Zenoss.render.default_uid_renderer(uid, name);
    },

    IpServiceClass: function(value, metadata, record) {
        // this is intended to set directly as a column renderer instead of
        // using Types.js. See the Ext.grid.ColumnModel.setRenderer
        // documentation
        var uid = record.data.serviceClassUid.replace(/\//g, '.');
        return Zenoss.render.serviceClass('ipservice', uid, value);
    },

    WinServiceClass: function(value, metadata, record) {
        // this is intended to set directly as a column renderer instead of
        // using Types.js. See the Ext.grid.ColumnModel.setRenderer
        // documentation
        var uid = record.data.serviceClassUid.replace(/\//g, '.');
        return Zenoss.render.serviceClass('winservice', uid, value);
    },

    serviceClass: function(page, uid, name) {
        var url = Ext.String.format('/zport/dmd/{0}#navTree:{1}', page, uid);
        return Zenoss.render.link(null, url, name);
    },

    nextHop: function(value){
        var link = "";
        if (value && value.uid && value.id) {
            link += Zenoss.render.IpAddress(value.uid, value.id);
            if (value.device && value.device.uid && value.device.id) {
                link += " (";
                link += Zenoss.render.Device(value.device.uid, value.device.id);
                link += ")";
            }
        }
        return link;
    },

    IpInterface: function(uid, name) {
        var deviceUid = uid.split('/os/interfaces/')[0];
        var url = deviceUid + '/devicedetail#deviceDetailNav:IpInterface:' + uid;
        return Zenoss.render.link(null, url, name);
    },

    IpAddress: function(uid, name) {
        return Zenoss.render.Network(uid, name);
    },

    Network: function(uid, name) {
        var url = '/zport/dmd/networks#networks:' + uid.replace(/\//g, '.');
        if (!name) {
            var parts = uid.split('/');
            name = parts[parts.length-1];
        }
        return Zenoss.render.link(null, url, name);
    },

    Process: function(uid, name) {
        var url = '/zport/dmd/process#processTree:' + uid.replace(/\//g, '.');
        if (!name) {
            var parts = uid.split('/');
            name = parts[parts.length-1];
        }
        return Zenoss.render.link(null, url, name);
    },

    HyperlinkTag : function(uid, name) {
        //When we get hyperlink tag return it back
        return Zenoss.render.link(null, uid, null);
    },

    eventSummaryRow:function (data, metadata, record){
        var msg = record.data.message;
        if (!msg || msg === "None" ) {
            msg = record.data.summary;
        }
        msg = Zenoss.render.conditionalEscaping(msg);
        msg = "<pre style='white-space:normal;'>" + msg + "</pre>";
        msg = msg.replace(/\"/g, '&quot;');
        metadata.tdAttr = 'data-qtip="' + Zenoss.render.conditionalEscaping(msg) + '" data-qwidth="500"';

        data = Zenoss.render.conditionalEscaping(data);
        return data;
    },

    zProperty: function(value, record) {
        // if value is an object or array, it must be stringified via JSON
        if (typeof value === "object") {
            value = JSON.stringify(value);
        }
        var severityFields = ['zEventSeverity', 'zFlappingSeverity'];
        if (severityFields.includes(record.data.id)) {
            if (value === -1) {
                value = 'Default (-1)';
            } else {
                value = Ext.String.format('{0} ({1})', Zenoss.util.convertSeverity(value), value);
                value = value.charAt(0).toUpperCase() + value.slice(1);
            }
        }
        return Ext.htmlEncode(value);
    }


}); // Ext.apply

})(); // End local namespace
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function () {
Ext.ns('Zenoss');

Ext.define("Zenoss.FlexButton", {
    extend: "Ext.button.Button",
    alias: ['widget.FlexButton'],
    constructor: function(config) {
        if ( config.menu && config.menu.items && config.menu.items.length === 1 ) {
            // Only has one menu item, lets just not have it display a menu
            config.originalMenu = Ext.apply({}, config.menu);

            // probably don't want to inherit the text
            var menuConfig = config.menu.items[0];
            Ext.apply(config, {listeners: menuConfig.listeners});
            Ext.destroyMembers(config, 'menu');
        }

        Zenoss.FlexButton.superclass.constructor.call(this, config);
    },
    add: function(config) {
        if ( !this.menu ) {
            // this button does not have a menu yet so we need to initialize it
            // update the config with things that may have changed since creation
            var menuConfig = {};
            if ( this.initialConfig.originalMenu ) {
                // originally had a menu, just use it
                Ext.apply(menuConfig, this.initialConfig.originalMenu);

            }
            else {
                // Have to generate a menu config from the original button
                menuConfig.items = [{
                    text: this.getText() ? this.getText() : this.tooltip,
                    listeners: this.initialConfig.listeners
                }];

            }

            this.split = true;
            // remove the old click handler
            var oldClick = this.initialConfig.listeners.click;
            this.un('click', oldClick);

            // Clear out properties that should be handled by the menu item
            this.on('render', function() {
                this.events['click'].clearListeners();
            }); // *TODO* This code makes me feel dirty; there must be a better way
            this.clearTip();

            this.menu = Ext.menu.MenuMgr.get(menuConfig);
            this.up('panel').doLayout();
        }

        return this.menu.add(config);
    }
});

})();
(function(){

Ext.ns('Zenoss', 'Zenoss.i18n');

// Provide a default; this gets filled in later when appropriate.
Zenoss.i18n._data = Zenoss.i18n._data || {};

Zenoss.i18n.translate = function(s, d) {
    var t = Zenoss.i18n._data[s];
    return t ? t : (d ? d: s);
};

// Shortcut
window._t = Zenoss.i18n.translate;
 
})();
/*
 * Tooltips.js
 */
(function(){ // Local scope
/*
 * Zenoss.registerTooltip
 *
 * Make QuickTips also accept a target component or component ID to attach a
 * tooltip to. It will attempt to discover the correct Ext.Element to which it
 * should attach the tip.
 *
 * Accepts the same config options as the Ext.ToolTip constructor.
 *
 */

Zenoss.TIPS = {};

/*
* Zenoss.ToolTip
* Causes tooltips to remain visible if the mouse is over the tip itself,
* instead of hiding as soon as the mouse has left the target.
*/
Zenoss.ToolTip = Ext.extend(Ext.ToolTip, {
   constructor: function(config) {
       Zenoss.ToolTip.superclass.constructor.call(this, config);
       this.on('render', this.attachHoverEvents, this);
   },
   attachHoverEvents: function() {
       var el = this.getEl();
       el.on('mouseenter', this.onMouseEnter, this);
       el.on('mouseleave', this.onTargetOut, this);
   },
   onMouseEnter: function() {
       var component = Ext.getCmp(this.target.id);

       // if the target has a visible menu,
       // allow the tooltip to autohide so that
       // it does not get in the way
       if(component && component.menu && !component.menu.hidden){
           return;
       } else {
           this.clearTimer('hide');
       }
   }
});

Zenoss.registerTooltip = function(config) {
    var target = config.target,
        initialConfig = Ext.apply({}, config),
        cmp = Ext.getCmp(target);
    if (Ext.isDefined(cmp)) {
        cmp.on('destroy', function(){
            Zenoss.TIPS[target] = initialConfig;
        });
    }

    if (Ext.get(target)) {
        var tip = new Zenoss.ToolTip(config);
        Zenoss.TIPS[target] = tip;
    } else {
        Zenoss.TIPS[target] = initialConfig;
    }
}; // Zenoss.registerTooltip

/*
 * Zenoss.registerTooltipFor
 *
 * Looks up any tooltips for a component id and registers them. Used for items
 * that don't exist when Zenoss.registerTooltip was called.
 *
 * If a tooltip is already registered, don't reregister it.
 *
 */
Zenoss.registerTooltipFor = function(target) {
    var t;
    if (Ext.isDefined(t = Zenoss.TIPS[target])) {
        if (!(t instanceof Ext.ToolTip)) {
            Zenoss.registerTooltip(t);
        }
    }
}; // Zenoss.registerTooltipFor


})(); // End local scope
(function(){
    Ext.ns('Zenoss.VTypes');

    var ipv4_regex = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$");
    var ipv6_regex = new RegExp("^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$");
    var ip_with_netmask_regex = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(/[0-9]+)$");
    var hex_regex = new RegExp("^#?([a-f]|[A-F]|[0-9]){3}(([a-f]|[A-F]|[0-9]){3,5})?$");
    var numcmp_regex = new RegExp("^(\>=|\<=|\>|\<|=)?\s*([0-9]+)$");
    var numrange_regex = new RegExp("^(-?[0-9]+)?:?(-?[0-9]+)?$");
    var range_regex = new RegExp("^([^:]*):?([^:]*)$");
    var alpha_num_space = new RegExp(/[a-z_\s\d]/i);
    var hostname_or_ip_regex = new RegExp(/[a-zA-Z0-9-_.\(\)\/:]/);

    /**
     * These are the custom validators defined for
     * our zenoss forms. The xtype/vtype custom is to have the
     * name of the vtype all lower case with no word separator.
     *
     **/
    var vtypes = {
        /**
         * Allows int comparisons like 4,>2,<=5
         */
        numcmp: function(val) {
            return numcmp_regex.test(val);
        },
        numcmpText: _t('Enter a valid comparison (ex: 4, <2, >=1)'),

        numrange: function(val) {
            var result, from, to;
            result = numrange_regex.exec(val);
            if (!result) {
                return false;
            }
            from = result[1];
            to = result[2];
            if (from && to) {
                return parseInt(from) <= parseInt(to);
            }
            return true;
        },
        numrangeText: _t('Enter a valid numeric range (ex: 2, 5:, 6:8, :9)'),

        floatrange: function(val) {
            var result, from, to;
            result = range_regex.exec(val);
            if (!result) {
                return false;
            }
            from = result[1];
            to = result[2];
            if (from && to) {
                from = parseFloat(from);
                to = parseFloat(to);
                return !isNaN(from) && !isNaN(to) && from <= to;
            }
            if (from) {
                return !isNaN(parseFloat(from));
            }
            if (to) {
                return !isNaN(parseFloat(to));
            }
            return true;
        },
        floatrangeText: _t('Enter a valid floating point range (ex: 1, 2.5:5.5, 2.5:, :9.5'),

        /**
         * The number must be greater than zero. Designed for us in NumberFields
         **/
        positive: function(val) {
            return (val >= 0);
        },
        positiveText: _t('Must be greater than or equal to 0'),

        /**
         * Between 0 and 1 (for float types)
         **/
        betweenzeroandone: function(val) {
            return (val >= 0 && val <=1);
        },
        betweenzeroandoneText: _t('Must be between 0 and 1'),
        ipaddress: function(val) {
            return ipv4_regex.test(val) || ipv6_regex.test(val);
        },
        ipv6address: function(val) {
            return ipv6_regex.test(val);
        },
        ipaddressText: _t('Invalid IP address'),

        ipaddresswithnetmask: function(val) {
            return ip_with_netmask_regex.test(val);
        },
        ipaddresswithnetmaskText: _t('You must enter a valid IP address with netmask.'),

        /**
         * Hex Number (for colors etc)
         **/
        hexnumber: function(val) {
            return hex_regex.test(val);
        },
        hexnumberText: _t('Must be a 6 or 8 digit hexadecimal value.'),

        /**
         * Modifies alpha number to allow spaces
         **/
        alphanumspace: function(val) {
            return alpha_num_space.test(val);
        },
        alphanumspaceText: _t('Must be an alphanumeric value or a space '),
        alphanumspaceMask: alpha_num_space,

        hostnameorIP: function(val) {
            return hostname_or_ip_regex.test(val);
        },
        hostnameorIPText: _t('Must be a valid hostname or IP address '),
        hostnameorIPMask: hostname_or_ip_regex
    };

    Ext.apply(Ext.form.VTypes, vtypes);
}());
/*
 * Utility for keeping track of navigation among subcomponents on a page and
 * restoring that state on page load.
 */
Ext.onReady(function(){

    var H = Ext.util.History;
    H.DELIMITER = ':';
    H.selectByToken = function(token) {
        if(token) {
            var parts = token.split(H.DELIMITER),
            mgr = Ext.getCmp(parts[0]),
            remainder = parts.slice(1).join(H.DELIMITER);
            if (mgr) {
                mgr.selectByToken(remainder);
            }
        }
    };
    H.events = H.events || {};
    H.addListener('change', H.selectByToken);

});
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2009, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.define("Zenoss.DisplayField", {
    extend: "Ext.form.DisplayField",
    alias: ['widget.displayfield'],
    constructor: function(config) {
        Ext.applyIf(config, {
            fieldClass: 'display-field'
        });
        Zenoss.DisplayField.superclass.constructor.call(this, config);
    },
    setRawValue: function(v) {
        if (v && Ext.isIE && typeof v === 'string') {
            v = v.replace(/\n/g, '<br/>');
        }
        Zenoss.DisplayField.superclass.setRawValue.call(this, v);
    }
});

Ext.define("Zenoss.EditorWithButtons", {
    extend: "Ext.Editor",
    alias: ['widget.btneditor'],
    onRender: function() {
        Zenoss.EditorWithButtons.superclass.onRender.apply(this, arguments);
        this.editorpanel = new Ext.Panel({
            frame: true,
            bodyStyle: Ext.isIE ? 'padding-bottom:5px' : '',
            buttonAlign: 'left',
            buttons: [{
                text:_t('Save'),
                handler: Ext.bind(this.completeEdit, this)
            }, {
                text:_t('Cancel'),
                handler: Ext.bind(this.cancelEdit, this)
            }],
            items: this.field
        });
        if (Ext.isIE) {
            // IE can't set the layer width properly; always gets set to 11189
            this.el.setWidth(450);
        }
        this.editorpanel.render(this.el).show();
    },
    onBlur: function(){
        // do nothing
    }
});

Ext.define("Zenoss.EditableField", {
    extend: "Zenoss.DisplayField",
    alias: ['widget.editable'],
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            cls: 'editable-field',
            overCls: 'editable-field-over',
            editEvent: 'click'
        });
        config.editor = Ext.applyIf(config.editor||{}, {
            xtype: 'btneditor',
            updateEl: !Ext.isIE,
            ignoreNoChange: true,
            autoSize: 'width',
            field: {
                xtype: 'textfield',
                selectOnFocus: true
            }
        });
        Zenoss.EditableField.superclass.constructor.call(this, config);
        this.initEditor();
        this.on('render', function(){
            this.getEl().on(this.editEvent, this.startEdit, this);
        }, this);
    },
    initEditor: function() {
        if (!(this.editor instanceof Ext.Editor)) {
            this.editor = Ext.create(this.editor);
        }
        var ed = this.editor;
        ed.on('beforecomplete', this.onBeforeComplete, this);
        if (Ext.isIE) {
            ed.field.origSetValue = ed.field.setValue;
            ed.field.setValue = Ext.bind(function(v){
                v = v.replace(/<BR\>/g, '\n');
                return this.origSetValue(v);
            }, ed.field);
        }
    },
    getForm: function() {
        var ownerCt = this.ownerCt;
        while (ownerCt && !(ownerCt instanceof Ext.FormPanel)) {
            ownerCt = ownerCt.ownerCt;
        }
        return ownerCt;
    },
    onBeforeComplete: function(t, val, startVal) {
        var opts = Ext.apply({}, this.getForm().baseParams||{});
        opts[this.name] = val;
        this.getForm().api.submit(opts, function(result) {
            if (result.success) {
                this.setValue(result[this.name] || val);
            } else {
                this.setValue(startVal);
            }
        }, this);
    },
    startEdit: function(e, t) {
        if (!this.disabled) {
            this.editor.startEdit(t);
        }
    }
});


Ext.define("Zenoss.EditableTextarea", {
    extend: "Zenoss.EditableField",
    alias: ['widget.editabletextarea'],
    constructor: function(config) {
        config.editor = config.editor || {};
        config.editor.field = Ext.applyIf(config.editor.field||{}, {
            xtype: 'textarea',
            grow: true
        });
        Zenoss.EditableTextarea.superclass.constructor.call(this, config);
    }
});

})();
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {

Ext.ns('Zenoss.form');

Ext.define("Zenoss.form.FieldRow", {

    alias: ['widget.fieldrow'],
    extend: "Ext.Container",
    constructor: function(config) {
         Ext.apply(config, {
             layout: 'hbox',
             align: 'top'
         });

         var items = config.items;
         config.items = [];

         Ext.each(items, function(item) {
             var cfg = {
                xtype: 'container',
                autoHeight: true,
                minWidth: 10,
                width: item.width,
                fieldDefaults: {
                    labelAlign: item.labelAlign || 'top'
                },
                layout: 'anchor',
                items: [item]
             };
             if (!item.width) {
                 cfg.flex = item.flex || 1;
             }
             config.items.push(cfg);
         });

         Zenoss.form.FieldRow.superclass.constructor.call(this, config);

     }
 });

})();
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


(function() {

Ext.ns('Zenoss.form');

Ext.define("Zenoss.form.InheritField", {
    extend: "Zenoss.form.FieldRow",
    alias: ['widget.inheritfield'],
    constructor: function(config) {
        var item;

        // Take the first member of items
        if (Ext.isObject(config.items)) {
            item = config.items;
        } else {
            item = config.items[0];
        }

        // Add the checkbox
        this.inheritbox = new Ext.form.Checkbox({
            fieldLabel: _t('Inherit?'),
            width: 60
        });
        config.items = [this.inheritbox, item];
        Zenoss.form.InheritField.superclass.constructor.call(this, config);
        this.field = this.items.items[1].items.items[0];
    },
    inherit: function() {
        return this.inheritbox.checked;
    },
    getValue: function() {
        return {
            inherited: this.inherit(),
            value: this.field.getValue()
        };
    },
    setValue: function(value) {
        if (Ext.isObject(value)) {
            this.inheritbox.setValue(
                Ext.isDefined(value.inherited) ? value.inherited : false
            );
            value = value.value;
        } else {
            this.inheritbox.setValue(false);
        }
        this.field.setValue(value);
    }
 });


})();
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/*
A widget that handles the fact that zProperties can be acquired from parents
in the dmd object hierarchy. The strategy is to use a field set. The title of
the field set is a friendly name for the zProperty (not the z* name). Inside
the field set are two lines. The user selects a line as being the active
setting by clicking on the radio button at the beginning of the row. The top
line has a label that reads 'Set Local Value:', and an appropriate widget for
the type of the zProp (checkbox, textfield, numberfield, or combobox). The 2nd
line states 'Inherit Value' and then indicates the acquired value and the path
of the ancestor from which the value is acquired.

Example usage:

var zMonitor = Ext.create({
    xtype: 'zprop',
    name: 'zMonitor',
    title: _t('Is Monitoring Enabled?'),
    localField: {
        xtype: 'select',
        model: 'local',
        store: [[true, 'Yes'], [false, 'No']],
        value: true
    }
});

zMonitor.setValues({
    isAcquired: false,
    localValue: false,
    acquiredValue: 'No',
    ancestor: '/Processes/Apache'
});

*/

(function() {

Ext.ns('Zenoss.form');


/* A radio button that allows for the boxLabel to be updated.
 */
Ext.define("Zenoss.form.Radio", {
    extend: "Ext.form.Radio",
    alias: ['widget.zradio'],

    setBoxLabel: function(boxLabel) {
        if (this.rendered) {
            this.boxLabel = boxLabel;
            this.boxLabelEl.update(boxLabel);
        } else {
            this.boxLabel = boxLabel;
        }
    }

});

/* A hidden field used internally by the ZProperty fieldset. This hidden field
   overrides getValue to return an object with isAcquired and localValue.
   Introduces the zpropFieldSet config prop which is a reference to the
   ZProperty field set.
 */
Ext.define("Zenoss.form.ZPropHidden", {
    extend: "Ext.form.Hidden",
    alias: ['widget.zprophidden'],

    getValue: function() {
        return {
            isAcquired: this.zpropFieldSet.acquiredRadio.getValue(),
            localValue: this.zpropFieldSet.localField.getValue(),

            // acquiredValue and ancestor aren't needed by the server, but it
            // is needed by reset which is called when the form is submitted
            acquiredValue: this.zpropFieldSet.acquiredValue,
            ancestor: this.zpropFieldSet.ancestor
        };
    },
    getRawValue: function() {
        return this.getValue();
    },
    setValue: function(values) {
        this.zpropFieldSet.setValues(values);
    },

    isDirty: function() {
        return this.zpropFieldSet.acquiredRadio.isDirty() || this.zpropFieldSet.localField.isDirty();
    }

});




/*
A field set that represents a zProperty.

The config parameter passed into the constructor must have a ref. The refOwner
must be the FormPanel.

Additional config keys:
    localField - config for the Ext.form.Field used to input a local setting
    name - string that that is submited to the server as the name

New public method:
    setValues - upon context change in the client code, set all the values
                of this composite widget
 */
Ext.define("Zenoss.form.ZProperty", {
    extend: "Ext.form.FieldSet",
    alias: ['widget.zprop'],

    constructor: function(config) {
        Ext.applyIf(config, {
            hideLabels: true,
            hideBorders: true,
            border:false,
            defaults:{border:false},
            items: [
                this.getLocalRadioConfig(config.localField),
                this.getAcquiredRadioConfig(),
                this.getHiddenFieldConfig(config.name)
            ]
        });
        Zenoss.form.ZProperty.superclass.constructor.call(this, config);
    },
    setValues: function(values) {
        // values has isAcquired, localValue, acquiredValue, and ancestor
        // localValue is the appropriate type
        // acquiredValue is always a string

        // setting the values this away marks the form clean and disables the
        // submit and cancel buttons

        if (!values) {
            return;
        }
        var basicForm = this.refOwner.getForm();
        basicForm.setValues([
            {id: this.localRadio.getName(), value: !values.isAcquired},
            {id: this.localField.getName(), value: values.localValue},
            {id: this.acquiredRadio.getName(), value: values.isAcquired}
        ]);

        // update the boxLabel with the acquiredValue and ancestor
        var boxLabel;
        if ( values.acquiredValue !== null && values.ancestor !== null ) {
            boxLabel = Ext.String.format('Inherit Value "{0}" from {1}', values.acquiredValue, values.ancestor);
            this.acquiredRadio.enable();
        } else {
            boxLabel = Ext.String.format('Inherit Value');
            this.acquiredRadio.disable();
        }
        this.acquiredRadio.setBoxLabel(boxLabel);
        this.acquiredValue = values.acquiredValue;
        this.ancestor = values.ancestor;
    },

    // private
    getHiddenFieldConfig: function(name) {
        return {
            xtype: 'zprophidden',
            name: name,
            zpropFieldSet: this
        };
    },

    // private
    getAcquiredRadioConfig: function() {
        return {
            xtype: 'zradio',
            ref: 'acquiredRadio',
            boxLabel: 'Inherit Value',
            scope: this,
            anchor: '75%',
            handler: function(acquiredRadio, checked) {
                this.localRadio.setValue(!checked);
            }
        };
    },

    //private
    getLocalRadioConfig: function(localField) {
        return {
            xtype: 'panel',
            layout: 'column',
            hideBorders: true,
            border:false,
            defaults: {
                xtype: 'panel',
                layout: 'anchor',
                border:false,
                hideLabels: true
            },
            items: [{
                width: 130,
                items: [{
                    xtype: 'radio',
                    ref: '../../localRadio',
                    boxLabel: _t('Set Local Value:'),
                    checked: true,
                    scope: this,
                    handler: function(localRadio, checked) {
                        this.acquiredRadio.setValue(!checked);
                    }
                }]
            }, {
                columnWidth: 0.94,
                items: [
                    // Set submitValue to false in case localField has a name
                    Ext.apply(localField, {
                        ref: '../../localField',
                        submitValue: false,
                        anchor: '75%',
                        listeners: {
                            scope: this,
                            focus: function() {
                                this.localRadio.setValue(true);
                            }
                        }
                    })
                ]
            }]
        };
    }

});

// A simple ComboBox that behaves like an HTML select tag
Ext.define("Zenoss.form.Select", {
    extend: "Ext.form.ComboBox",
    alias: ['widget.select'],

    constructor: function(config){
        Ext.applyIf(config, {
            allowBlank: false,
            triggerAction: 'all',
            typeAhead: false,
            forceSelection: true
        });
        Zenoss.form.Select.superclass.constructor.call(this, config);
    }

});

})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function() {

Ext.ns('Zenoss.form');

Ext.define("Zenoss.form.IDField", {
    alias: ['widget.idfield'],
    extend: "Ext.form.TextField",
    anchor:'80%',
    /*
    * Context on which to check for id validity. Defaults to
    * Zenoss.env.PARENT_CONTEXT.
    */
    context: null,
    /*
    * Limit characters to those accepted by ObjectManager
    */
    maskRe: /[a-zA-Z0-9-_~,.$\(\)# @]/,
    _serverIsValid: false,
    /*
    * Validator function that makes a request to the parent context and calls
    * the checkValidId method.
    */
    validator: function(value) {
        var context = this.context || Zenoss.env.PARENT_CONTEXT;

        // Don't bother with empty values
        if (Ext.isEmpty(value)) {
            return true;
        }
        // if the value has not changed do not send an ajax request
        if(typeof this._previousValue !== 'undefined'){
            if (value === this._previousValue) {
                return this.reportResponse(this._previousResponseText);
            }
        }
        this._previousValue = value;

        if (this.vtransaction) {
            Ext.Ajax.abort(this.vtransaction);
        }
        function callback(response) {
            this._previousResponseText = response.responseText;
            return this.reportResponse(response.responseText);
        }
        this.vtransaction = Ext.Ajax.request({
            url: context + '/checkValidId?id='+value,
            method: 'GET',
            success: callback,

            failure: function(response) {
                this.markInvalid(_t('That name is invalid or is already in use.'));
                this.fireEvent('validitychange', this, false);
            },
            scope: this
        });
        return true;
    },
    validate: function() {
        var result = this.callParent(arguments);
        return result && this._serverIsValid;
    },
    /**
    * Interprets a response from the server to determine if this field is valid.
    **/
    reportResponse: function(responseText) {
        if (responseText === "True") {
            this.fireEvent('validitychange', this, true);
            this._serverIsValid = true;
            return true;
        }
        // the server responds with a string of why it is invalid
        this._serverIsValid = false;
        this.markInvalid(
            _t('That name is invalid: ') + ' ' + responseText
        );

        // let any event listeners know that we are invalid
        this.fireEvent('validitychange', this, false);

        return false;
    }
});

})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss', 'Zenoss.dialog');

/**
 * @class Zenoss.dialog.BaseWindow
 * @extends Ext.window.Window
 * A modal window with some defaults. Will auto focus on the first
 * textfield and make the enter key hit submit
 *
 **/
Ext.define("Zenoss.dialog.BaseWindow", {
    extend: "Ext.window.Window",
    alias: ['widget.basewindow'],
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            //layout: (Ext.isIE) ? 'form': 'fit',
            plain: true,
            buttonAlign: 'left',
            modal: true,
            constrain: true
        });
        Zenoss.dialog.BaseWindow.superclass.constructor.apply(this, arguments);
    },
    initEvents: function() {
        Zenoss.dialog.BaseWindow.superclass.initEvents.apply(this, arguments);
        this.on('show', this.focusFirstTextField, this);
        this.on('show', this.registerEnterKey, this);
    },
    focusFirstTextField: function() {
        // go through our items and find a text field

        var fields = this.query("textfield");
        if (fields.length) {
            fields[0].focus(false, 300);
        }
    },
    registerEnterKey: function() {
        var km = this.getKeyMap();
        km.on(13, function(){
            var button, el;

            // make sure we are not focused on text areas
            if (document.activeElement) {
                el = document.activeElement;
                if (el.type === "textarea" || el.type === 'button') {
                    return;
                }
            }

            // the button is on the window
            var buttons = this.query("button");
            if (buttons && buttons.length) {
                button = buttons[0];
            }else{
                // the button is on a form
                var forms = this.query('form');
                if (forms) {
                    if (forms.length && forms[0]){
                        var form = forms[0];
                        if (form.query("button").length) {
                            button = form.query("button")[0];
                        }
                    }
                }
            }

            if (button && !button.disabled){
                button.handler(button);
            }
        }, this);

    },
    /**
     * Make sure any dialog appears above loadmasks.
     **/
    show: function() {
        var oldSeed = Ext.WindowManager.zseed;
        Ext.WindowManager.zseed = 20000;
        this.callParent(arguments);
        Ext.WindowManager.zseec = oldSeed;
    }

});



/**
 * @class Zenoss.dialog.BaseDialog
 * @extends Zenoss.dialog.BaseWindow
 * A modal dialog with Zenoss styling. Subclasses should specify a layout.
 * @constructor
 */
Ext.define("Zenoss.dialog.BaseDialog", {
    extend: "Zenoss.dialog.BaseWindow",
    constructor: function(config) {
        Ext.applyIf(config, {
            width: 310,
            closeAction: 'hide',
            buttonAlign: 'left',
            padding: 10,
            autoHeight: true
        });
        this.callParent([config]);
    }
});

function destroyWindow(button) {
    var win = button.up('window');
    if (win){
        return win.destroy();
    }
    if (button.ownerCt !== undefined){
        var container = button.ownerCt.ownerCt;
        if (container.ownerCt !== undefined){
            container.ownerCt.destroy();
        }else{
            container.destroy();
        }
    }
}

/**
 * @class Zenoss.dialog.HideDialogButton
 * @extends Ext.Button
 * A button that destroys its window.
 * @constructor
 */
Ext.define("Zenoss.dialog.DialogButton", {
    ui: 'dialog-dark',
    extend: "Ext.button.Button",
    alias: ['widget.DialogButton'],
    constructor: function(config) {
        var h = config.handler;
        config.handler = h ? Ext.Function.createSequence(h, destroyWindow) : destroyWindow;
        Zenoss.dialog.DialogButton.superclass.constructor.call(this, config);
    },
    setHandler: function(handler, scope) {
        var h = handler ? Ext.Function.createSequence(handler, destroyWindow) : destroyWindow;
        Zenoss.dialog.DialogButton.superclass.setHandler.call(this, h, scope);
    }
});



function hideWindow(button){
    button.ownerCt.ownerCt.hide();
}

/**
 * @class Zenoss.dialog.HideDialogButton
 * @extends Ext.Button
 * A button that hides it's window.
 * @constructor
 */
Ext.define("Zenoss.dialog.HideDialogButton", {
    ui: 'dialog-dark',
    extend: "Ext.button.Button",
    alias: ['widget.HideDialogButton'],
    constructor: function(config) {
        var h = config.handler;
        config.handler = h ? Ext.Function.createSequence(h, hideWindow) : hideWindow;
        Zenoss.dialog.HideDialogButton.superclass.constructor.call(this, config);
    },
    setHandler: function(handler, scope) {
        var h = handler ? Ext.Function.createSequence(handler, hideWindow) : hideWindow;
        Zenoss.dialog.HideDialogButton.superclass.setHandler.call(this, h, scope);
    }

});



Zenoss.dialog.CANCEL = {
    xtype: 'DialogButton',
    text: _t('Cancel')
};

/**
 * @class Zenoss.MessageDialog
 * @extends Zenoss.dialog.BaseDialog
 * A modal dialog window with Zenoss styling and a fit layout.  This window
 * meant to be instantiated once per page, and hidden each time the user
 * closes it.  Includes an OK and Cancel button.
 * @constructor
 */
Ext.define("Zenoss.MessageDialog", {
    extend: "Zenoss.dialog.BaseDialog",
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'fit',
            items: [ {
                xtype: 'label',
                text: config.message,
                ref: 'messagelabel'
                } ],
            buttons: [
                {
                    xtype: 'HideDialogButton',
                    text: _t('OK'),
                    handler: config.okHandler
                }, {
                    xtype: 'HideDialogButton',
                    text: _t('Cancel'),
                    handler: config.cancelHandler
                }
            ]
        });
        Zenoss.MessageDialog.superclass.constructor.call(this, config);
    },
    setText: function(text) {
        this.messagelabel.setText(text);
    }
});

/**
 * @class Zenoss.dialog.SimpleMessageDialog
 * @extends Zenoss.dialog.BaseDialog
 * A modal dialog window with Zenoss styling and a fit layout. No buttons are
 * included
 * @constructor
 */
Ext.define("Zenoss.dialog.SimpleMessageDialog", {
    extend: "Zenoss.dialog.BaseDialog",
    /**
     * message to be displayed on dialog
     * @param {Object} config
     */
    message: null,
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'anchor',
            items: [{
                html: config.message
            },{
                // add a spacer between the text and the buttons so it is not squished together
                height: 10
            }],
            closeAction: 'destroy'
        });
        Zenoss.MessageDialog.superclass.constructor.call(this, config);
    }
});

/**
 * @class Zenoss.FormDialog
 * @extends Zenoss.dialog.BaseWindow
 * A modal dialog window with Zenoss styling and a form layout.  This window
 * meant to be instantiated multiple times per page, and destroyed each time
 * the user closes it.
 * @constructor
 */
Ext.define("Zenoss.FormDialog", {
    extend: "Zenoss.dialog.BaseWindow",
    constructor: function(config) {
        var me = this;
        config = config || {};
        config.formListeners = config.formListeners || {};
        Ext.applyIf(config.formListeners, {
            validitychange: function(form, isValid) {
                me.query('DialogButton')[0].setDisabled(!isValid);
            }
        });
        var form = new Ext.form.FormPanel({
            id: config.formId,
            minWidth: 300,
            ref: 'editForm',
            fieldDefaults: {
                labelAlign: 'top'
            },
            autoScroll: true,
            defaults: {
                xtype: 'textfield'
            },
            items: config.items,
            html: config.html,
            listeners: config.formListeners,
            paramsAsHash: true,
            api: config.formApi || {}
        });

        // Remove config properties that don't pertain to the window
        Ext.destroyMembers(config, 'items', 'formApi', 'formId', 'html');

        Ext.applyIf(config, {
            // ie renders window correctly on when layout is set to form
            // this may change in future ext/ie version
            //layout: (Ext.isIE) ? 'form': 'fit',
            plain: true,
            buttonAlign: 'left',
            autoScroll: true,
            width: 375,
            modal: true,
            padding: 10
        });

        Zenoss.FormDialog.superclass.constructor.call(this, config);

        this.add(form);
        this.form = form;
    },

    getForm: function() {
        return this.form;
    }
});


Ext.define("Zenoss.dialog.CloseDialog",{
    extend: "Zenoss.dialog.BaseWindow",
    constructor: function(config) {
        Ext.applyIf(config, {
            width: 310,
            plain: true,
            layout: 'auto',
            buttonAlign: 'left',
            padding: 10,
            modal: true
        });
        Zenoss.dialog.CloseDialog.superclass.constructor.call(this, config);
    }
});


/**
 * @class Zenoss.HideFormDialog
 * @extends Zenoss.dialog.BaseDialog
 * A modal dialog window with Zenoss styling and a form layout.  This window
 * meant to be instantiated once per page, and hidden each time the user
 * closes it.
 * @constructor
 */
Ext.define("Zenoss.HideFormDialog", {
    extend: "Zenoss.dialog.BaseDialog",
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'anchor',
            fieldDefaults: {
                labelAlign: 'top'
            }
        });
        Zenoss.HideFormDialog.superclass.constructor.call(this, config);
    }
});

/**
 * @class Zenoss.SmartFormDialog
 * @extends Zenoss.FormDialog
 * A modal dialog window with Zenoss styling and a form layout.  This window
 * meant to be instantiated once and then thrown away after use.
 *
 * It smartly cleans up it's own form items when "hidden" and provides a better
 * handler mechanism for the callback, returning an object with properties of
 * all form values.
 * @constructor
 */
Ext.define("Zenoss.SmartFormDialog", {
    extend: "Zenoss.FormDialog",
    alias: 'widget.smartformdialog',
    message: '',
    submitHandler: null,
    constructor: function(config) {
        var me = this;
        this.listeners = config.listeners;
        Ext.applyIf(this.listeners, {
            validitychange: function(form, isValid) {
                var btn = me.query("button[ref='buttonSubmit']")[0];
                btn.setDisabled(!isValid);
            }
        });
        Ext.applyIf(config, {
            buttons: [{
                xtype: 'DialogButton',
                text: _t('Submit'),
                disabled: true,
                type: 'submit',
                ref: 'buttonSubmit'
             }, {
                xtype: 'DialogButton',
                ref: 'buttonCancel',
                text: _t('Cancel')
            }],
            modal: true,
            closeAction: 'destroy'
        });

        Zenoss.SmartFormDialog.superclass.constructor.call(this, config);

        if ( config.message || this.message ) {
            this.insert(0, {
                xtype: 'label',
                ref: 'label',
                text: config.message || this.message
            });
        }
    },
    setSubmitHandler: function(callbackFunction) {
        var btn = this.query("button[ref='buttonSubmit']")[0];

        if (callbackFunction === null) {
            btn.setHandler(null);
        }
        else {
            btn.setHandler(Ext.bind(function() {
                var form = this.getForm();
                var values = form.getValues();
                return callbackFunction(values);
            }, this));
        }
    },
    initComponent: function() {
        this.callParent();
        if (this.submitHandler) {
            this.setSubmitHandler(this.submitHandler);
            delete this.submitHandler;
        }
        this.on('show', function() {
            this.getForm().getForm().checkValidity();
        }, this);
    }
});



/**
 * @class Zenoss.dialog.ErrorDialog
 * @extends Zenoss.dialog.BaseDialog
 * A modal dialog window with Zenoss styling and a fit layout.
 * @constructor
 */
Ext.define("Zenoss.dialog.ErrorDialog", {
    extend: "Zenoss.dialog.BaseDialog",
    title:_t('Error'),
    message: null,
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'fit',
            cls:'errorbox',
            items: [{
                html: config.message,
                buttons: [{
                    xtype: 'DialogButton',
                    text: _t('OK')
                }]
            }],
            closeAction: 'destroy'
        });
         this.callParent([config]);
    },
    initComponent: function(){
        this.callParent(arguments);
        this.show();
    }
});


/**
 * @class Zenoss.HideFitDialog
 * @extends Zenoss.dialog.BaseWindow
 * A modal dialog window with Zenoss styling and a fit layout.
 * @constructor
 */
Ext.define("Zenoss.HideFitDialog", {
    extend: "Zenoss.dialog.BaseWindow",
    constructor: function(config) {
        Ext.applyIf(config, {
            width: 600,
            height: 300,
            closeAction: 'hide',
            plain: true,
            buttonAlign: 'left',
            padding: 10,
            modal: true
        });
        Zenoss.HideFitDialog.superclass.constructor.call(this, config);
    }
});

/**
 * Works in conjunction with Zenoss.dialog.DialogFormPanel. Loads a formPanel
 * with ID diynamic-dialog-panel and submits the form on the panel when the
 * submit button on this dialog is pressed.
 */
Ext.define("Zenoss.dialog.DynamicDialog", {
    extend: "Zenoss.dialog.BaseDialog",
    initEvents: function(){
        Zenoss.dialog.DynamicDialog.superclass.initEvents.call(this);
        this.body.getLoader().on('failure', function(el, response) {
            el.update("Failed to load dialog");
        });
    },
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            layout: 'fit',
            modal: true,
            stateful: false,
            closeAction: 'close',
            minHeight: '150',
            buttons: [{
                xtype: 'DialogButton',
                text: _t('Submit'),
                handler: Ext.bind(this.submitHandler, this)
            },{
                xtype: 'DialogButton',
                text: _t('Cancel')
            }]
        });
        Ext.apply(config, {
            id: 'dynamic-dialog'
        });
        Zenoss.dialog.DynamicDialog.superclass.constructor.call(this, config);
    },
    submitHandler: function(b, event){
        var formPanel = Ext.getCmp('dynamic-dialog-panel');
        var form = formPanel.getForm();
        var params = {};
        if (Ext.isDefined(formPanel.submitName) && formPanel.submitName !== null){
            params[formPanel.submitName] = 'OK';
        }
        form.submit({
            params: params,
            success: Ext.bind(function(form, action){
                Zenoss.message.success(_t('{0} finished successfully'), this.title);
            }, this),
            failure: function(form, action){
                Zenoss.message.error(_t('{0} had errors'), this.title);
            }
        });
    }
});

/**
 * Used to create dialogs that will be added dynamically
 */
Ext.define("Zenoss.dialog.DialogFormPanel", {
    extend: "Ext.form.FormPanel",
    alias: ['widget.DialogFormPanel'],
    /**
     * whether or not the result of submitting the form associated with this
     * panel will return a json result. Default true
     */
    jsonResult: true,
    /**
     * extra parameter to send when submitted
     */
    submitName: null,
    /**
     * name of an existing from to be submitted; if not defined a new form
     * will be created.  Primarily used for backwards compatibility so existing
     * dialog forms don't have to be entirely rewritten.
     */
    existingFormId: null,
    constructor: function(config) {
        config = config || {};
        Ext.apply(config, {
            id: 'dynamic-dialog-panel'
        });
        Zenoss.dialog.DialogFormPanel.superclass.constructor.call(this, config);
    },
    /**
     * private; override from base class so that a basic form can be created
     * to point at an existing from if configured and also set a different
     * response reader if expected result from submit is not JSON
     */
    createForm: function(){
        var config = Ext.applyIf({listeners: {}}, this.initialConfig);
        if (!this.jsonResult) {
            config.errorReader = {
                read: function(xhr) {
                    var success = true;
                    //TODO scan result for exceptions/errors
                    if (xhr.status !== 200) {
                        success = false;
                    }
                    return {
                        records: [],
                        success: success
                    };
                }
            };
        }
        var formId = null;
        if (this.existingFormId !== null){
            formId = this.existingFormId;
        }
        return new Ext.form.BasicForm(formId, config);
    }
});



})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {
    Ext.namespace('Zenoss.form');
    /**
     * Config Options:
     * - record (required) - the record we are editing, it is attached to each item
     * - items - received from the Schema declaration
     * - directFn - The server call when we press Save
     * - saveHandler - the router callback from after we save
     **/
    Ext.define("Zenoss.form.DataSourceEditDialog", {
        extend: "Zenoss.dialog.BaseWindow",
        alias: ['widget.datasourceeditdialog'],
        constructor: function(config) {
            config = config || {};

            // verify we received the record we are editing
            var record = config.record,
                items = config.items,
                i,
                autoHeight = true,
                that = this; // used in the save handler closure
            this.itemCount = 0;
            // make sure the record was passed in
            if (!record) {
                throw "EditDialog did not recieve a record to edit (config.record is undefined)";
            }

            if (!config.singleColumn) {
                items = this.sortItems(items, record);
            }else{
                // the items come in the form of a single fieldset
                // and we can not have a fieldset because it looks ugly in a dialog
                items = items.items[0].items;
                for(i=0; i<items.length; i+=1) {
                    items[i].record = record;
                    this.itemCount += 1;
                    // datapointitemselectors are tall
                    if (items[i]['xtype'] === 'datapointitemselector') {
                        this.itemCount += 4;
                    }
                }
            }

            // Since ext has no maxheight property, we need to only use autoheight if we don't
            // have a lot of items. Otherwise the dialog expands off the screen.
            if (this.itemCount > 10) {
                autoHeight = false;
            }

            Ext.apply(config, {
                layout: 'anchor',
                plain: true,
                buttonAlign: 'left',
                autoScroll: true,
                constrain: true,
                modal: true,
                padding: 10,
                height: 500,
                autoHeight: autoHeight,
                items: [{
                    xtype:'form',
                    minWidth: 300,
                    ref: 'editForm',
                    fieldDefaults: {
                        labelAlign: 'top'
                    },
                    autoScroll:true,

                    defaults: {
                        xtype: 'textfield',
                        anchor: '85%'
                    },
                    listeners: {
                        /**
                         * Sets the windows submit button to be disabled when the form is not valid
                         **/
                        validitychange: function(formPanel, valid){
                            var dialogWindow = that;
                            // check security first
                            if (Zenoss.Security.hasPermission('Manage DMD')) {
                                dialogWindow.submitButton.setDisabled(!valid);
                            }
                        }
                    },
                    items: items

                }],
                buttons: [{
                    xtype: 'DialogButton',
                    ref: '../submitButton',
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    text: _t('Save'),
                    handler: function () {
                        var form = that.editForm.form,
                            dirtyOnly = true,
                            values = form.getFieldValues(dirtyOnly);

                        values.uid = record.uid;
                        config.directFn(values, config.saveHandler);
                    }
                },
                    Zenoss.dialog.CANCEL
                ]

            });
            Zenoss.form.DataSourceEditDialog.superclass.constructor.apply(this, arguments);
        },

        /**
         * This function returns a two column layout with the fields auto divided between the
         * column according to the order they are defined.
         * If you specify a group in your datasource schema, they will appear in that "panel" on this dialog
         **/
        sortItems: function(fieldsets, record) {
            var panel = [], i, j, currentPanel,
            item, fieldset, tmp, header, textareas;

            // items comes back from the server in the form
            // fieldsets.items[0] = fieldset
            fieldsets = fieldsets.items;

            // The datasources have a convention to where the first items are
            // ungrouped.
            // This section makes sure the non-titled one is first.
            // (It swaps the titled first fieldset with the untitled fieldset)
            if (fieldsets[0].title) {
                tmp = fieldsets[0];
                for (i=0; i < fieldsets.length; i += 1) {
                    if (!fieldsets[i].title) {
                        fieldsets[0] = fieldsets[i];
                        fieldsets[i] = tmp;
                        break;
                    }
                }
            }

            // this creates a new panel for each group of items that come back from the server
            for (i =0; i < fieldsets.length; i += 1) {
                fieldset = fieldsets[i];

                // format the title a little funny
                if (fieldset.title) {
                    header = {
                        xtype: 'panel',
                        layout: 'anchor',
                        html: '<br /><br /><h1>' + fieldset.title + '</h1>'
                    };
                }else {
                    header = null;
                }

                textareas = [];

                currentPanel = {
                    xtype:'panel',
                    layout: 'column',
                    items: [{
                        layout:'anchor',
                        border:false,
                        columnWidth: 0.5,
                        items: []
                    },{
                        layout:'anchor',
                        border:false,
                        columnWidth: 0.5,
                        items: []
                    }]
                };

                // alternate which column this item belongs too
                for(j = 0; j < fieldset.items.length; j += 1) {
                    item = fieldset.items[j];
                    this.itemCount += 1;
                    item.record = record;

                    // we want to keep text areas to put them in a single column panel
                    if (item.xtype.search(/textarea/) >= 0) {
                        textareas.push(item);
                    }else{
                        currentPanel.items[j%2].items.push(item);
                    }
                }

                // if we have a header set display it
                if (header) {
                    panel.push(header);
                }

                // add the non-textarea fields
                panel.push(currentPanel);

                // put text areas in a single column layout
                if (textareas.length) {
                    for (j=0; j<textareas.length; j += 1) {
                        panel.push({
                            xtype: 'panel',
                            layout: 'anchor',
                            items: textareas[j]
                        });
                    }
                }

            }// fieldsets

            return panel;
        }
    });


}());
(function() {
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2012, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


Ext.ns('Zenoss.model');

    // Common fields used by all tree models
    Zenoss.model.BASE_TREE_FIELDS = [
        {
            name: 'hidden',
            type: 'boolean'
        },
        {
            name: 'leaf',
            type: 'boolean'
        },
        {
            name: 'uid',
            type: 'string'
        },
        {
            name: 'text'
        },
        {
            name: 'id',
            type: 'string'
        },
        {
            name: 'path',
            type: 'string'
        },
        {
            name: 'iconCls',
            type: 'string'
        },
        {
            name: 'uuid',
            type: 'string'
        }
    ];

    // A model defined which uses the default tree fields
    Ext.define("Zenoss.model.Tree", {
        extend: 'Ext.data.Model',
        fields: Zenoss.model.BASE_TREE_FIELDS
    });
}());
(function() {
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2012, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


Ext.ns('Zenoss.model');

    /**
     * Model that defines a uid and name
     **/
    Ext.define("Zenoss.model.Basic", {
        extend: 'Ext.data.Model',
        idProperty: 'uid',
        fields: ['uid', 'name']
    });
    
    /**
     * Model that defines a uid and label
     **/
    Ext.define("Zenoss.model.Label", {
        extend: 'Ext.data.Model',
        idProperty: 'uid',
        fields: ['uid', 'label']
    });    

    /**
     * Like the basic model but defined with the UUID instead of the uid
     **/
    Ext.define("Zenoss.model.BasicUUID", {
        extend: 'Ext.data.Model',
        idProperty: 'uuid',
        fields: ['uuid', 'name']
    });

    /**
     * Store that just defines a name
     **/
    Ext.define("Zenoss.model.Name", {
        extend: 'Ext.data.Model',
        idProperty: 'name',
        fields: ['name']
    });

    Ext.define("Zenoss.model.NameValue", {
        extend: 'Ext.data.Model',
        idProperty: 'value',
        fields: ['name', 'value']
    });

    Ext.define("Zenoss.model.IdName", {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: ['id', 'name']
    });

    Ext.define("Zenoss.model.ValueText", {
        extend: 'Ext.data.Model',
        idProperty: 'value',
        fields: ['value', 'text']
    });

}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2009, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function () {

    Ext.ns('Zenoss');

    /**
     * @class Zenoss.HierarchyTreePanel
     * @extends Ext.tree.TreePanel
     * The primary way of navigating one or more hierarchical structures. A
     * more advanced Subsections Tree Panel. Configurable as a drop target.
     * Accepts array containing one or more trees (as nested arrays). In at
     * least one case data needs to be asynchronous. Used on screens:
     *   Device Classification Setup Screen
     *   Devices
     *   Device
     *   Event Classification
     *   Templates
     *   Manufacturers
     *   Processes
     *   Services
     *   Report List
     * @constructor
     */



    /**
     * The default sort for hierarchical tree panels.
     * Show anything with a folder icon first and then sort Alpha.
     * by the "text" property.
     * To override this pass a custom sort function in the Tree Panel's
     * config.sortFn property.
     **/
    function sortTreeNodes(o1, o2) {
        function getText(object) {
            // text is sometimes an object and sometimes a string
            if (Ext.isObject(object.get('text'))) {
                return object.get('text').text.toLowerCase();
            }
            return object.get('text').toLowerCase();
        }

        function alphcmp(obj1, obj2) {
            var text1 = getText(obj1),
            text2 = getText(obj2);

            // sort by text
            if (text1 === text2) {
                return 0;
            }
            return text1 < text2 ? -1 : 1;
        }


        // always show folders first
        if (o1.get('iconCls') === 'folder' &&  o2.get('iconCls') !== 'folder'){
            return -1;
        }
        if (o2.get('iconCls') === 'folder' && o1.get('iconCls') !== 'folder') {
            return 1;
        }

        // otherwise sort by text
        return alphcmp(o1, o2);
    }

    Zenoss.sortTreeNodes = sortTreeNodes;


    /**
     * Base Tree Selection model for zenoss. Defines
     * the getSelectedNode method that existed in 3.X trees.
     **/
    Ext.define('Zenoss.TreeSelectionModel', {
        extend:'Ext.selection.TreeModel',
        getSelectedNode:function () {
            var selections = this.getSelection();
            if (selections.length) {
                return selections[0];
            }
            return null;
        }

    });

    Ext.define('Zenoss.HierarchyTreePanelSearch', {
        extend:'Ext.Panel',
        alias:['widget.HierarchyTreePanelSearch'],
        constructor:function (config) {
            var oldConfig = config;
            config = {
                cls:'x-hierarchy-search-panel',
                bodyStyle:'background-color:#d4e0ee;',
                items:[
                    {
                        xtype:'searchfield',
                        id:config.id || Ext.id(),
                        height: 25,
                        hidden:!Zenoss.settings.enableTreeFilters,
                        cls:'x-hierarchy-search',
                        enableKeyEvents:true,
                        ref:'searchfield'
                    },
                    {
                        xtype:'panel',
                        ui:'hierarchy',
                        padding:'5px 0 0 0',
                        items:oldConfig.items,
                        flex:1,
                        autoScroll:true,
                        regSearchListeners:function (listeners) {
                            this.ownerCt.query('.searchfield')[0].on(listeners);
                        }
                    }
                ],
                layout:{
                    type:'vbox',
                    align:'stretch'
                },
                listeners: {
                    afterrender: function(t){
                        // fixes 20000px width bug on the targetEl div bug in Ext
                        t.searchfield.container.setWidth(t.ownerCt.getWidth());
                    }
                }
            };

            Zenoss.HierarchyTreePanelSearch.superclass.constructor.call(this, config);
        }
    });

    /**
     *  Right click handlers for nodes.
     **/
    Zenoss.treeContextMenu = function (view, node, item, index, e, opti) {
        // Register the context node with the menu so that a Menu Item's handler function can access
        // it via its parentMenu property.
        var tree = view.panel;
        if (!tree.contextMenu) {
            tree.contextMenu = new Ext.menu.Menu({
                items:[
                    {
                        ref:'refreshtree',
                        text:_t('Refresh Tree'),
                        handler:function (item, e) {
                            var tree = item.parentMenu.tree;
                            tree.getStore().load({
                                callback:function (records, opts, success) {
                                    if (success) {
                                        var childNodes = opts.response.result[0];
                                        // Remove old records and update root node with the fresh data
                                        this.getRootNode().removeAll();
                                        this.getRootNode().appendChild(childNodes);
                                        if (this.getRootNode().childNodes.length) {
                                            this.getRootNode().childNodes[0].expand();
                                        }
                                    }
                                }
                            });
                        }
                    },
                    {
                        ref:'expandall',
                        text:_t('Expand All'),
                        handler:function (item, e) {
                            var tree = item.parentMenu.tree;
                            tree.expandAll();
                        }
                    },
                    {
                        ref:'collapsall',
                        text:_t('Collapse All'),
                        handler:function (item, e) {
                            var tree = item.parentMenu.tree;
                            tree.collapseAll();
                            // by default we usually expand the first child
                            tree.getRootNode().expand();
                            if (tree.getRootNode().childNodes.length) {
                                tree.getRootNode().childNodes[0].expand();
                            }
                        }
                    },
                    '-',
                    {
                        ref:'expandnode',
                        text:_t('Expand Node'),
                        handler:function (item, e) {
                            var node = item.parentMenu.contextNode;
                            if (node) {
                                node.expand(true, true);
                            }
                        }
                    },
                    {
                        ref:'newwindow',
                        text:_t('Open in New Window'),
                        handler:function (item, e) {
                            var node = item.parentMenu.contextNode,
                                tree, path,
                                href = window.location.protocol + '//' + window.location.host + window.location.pathname;
                            if (node && node.data.uid) {
                                tree = item.parentMenu.tree;
                                path = tree.createDeepLinkPath(node);
                                window.open(href + '#' + path);
                            }
                        }
                    }
                ]
            });
        }
        var c = tree.contextMenu;
        c.tree = tree;
        c.contextNode = node;
        e.preventDefault();
        c.showAt(e.getXY());
    };


    /**
     * @class Zenoss.HierarchyTreePanel
     * @extends Ext.tree.Panel
     * Base classe for most of the trees that appear on the left hand side
     * of various pages
     **/
    Ext.define('Zenoss.HierarchyTreePanel', {
        extend:'Ext.tree.Panel',
        alias:['widget.HierarchyTreePanel'],
        constructor:function (config) {
            Ext.applyIf(config, {
                enableDragDrop:true,
                loadMask: true
            });
            config.listeners = config.listeners || {};
            Ext.applyIf(config.listeners, {
                itemcontextmenu:Zenoss.treeContextMenu,
                scope:this
            });

            config.viewConfig = config.viewConfig || {};
            if (config.enableDragDrop) {
                var dd_permission = Zenoss.Security.hasPermission('Change Device');
                if (config.forceEnableDd) {
                    dd_permission = true;
                }
                Ext.applyIf(config.viewConfig, {
                    loadMask:config.loadMask,
                    plugins:{
                        ptype:'treeviewdragdrop',
                        enableDrag:dd_permission,
                        enableDrop:dd_permission,
                        ddGroup:config.ddGroup
                    }
                });
            } else {
                Ext.applyIf(config.viewConfig, {
                    loadMask:config.loadMask
                });
            }
            Ext.applyIf(config, {
                ui:'hierarchy',
                frame:false,
                useArrows:true,
                autoScroll:true,
                manageHeight: false,
                relationshipIdentifier:null,
                containerScroll:true,
                selectRootOnLoad:true,
                rootVisible:false,
                rootDepth:config.rootVisible ? 0 : 1,
                allowOrganizerMove:true,
                pathSeparator:"/",
                nodeIdSeparator:".",
                hideHeaders:true,
                columns:[
                    {
                        xtype:'treecolumn',
                        flex:1,
                        dataIndex:'text',
                        renderer:function (value, l, n) {
                            if (Ext.isString(value)) {
                                return value;
                            }
                            var parentNode = n.parentNode,
                                safeText = Ext.String.htmlEncode(value.text),
                                safeDescription = Ext.String.htmlEncode(value.description),
                                count;

                            if (Ext.isEmpty(value.count)) {
                                count = "";
                            } else {
                                count = Ext.String.format(" <span title='{0}'>({1})</span>", safeDescription, value.count);
                            }
                            if (parentNode.data.root === true) {
                                return Ext.String.format("<span class='rootNode'>{0}{1}</span>", safeText, count);
                            } else {
                                return Ext.String.format("<span class='subNode'>{0}</span>{1}", safeText, count);
                            }

                        }
                    }
                ]

            });
            if (config.router) {
                Ext.applyIf(config, {
                    addNodeFn:config.router.addNode,
                    deleteNodeFn:config.router.deleteNode
                });
            }
            else {
                Ext.applyIf(config, {
                    addNodeFn:Ext.emptyFn,
                    deleteNodeFn:Ext.emptyFn
                });
            }
            var root = config.root || {};
            if (config.directFn && !config.loader) {
                var modelId = Ext.String.format('Zenoss.tree.{0}Model', config.id);

                Ext.define(modelId, {
                    extend:'Ext.data.Model',
                    treeId:config.id,
                    idProperty:config.idProperty || 'id',
                    getOwnerTree:function () {
                        return Ext.getCmp(this.treeId);
                    },
                    /**
                     * Used by the tree store to determine what
                     * to send to the server
                     **/
                    getId:function () {
                        return this.get("uid");
                    },
                    proxy:{
                        simpleSortMode: true,
                        type:'direct',
                        directFn:config.directFn,
                        paramOrder:['uid']
                    },
                    fields:Zenoss.model.BASE_TREE_FIELDS.concat(config.extraFields || [])
                });
                config.store = new Ext.create('Ext.data.TreeStore', {
                    clearOnLoad: false,
                    model:modelId,
                    nodeParam:'uid',
                    defaultRootId:root.uid,
                    remoteSort: false,
                    sorters: {
                        sorterFn: config.sortFn || Zenoss.sortTreeNodes,
                        direction: 'asc'
                    },
                    uiProviders:{
                        // 'hierarchy': Zenoss.HierarchyTreeNodeUI
                    }
                });
                Ext.destroyMembers(config, 'directFn', 'ddGroup');
            }
            Ext.applyIf(root, {
                id:root.id,
                uid:root.uid,
                text:_t(root.text || root.id)
            });
            this.root = root;
            this.stateHash = {};
            if (config.stateful) {
                this.stateEvents = this.stateEvents || [];
                this.stateEvents.push('expandnode', 'collapsenode');
            }

            Zenoss.HierarchyTreePanel.superclass.constructor.apply(this, arguments);
        },
        setNodeVisible:function (nodeId, visible) {
            var node = this.getStore().getNodeById(nodeId),
                view = this.getView(),
                el = Ext.fly(view.getNodeByRecord(node));
            if (el) {
                el.setVisibilityMode(Ext.Element.DISPLAY);
                el.setVisible(visible);
            }
        },
        getState:function () {
            return {stateHash:this.stateHash};
        },
        applyState:function (state) {
            if (state) {
                Ext.apply(this, state);
                this.setStateListener();
            }
        },
        setStateListener:function () {
            this.store.on({
                load:{ scope:this, fn:function () {
                    for (var p in this.stateHash) {
                        if (this.stateHash.hasOwnProperty(p)) {
                            this.expandPath(this.stateHash[p]);
                        }
                    }
                }}
            });
        },
        initEvents:function () {
            var me = this;
            Zenoss.HierarchyTreePanel.superclass.initEvents.call(this);

            if (this.selectRootOnLoad && !Ext.History.getToken()) {
                this.getRootNode().on('expand', function () {
                    // The first child is our real root
                    if (this.getRootNode().firstChild) {
                        me.addHistoryToken(me.getView(), this.getRootNode().firstChild);
                        me.getRootNode().firstChild.expand();
                        me.getSelectionModel().select(this.getRootNode().firstChild);
                    }
                }, this, {single:true});
            } else {

                // always expand the first shown root if we can
                this.getRootNode().on('expand', function () {
                    if (this.getRootNode().firstChild) {
                        this.getRootNode().firstChild.expand();
                    }
                }, this, {single:true});
            }
            this.addEvents('filter');
            this.on('itemclick', this.addHistoryToken, this);
            this.on({
                beforeexpandnode:function (node) {
                    this.stateHash[node.id] = node.getPath();
                },
                beforecollapsenode:function (node) {
                    delete this.stateHash[node.id];
                    var tPath = node.getPath();
                    for (var t in this.stateHash) {
                        if (this.stateHash.hasOwnProperty(t)) {
                            if (-1 !== this.stateHash[t].indexOf(tPath)) {
                                delete this.stateHash[t];
                            }
                        }
                    }
                }
            });    // add some listeners for state
        },
        addHistoryToken:function (view, node) {
            Ext.History.add(this.id + Ext.History.DELIMITER + node.get('id'));
        },
        update:function (data) {
            function doUpdate(root, data) {
                Ext.each(data, function (datum) {
                    var node = root.findChild('id', datum.id);
                    if (node) {
                        node.data = datum;
                        node.setText(node.data.text);
                        doUpdate(node, datum.children);
                    }
                });
            }

            doUpdate(this.getRootNode(), data);

        },
        selectByToken:function (nodeId) {
            nodeId = unescape(nodeId);
            var root = this.getRootNode(),
                selNode = Ext.bind(function () {
                    var sel = this.getSelectionModel().getSelectedNode();

                    if (!(sel && nodeId === sel.id)) {
                        var path = this.getNodePathById(nodeId);
                        this.selectNodeByPath(path);
                    }
                }, this);

            if (!root.isLoaded()) {
                // Listen on expand because if we listen on the store's load expand
                // gets double-called.
                root.on('expand', selNode, this, {single:true});
            } else {
                selNode();
            }
        },

        /**
         * Expand the tree along the given path.
         *
         * This is a copy of the the expandNode function from ExtJS with
         * modifications to detect and skip path segments that do not
         * exist in the tree.
         *
         * @param {String} path The path to expand. The path should include a leading separator.
         * @param {String} [field] The field to get the data from. Defaults to the model idProperty.
         * @param {String} [separator='/'] A separator to use.
         * @param {Function} [callback] A function to execute when the expand finishes. The callback will be called with
         * (success, lastNode) where success is if the expand was successful and lastNode is the last node that was expanded.
         * @param {Object} [scope] The scope of the callback function
         */
        expandNodeByPath: function(path, field, separator, callback, scope) {
            var me = this,
                current = me.getRootNode(),
                index = 1,
                keys,
                expander;

            field = field || me.getRootNode().idProperty;
            separator = separator || '/';

            if (Ext.isEmpty(path)) {
                Ext.callback(callback, scope || me, [false, null]);
                return;
            }

            keys = path.split(separator);
            if (current.get(field) !== keys[1]) {
                Ext.callback(callback, scope || me, [false, current]);
                return;
            }

            expander = function() {
                if (++index === keys.length) {
                    Ext.callback(callback, scope || me, [true, current]);
                    return;
                }
                var node = current.findChild(field, keys[index]);
                if (node) {
                    current = node;
                }
                current.expand(false, expander);
            };
            current.expand(false, expander);
        },

        /**
         * Expand the tree to the path of a particular node, then select it.
         *
         * This is a copy of the selectNode function from ExtJS with the
         * modification to call the expandNodeByPath function defined above.
         *
         * @param {String} path The path to select. The path should include a leading separator.
         * @param {String} [field] The field to get the data from. Defaults to the model idProperty.
         * @param {String} [separator='/'] A separator to use.
         * @param {Function} [callback] A function to execute when the select finishes. The callback will be called with
         * (bSuccess, oLastNode) where bSuccess is if the select was successful and oLastNode is the last node that was expanded.
         * @param {Object} [scope] The scope of the callback function
         */
        selectNodeByPath: function(path, field, separator, callback, scope) {
            var me = this,
                root,
                keys,
                last;

            field = field || me.getRootNode().idProperty;
            separator = separator || '/';

            keys = path.split(separator);
            last = keys.pop();
            if (keys.length > 1) {
                me.expandNodeByPath(
                    keys.join(separator), field, separator,
                    function(success, node) {
                        var lastNode = node;
                        if (success && node) {
                            node = node.findChild(field, last);
                            if (node) {
                                me.getSelectionModel().select(node);
                                Ext.callback(callback, scope || me, [true, node]);
                                return;
                            }
                        }
                        Ext.callback(callback, scope || me, [false, lastNode]);
                    },
                    me);
            } else {
                root = me.getRootNode();
                if (root.getId() === last) {
                    me.getSelectionModel().select(root);
                    Ext.callback(callback, scope || me, [true, root]);
                } else {
                    Ext.callback(callback, scope || me, [false, null]);
                }
            }
        },

        /**
         * Given a nodeId this returns the full path to the node. By convention
         * nodeIds are the uid with a "." in place of a "/". So for example on the
         * infrastructure page this method would recieve ".zport.dmd.Device.Server" and
         * return "/Devices/.zport.dmd.Devices/.zport.dmd.Devices.Server"
         *
         * Override this method if your tree implements a custom path setup
         **/
        getNodePathById:function (nodeId) {
            var depth = this.root.uid.split('/').length - this.rootDepth,
                parts = nodeId.split(this.nodeIdSeparator),
                path = [],
                segment = Ext.Array.splice(parts, 0, depth + 1).join(this.nodeIdSeparator);

            // grab the first depth pieces of the id (e.g. .zport.dmd.Devices)
            path.push(this.initialConfig.root.id);
            // each segment of the path will have the previous segment as a piece of it
            path.push(segment);

            Ext.each(parts, function (piece) {
                // We need to skip the piece of the path that represents the
                // relationship between the organizer and the object:
                // e.g.: .zport.dmd.Something.SampleOrganizer.{relationshipIdentifier}.myObject
                // We do still need to add it to the segment that is reused for
                // each piece of the overall path.
                segment = segment + this.nodeIdSeparator + piece;
                if (piece !== this.relationshipIdentifier) {
                    path.push(segment);
                }
                else {
                    // stop iterating over the path once we've found the
                    // relationshipIdentifier, but make sure to push on the
                    // last 'chunk'.
                    // Trying to get something like this:
                    // foo.bar.baz.{relationshipIdentifier}.monkey =>
                    //     ["foo.bar.baz", "monkey"]
                    var idPartsWithoutRelationshipId = nodeId.split(
                            this.nodeIdSeparator + this.relationshipIdentifier + this.nodeIdSeparator);

                    if (idPartsWithoutRelationshipId.length > 1) {
                        path.push(segment + this.nodeIdSeparator + idPartsWithoutRelationshipId.pop());
                        // stop Ext.each iteration - this prevents generating
                        // segements for any further parts of the 'path' that
                        // may have been generated by splitting by the nodeIdSeparator
                        // such as in the case of mibs:
                        // .zport.dmd.Mibs.TestMib.mibs.1.2.4.5.6.6.7.2
                        return false;
                    }
                }
            }, this);
            return "/" + path.join(this.pathSeparator);
        },
        /**
         * This takes a node anywhere in the hierarchy and
         * will go back up to the parents and expand until it hits
         * the root node. This is a workaround for selectPath being nonfunctional
         * in Ext4
         *@param Ext.data.NodeInterface child
         **/
        expandToChild:function (child) {
            var parentNode = child.parentNode;

            // go back up and expand to this point
            while (parentNode) {
                // at the pseudo root nothing is further up
                if (Ext.isEmpty(parentNode.get('path'))) {
                    break;
                }

                if (!parentNode.isExpanded() && parentNode.hasChildNodes()) {
                    parentNode.expand();
                }
                parentNode = parentNode.parentNode;
            }
        },
        createDeepLinkPath:function (node) {
            var path = this.id + Ext.History.DELIMITER + node.get("uid").replace(/\//g, '.');
            return path;
        },
        afterRender:function () {
            Zenoss.HierarchyTreePanel.superclass.afterRender.call(this);
            var liveSearch = Zenoss.settings.enableLiveSearch,
                listeners = {
                    scope:this,
                    keypress:function (field, e) {
                        if (e.getKey() === e.ENTER) {
                            this.filterTree(field);
                        }
                    }
                };

            if (liveSearch) {
                listeners.change = this.filterTree;
            }
            if (this.searchField && this.ownerCt.regSearchListeners) {
                this.ownerCt.regSearchListeners(listeners);
            }
            this.getRootNode().expand(false, true, function (node) {
                node.expandChildNodes();
            });
        },
        expandAll:function () {
            // we have a hidden pseudo-root so we need to
            // expand all from the first visible root
            if (this.getRootNode().firstChild) {
                this.getRootNode().firstChild.expand(true);
            } else {
                this.callParent(arguments);
            }
        },
        filterTree:function (e) {
            if (!this.onFilterTask) {
                this.onFilterTask = new Ext.util.DelayedTask(function () {
                    this.doFilter(e);
                }, this);
            }

            this.onFilterTask.delay(1000);
        },
        postFilter: function(){
            var rootNode = this.getRootNode(),
                childNodes = rootNode.childNodes;


            // select the first leaf
            while (childNodes.length) {
                if (childNodes[0].childNodes.length) {
                    childNodes = childNodes[0].childNodes;
                } else {
                    break;
                }
            }

            this.getSelectionModel().select(childNodes[0]);

            // and then focus on back on the filter text
            this.up('HierarchyTreePanelSearch').down('searchfield').focus([false]);
        },
        getFilterFn: function(text) {
            var regex = new RegExp(Ext.String.escapeRegex(text),'i');
            var fn = function(item){
                // text can be either an object with the property text or a string
                var attr = item.get('text');
                if (Ext.isObject(attr)) {
                    attr = attr.text;
                }
                return regex.test(attr);
            };
            return fn;
        },
        doFilter:function (e) {
            var text = e.getValue(),
                store = this.getStore();
            store.clearFilter(true);

            this.fireEvent('filter', e);
            if (text) {
                store.filter(new Ext.util.Filter({
                    filterFn: this.getFilterFn(text)
                }));
            }
        },

        addNode:function (type, id) {
            this.addChildNode({type:type, id:id});
        },

        addChildNode:function (params) {
            var selectedNode = this.getSelectionModel().getSelectedNode();
            var parentNode;
            if (selectedNode.leaf) {
                parentNode = selectedNode.parentNode;
            } else {
                parentNode = selectedNode;
            }
            var contextUid = parentNode.data.uid;
            Ext.applyIf(params, {
                contextUid:contextUid
            });

            this.addTreeNode(params);
        },

        addTreeNode:function (params) {
            var callback = function (provider, response) {
                var result = response.result;
                var me = this;
                if (result.success) {
                    // look for another node on result and assume it's the new node, grab it's id
                    // TODO would be best to normalize the names of result node
                    var nodeId = Zenoss.env.PARENT_CONTEXT + '/' + params.id;
                    this.getStore().load({
                        callback:function () {
                            nodeId = nodeId.replace(/\//g, '.');
                            me.selectByToken(nodeId);
                        }
                    });

                    this.refresh();
                }
                else {
                    Ext.Msg.alert('Error', result.msg);
                }
            };

            this.addNodeFn(params, Ext.bind(callback, this));
        },

        deleteSelectedNode:function () {
            var node = this.getSelectionModel().getSelectedNode();
            var me = this;
            var parentNode = node.parentNode;
            var uid = node.get('uid');
            var params = {uid:uid};

            function callback(provider, response) {
                // Only update the UI if the response indicates success
                if (Zenoss.util.isSuccessful(response)) {
                    // Select the parent node since the current one was deleted.
                    me.getSelectionModel().select(parentNode);

                    // Refresh the parent node's tree to remove our node.
                    me.getStore().load({
                        callback:function () {
                            me.selectByToken(parentNode.get('uid'));
                            me.getRootNode().firstChild.expand();
                        }
                    });
                }
            }

            // all hierarchytreepanel's have an invisible root node with depth of 0
            if (node.getDepth() <= 1) {
                Zenoss.message.error(_t('You can not delete the root node'));
                return;
            }

            this.deleteNodeFn(params, callback);
            this.refresh();
        },

        canMoveOrganizer:function (organizerUid, targetUid) {
            var orgPieces = organizerUid.split('/'),
                targetPieces = targetUid.split('/');

            // make sure we can actually move organizers
            if (!this.allowOrganizerMove) {
                return false;
            }

            // Relying on a coincidence that the third item
            // is the top level organizer (e.g. Locations, Groups)
            return orgPieces[3] === targetPieces[3];
        },
        refresh:function (callback, scope) {
            // Disable ???Loading?????? mask during the request
            this.view.loadMask.disabled = true;
            this.getStore().load({
                scope:this,
                callback:function (records, opts, success) {
                    if (success) {
                        var childNodes = opts.response.result[0];
                        // Remove old records and update root node with the fresh data
                        this.getRootNode().removeAll();
                        this.getRootNode().appendChild(childNodes);
                        if (this.getRootNode().childNodes.length) {
                            this.getRootNode().childNodes[0].expand();
                        }
                    }
                }
            });
        }
    }); // HierarchyTreePanel

})();
Ext.ns('Zenoss');

/**
 * @class Zenoss.SearchField
 * @extends Ext.form.TextField
 * @constructor
 */
Ext.define("Zenoss.SearchField", {
    extend: "Ext.form.TextField",
    alias: ['widget.searchfield'],
    constructor: function(config){
        config = Ext.applyIf(config||{}, {
            validationDelay: 500,
            selectOnFocus: true
        });
        config.cls += ' x-field-search';
        Zenoss.SearchField.superclass.constructor.apply(this, arguments);
    },
    getClass: function(){
        var cls = this.altCls ? this.altCls : 'searchfield';
        return this.black ? cls + '-black' : cls;
    },
    onRender: function() {
        Zenoss.SearchField.superclass.onRender.apply(this, arguments);
        this.wrap = this.el.boxWrap(this.getClass());
        if (this.bodyStyle) {
            this.wrap.setStyle(this.bodyStyle);
        }
        this.resizeEl = this.positionEl = this.wrap;
        this.syncSize();
    },
    syncSize: function(){
        this.el.setBox(this.el.parent().getBox());
    }

}); // Ext.extend


/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {

Ext.ns('Zenoss.form');

Ext.define("Zenoss.form.LinkField", {
    extend: "Ext.form.DisplayField",
    alias: ['widget.linkfield'],
    initComponent: function() {
        this.callParent(arguments);
        // make sure our value is established
        // before rendering
        this.setValue(this.value);
    },
    getValue: function() {
        return this.rawValue;
    },
    setValue: function(value) {
        var origValue = value;
        if (Ext.isEmpty(value)) {
            value = _t('None');
        } else {
            if (Ext.isArray(value)){
                var items = [];
                Ext.each(value, function(v){
                    items.push(Zenoss.render.link(v));
                });
                value = items.join('<br/>');
            } else {
                value = Zenoss.render.link(value);
            }
        }
        this.callParent([value]);
        this.rawValue = origValue;
    }
 });


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function() {

var ZF = Ext.ns('Zenoss.form');

Ext.form.TextArea.prototype.grow = true;
Ext.form.TextArea.prototype.growMin = 20;


ZF.createDirectSubmitFunction = function(router) {

    // called in the Ext.form.Action.DirectSubmit.run method
    return function(formElDom, successFunction, directSubmitAction) {

        var form = directSubmitAction.form;

        // copy field values to the info object for fields that
        //   * are dirty (param passed to getFieldValues above)
        //   * have submitValue true
        //   * have an id or name set on them
        var info = {};
        var dirtyFieldValues = form.getValues(false, true);

        Ext.iterate(dirtyFieldValues, function(key, value) {
            var field = form.findField(key);
            if ( field.submitValue !== false && field.getName().indexOf("ext-gen") !== 0 ) {
                info[key] = value;
            }
        });


        // add the forms baseParams to the info object (often includes uid)
        Ext.applyIf(info, directSubmitAction.getParams());

        Ext.iterate(info, function(key, value) {
            if (key.indexOf("ext-gen") !== 0 ) {
                info[key] = value;
            }else {
                delete info[key];
            }
        });

        // define a callback to run after server responds
        var callback = function() {
            form.clearInvalid();
            form.setValues(dirtyFieldValues); // isDirty() will return false now
            form.afterAction(directSubmitAction, true);
            form.reset();
            Zenoss.message.info(_t("Details updated successfully"));
        };

        // the remote call
        router.setInfo(info, callback, directSubmitAction);

    };
};

Ext.define("Zenoss.form.BaseDetailForm", {
    extend: "Ext.form.FormPanel",
    alias: ['widget.basedetailform'],
    contextUid: null,
    isLoadInProgress: false,
    constructor: function(config){
        // Router doesn't technically matter, since they all do getInfo, but
        // getForm is definitely defined on DeviceRouter
        var router = config.router || Zenoss.remote.DeviceRouter;
        config.baseParams = Ext.applyIf(config.baseParams||{
            uid: config.contextUid
        });

        config = Ext.applyIf(config||{}, {
            paramsAsHash: true,
            autoScroll: 'y',
            cls: 'detail-form-panel',
            buttonAlign: 'left',
            fieldDefaults: {
                labelAlign: 'top'
            },
            trackResetOnLoad: true,
            permission: 'Manage Device',
            api: {
                submit: ZF.createDirectSubmitFunction(router),
                load: router.getInfo
            }
        });
        var hasPermission = function() {
            var perm = !Zenoss.Security.doesNotHavePermission(config.permission);
            if (Ext.isDefined(config.userCanModify)) {
                return config.userCanModify && perm;
            } else {
                return perm;
            }
        };
        if (hasPermission()) {
            Ext.apply(config, {
                buttons:  [{
                    xtype: 'button',
                    ref: '../savebtn',
                    text: _t('Save'),
                    disabled: true,
                    cls: 'detailform-submit-button',
                    handler: function(btn, e) {
                        this.refOwner.getForm().submit();
                    }
                },{
                    xtype: 'button',
                    ref: '../cancelbtn',
                    disabled: true,
                    text: _t('Cancel'),
                    cls: 'detailform-cancel-button',
                    handler: function(btn, e) {
                        this.refOwner.getForm().reset();
                    }
                }]
            });
        }
        ZF.BaseDetailForm.superclass.constructor.call(this, config);
    },
    initComponent: function() {
        this.callParent(arguments);
        var form = this.getForm();
        form.on('dirtychange', function(form, dirty, options ) {
            if (dirty && form.isValid()) {
                this.setButtonsDisabled(false);
            } else {
                this.setButtonsDisabled(true);
            }
        }, this);
        form.on('validityChange', function(f, valid, eOpts) {
            this.doButtons();
        }, this);
    },
    hasPermission: function() {
        var perm = !Zenoss.Security.doesNotHavePermission(this.permission);
        if (Ext.isDefined(this.userCanModify)) {
            return this.userCanModify && perm;
        } else {
            return perm;
        }
    },
    setButtonsDisabled: function(b) {
        if (this.hasPermission()) {
            this.savebtn.setDisabled(b);
            this.cancelbtn.setDisabled(b);
        }
    },
    doButtons: function() {
        this.setButtonsDisabled(!this.form.isDirty());
    },
    onFieldAdd: function(field) {
        if (!field.isXType('displayfield')) {
            this.mon(field, 'valid', this.doButtons, this);
        }
    },
    getFieldNames: function() {
        var keys = [];
        for (var k in this.getForm().getValues(false, false)) {
            if (Ext.Array.indexOf(keys, k)===-1) {
                keys.push(k);
            }
        }
        return keys;
    },
    load: function() {
        var o = Ext.apply({keys:this.getFieldNames()}, this.baseParams);
        this.form.load(o, function(result) {
            this.form.setValues(result.data);
            this.form.reset();
            this.doLayout();
        }, this);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.baseParams.uid = uid;
        this.isLoadInProgress = true;
        this.load();
    }
});

})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {

var ZF = Ext.ns('Zenoss.form');

Ext.define("Zenoss.form.AutoFormPanel", {
    extend: "Zenoss.form.BaseDetailForm",
    alias: ['widget.autoformpanel']
});



/*
* Zenoss.form.getGeneratedForm
* Accepts a uid and a callback.
* Asks the router for a form for the object represented by uid.
* Returns a config object that can be added to a container to render the form.
*/
ZF.getGeneratedForm = function(uid, callback, router) {
    // Router doesn't technically matter, since they all do getInfo, but
    // getForm is definitely defined on DeviceRouter
    router = router || Zenoss.remote.DeviceRouter;
    router.getForm({uid:uid}, function(response){
        callback(Ext.apply({
            xtype:'autoformpanel',
            contextUid: uid,
            layout: 'column',
            defaults: {
                layout: 'anchor',
                bodyStyle: 'padding:10px',
                fieldDefaults: {
                    labelAlign: 'top'
                },
                columnWidth: 0.5
            }
        }, response.form));
    });
};

Ext.define("Zenoss.form.AutoFormCombo", {
    extend: "Ext.form.ComboBox",
    alias: ['widget.autoformcombo'],
     constructor: function(config) {
         config = Ext.applyIf(config||{}, {
             editable: false,
             forceSelection: true,
             autoSelect: true,
             triggerAction: 'all',
             queryMode: 'local',
             store: config.values || []
         });
         Zenoss.form.AutoFormCombo.superclass.constructor.call(this, config);
     }

});


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2009, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');

function objectRenderer(obj) {
    if (obj) {
        return obj.name;
    }
    return "";
}
var deviceColumns = [
    {
        dataIndex: 'name',
        header: _t('Device'),
        id: 'name',
        flex: 1,
        hideable: false,
        renderer: function(name, row, record) {
            return Zenoss.render.Device(record.data.uid, name);
        }
    },{
        id: 'snmpSysName',
        dataIndex: 'snmpSysName',
        width: 150,
        hidden: true,
        header: _t('System Name')
    },{
        width: 100,
        dataIndex: 'ipAddress',
        header: _t('IP Address'),
        renderer: function(ip, row, record) {
            return record.data.ipAddressString;
        }
    },{
        dataIndex: 'uid',
        header: _t('Device Class'),
        width: 120,
        renderer: Zenoss.render.DeviceClass
    },{
        id: 'status',
        dataIndex: 'status',
        sortable: true,
        hidden: true,
        filter: {
            xtype: 'multiselect-devicestatus'
        },
        header: _t('Device Status'),
        renderer: function(status, row, record) {
            switch(record.data.status){
                case true: return Zenoss.render.pingStatus('Up');
                case false: return Zenoss.render.pingStatus('Down');
                default: return Zenoss.render.pingStatus(null);
            }
        },
        width: 80
    },{
        id: 'productionState',
        dataIndex: 'productionState',
        width: 100,
        filter: {
            xtype: 'multiselect-prodstate'
        },
        header: _t('Production State'),
        renderer: function(value) {
            return Zenoss.env.PRODUCTION_STATES_MAP[value];
        }

    },{
        id: 'serialNumber',
        dataIndex: 'serialNumber',
        width: 100,
        hidden: true,
        header: _t('Serial Number')
    },{
        id: 'tagNumber',
        dataIndex: 'tagNumber',
        width: 100,
        hidden: true,
        header: _t('Tag Number')
    },{
        id: 'hwManufacturer',
        dataIndex: 'hwManufacturer',
        width: 100,
        header: _t('Hardware Manufacturer'),
        hidden: true,
        renderer: objectRenderer
    },{
        id: 'hwModel',
        dataIndex: 'hwModel',
        hidden: true,
        width: 100,
        header: _t('Hardware Model'),
        renderer: objectRenderer
    },{
        id: 'osManufacturer',
        dataIndex: 'osManufacturer',
        width: 100,
        header: _t('OS Manufacturer'),
        hidden: true,
        renderer: objectRenderer
    },{
        id: 'osModel',
        dataIndex: 'osModel',
        width: 150,
        hidden: true,
        header: _t('OS Model'),
        renderer: objectRenderer
    },{
        dataIndex: 'collector',
        width: 100,
        hidden: true,
        header: _t('Collector')
    },{
        id: 'priority',
        dataIndex: 'priority',
        width: 100,
        hidden: true,
        filter: {
            xtype: 'multiselect-devicepriority'
        },
        header: _t('Priority'),
        renderer: function(value) {
            return Zenoss.env.PRIORITIES_MAP[value];
        }
    },{
        dataIndex: 'systems',
        width: 100,
        hidden: true,
        sortable: true,
        header: _t('Systems'),
        renderer: function(systems) {
            var links = [];
            if (systems) {
                Ext.each(systems, function(system){
                    links.push(Zenoss.render.DeviceSystem(system.uid, system.name));
                });
            }
            return links.join(" | ");
        }
    },{
        dataIndex: 'groups',
        width: 100,
        hidden: true,
        sortable: true,
        header: _t('Groups'),
        renderer: function(groups) {
            var links = [];
            if (groups) {
                Ext.each(groups, function(group){
                    links.push(Zenoss.render.DeviceGroup(group.uid, group.name));
                });
            }
            return links.join(" | ");

        }
    },{
        dataIndex: 'location',
        width: 100,
        hidden: true,
        sortable: false,
        header: _t('Location'),
        renderer: function(loc){
            if (loc){
                return Zenoss.render.DeviceLocation(loc.uid, loc.name);
            }
            return '';
        }
    },{
        id: 'worstevents',
        sortable: true,
        filter: false,
        width: 75,
        dataIndex: 'events',
        header: _t('Events'),
        renderer: function(ev, ignored, record) {
            var table = Zenoss.render.worstevents(ev),
            url = record.data.uid + '/devicedetail?filter=default#deviceDetailNav:device_events';
            if (table){
                table = table.replace('table', 'table onclick="location.href=\''+url+'\';"');
            }
            return table;
        }
    }
];

Ext.define("Zenoss.DeviceGridSelectionModel", {
    extend:"Zenoss.ExtraHooksSelectionModel",

    // Default to 'MULTI'-selection mode.
    mode: 'MULTI',

    /** _selectAll
     *
     * Set to true when the user has chosen the 'select all' records.
     *
     * @private
     * @type boolean
     */
    _selectAll: false,

    /** _excludedRecords
     *
     * Stores the set of records (IDs, hashes, ??) specifically
     * deselected from the set of all selected records.
     *
     * @private
     * @type dictionary
     */
    _excludedRecords: {},

    constructor: function(config) {
        this.callParent([config]);
        this.on('select', this._includeRecord, this);
        this.on('deselect', this._excludeRecord, this);
    },

    /** _includeRecord
     *
     * Include the record in the selection. Removes the record from the
     * set of excluded records.
     *
     * @private
     * @param sm {Zenoss.DeviceGridSelectionModel}
     * @param record {Zenoss.device.DeviceModel}
     * @param index {Integer}
     */
    _includeRecord: function(sm, record) {
        if (record && this._selectAll) {
            delete this._excludedRecords[record.getId()];
        }
    },

    /** _excludeRecord
     *
     * Exclude the record from the selection. Includes the record in the
     * set of excluded records.
     *
     * @private
     * @param sm {Zenoss.DeviceGridSelectionModel}
     * @param record {Zenoss.device.DeviceModel}
     * @param index {Integer}
     */
    _excludeRecord: function(sm, record) {
        if (record && this._selectAll) {
            this._excludedRecords[record.getId()] = true;
        }
    },

    /** _handleStoreDataChange
     *
     * Callback for handling changes to the grid's datastore.  When the
     * _selectAll flag is set, this function removes the current selection
     * and selects all the records currently in the datastore.
     *
     * @private
     */
    _handleStoreDataChange: function() {
        if (this._selectAll) {
            this.suspendEvents();
            var data = this.store.data.filterBy(
                    function(item) {
                        return (! this._excludedRecords[item.getId()]);
                    },
                    this
                );
            this.select(data.items, false, true);
            this.resumeEvents();
            this.fireEvent('selectionchange', this);
        }
    },

    /** bind
     *
     * The grid will use this method to bind the datastore to the
     * grid's selection model.
     *
     * @override
     * @param store {Ext.data.Store}
     * @param initial {boolean}
     */
    bind: function(store, initial){
        if (!initial && this.store) {
            if (store !== this.store && this.store.autoDestroy) {
                this.store.destroyStore();
            } else {
                this.store.un("datachanged", this._handleStoreDataChange, this);
            }
        }
        if (store) {
            store = Ext.data.StoreManager.lookup(store);
            store.on("datachanged", this._handleStoreDataChange, this);
        }
        this.store = store;
        if (store && !initial) {
            this.refresh();
        }
    },

    /** selectAll
     *
     * Sets the _selectAll flag before selecting 'all' the records.
     *
     * @override
     */
    selectAll: function() {
        this._selectAll = true;
        this.callParent([true]);
    },

    /** selectNone
     *
     * Unsets the _selectAll flag before deselecting all the records.
     */
    selectNone: function() {
        this._selectAll = false;
        // reset the set of excluded records to the empty set.
        this._excludedRecords = {};
        // Deselect all the records without firing an event for each
        // selected record.
        this.deselectAll([true]);
        this.fireEvent('selectionchange', this);
    },

    /** deselectAll
     *
     * Doesn't deselect anything if the _selectAll flag is set.
     * This works around other parts of the ExtJS framework that
     * invoke this method without passing suppressEvents = true.
     *
     * @override
     */
    deselectAll: function() {
        if (! this._selectAll) {
            this.callParent(arguments);
        }
    },

    /** clearSelections
     *
     * Clears out all the selections only if the _selectAll flag
     * is not set.
     *
     * @override
     */
    clearSelections: function() {
        if (this.isLocked() || this._selectAll) {
            return;
        }

        // Suspend events to avoid firing the whole chain for every row
        this.suspendEvents();

        // make sure all rows are deselected so that UI renders properly
        // base class only deselects rows it knows are selected; so we need
        // to deselect rows that may have been selected via selectstate
        this.deselect(this.store.data.items, true);

        // Bring events back and fire one selectionchange for the batch
        this.resumeEvents();

        this.fireEvent('selectionchange', this);

        this.callParent(arguments);
    }
});


Ext.define('Zenoss.device.DeviceModel',{
    extend: 'Ext.data.Model',
    fields: [
        {name: 'uid', type: 'string'},
        {name: 'name', type: 'string'},
        {name: 'snmpSysName', type: 'string'},
        {name: 'ipAddress', type: 'int'},
        {name: 'ipAddressString', type: 'string'},
        {name: 'status', type: 'auto'},
        {name: 'productionState', type: 'string'},
        {name: 'serialNumber', type: 'string'},
        {name: 'tagNumber', type: 'string'},
        {name: 'hwManufacturer', type: 'object'},
        {name: 'hwModel', type: 'object'},
        {name: 'osManufacturer', type: 'object'},
        {name: 'osModel', type: 'object'},
        {name: 'collector', type: 'string'},
        {name: 'priority', type: 'string'},
        {name: 'systems', type: 'object'},
        {name: 'groups', type: 'object'},
        {name: 'location', type: 'object'},
        {name: 'events', type: 'object'},
        {name: 'availability', type: 'float'},
        {name: 'pythonClass', type: 'string'}
    ],
    idProperty: 'uid'
});

/**
 * @class Zenoss.DeviceStore
 * @extend Zenoss.DirectStore
 * Direct store for loading devices
 */
Ext.define("Zenoss.DeviceStore", {
    alias: ['widget.DeviceStore'],
    extend: "Zenoss.DirectStore",
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            autoLoad: false,
            pageSize: Zenoss.settings.deviceGridBufferSize,
            model: 'Zenoss.device.DeviceModel',
            initialSortColumn: "name",
            directFn: Zenoss.remote.DeviceRouter.getDevices,
            root: 'devices'
        });
        this.callParent(arguments);
    }
});

/**
 * @class Zenoss.DeviceGridPanel
 * @extends Zenoss.FilterGridPanel
 * Main grid panel for displaying a device. Used on the It Infrastructure page.
 **/
Ext.define("Zenoss.DeviceGridPanel", {
    extend: "Zenoss.FilterGridPanel",
    alias: ['widget.DeviceGridPanel', 'widget.SimpleDeviceGridPanel'],
    lastHash: null,
    constructor: function(config) {
        var storeConfig = config.storeCfg || {};
        var store = Ext.create('Zenoss.DeviceStore', storeConfig);

        Ext.applyIf(config, {
            store: store,
            columns: deviceColumns
        });

        this.callParent(arguments);
        this.on('itemdblclick', this.onItemDblClick, this);
    },

    onItemDblClick: function(view, record) {
        window.location = record.get("uid");
    },
    applyOptions: function(options){
        // only request the visible columns
        var visibleColumns = Zenoss.util.filter(this.columns, function(c){
                return !c.hidden;
            }),
            keys = Ext.Array.pluck(visibleColumns, 'dataIndex');

        keys.push('ipAddressString');
        keys.push('pythonClass');
        Ext.apply(options.params, {
            keys: keys
        });
    }
});

function showComponentLockingDialog(msg, locking, funcs) {
        Ext.create('Zenoss.dialog.LockForm', {
            applyOptions: function(values) {
                Ext.applyIf(values, funcs.fetcher());
            },
            title: msg === "" ? _t("Lock Device") : _t("Lock Devices"),
            message: msg,
            updatesChecked: locking.updates,
            deletionChecked: locking.deletion,
            sendEventChecked: locking.events,
            submitFn: function(values) {
                funcs.REMOTE.lockDevices(values, funcs.saveHandler);
            }
        }).show();
}

/**********************************************************************
 *
 * Device Actions
 *
 */
    /**
     * Drop down of action items that you can use against
     * a device. The two required parameters are
     *@param 1. saveHandler = function to be called after the action (refresh the grid etc)
     *@param 2. deviceFetcher = function that returns the list of device records
     *@class Zenoss.DeviceActionMenu
     *@extends Ext.Button
     **/
    Ext.define("Zenoss.DeviceActionMenu", {
        alias: ['widget.deviceactionmenu'],
        extend: "Ext.Button",
        constructor: function(config) {
            config = config || {};
            if (!config.saveHandler) {
                throw "Device Action Menu did not receive a save handler";
            }
            if (!config.deviceFetcher) {
                throw "Device Action Menu did not receive a device fetcher";
            }
            var fetcher = config.deviceFetcher,
                saveHandler = config.saveHandler,
                REMOTE = Zenoss.remote.DeviceRouter;

            Ext.applyIf(config, {
                text: _t('Actions'),
                disabled: Zenoss.Security.doesNotHavePermission('Delete Device'),
                menu: {
                    items: [
                        new Zenoss.Action({
                            text: _t('Lock Devices') + '...',
                            iconCls: 'lock',
                            permission: 'Change Device',
                            handler: function() {
                            var sel = fetcher().uids,
                                funcs = {'fetcher': fetcher, 'saveHandler': saveHandler, 'REMOTE': REMOTE};

                                if(sel.length === 0){
                                    Zenoss.message.warning(_t("Please select 1 or more devices to lock"));
                                    return;
                                }
                                if(sel.length > 1){
                                    showComponentLockingDialog(_t("To view locked state, select one device at a time."), "", funcs);
                                }else{
                                    REMOTE.getInfo({
                                        uid: fetcher().uids[0],
                                        keys: ['locking']
                                    }, function(result){
                                        if (result.success) {
                                            showComponentLockingDialog("", result.data.locking, funcs);
                                        }
                                    });
                                }

                            }
                        }),
                        new Zenoss.Action({
                            text: _t('Reset IP'),
                            iconCls: 'set',
                            permission: 'Change Device',
                            handler: function(){
                                new Zenoss.dialog.SimpleMessageDialog({
                                    message: Ext.String.format(_t('Are you sure you want to reset the IP addresses of these devices to the results of a DNS lookup?')),
                                    title: _t('Reset IP'),
                                    buttons: [{
                                        xtype: 'DialogButton',
                                        text: _t('OK'),
                                        handler: function() {
                                            REMOTE.resetIp(fetcher(), saveHandler);
                                        }
                                    }, {
                                        xtype: 'DialogButton',
                                        text: _t('Cancel')
                                    }]
                                }).show();
                            }
                        }),
                        /*
                         * Currently causes a bus error on multiple devices: http://dev.zenoss.org/trac/ticket/6142
                         * Commenting out until that is fixed
                         *
                        resetCommunity: new Zenoss.Action({
                            text: _t('Reset Community'),
                            iconCls: 'set',
                            permission: 'Change Device',
                            handler: function(){
                                Ext.Msg.show({
                                    title: _t('Reset Community'),
                                    msg: _t('Are you sure you want to reset the SNMP '+
                                            'community strings of these devices?'),
                                    buttons: Ext.Msg.YESNO,
                                    fn: function(r) {
                                        switch(r) {
                                          case 'no':
                                            break;
                                          case 'yes':
                                            REMOTE.resetCommunity(gridOptions(), resetGrid);
                                            break;
                                        default:
                                            break;
                                        }
                                    }
                                });
                            }
                        }),
                        */
                        new Zenoss.Action({
                            text: _t('Set Production State')+'...',
                            iconCls: 'set',
                            permission: 'Change Device Production State',
                            handler: function(){
                                var win = new Zenoss.FormDialog({
                                    title: _t('Set Production State'),
                                    modal: true,
                                    width: 310,
                                    height: 150,
                                    items: [{
                                        xtype: 'ProductionStateCombo',
                                        fieldLabel: _t('Select a production state'),
                                        id: 'prodstate',
                                        listeners: {
                                            'select': function(){
                                                Ext.getCmp('prodstateok').enable();
                                            }
                                        }
                                    }],
                                    buttons: [{
                                        xtype: 'DialogButton',
                                        id: 'prodstateok',
                                        disabled: true,
                                        text: _t('OK'),
                                        handler: function(){
                                            var opts = Ext.apply(fetcher(), {
                                                prodState:Ext.getCmp('prodstate').getValue()
                                            });
                                            REMOTE.setProductionState(opts, saveHandler);
                                        }
                                    }, Zenoss.dialog.CANCEL
                                             ]
                                });
                                win.show();
                            }
                        }),
                        new Zenoss.Action({
                            text: _t('Set Priority')+'...',
                            iconCls: 'set',
                            permission: 'Change Device',
                            handler: function(){
                                var win = new Zenoss.FormDialog({
                                    title: _t('Set Priority'),
                                    modal: true,
                                    width: 310,
                                    height: 150,
                                    items: [{
                                        xtype: 'PriorityCombo',
                                        id: 'device_action_priority',
                                        fieldLabel: _t('Select a priority'),
                                        listeners: {
                                            'select': function(){
                                                Ext.getCmp('priorityok').enable();
                                            }
                                        }
                                    }],
                                    buttons: [{
                                        xtype: 'DialogButton',
                                        id: 'priorityok',
                                        disabled: true,
                                        text: _t('OK'),
                                        handler: function(){
                                            var opts = Ext.apply(fetcher(), {
                                                priority: Ext.getCmp('device_action_priority').getValue()
                                            });
                                            REMOTE.setPriority(opts, saveHandler);
                                        }
                                    }, Zenoss.dialog.CANCEL
                                             ]
                                });
                                win.show();
                            }
                        }),
                        new Zenoss.Action({
                            text: _t('Set Collector') + '...',
                            iconCls: 'set',
                            permission: 'Change Device',
                            handler: function(){
                                var win = new Zenoss.FormDialog({
                                    title: _t('Set Collector'),
                                    modal: true,
                                    width: 310,
                                    height: 180,
                                    items: [{
                                        xtype: 'combo',
                                        fieldLabel: _t('Select a collector'),
                                        id: 'collector',
                                        queryMode: 'local',
                                        store: new Ext.data.ArrayStore({
                                            data: Zenoss.env.COLLECTORS,
                                            fields: ['name']
                                        }),
                                        valueField: 'name',
                                        displayField: 'name',
                                        forceSelection: true,
                                        editable: false,
                                        listeners: {
                                            'select': function(){
                                                Ext.getCmp('collectorok').enable();
                                            }
                                        }
                                    }],
                                    buttons: [{
                                        xtype: 'DialogButton',
                                        id: 'collectorok',
                                        disabled: true,
                                        text: _t('OK'),
                                        handler: function(){
                                            var opts = Ext.apply(fetcher(), {
                                                collector: Ext.getCmp('collector').getValue()
                                            });
                                            opts.asynchronous = Zenoss.settings.deviceMoveIsAsync(opts.uids);
                                            REMOTE.setCollector(opts, saveHandler);
                                        }
                                    }, Zenoss.dialog.CANCEL
                                             ]
                                });
                                win.show();
                            }
                        })]
                }
            });
            Zenoss.DeviceActionMenu.superclass.constructor.apply(this, arguments);
        }
    });



    // Extension point for adding new devices for zenpacks
    Zenoss.customDeviceAdder = {};
    Zenoss.registerAddDeviceMethod = function(uid, fn ) {
        Zenoss.customDeviceAdder[uid] = fn;
    };
    Zenoss.getCustomDeviceAdder = function(uid) {
        return Zenoss.customDeviceAdder[uid];
    };

})(); // end of function namespace scoping
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/

(function(){
    Ext.ns('Zenoss.events');
    Ext.ns('Zenoss.eventclasses');

    Zenoss.events.getRowClass = function(record) {
        var stateclass = record.get('eventState')==='New' ?
            'unacknowledged':'acknowledged';
        var sev = Zenoss.util.convertSeverity(record.get('severity'));
        var rowcolors = Ext.state.Manager.get('rowcolor') ? 'rowcolor rowcolor-' : '';
        return rowcolors + sev + '-' + stateclass + ' ' + stateclass;
    };

    /*
     * Show the dialog that allows one to add an event.
     */
    function showAddEventDialog(gridId) {
        if (Ext.getCmp('addeventwindow')) {
            Ext.getCmp('addeventwindow').show();
            return;
        }
        var device;
        if (Zenoss.env.device_uid) {
            device = Zenoss.env.device_uid.split("/").reverse()[0];
        }
        var collectors = new Ext.data.ArrayStore({
                        data: Zenoss.env.COLLECTORS,
                        fields: ['name']
                    })
        var defaultValue;
        if (Zenoss.env.COLLECTORS[0].indexOf('localhost') > -1){
            defaultValue = "localhost";
        } else {
            defaultValue = null;
        }
        var addevent = Ext.create('Zenoss.dialog.BaseWindow', {
            title: _t('Create Event'),
            id: 'addeventwindow',
            layout: 'fit',
            autoHeight: true,
            modal: true,
            width: 310,
            closeAction: 'hide',
            plain: true,
            items: [{
                id: 'addeventform',
                xtype: 'form',
                defaults: {width: 290},
                autoHeight: true,
                frame: false,
                listeners: {
                    validitychange: function(form, isValid){
                        addevent.query('DialogButton')[0].setDisabled(!isValid);
                    }
                },
                fieldDefaults: {
                    labelWidth: 100
                },
                items: [{
                    xtype: 'textarea',
                    name: 'summary',
                    id: 'add_event_summary_textarea',
                    fieldLabel: _t('Summary'),
                    allowBlank: false
                },{
                    xtype: 'textfield',
                    fieldLabel: _t('Device'),
                    id: 'add_event_device_textfield',
                    name: 'device',
                    allowBlank: false,
                    value: device
                },{
                    xtype: 'textfield',
                    fieldLabel: _t('Component'),
                    id: 'add_event_component_textfield',
                    name: 'component'
                },{
                    fieldLabel: _t('Severity'),
                    name: 'severity',
                    xtype: 'combo',
                    id: 'add_event_severity_combo',
                    store: Zenoss.env.SEVERITIES,
                    typeAhead: true,
                    allowBlank: false,
                    forceSelection: true,
                    triggerAction: 'all',
                    value: 5,
                    selectOnFocus: true,
                    listConfig: {
                        id: 'add_event_severity_combo_list'
                    }
                },{
                    xtype: 'textfield',
                    fieldLabel: _t('Event Class Key'),
                    id: 'add_event_key_textfield',
                    name: 'evclasskey'
                },{
                    fieldLabel: _t('Event Class'),
                    name: 'evclass',
                    xtype: 'combo',
                    id: 'add_event_evclass_combo',
                    allowBlank: true,
                    store: Zenoss.env.EVENT_CLASSES,
                    typeAhead: true,
                    forceSelection: true,
                    triggerAction: 'all',
                    selectOnFocus: true,
                    listConfig: {
                        resizable: true,
                        id: 'add_event_evclass_combo_list'
                    }
                },{
                    fieldLabel: _t('Collector'),
                    name: 'monitor',
                    xtype: 'combo',
                    allowBlank: false,
                    store: collectors,
                    valueField: 'name',
                    displayField: 'name',
                    typeAhead: true,
                    forceSelection: true,
                    triggerAction: 'all',
                    selectOnFocus: true,
                    value: defaultValue,
                    listConfig: {
                        resizable: true
                    }
                }],
                buttons: [{
                    text: _t('Submit'),
                    xtype: 'DialogButton',
                    id: 'add_event_submit_button',
                    formBind: true,
                    handler: function(){
                        var form = Ext.getCmp('addeventform');
                        Zenoss.remote.EventsRouter.add_event(
                            form.getForm().getValues(),
                            function(){
                                addevent.hide();
                                var grid = Ext.getCmp(gridId);
                                grid.refresh();
                            }
                        );
                    }
                },{
                    text: _t('Cancel'),
                    xtype: 'DialogButton',
                    id: 'add_event_cancel_button',
                    handler: function(){
                        addevent.hide();
                    }
                }]
            }]

        });
        addevent.show();
    }


    /*
     * Show the dialog that allows one to classify an event
     */
    function showClassifyDialog(gridId) {

        Zenoss.events.launchMappingDialog = function(uid){
            var grid = null;
            var data = {};
            data.whichPanel = 'default';
            data.uid = "/zport/dmd/"+uid.split("| ")[1];
            Zenoss.eventclasses.mappingDialog(grid, data);
        };


        var win = new Zenoss.dialog.BaseWindow({
            title: _t('Classify Events'),
            id: 'reclassify_events_window',
            width: 300,
            autoHeight: true,
            modal: true,
            plain: true,
            items: [{
                id: 'classifyEventForm',
                xtype: 'form',
                monitorValid: true,
                autoHeight: true,
                frame: false,
                items: [{
                    padding: 10,
                    style: {'font-size':'10pt'},
                    html: _t('Select the event class with which'+
                             ' you want to associate these events.')
                },{
                    xtype: 'combo',
                    store: Zenoss.env.EVENT_CLASSES,
                    typeAhead: true,
                    allowBlank: false,
                    forceSelection: true,
                    triggerAction: 'all',
                    width: 180,
                    style: {'margin-left':'100px'},
                    listConfig: {
                        resizable: true,
                        id: 'classify_event_combo_list'
                    },
                    emptyText: _t('Select an event class'),
                    selectOnFocus: true,
                    id: 'evclass_combo'
                }],
                listeners: {
                    fieldvaliditychange: function(form, field, isValid) {
                        Ext.getCmp('classifyEventFormSubmitButton').setDisabled(!isValid);
                    },
                    scope: win
                },
                buttons: [{
                    text: _t('Submit'),
                    xtype: 'DialogButton',
                    id: 'classifyEventFormSubmitButton',
                    disabled: true,
                    handler: function(){
                        var cb = Ext.getCmp('evclass_combo'),
                        grid = Ext.getCmp(gridId),
                        sm = grid.getSelectionModel(),
                        rs = sm.getSelection(),
                        evrows = [];
                        Ext.each(rs, function(record){
                            evrows[evrows.length] = record.data;
                        });
                        if (!evrows.length) {
                            win.hide();
                            new Zenoss.dialog.ErrorDialog({message: _t('No events were selected.')});
                        } else {
                            Zenoss.remote.EventsRouter.classify({
                                'evclass': cb.getValue(),
                                'evrows': evrows
                            }, function(result){
                                win.destroy();
                                var title = result.success ? _t('Classified'): _t('Error');

                                Ext.MessageBox.show({
                                    title: title,
                                    msg: '<a href="javascript:void(0)" onclick="Zenoss.events.launchMappingDialog(\''+result.msg+'\')" >Edit new mapping</a>',//Ext.htmlDecode(result.msg),
                                    buttons: Ext.MessageBox.OK
                                });
                            });
                        }
                    }
                },{
                    text: _t('Cancel'),
                    xtype: 'DialogButton',
                    handler: function(){
                        win.destroy();
                    }
                }]
            }]

        });

        win.show();
    }

    Zenoss.events.EventPanelToolbarActions = {
        acknowledge: new Zenoss.ActionButton({
            iconCls: 'acknowledge',
            tooltip: _t('Acknowledge events (Ctrl-Shift-a)'),
            permission: 'Manage Events',
            itemId: 'acknowledge',
            handler: function() {
                Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.acknowledge);
            }
        }),
        close: new Zenoss.ActionButton({
            iconCls: 'close',
            tooltip: _t('Close events (Ctrl-Shift-c)'),
            permission: 'Manage Events',
            itemId: 'close',
            handler: function() {
                Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.close);
            }
        }),
        reclassify: new Zenoss.ActionButton({
            iconCls: 'classify',
            tooltip: _t('Reclassify an event'),
            permission: 'Manage Events',
            itemId: 'classify',
            handler: function(button) {
                var gridId = button.ownerCt.ownerCt.id;
                showClassifyDialog(gridId);
            }
        }),
        reopen: new Zenoss.ActionButton({
            iconCls: 'unacknowledge',
            tooltip: _t('Unacknowledge events (Ctrl-Shift-u)'),
            permission: 'Manage Events',
            itemId: 'unacknowledge',
            handler: function() {
                Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.reopen);
            }
        }),
        unclose: new Zenoss.ActionButton({
            iconCls: 'reopen',
            tooltip: _t('Reopen events (Ctrl-Shift-o)'),
            permission: 'Manage Events',
            itemId: 'reopen',
            handler: function() {
                Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.reopen);
            }
        }),
        newwindow: new Zenoss.ActionButton({
            iconCls: 'newwindow',
            permission: 'View',
            tooltip: _t('Go to event console'),
            handler: function(btn) {
                var grid = btn.grid || this.ownerCt.ownerCt,
                curState = Ext.state.Manager.get('evconsole') || {},
                filters = curState.filters || {},
                opts = filters.options || {},
                pat = /devices\/([^\/]+)(\/.*\/([^\/]+)$)?/,
                matches = grid.view.getContext().match(pat),
                st, url;

                // on the device page
                if (matches) {
                    opts.device = matches[1];
                    if (matches[3]) {
                        opts.component = matches[3];
                    }
                }
                filters.options = opts;
                curState.filters = filters;
                st = encodeURIComponent(Zenoss.util.base64.encode(Ext.encode(curState)));
                url = '/zport/dmd/Events/evconsole?state=' + st;
                window.open(url, '_newtab', "");
            }
        }),
        refresh: new Zenoss.ActionButton({
            iconCls: 'refresh',
            permission: 'View',
            tooltip: _t('Refresh events'),
            handler: function(btn) {
                var grid = btn.grid || this.ownerCt.ownerCt;
                if(grid.getComponent("event_panel")) {
                    grid = grid.getComponent("event_panel");
                }
                grid.refresh();
            }
        })
    };

    Ext.define("Zenoss.model.EventType", {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: ['id', 'event_type']
    });

    Ext.define("Zenoss.form.ColumnItemSelector", {
        extend:"Ext.ux.form.ItemSelector",
        constructor: function(state_id, config) {
            var cols = Zenoss.env.getColumnDefinitions();
            var cols_to_display = Zenoss.env.getColumnIdsToRender(state_id);
            var data = [];
            Ext.Array.each(cols, function(col) {
                data.push([col.id, col.header]);
            });
            Ext.applyIf(config, {
                name: 'columnItemSelector',
                id: 'columns_item_selector',
                imagePath: "/++resource++zenui/img/xtheme-zenoss/icon",
                valueField: 'id',
                displayField: 'name',
                autoScroll:true,
                minHeight:500,
                maxHeight:500,
                height:500,
                store:  Ext.create('Ext.data.ArrayStore', {
                    data: data,
                    model: 'Zenoss.model.IdName',
                    sorters: [{
                        property: 'name',
                        direction: 'ASC'
                    }]
                }),
                value: cols_to_display
            });
            this.superclass.constructor.call(this, config);
        }
    });

    Ext.define('Zenoss.events.ColumnConfigDialog',{
        extend:'Zenoss.dialog.BaseWindow',
        title: _t('Column Configuration'),
        id: 'events_column_config_dialog',
        minHeight: 600,
        minWidth: 600,
        width: 600,
        height: 600,
        modal: true,
        margins: {top:2, left:2, right: 2, bottom:20},
        closeAction: 'destroy',
        plain: true,
        buttons: [
            {
                text: _t('Submit'),
                xtype: 'DialogButton',
                formBind: true,
                handler: function(){
                    var columns = Ext.getCmp('columns_item_selector').value;
                    var dialog = Ext.getCmp('events_column_config_dialog');
                    var grid = dialog.grid;

                    Zenoss.env.recreateGridWithNewColumns(grid, columns);
                }
            },
            {
                text: _t('Cancel'),
                xtype: 'DialogButton',
                handler: function(){
                    this.hide();
                }
            }
        ],
        constructor: function(grid)
        {
            this.superclass.constructor.call(this, { layout: {type:'vbox', align: 'stretch'} });

            this.grid = grid;

            var header_panel = Ext.widget('panel', {
                xtype: 'panel',
                margins: {top:10, left:2, right: 2, bottom:2},
                layout: {
                    type: 'hbox'
                },
                items: [ {
                    flex: 1,
                    xtype: 'panel',
                    html: "<center><b>Available</b></center>"
                },{
                    flex: 1,
                    xtype: 'panel',
                    html: "<center><b>Selected</b></center>"
                }
                ]
            });

            var item_selector = Ext.create('Zenoss.form.ColumnItemSelector', this.grid.stateId, {});

            var body_panel = Ext.widget('panel', {
                layout: {type:'fit', autoSize: true},
                margins: {top:0, left:10, right: 10, bottom: 10},
                items:[ item_selector ]
            });

            this.add(header_panel);
            this.add(body_panel);
        }
    });

    Zenoss.events.showColumnConfigDialog = function(grid)
    {
        var dialog = Ext.create('Zenoss.events.ColumnConfigDialog', grid);
        dialog.show();
    };

    Zenoss.EventConsoleTBar = Ext.extend(Zenoss.LargeToolbar, {
        constructor: function(config){
            var gridId = config.gridId,
                showActions = true,
                showCommands = true,
                configureMenuItems,
                tbarItems = config.tbarItems || [],
                eventSpecificTbarActions = ['acknowledge', 'close', 'reopen', 'unacknowledge', 'addNote'];
            if (!gridId) {
                throw ("Event console tool bar did not receive a grid id");
            }
            configureMenuItems = [{
                id: 'rowcolors_checkitem',
                xtype: 'menucheckitem',
                text: _t('Show severity row colors'),
                handler: function(checkitem) {
                    var checked = checkitem.checked;
                    var grid = Ext.getCmp(gridId);
                    grid.toggleRowColors(checked);
                }
            },{
                id: 'clearfilters',
                text: _t('Clear filters'),
                listeners: {
                    click: function(){
                        Ext.getCmp(gridId).clearFilters();
                    }
                }
            },{
                id: 'adjust_columns_item_selector',
                text: _t('Adjust columns'),
                listeners: {
                click: function(){
                    var grid = Ext.getCmp(gridId);
                    Zenoss.events.showColumnConfigDialog(grid);
                    }
                }
            },{
                text: _t("Restore defaults"),
                handler: function(){
                    new Zenoss.dialog.SimpleMessageDialog({
                        message: Ext.String.format(_t('Are you sure you want to restore ' +
                                                   'the default configuration? All' +
                                                   ' filters, column sizing, and column order ' +
                                                   'will be lost.')),
                        title: _t('Confirm Restore'),
                        buttons: [{
                            xtype: 'DialogButton',
                            text: _t('OK'),
                            handler: function() {
                                var grid = Ext.getCmp(gridId);
                                grid.resetGrid();
                            }
                        }, {
                            xtype: 'DialogButton',
                            text: _t('Cancel')
                        }]
                    }).show();
                }
            }];

            if (!_global_permissions()['manage events']) {
                configureMenuItems.unshift({
                    id: 'excludenonactionables_checkitem',
                    xtype: 'menucheckitem',
                    text: _t('Only show actionable events'),
                    handler: function(checkitem) {
                        var checked = checkitem.checked;
                        var grid = Ext.getCmp(gridId);
                        var tbar = grid.tbar;
                        if (tbar && tbar.getComponent) {
                            Ext.each(eventSpecificTbarActions, function(actionItemId) {
                                var cmp = tbar.getComponent(actionItemId);
                                if (cmp) {
                                    cmp.filtered = checked;
                                    cmp.updateDisabled();
                                }
                            });
                        }
                        grid.toggleNonActionables(checked);
                    }
                });
            }

            if (/^\/zport\/dmd\/Events/.test(window.location.pathname)) {
                configureMenuItems.splice(2, 0, {
                    text: _t('Save this configuration...'),
                    handler: function(){
                        var grid = Ext.getCmp(gridId),
                        link = grid.getPermalink();
                        new Zenoss.dialog.ErrorDialog({
                            message: Ext.String.format(_t('<div class="dialog-link">' +
                                                       'Drag this link to your bookmark bar ' +
                                                       '<br/>to return to this configuration later.' +
                                                       '<br/><br/><a href="' +
                                                       link +
                                                       '">Resource Manager: Events</a></div>')),
                            title: _t('Save Configuration')
                        });
                    }
                });
            }

            // actions
            if (Ext.isDefined(config.actionsMenu)) {
                showActions = config.actionsMenu;
            }
            if (showActions) {
                tbarItems.push({
                    id: 'event-actions-menu',
                    text: _t('Actions'),
                    xtype: 'deviceactionmenu',
                    deviceFetcher: function() {
                        var grid = Ext.getCmp(gridId),
                        sm = grid.getSelectionModel(),
                        rows = sm.getSelection(),
                        pluck = Ext.Array.pluck,
                        uids = pluck(pluck(pluck(rows, 'data'), 'device'), 'uid'),
                        opts =  {
                            uids: uids,
                            ranges: [],
                            hashcheck: 'none'
                        };
                        opts.params = grid.filterRow.getSearchValues();
                        // filter out the nulls
                        opts.uids = Zenoss.util.filter(opts.uids, function(uid){
                            return uid;
                        });

                        return opts;
                    },
                    saveHandler: Ext.emptyFn
                });
            }

            // commands
            if (Ext.isDefined(config.commandsMenu)) {
                showCommands = config.commandsMenu;
            }
            if (showCommands) {
                tbarItems.push({
                    id: 'event-commands-menu',
                    text: _t('Commands'),
                    hidden: !showCommands,
                    disabled: Zenoss.Security.doesNotHavePermission('Run Commands'),
                    setContext: function(uid) {
                        if (!uid) {
                            uid = '/zport/dmd/Devices';
                        }
                        var me = Ext.getCmp('event-commands-menu'),
                        menu = me.menu;
                        // load the available commands from the server
                        // commands are based on context
                        Zenoss.remote.DeviceRouter.getUserCommands({uid:uid}, function(data) {
                            menu.removeAll();
                            Ext.each(data, function(d) {
                                menu.add({
                                    text:d.id,
                                    tooltip:d.description,
                                    handler: function(item) {
                                        var command = item.text,
                                            grid = Ext.getCmp(gridId),
                                            sm = grid.getSelectionModel(),
                                            selections = sm.getSelection(),
                                            devids = Ext.Array.pluck(Ext.Array.pluck(Ext.Array.pluck(selections, 'data'), 'device'), 'uid');

                                        // filter out the none device events
                                        devids = Zenoss.util.filter(devids, function(uid){ return uid; });
                                        if (devids.length) {

                                            // only run commands for the visible devices
                                            var win = new Zenoss.CommandWindow({
                                                uids: devids,
                                                target: uid + '/run_command',
                                                command: command
                                            });
                                            win.show();
                                        }
                                    }
                                });
                            });
                        });
                    },
                    menu: {}
                });
            }
            this.gridId = gridId;

            if (!config.hideDisplayCombo) {
                tbarItems.push('-');
                tbarItems.push(Ext.create('Ext.toolbar.TextItem', {
                    hidden: config.hideDisplayCombo || false,
                    text: _t('Display: ')
                }));
                tbarItems.push(Ext.create('Ext.form.ComboBox', {
                    id: 'history_combo',
                    hidden: config.hideDisplayCombo || false,
                    name: 'event_display',
                    queryMode: 'local',
                    store: new Ext.data.SimpleStore({
                        model: 'Zenoss.model.EventType',
                        data: [[0,'Events'],[1,'Event Archive']]
                    }),
                    displayField: 'event_type',
                    valueField: 'id',
                    width: 120,
                    value: 0,
                    triggerAction: 'all',
                    forceSelection: true,
                    editable: false,
                    listeners: {
                        select: function(selection) {
                            var archive = selection.value === 1,
                                grid = Ext.getCmp(gridId),
                                yesterday = new Date();

                            // reload the grid. changing the filters
                            grid.setStoreParameter('archive', archive);

                            // if history set default lastseen to yesterday
                            if (archive) {
                                yesterday.setDate(yesterday.getDate() - 1);
                                grid.setFilter('lastTime', yesterday);
                            }else{
                                grid.setFilter('lastTime', null);
                            }
                        }
                    }
                }));

            }
            if (config.newwindowBtn) {
                tbarItems.push('-');
                tbarItems.push(Zenoss.events.EventPanelToolbarActions.newwindow);
            }

            Zenoss.EventActionManager.configure({
                onFinishAction: function() {
                    var grid = Ext.getCmp(gridId);
                    if(Ext.get('event_panel')){
                        grid = Ext.getCmp('event_panel');
                    }
                    if (grid) {
                        grid.refresh();
                    }
                    var dpanel = Ext.getCmp('dpanelcontainer');
                    if (dpanel && dpanel.isVisible()) {
                        dpanel.refresh();
                    }
                },
                findParams: function() {
                    var grid = Ext.getCmp(gridId);
                    if(Ext.get('event_panel')){
                        grid = Ext.getCmp('event_panel');
                    }
                    if (grid) {
                        var params =  grid.getSelectionParameters();
                        if (Zenoss.env.device_uid) {
                            params.uid = Zenoss.env.device_uid;
                        }
                        return params;
                    }
                }
            });

            Ext.applyIf(config, {
                ref: 'tbar',
                listeners: {
                    beforerender: function(){
                        var grid = Ext.getCmp(gridId),
                        tbar = this;
                        if (tbar.getComponent) {
                            Ext.each(eventSpecificTbarActions, function(actionItemId) {
                                var cmp = tbar.getComponent(actionItemId);
                                if (cmp) {
                                    cmp.filtered = grid.excludeNonActionables;
                                }
                            });
                        }
                    },
                    afterrender: function(){
                        var grid = Ext.getCmp(gridId),
                        store = grid.getStore(),
                        tbar = this,
                        view = grid.getView();
                        if(store.buffered) {
                            store.on('guaranteedrange', this.doLastUpdated);
                        } else {
                            store.on('load', this.doLastUpdated);
                        }
                        view.on('buffer', this.doLastUpdated);

                        view.on('filterchange', function(){
                            tbar.refreshmenu.setDisabled(!view.isValid());

                            // Hook up the "Last Updated" text
                            if ( !view.isValid() ) {
                                var box = Ext.getCmp('lastupdated');
                                box.setText(_t(''));
                            }
                        });
                        // set up the commands menu
                        var context = Zenoss.env.device_uid || Zenoss.env.PARENT_CONTEXT;
                        if (context === "/zport/dmd/Events") {
                            context = location.pathname.replace('/viewEvents', '');
                        }
                        this.setContext(context);
                    },
                    scope: this
                },
                items: Ext.Array.union([
                    // create new instances of the action otherwise Ext won't render them (probably a bug in 4.1)
                    new Zenoss.ActionButton({
                        iconCls: 'acknowledge',
                        tooltip: _t('Acknowledge events'),
                        permission: 'Manage Events',
                        id: 'events_toolbar_ack',
                        itemId: 'acknowledge',
                        handler: function() {
                            Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.acknowledge);
                        }
                    }),
                    new Zenoss.ActionButton({
                        iconCls: 'close',
                        tooltip: _t('Close events'),
                        permission: 'Manage Events',
                        itemId: 'close',
                        id: 'events_toolbar_close_events',
                        handler: function() {
                            Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.close);
                        }
                    }),
                    new Zenoss.ActionButton({
                        iconCls: 'classify',
                        tooltip: _t('Reclassify an event'),
                        permission: 'Manage Events',
                        itemId: 'classify',
                        id: 'events_toolbar_reclassify_event',
                        handler: function(button) {
                            var gridId = button.ownerCt.ownerCt.id;
                            showClassifyDialog(gridId);
                        }
                    }),
                    new Zenoss.ActionButton({
                        iconCls: 'unacknowledge',
                        tooltip: _t('Unacknowledge events'),
                        permission: 'Manage Events',
                        itemId: 'unacknowledge',
                        id: 'events_toolbar_unack',
                        handler: function() {
                            Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.reopen);
                        }
                    }),
                    new Zenoss.ActionButton({
                        iconCls: 'reopen',
                        tooltip: _t('Reopen events'),
                        permission: 'Manage Events',
                        itemId: 'reopen',
                        id: 'events_toolbar_reopen',
                        handler: function() {
                            Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.reopen);
                        }
                    }),
                    new Zenoss.ActionButton({
                        iconCls: 'addslide',
                        tooltip: _t('Add Log'),
                        permission: 'Manage Events',
                        itemId: 'addNote',
                        handler: function() {
                            var grid = Ext.getCmp(gridId),
                                sm = grid.getSelectionModel(),
                                selected = sm.getSelection(),
                                data = Ext.pluck(selected, "data"),
                                uuids = Ext.pluck(data, "evid"),
                                addNoteWindow;
                            addNoteWindow = Ext.create('Zenoss.dialog.BaseWindow', {
                                title: _t('Add Note'),
                                id: 'addNoteWindow',
                                layout: 'fit',
                                autoHeight: true,
                                modal: true,
                                width: 310,
                                plain: true,
                                items: [{
                                    id: 'addNoteForm',
                                    xtype: 'form',
                                    defaults: {width: 290},
                                    autoHeight: true,
                                    frame: false,
                                    fieldDefaults: {
                                        labelWidth: 100
                                    },
                                    items: [{
                                        xtype: 'textarea',
                                        maxHeight: Ext.getBody().getViewSize().height * 0.4,
                                        name: 'note',
                                        fieldLabel: _t('note'),
                                        allowBlank: false
                                    }],
                                    buttons: [{
                                        text: _t('Submit'),
                                        xtype: 'DialogButton',
                                        formBind: true,
                                        handler: function() {
                                            var form = Ext.getCmp('addNoteForm'),
                                                note = form.getValues().note;

                                            Ext.each(uuids, function(uuid) {
                                                Zenoss.remote.EventsRouter.write_log(
                                                {
                                                    evid: uuid,
                                                    message: note
                                                });
                                            });
                                        }
                                    },{
                                        text: _t('Cancel'),
                                        xtype: 'DialogButton',
                                        handler: function(){
                                            addNoteWindow.hide();
                                        }
                                    }]
                                }]
                            });
                            addNoteWindow.show();
                        }
                    }),
                    new Zenoss.ActionButton({
                        iconCls: 'add',
                        tooltip: _t('Add an event'),
                        permission: 'Manage Events',
                        id: 'add_event_main_button',
                        handler: function() {
                            showAddEventDialog(gridId);
                        }
                    }),
                    {
                        xtype: 'tbseparator'
                    },
                    Zenoss.events.EventPanelToolbarSelectMenu,
                    {
                        text: _t('Export'),
                        id: 'export-button',
                        //iconCls: 'export',
                        menu: {
                            items: [{
                                text: 'XML',
                                handler: function(){
                                    var context = Zenoss.env.device_uid || Zenoss.env.PARENT_CONTEXT;
                                    if (context === "/zport/dmd/Events") {
                                        context = location.pathname.replace('/viewEvents', '');
                                    }

                                    var grid = Ext.getCmp(gridId),
                                        state = grid.getState(),
                                        historyCombo = Ext.getCmp('history_combo'),
                                        params = {
                                            type: 'xml',
                                            isHistory: false,
                                            params: {
                                                uid: context,
                                                fields: Ext.Array.pluck(state.columns, 'id'),
                                                sort: state.sort.property,
                                                dir: state.sort.direction,
                                                params: grid.getExportParameters()
                                            }
                                        };
                                    if (historyCombo && historyCombo.getValue() === 1) {
                                        params.isHistory = true;
                                    }
                                    Ext.get('export_body').dom.value =
                                        Ext.encode(params);
                                    Ext.get('exportform').dom.submit();
                                }
                            }, {
                                text: 'CSV',
                                handler: function(){
                                    var context = Zenoss.env.device_uid || Zenoss.env.PARENT_CONTEXT;
                                    if (context === "/zport/dmd/Events") {
                                        context = location.pathname.replace('/viewEvents', '');
                                    }
                                    var grid = Ext.getCmp(gridId),
                                    state = Ext.getCmp(gridId).getState(),
                                    historyCombo = Ext.getCmp('history_combo'),
                                    params = {
                                        type: 'csv',
                                        params: {
                                            uid: context,
                                            fields: Ext.Array.pluck(state.columns, 'id'),
                                            sort: state.sort.property,
                                            dir: state.sort.direction,
                                            params: grid.getExportParameters()
                                        }
                                    };
                                    if (historyCombo && historyCombo.getValue() === 1) {
                                        params.isHistory = true;
                                    }
                                    Ext.get('export_body').dom.value =
                                        Ext.encode(params);
                                    Ext.get('exportform').dom.submit();
                                }
                            }]
                        }
                    },
                    {
                        text: _t('Configure'),
                        id: 'configure-button',
                        //iconCls: 'customize',
                        menu: {
                            items: configureMenuItems
                        }
                    },{
                        xtype: 'tbfill'
                    },{
                        id: 'lastupdated',
                        xtype: 'tbtext',
                        cls: 'lastupdated',
                        text: _t('Updating...')
                    },{
                        xtype: 'refreshmenu',
                        ref: 'refreshmenu',
                        id: 'refresh-button',
                        refreshWhenHidden: true,
                        iconCls: 'refresh',
                        text: _t('Refresh'),
                        handler: function() {
                            var grid = Ext.getCmp(gridId);
                            if (grid.isVisible(true)) {
                                // Indicating grid updating progress
                                var box = Ext.getCmp('lastupdated');
                                box.setText(_t('<span>Updating... </span><img src="/++resource++zenui/img/ext4/icon/circle_arrows_ani.gif" width=12 height=12>'));
                                grid.refresh();
                            }
                        },
                        pollHandler: function(btn) {
                            var grid = Ext.getCmp(gridId);
                            if (grid.refresh_in_progress === 0) {
                                this.handler(btn);
                            }
                        }
                    }
                ], tbarItems)
            });
            Zenoss.EventConsoleTBar.superclass.constructor.call(this, config);
        },
        doLastUpdated: function() {
            var box = Ext.getCmp('lastupdated'),
            dtext = Zenoss.date.renderWithTimeZone(new Date()/1000);
            dtext += " (" + Zenoss.USER_TIMEZONE + ")";
            box.setText(_t('Last updated at ') + dtext);
        },
        setContext: function(uid) {
            var commands = Ext.getCmp('event-commands-menu');
            if (commands) {
                commands.setContext(uid);
            }
        }
    });

    /**
     * @class Zenoss.EventPanelSelectionModel
     * @extends Zenoss.ExtraHooksSelectionModel
     *
     */
        Ext.define("Zenoss.EventPanelSelectionModel", {
            extend:"Zenoss.ExtraHooksSelectionModel",
            selectState: null,
            badIds: {},
            mode: 'MULTI',
            constructor: function(config){
                this.callParent([config]);
                this.on('select', this.handleRowSelect, this);
                this.on('deselect', this.handleRowDeSelect, this);
                this.on('selectionchange', function(selectionmodel) {
                    // Disable buttons if nothing selected (and vice-versa)
                    var actionsToChange = ['acknowledge', 'close', 'reopen',
                                           'unacknowledge', 'classify',
                                           'addNote', 'event-actions-menu'],
                        newDisabledValue = !selectionmodel.hasSelection() && selectionmodel.selectState !== 'All',
                        tbar = this.getGrid().tbar,
                        history_combo = Ext.getCmp('history_combo'),
                        archive = Ext.isDefined(history_combo) ? history_combo.getValue() === 1 : false;
                    if (archive) {
                        // These are always disabled on the archive event console
                        tbar.getComponent('acknowledge').setDisabled(true);
                        tbar.getComponent('close').setDisabled(true);
                        tbar.getComponent('reopen').setDisabled(true);
                        tbar.getComponent('unacknowledge').setDisabled(true);

                        // This is conditionally enabled/disabled based on selection
                        tbar.getComponent('classify').setDisabled(newDisabledValue);
                    }
                    else {
                        // tbar is not present on component event consoles
                        if (tbar && tbar.getComponent) {
                            Ext.each(actionsToChange, function(actionItemId) {
                                if(tbar.getComponent(actionItemId)){
                                    tbar.getComponent(actionItemId).setDisabled(newDisabledValue);
                                }
                            });
                        }
                    }
                });


            },
            getGrid: function() {
                if (!Ext.isDefined(this.grid)) {
                    this.grid = Ext.getCmp(this.gridId);
                }
                return this.grid;
            },
            handleRowSelect: function(sm, record){
                if (record) {
                    delete this.badIds[record.get("evid")];
                }
            },
            handleRowDeSelect: function(sm, record){
                if (this.selectState && record) {
                    this.badIds[record.get("evid")] = 1;
                }
            },
            onStoreLoad: function() {
                var store = this.grid.getStore();
                if (this.selectState === 'All') {
                    this.suspendEvents();
                    var items = Zenoss.util.filter(store.data.items, function(item){
                        return (! this.badIds[item.get('evid')]);
                    }, this);
                    this.select(items, false, true);
                    this.resumeEvents();
                    this.fireEvent('selectionchange', this);
                }
            },
            selectEventState: function(state){
                var me = this,
                    grid = this.getGrid(),
                    store = grid.getStore();
                if (state === 'All') {
                    // suppress events
                    return this.selectAll(true);
                }
                this.clearSelections(true);
                // Suspend events to avoid firing the whole chain for every row
                this.suspendEvents();

                Ext.each(store.data.items, function(record){
                    if (record) {
                        if (record.data.eventState === state) {
                            me.select(record, true);
                        }
                    }
                });
                this.selectState = state;

                // Bring events back and fire one selectionchange for the batch
                this.resumeEvents();
                this.fireEvent('selectionchange', this);
            },
            clearSelectState: function() {
                this.selectState = null;
                this.grid.getStore().un('datachanged', this.onStoreLoad, this);
                this.grid.disableSavedSelection(false);
            },
            setSelectState: function(state) {
                this.selectState = state;
                if (state === 'All') {
                    this.grid.getStore().on('datachanged', this.onStoreLoad, this);
                    this.grid.disableSavedSelection(true);
                }
            },
            selectNone: function(){
                this.clearSelections(true);
                this.clearSelectState();
                // Fire one selectionchange to make buttons figure out their
                // disabledness
                this.fireEvent('selectionchange', this);
            },
            clearSelections: function(fast){
                if (this.isLocked() || !this.grid) {
                    return;
                }

                // Suspend events to avoid firing the whole chain for every row
                this.suspendEvents();
                if(!fast){
                    //make sure all rows are deselected so that UI renders properly
                    //base class only deselects rows it knows are selected; so we need
                    //to deselect rows that may have been selected via selectstate
                    this.deselect(this.grid.getStore().data.items);
                }
                // Bring events back and fire one selectionchange for the batch
                this.resumeEvents();
                this.fireEvent('selectionchange', this);

                this.badIds = {};
                Zenoss.EventPanelSelectionModel.superclass.clearSelections.apply(this, arguments);
            }
        });

    /**
     * @class Zenoss.EventsJsonReader
     * @extends Zenoss.ExtraHooksSelectionModel
     *
     * Subclass the Ext JsonReader so that we can override how data is fetched
     * from a record that is returned by the router. Custom details use keys that
     * contain dots (zenpacks.foo.bar.baz) so we need to force key-method access.
     */
    Ext.define("Zenoss.EventsJsonReader", {
        extend: "Ext.data.reader.Json",
        alias: 'reader.events',
        useSimpleAccessors: true,
        createAccessor : (function(){
            return function(expr) {
                return function(obj){
                    return obj[expr];
                };
            };
        }())
    });



    /**
     * @class Zenoss.events.Store
     * @extend Zenoss.DirectStore
     * Direct store for loading ip addresses
     */
    Ext.define("Zenoss.events.Store", {
        extend: "Zenoss.DirectStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.events.Model',
                initialSortColumn: 'severity',
                initialSortDirection: 'DESC',
                pageSize: Zenoss.settings.eventConsoleBufferSize,
                proxy: {
                    type: 'direct',
                    simpleSortMode: true,
                    directFn: config.directFn || Zenoss.remote.EventsRouter.query,
                    reader: {
                        type: 'events',
                        root: 'events',
                        totalProperty: 'totalCount'
                    }
                }


            });
            this.callParent(arguments);
        }
    });

    Zenoss.events.customColumns = {};
    Zenoss.events.registerCustomColumn = function(dataIndex, obj) {
        Zenoss.events.customColumns[dataIndex] = obj;
    };

    Zenoss.events.eventFields = [];
    /**
     * This is how zenpack authors can register event fields that they want pulled from the server,
     * but may not be visible in a colum. So if you add the following line to your javascript:
     *     Zenoss.events.registerEventField("ipAddress");
     * On every browser request the ipAddress field will be populated
     **/
    Zenoss.events.registerEventField = function(dataIndex) {
        Zenoss.events.eventFields.push(dataIndex);
    };

    /**
     * @class Zenoss.events.Grid
     * @extends Zenoss.FilterGridPanel
     * Base Class for the event panels
     **/
    Ext.define('Zenoss.events.Grid', {
        extend: 'Zenoss.FilterGridPanel',
        rowcolors: false,
        excludeNonActionables: false,
        constructor: function(config) {
            config = config || {};
            config.viewConfig = config.viewConfig || {};
            Ext.applyIf(config.viewConfig, {
                getRowClass: Zenoss.events.getRowClass,
        enableTextSelection: true
            });

            this.callParent(arguments);
            this.on('itemclick', this.onItemClick, this );
            this.on('filterschanged', this.onFiltersChanged, this);
            this.excludeNonActionables = !_global_permissions()['manage events'] && Ext.state.Manager.get('excludeNonActionables');
            this.getStore().autoLoad = true;
        },
        initComponent: function() {
            this.getSelectionModel().grid = this;
             // create keyboard shortcuts for the main event console
            function nonInputHandler(func, scope){

                return function(key, e) {
                    if (e.target.tagName !== "INPUT" && e.target.tagName !== "TEXTAREA") {
                        Ext.bind(func, scope)(key, e);
                        e.preventDefault();
                    }
                };
            }
            this.map = new Ext.util.KeyMap({
                target: document.body,
                binding: [{
                    key:  Ext.EventObject.A,
                    ctrl: true,
                    shift: false,
                    alt: false,
                    scope: this,
                    fn: nonInputHandler(function() {
                        this.getSelectionModel().selectEventState('All');
                    }, this)
                },{
                    key:  Ext.EventObject.ESC,
                    ctrl: false,
                    shift: false,
                    alt: false,
                    scope:this,
                    fn: nonInputHandler(function() {
                        this.getSelectionModel().clearSelections();
                        this.getSelectionModel().clearSelectState();
                    }, this)
                }, {
                    // acknowledge
                    key: Ext.EventObject.A,
                    shift: true,
                    ctrl: true,
                    alt: false,
                    fn: nonInputHandler(function() {
                        Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.acknowledge);
                    }, this)
                }, {
                    // close
                    key: Ext.EventObject.C,
                    shift: true,
                    ctrl: true,
                    alt: false,
                    fn: nonInputHandler(function() {
                        Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.close);
                    }, this)
                }, {
                    // reopen
                    key: Ext.EventObject.O,
                    shift: true,
                    ctrl: true,
                    alt: false,
                    fn: nonInputHandler(function() {
                        Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.reopen);
                    }, this)
                }, {
                    // unack
                    key: Ext.EventObject.U,
                    shift: true,
                    ctrl: true,
                    alt: false,
                    fn: nonInputHandler(function() {
                        Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.reopen);
                    }, this)
                }]

            });
            this.on('show', function(){ this.map.enable();}, this);
            this.on('hide', function(){ this.map.disable();}, this);

            this.callParent(arguments);
        },
        onItemClick: function(){
            this.getSelectionModel().clearSelectState();
        },
        setGetFilterParameters: function() {
            var params = Ext.Object.fromQueryString(location.search),
                cleared = false;
            Ext.each(["severity", "eventState", "device", "component", "eventClass", "summary"], function(param) {
                var v = params[param];
                if (Ext.isDefined(v)) {
                    if (!cleared) {
                        // Clear filters first if anything is set, so we don't merge with existing state
                        this.clearFilters();
                        cleared = true;
                    }
                    // Do any modification of the values
                    switch (param) {
                        case "eventState":
                        case "severity":
                            v = v.split(',').map(function(item) {
                              return parseInt(item, 10);
                            });
                            break;
                        default:
                            // Let it go through
                            break;
                    }
                    // Set the filter value
                    this.setFilter(param, v);
                }
            }, this);
        },
        listeners: {
            beforerender: function(){
                this.rowcolors = Ext.state.Manager.get('rowcolor');
                // Some event consoles (Impact Events) do not use severity
                // config colors.  Check and see if it's being used before
                // trying to use it.
                var rowcolorsCheckItem = Ext.getCmp('rowcolors_checkitem');
                if (rowcolorsCheckItem) {
                    rowcolorsCheckItem.setChecked(this.rowcolors);
                }

                var excludeNonActionablesCheckItem = Ext.getCmp('excludenonactionables_checkitem');
                if (excludeNonActionablesCheckItem) {
                    excludeNonActionablesCheckItem.setChecked(this.excludeNonActionables);
                }
                this.setGetFilterParameters();
            }
        },
        applyOptions: function() {
            var store = this.getStore(),
                keys,
                columns = this.headerCt.getGridColumns();

            columns = Zenoss.util.filter(columns, function(col) {
                return !col.hidden;
            });
            keys = Ext.Array.pluck(columns, "dataIndex");
            // always have these fields for reclassifying
            keys = Ext.Array.union(keys, ["evid", "eventClass", "eventClassKey", "message"]);

            // grab any fields zenpack authors may add
            keys = Ext.Array.union(keys, Zenoss.events.eventFields);
            store.setBaseParam("keys", keys);
            store.setParamsParam("excludeNonActionables", this.excludeNonActionables);
        },
        getSelectionParameters: function() {
            var grid = this,
            sm = grid.getSelectionModel(),
            uid,
            evids = [],  // Event IDs selected
            sels = sm.getSelection();  // UI records selected
            if(Ext.isEmpty(sels)){ // if nothing is selected, check and see if there's an event_panel
                if(Ext.get('event_panel')) {
                    sels = Ext.getCmp('event_panel').getSelectionModel().getSelection();
                }
            }
            var selectedAll = (sm.selectState === 'All');
            if (selectedAll) {
                // If we are selecting all, we don't want to send back any evids.
                // this will make the operation happen on the filter's result
                // instead of whatever the view seems to have selected.
                sels = [];
            } else {
                Ext.each(sels, function(record){
                    evids[evids.length] = record.data.evid;
                });
            }

            // Don't run if nothing is selected.
            if (!selectedAll && Ext.isEmpty(sels)) {
                return false;
            }
            // if we are a contextual event console ALWAYS send the uid
            if (this.uid !== '/zport/dmd'){
                uid = this.uid;
            }
            var params = {
                evids: evids,
                excludeIds: sm.badIds,
                uid: uid
            };
            Ext.apply(params, this.getUpdateParameters());
            return params;
        },
        clearFilters: function(){
            this.filterRow.clearFilters();
        },
        onFiltersChanged: function(grid) {
            // ZEN-4441: Clear selections whenever the filter changes.
            var sm = grid.getSelectionModel();
            sm.clearSelections();
            sm.clearSelectState();
        },
        /*
         * Create parameters used for exporting events. This differs from
         * getSelectionParameters in that if no events are selected, all of
         * the events matching the current filters are exported.
         */
        getExportParameters: function() {
            var params = this.getSelectionParameters();
            if (params === false) {
                params = {
                    evids: [],
                    excludeIds: {}
                };
                Ext.apply(params, this.getUpdateParameters());
            }
            return params;
        },
        /*
         * Build parameters for updates (don't need to include sort information).
         */
        getUpdateParameters: function() {
            var o = {};
            o.params = this.filterRow.getSearchValues();
            o.params.excludeNonActionables = this.excludeNonActionables;
            return o;
        },
        toggleRowColors: function(bool) {
            this.rowcolors = bool;
            Ext.state.Manager.set('rowcolor', bool);
            this.refresh();
        },
        toggleNonActionables: function(bool) {
            this.excludeNonActionables = bool;
            Ext.state.Manager.set('excludeNonActionables', bool);
            this.refresh();
        },
        restoreURLState: function() {
            var qs = window.location.search.replace(/^\?/, ''), state;
            var decoded = Ext.urlDecode(qs);

            if (decoded.state) {
                try {
                    state = Ext.decode(Zenoss.util.base64.decode(decodeURIComponent(decoded.state)));
                } catch(e) { }
            //in case parameters are not encoded
            } else {
                state = {"filters": decoded};
            }

            Ext.state.Manager.set(this.stateId, state);
            this.fireEvent('recreateGrid', this);
        },
        clearURLState: function() {
            var qs = Ext.urlDecode(window.location.search.replace(/^\?/, ''));
            if (qs.state) {
                delete qs.state;
                qs = Ext.urlEncode(qs);
                if (qs) {
                    window.location.search = '?' + Ext.urlEncode(qs);
                } else {
                    window.location.search = '';
                }
            }
        },
        getPermalink: function() {
            var l = window.location,
            path = l.protocol + '//' + l.host + l.pathname + l.hash,
            st = Zenoss.util.base64.encode(Ext.encode(this.getState()));
            return path + '?state=' + st;
        },
        resetGrid: function() {
            Ext.state.Manager.clear(this.stateId);
            this.clearFilters();
            this.fireEvent('recreateGrid', this);
        },
        updateRows: function(){
            this.refresh();
        }
    });

    /**
     * @class Zenoss.SimpleEventGridPanel
     * @extends Zenoss.events.Grid
     * Shows events in a grid panel similar to that on the event console.
     * Fixed columns.
         * @constructor
         */
    Ext.define("Zenoss.SimpleEventGridPanel", {
            extend:"Zenoss.events.Grid",
            alias: ['widget.SimpleEventGridPanel'],
        constructor: function(config){

            var id = config.id || Ext.id();
            config.viewConfig = config.viewConfig || {};
                Ext.applyIf(config.viewConfig, {
                    getRowClass:  Zenoss.events.getRowClass
                });
            Ext.applyIf(config, {
                id: 'eventGrid' + id,
                stateId: Zenoss.env.EVENTSGRID_STATEID || 'default_eventsgrid',
                enableDragDrop: false,
                stateful: true,
                rowSelectorDepth: 5,
                store: Ext.create('Zenoss.events.Store', {}),
                appendGlob: true,
                selModel: new Zenoss.EventPanelSelectionModel({
                    grid: this
                }),
                defaultFilters: {
                    severity: [Zenoss.SEVERITY_CRITICAL, Zenoss.SEVERITY_ERROR, Zenoss.SEVERITY_WARNING, Zenoss.SEVERITY_INFO],
                    eventState: [Zenoss.STATUS_NEW, Zenoss.STATUS_ACKNOWLEDGED]
                },
                viewConfig: {
                    getRowClass:  Zenoss.events.getRowClass
                }
            }); // Ext.applyIf
            Zenoss.SimpleEventGridPanel.superclass.constructor.call(this, config);
            this.on('itemdblclick', this.onRowDblClick, this);
        }, // constructor
        onRowDblClick: function(view, record) {
            var evid = record.get('evid'),
                url = '/zport/dmd/Events/viewDetail?evid='+evid;
            window.open(url, evid.replace(/-/g,'_'),
                        "status=1,width=600,height=500,resizable=1");
        },
        initComponent: function() {
            this.callParent(arguments);

            /**
             * @event eventgridrefresh
             * Fires when the events grid is refreshed.
             * @param {Zenoss.SimpleEventGridPanel} this The gridpanel.
             */
            this.addEvents('eventgridrefresh');
        },
        /**
         *Since on a regular event console you can not choose which columns
         * are present we are overriding the default implementation of getState
         * to not remember the widths of columns that are not visible.
         * This is necessary because of our column definitions were creating
         * cookies larger than 8192 (the default zope max cookie size)
         **/
        getState: function(){

            var val = Zenoss.SimpleEventGridPanel.superclass.getState.call(this);
            // do not store the state of the hidden ones
            val.columns = Zenoss.util.filter(val.columns, function(col) {
                return !col.hidden;
            });
            return val;
        },
        refresh: function() {
            this.callParent(arguments);
            this.fireEvent('eventgridrefresh', this);
        }
    }); // SimpleEventGridPanel

    // Define all of the items that could be shown in an EventConsole toolbar.
    Zenoss.events.EventPanelToolbarSelectMenu = {
        text: _t('Select'),
        id: 'select-button',
        menu:{
            xtype: 'menu',
            items: [{
                text: _t("All"),
                tooltip: _t('Ctrl-a'),
                handler: function(){
                    var grid = Ext.getCmp('select-button').ownerCt.ownerCt,
                    sm = grid.getSelectionModel();
                    sm.selectEventState('All');
                    sm.setSelectState("All");
                }
            },{
                text: 'None',
                tooltip: _t('Esc'),
                handler: function(){
                    var grid = Ext.getCmp('select-button').ownerCt.ownerCt,
                    sm = grid.getSelectionModel();
                    sm.clearSelections();
                    sm.clearSelectState();
                }
            }]
        }
    };


    Ext.define("Zenoss.EventGridPanel", {
        extend: "Zenoss.SimpleEventGridPanel",
        alias: ['widget.EventGridPanel'],
        border:false,
        constructor: function(config) {
            Ext.applyIf(config, {
                tbar: new Zenoss.EventConsoleTBar({
                    gridId: config.id,
                    actionsMenu: config.actionsMenu,
                    commandsMenu: config.commandsMenu
                })
            });
            Zenoss.EventGridPanel.superclass.constructor.call(this, config);
        },
        onRowDblClick: function(view, record) {
            var evid = record.get('evid'),
                combo = Ext.getCmp('history_combo'),
                history = (combo.getValue() === '1') ? 'History' : '',
                url = '/zport/dmd/Events/view'+history+'Detail?evid='+evid;
            window.open(url, evid.replace(/-/g,'_'),
                        "status=1,width=600,height=500,resizable=1");
        },
        setContext: function(uid){
            if (uid) {
                this.uid = uid;
            }

            var toolbar = this.getDockedItems('toolbar')[0];
            if (toolbar && Ext.isDefined(toolbar.setContext)) {
                toolbar.setContext(uid);
            }
        }
    });

    Ext.define("Zenoss.EventRainbow", {
        extend:"Ext.toolbar.TextItem",
        alias: ['widget.eventrainbow'],
        constructor: function(config) {
            var severityCounts = {
                critical: {count: 0, acknowledged_count: 0},
                error:    {count: 0, acknowledged_count: 0},
                warning:  {count: 0, acknowledged_count: 0},
                info:     {count: 0, acknowledged_count: 0},
                debug:    {count: 0, acknowledged_count: 0},
                clear:    {count: 0, acknowledged_count: 0}
            };
            config = Ext.applyIf(config || {}, {
                height: 45,
                directFn: Zenoss.util.isolatedRequest(Zenoss.remote.DeviceRouter.getInfo),
                text: Zenoss.render.events(severityCounts, config.count || 3)
            });
            Zenoss.EventRainbow.superclass.constructor.call(this, config);
        },
        setContext: function(uid){
            if (uid) {
                this.uid = uid;
                this.refresh();
            }
        },
        refresh: function(){
            this.directFn({uid:this.uid, keys:['events']}, function(result){
                if (Zenoss.env.contextUid && Zenoss.env.contextUid !== this.uid) {
                    return;
                }
                this.updateRainbow(result.data.events);
            }, this);
        },
        updateRainbow: function(severityCounts) {
            this.setText(Zenoss.render.events(severityCounts, this.count));
        }
    });

    // Functions related to creating the grid with only the columns that are going to be visible.
    // We used to create the grid with all the available columns and then hide the ones that we dont want to display.
    // From now on, the grid will only contain the visible columns.
    // Grids that use this functionality:
    //
    //      -- File --             -- state_id --
    //      EvConsole.js            evconsole
    //      itinfrastructure.js     infrastructure_events
    //      devdetail.js            device_events
    //      EvHistory.js            histconsole
    //

    var getColumnDefinitionById = function(col_id)
    {
        var col_def;
        Ext.each(Zenoss.env.getColumnDefinitions(), function(col) {
            if (col.id === col_id) {
                col_def = col;
                return false;
            }
        });
        return col_def;
    };

    var getDefaultColumnIdsToRender = function()
    {
        var column_ids_to_render = [];
        Ext.each(Zenoss.env.COLUMN_DEFINITIONS, function(col) {
            if( !col.hidden) {
                column_ids_to_render.push(col.id);
            }
        });

        return column_ids_to_render;
    };

    // The columns to render are extracted from Zenoss.env.COLUMN_DEFINITIONS and
    // the pre existing grid state.
    Zenoss.env.getColumnIdsToRender = function(state_id)
    {
        var state = Ext.state.Manager.get(state_id);
        var column_ids_to_render = [];

        if (state !== undefined && state.columns !== undefined && state.columns.length > 0)
        {
            Ext.each(state.columns, function(col_state) {
                var col_def = getColumnDefinitionById(col_state.id);
                if ( (col_state.hidden === undefined && !col_def.hidden) || (col_state.hidden !== undefined && col_state.hidden === false) )
                {
                    column_ids_to_render.push(col_state.id);
                }
            });
        }
        else
        {
            column_ids_to_render = getDefaultColumnIdsToRender(state_id);
        }

        return column_ids_to_render;
    };

    var getColumnStateById = function(state_id, col_id)
    {
        var state = Ext.state.Manager.get(state_id);
        var col_state;

        if (state !== undefined && state.columns !== undefined && state.columns.length > 0)
        {
            Ext.each(state.columns, function(c_state) {
                if (c_state.id === col_id)
                {
                    col_state = c_state;
                    return false; // to exit the Ext.each
                }
            });
        }

        return col_state;
    };

    Zenoss.env.recreateGridWithNewColumns = function(grid, col_ids)
    {
        // We modify the grid's state to reflect the new column settings
        // this way when the grid is recreated picks the new config.
        var state_id = grid.stateId;

        // We iterate the new cols and search for previous setting for
        // them in the state. If found in the state we re use them
        // in order to preserve the user's settings
        var new_cols_state = [];
        Ext.each(col_ids, function(col_id) {
            var col_state = getColumnStateById(state_id, col_id);
            if (col_state === undefined)
            {
                var col_def = getColumnDefinitionById(col_id);
                col_state = {};
                col_state.id = col_id;
                if (col_def.hidden) {
                    col_state.hidden = false;
                }
            }
            new_cols_state.push(col_state);
        });

        // Save the new state
        var state = Ext.state.Manager.get(state_id);

        if (state !== undefined)
        {
            state.columns = new_cols_state;
            Ext.state.Manager.set(state);
        }

        // Sends this event so each grid can recreate itself
        grid.fireEvent('recreateGrid', grid);
    };

    Zenoss.env.getColumnDefinitionsToRender = function(state_id)
    {
        var column_ids = Zenoss.env.getColumnIdsToRender(state_id);
        var columns_to_render = [];

        Ext.each(Zenoss.env.COLUMN_DEFINITIONS, function(col) {
            var index = Ext.Array.indexOf(column_ids, col.id);
            if (index > -1) {
                columns_to_render.push(col);
            }
        });
        return columns_to_render;
    };

// End of functions related to saving the state the columns to display

})(); // end of function namespace scoping
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');




/**
 * @class Zenoss.InstanceModel
 * @extends Ext.data.Model
 * Field definitions for the instances, matches up with the columns above
 **/
Ext.define('Zenoss.InstanceModel',  {
    extend: 'Ext.data.Model',
    idProperty: 'uid',
    fields: [
        {name: 'device'},
        {name: 'name'},
        {name: 'description'},
        {name: 'minProcessCount'},
        {name: 'maxProcessCount'},
        {name: 'monitored'},
        {name: 'pingStatus'}
    ]
});

/**
 * @class Zenoss.InstanceStore
 * @extend: Zenoss.DirectStore
 * Base store for the instances of things
 */
Ext.define("Zenoss.InstanceStore", {
    extend:"Zenoss.DirectStore",
    alias: ['widget.InstanceStore'],

    constructor: function(config) {
        Ext.applyIf(config, {
            model: 'Zenoss.InstanceModel',
            root: 'data',
            initialSortColumn: 'name'
        });
        this.callParent(arguments);
    }

});


Ext.define("Zenoss.SimpleInstanceGridPanel", {
    extend:"Zenoss.BaseGridPanel",
    alias: ['widget.SimpleInstanceGridPanel'],

    constructor: function(config) {
        var instanceColumns = [{
            id: 'instanceName',
            dataIndex: config.nameDataIndex || 'name',
            header: _t('Name'),
            flex: 0.25,
            width: 400
        },{
            id: 'device',
            dataIndex: 'device',
            flex: 0.75,
            header: _t('Device'),
            width: 200,
            sortable: false,
            renderer: function(device){
                return Zenoss.render.link(device.uid, undefined,
                                          device.name);
            }
        }, {
            dataIndex: 'minProcessCount',
            header: _t('Min Threshold'),
            width: 85,
            sortable: false,
            hidden: !config.showProcessCount
        }, {
            dataIndex: 'maxProcessCount',
            header: _t('Max Threshold'),
            width: 85,
            sortable: false,
            hidden: !config.showProcessCount
        }, {
            id: 'monitored',
            dataIndex: 'monitored',
            header: _t('Monitored'),
            width: 70,
            sortable: false
        }, {
            id: 'status',
            dataIndex: 'pingStatus',
            header: _t('Status'),
            renderer: Zenoss.render.pingStatus,
            width: 60,
            sortable: false
        }];

        Ext.applyIf(config, {
            columns: config.columns || instanceColumns,
            store: config.store || Ext.create('Zenoss.InstanceStore',{
                directFn: config.directFn,
                pageSize: config.pageSize
            }),
            selModel: config.sm || Ext.create('Zenoss.ExtraHooksSelectionModel', {
                mode: 'SINGLE'
            })
        });
        this.callParent(arguments);
    }
});

// supply the instances implementation in the config
Ext.define("Zenoss.SimpleCardPanel", {
    extend:"Ext.Panel",
    alias: ['widget.simplecardpanel'],

    constructor: function(config) {
        this.contextUid = null;
        Ext.applyIf(config, {
            layout: 'card',
            activeItem: 0,
            placeholder: {},
            height: Math.min(((Ext.getCmp('viewport').getHeight() - 75)/5)+30, 200),
            tbar: {
                xtype: 'consolebar',
                centerPanel: 'detail_panel',
                title: _t('Display: '),
                parentPanel: this,
                leftItems: [{
                    xtype: 'select',
                    ref: '../displaySelect',
                    queryMode: 'local',
                    value: config.instancesTitle,
                    store: [config.instancesTitle, _t('Configuration Properties')],
                    listeners: {
                        select: function(displaySelect, records) {
                            var index = records[0].index;
                            this.layout.setActiveItem(index);
                        },
                        scope: this
                    }
                }]
            },
            items: [
                config.instances,
            {
                xtype: 'configpropertypanel',
                id: 'config_property_panel',
                ref: 'zPropertyEdit',
                displayFilters: false,
                viewName: 'zPropertyEdit',
                listeners: config.zPropertyEditListeners
            }]
        });
        this.callParent(arguments);
    },
    setContext: function(uid) {
        // only reload our datastores if we are not collapsed
        this.contextUid = uid;
        if (!this.collapsed){
            this.loadInstances(uid);
        }
    },
    loadInstances: function(){
        if (!this.contextUid){
            return;
        }
        var contextUid = this.contextUid;
        // if we have not set our store since we last updated the
        // context uid update it now
        this.items.each(function(item) {
            item.setContext(contextUid);
        });
        this.contextUid = null;
    }

});

// has the instances used by the Netwoks, Processes and the two Services pages
Ext.define("Zenoss.InstanceCardPanel", {
    extend:"Zenoss.SimpleCardPanel",
    alias: ['widget.instancecardpanel'],

    constructor: function(config) {
        var gridId = config.gridId || Ext.id();

        Ext.applyIf(config, {
            instances: {
                xtype: 'SimpleInstanceGridPanel',
                id: gridId,
                bufferSize: config.bufferSize,
                nearLimit: config.nearLimit,
                directFn: config.router.getInstances,
                nameDataIndex: config.nameDataIndex || "name",
                columns: config.columns,
                sm: config.sm,
                store: config.store,
                showProcessCount: config.showProcessCount
            }
        });
        this.callParent(arguments);
        this.gridId = gridId;
    },
    getInstancesGrid: function() {
        return Ext.getCmp(this.gridId);
    }

});

})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');

/**
 * @class Zenoss.ViewButton
 * @extends Ext.Button
 * A button that toggles between cards in a panel with a card layout.
 * @constructor
 */
Ext.define("Zenoss.ViewButton", {
    extend:"Ext.Button",
    alias: ['widget.ViewButton'],

    constructor: function(userConfig) {

        var baseConfig = {
            enableToggle: true,
            toggleGroup: 'CardButtonPanel',
            allowDepress: false
        };

        var config = Ext.apply(baseConfig, userConfig);
        Zenoss.ViewButton.superclass.constructor.call(this, config);
    }

});



/**
 * @class Zenoss.CardButtonPanel
 * @extends Ext.Button
 * A Panel with a card layout and toolbar buttons for switching between the
 * cards.
 * @constructor
 */
Ext.define("Zenoss.CardButtonPanel", {
    extend:"Ext.Panel",
    alias: ['widget.CardButtonPanel'],

    constructor: function(config) {
        // Inner secret closure function to create the handler
        function createToggleHandler(cardPanel, panel) {
            return function(button, pressed) {
                if (pressed) {
                    cardPanel.fireEvent('cardchange', panel);
                    cardPanel.getLayout().setActiveItem(panel.id);
                }
            };
        }

        function syncButtons(me) {
            var tb = me.getDockedItems('toolbar')[0];
            for (var idx=0; idx < me.items.getCount(); ++idx) {
                var newComponent = me.items.get(idx);

                if (newComponent instanceof Ext.Panel) {
                    tb.add({
                        xtype: 'ViewButton',
                        id: 'button_' + newComponent.id,
                        text: Ext.clean(newComponent.buttonTitle,
                                        newComponent.title, 'Undefined'),
                        pressed: (newComponent === me.layout.activeItem),
                        iconCls: newComponent.iconCls,
                        toggleHandler: createToggleHandler(me, newComponent)
                    });
                }
            }
        }

        Ext.applyIf(config, {
            id: 'cardPanel',
            layout: 'card',
            activeItem: 0
        });

        Ext.apply(config, {
            header: false,
            tbar: [{
                xtype: 'tbtext',
                text: _t('View: ')
            }]
        });

        this.addEvents('cardchange');
        this.on('afterrender', syncButtons, this);
        this.listeners = config.listeners;
        Zenoss.CardButtonPanel.superclass.constructor.call(this, config);
    }
});



})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');


var oldActiveItem = Ext.layout.CardLayout.prototype.setActiveItem;
var oldInitEvents = Ext.layout.CardLayout.prototype.initEvents;

Ext.override(Ext.layout.CardLayout, {
    initEvents: function() {
        oldInitEvents.apply(this, arguments);
        this.owner.addEvents('cardchange');
    },
    setActiveItem: function(item) {
        oldActiveItem.apply(this, arguments);
        this.owner.fireEvent('cardchange', this.owner, item);
    }
});

Ext.define("Zenoss.ContextCardPanel", {
    extend:"Ext.Panel",
    alias: ['widget.contextcardpanel'],
    contextUid: null,
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: {
                type: 'card',
                deferredRender: true
            }
        });
        Zenoss.ContextCardPanel.superclass.constructor.call(this, config);
        this.on('cardchange', this.cardChangeHandler, this);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.cardChangeHandler(this.layout.activeItem);
    },
    cardChangeHandler: function(panel) {
        if (panel!==this && panel.setContext) {
            panel.setContext(this.contextUid);
        }
    }

});




})();
/* globals unescape:true */
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

var router, treeId, initTreeDialogs;

router = Zenoss.remote.TemplateRouter;
treeId = 'templateTree';



/**
 * The two default views
 **/
Ext.ns('Zenoss', 'Zenoss.templates');
Zenoss.templates.templateView = 'template';
Zenoss.templates.deviceClassView = 'deviceClass';


initTreeDialogs = function(tree) {

    new Zenoss.HideFormDialog({
        id: 'addTemplateDialog',
        title: _t('Add Monitoring Template'),
        items: {
            xtype: 'textfield',
            id: 'idTextfield',
            fieldLabel: _t('Name'),
            allowBlank: false
        },
        listeners: {
            'hide': function() {
                Ext.getCmp('idTextfield').setValue('');
            }
        },
        buttons: [
            {
                xtype: 'HideDialogButton',
                text: _t('Submit'),
                handler: function() {
                    var id = Ext.getCmp('idTextfield').getValue();
                    tree.addTemplate(id);
                }
            }, {
                xtype: 'HideDialogButton',
                text: _t('Cancel')
            }
        ]
    });

    new Zenoss.MessageDialog({
        id: 'deleteNodeDialog',
        title: _t('Delete Tree Node'),
        message: _t('The selected node will be deleted.'),
        okHandler: function(){
            tree.deleteTemplate();
        }
    });

};

Ext.ns('Zenoss');

/**
 * @class Zenoss.TemplateTreePanel
 * @extends Ext.tree.TreePanel
 * @constructor
 */
Ext.define("Zenoss.TemplateTreePanel", {
    alias: ['widget.TemplateTreePanel'],
    extend:"Zenoss.HierarchyTreePanel",

    constructor: function(config) {
        var currentView = config.currentView,
            directFn = router.getTemplates;
        if (currentView === Zenoss.templates.deviceClassView) {
            directFn = router.getDeviceClassTemplates;
        }
        this.currentView = currentView;

        Ext.applyIf(config, {
            id: treeId,
            rootVisible: false,
            autoScroll: true,
            containerScroll: true,
            useArrows: true,
            searchField: true,
            loadMask: true,
            router: router,
            cls: 'x-tree-noicon',
            idProperty: 'id',
            directFn: directFn,
            nodeName: 'Templates',
            root: {
                id: 'root',
                uid: '/zport/dmd/Devices',
                text: _t('Templates')
            }
        });

        this.callParent(arguments);
        initTreeDialogs(this);
        this.on('buttonClick', this.buttonClickHandler, this);
    },
    buttonClickHandler: function(buttonId) {
        switch(buttonId) {
            case 'addButton':
                Ext.getCmp('addTemplateDialog').show();
                break;
            case 'deleteButton':
                Ext.getCmp('deleteNodeDialog').show();
                break;
            default:
                break;
        }
    },

    addTemplate: function(id) {
        var rootNode, contextUid, params, tree;
        rootNode = this.getRootNode();
        contextUid = rootNode.data.uid;
        params = {contextUid: contextUid, id: id};
        tree = this;
        function callback(provider, response) {
            var result, nodeConfig, node, leaf;
            result = response.result;
            if (result.success) {
                nodeConfig = response.result.nodeConfig;
                node = tree.getLoader().createNode(nodeConfig);
                rootNode.appendChild(node);
                node.expand();
                leaf = node.childNodes[0];
                leaf.select();
            } else {
                Ext.Msg.alert('Error', result.msg);
            }
        }
        router.addTemplate(params, callback);
    },

    deleteTemplate: function() {
        var node, params, me;
        node = this.getSelectionModel().getSelectedNode();
        params = {uid: node.data.uid};
        me = this;
        function callback() {
            me.getRootNode().reload();
        }
        router.deleteTemplate(params, callback);
    },

    createDeepLinkPath: function(node) {
        var path;
        if (this.currentView !== Zenoss.templates.deviceClassView) {
            path = this.id + Ext.History.DELIMITER + node.data.uid;
        }else {
            path = this.id + Ext.History.DELIMITER + node.get("id");
        }

        return path;
    },
    onExpandnode: function(node) {
        // select the first template when the base URL is accessed without a
        // history token and without a filter value
        if ( ! this.searchField.getValue() && ! Ext.History.getToken() ) {
            if ( node === this.getRootNode() ) {
                node.childNodes[0].expand();
            } else {
                node.childNodes[0].select();
            }
        }
    },
    selectByToken: function(id) {
        if (this.currentView === Zenoss.templates.deviceClassView){
            this.callParent([unescape(id.replace(/\//g, '.'))]);
        }else{
            this.templateViewSelectByToken(id);
        }
    },
    getFilterFn: function(text) {
        if (this.currentView !== Zenoss.templates.deviceClassView){
            var regex = new RegExp(Ext.String.escapeRegex(text),'i');
            var fn = function(item){
                return regex.test(item.get('id'));
            };
            return fn;
        }
        // match against the displayed text
        return this.callParent(arguments);
    },
    addHistoryToken: function(view, node) {
        Ext.History.add(this.id + Ext.History.DELIMITER + node.get('uid'));
    },
    templateViewSelectByToken: function(uid) {
        var root = this.getRootNode(),
            selNode = Ext.bind(function(){
                // Translates from the History token into the
                // path for the tree
                // for example this:
                //     "/zport/dmd/Devices/rrdTemplates/ethernetCsmacd"
                // turns into:
                //     "/root/ethernetCsmacd/ethernetCsmacd..Devices"
                var templateSplit, pathParts, nameParts,
                templateName, dmdPath, path;

                if (uid.search('/rrdTemplates/') !== -1) {
                    // this is a device class
                    templateSplit = unescape(uid).split('/rrdTemplates/');
                    pathParts = templateSplit[0].split('/');
                    nameParts = templateSplit[1].split('/');
                    templateName = nameParts[0];
                } else {
                    // it is a template on a device
                    pathParts = uid.split('/');
                    templateName = pathParts.pop();

                    // determine if this is a component
                    var devicesIndex = pathParts.indexOf("devices");
                    if((pathParts.length - 1) > devicesIndex){
                        pathParts.splice(devicesIndex + 2, 1);
                        pathParts.splice(devicesIndex, 1);
                    }
                }

                if ( pathParts.length === 4 ) {
                    // Defined at devices, special case, include 'Devices'
                    dmdPath = 'Devices';
                } else {
                    // all the DeviceClass names under Devices separated by dots
                    dmdPath = pathParts.slice(4).join('.');
                }
                path = Ext.String.format('/root/{0}/{0}..{1}', templateName, dmdPath);
                this.selectPath(path);
            }, this);
        if (!root.isLoaded()) {
            // Listen on expand because if we listen on the store's load expand
            // gets double-called.
            root.on('expand', selNode, this, {single: true});
        } else {
            selNode();
        }

    },

    manualSelect: function(uid, templateName) {
        var theTree = this;
        var callback = function(success) {
            if (!success) {
                theTree.getRootNode().eachChild(function(node) {
                    if (templateName === node.data.id){
                        node.eachChild(function(node){
                            if (uid === node.data.uid) {
                                node.select();
                                return false;
                            }
                        });
                        return false;
                    }
                });
            }
        };
        return callback;
    }

});



})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {

    var ZD = Ext.ns('Zenoss.devices');

    Ext.define("Zenoss.form.SmartCombo", {
        extend: "Ext.form.ComboBox",
        alias: ['widget.smartcombo'],
        constructor: function(config) {
            config = Ext.applyIf(config || {}, {
                queryMode: config.autoLoad !== false ? 'local':'remote',
                store: new Zenoss.NonPaginatedStore({
                    directFn: config.directFn,
                    root: config.root || 'data',
                    model: config.model || 'Zenoss.model.NameValue',
                    initialSortColumn: config.initialSortColumn || 'name',
                    initialSortDirection: config.initialSortDirection || 'ASC'
                }),
                valueField: 'value',
                displayField: 'name',
                forceSelection: true,
                editable: false,
                autoSelect: true,
                selectOnFocus: false,
                triggerAction: 'all'
            });
            this.callParent([config]);
            if (this.autoLoad!==false) {
                this.getStore().load();
            }
        },
        getValue: function() {
            return this.callParent(arguments) || this.getRawValue();
        },
        getStore: function() {
            return this.store;
        }
    });

    Ext.define("Zenoss.model.ValueIntModel", {
        extend: 'Ext.data.Model',
        idProperty: 'name',
        fields: [
            { name: 'name', type: 'string'},
            { name: 'value', type: 'int'}
        ]
    });

    Ext.define("Zenoss.devices.PriorityCombo", {
        extend:"Zenoss.form.SmartCombo",
        alias: ['widget.PriorityCombo'],
        constructor: function(config) {
            config = Ext.apply(config || {}, {
                directFn: Zenoss.remote.DeviceRouter.getPriorities,
                cls: 'prioritycombo',
                model: 'Zenoss.model.ValueIntModel',
                initialSortColumn: 'value',
                initialSortDirection: 'DESC'

            });
            this.callParent([config]);
        },
        getValue: function() {
            // This method is being overridden because the check in SmartCombo
            // will not allow zero as a value; it will fallback and send the
            // raw value, which for Priority is the string "Trivial".
            var result = this.callParent(arguments);
            if (Ext.isString(result)) {
                Zenoss.env.initPriorities();
                Ext.each(Zenoss.env.PRIORITIES, function(item) {
                    if (item.name === result) {
                        result = item.value;
                        return false; // break
                    }
                });
            }
            return result;
        }
    });



    Ext.define("Zenoss.devices.DevicePriorityMultiselectMenu", {
        extend:"Zenoss.MultiselectMenu",
        alias: ['widget.multiselect-devicepriority'],
        constructor: function(config) {
            config = Ext.apply(config || {}, {
                text:'...',
                cls: 'x-btn x-btn-default-toolbar-small',
                source: Zenoss.env.priorities,
                defaultValues: []
            });
            ZD.DevicePriorityMultiselectMenu.superclass.constructor.call(this, config);
        }
    });


    Ext.define("Zenoss.devices.ProductionStateCombo", {
        extend:"Zenoss.form.SmartCombo",
        alias: ['widget.ProductionStateCombo'],
        constructor: function(config) {
            config = Ext.apply(config || {}, {
                directFn: Zenoss.remote.DeviceRouter.getProductionStates,
                model: 'Zenoss.model.ValueIntModel',
                initialSortColumn: 'value',
                initialSortDirection: 'DESC'
            });
            this.callParent([config]);
        }
    });


    Ext.define("Zenoss.devices.DeviceStatusMultiselectMenu", {
        extend:"Zenoss.MultiselectMenu",
        alias: ['widget.multiselect-devicestatus'],
        constructor: function(config) {
            var defaults = [];
            config = Ext.apply(config || {}, {
                text: '...',
                cls: 'x-btn x-btn-default-toolbar-small',
                source: [
                    {name: 'Up', value: true},
                    {name: 'Down', value: false},
                    {name: 'Unknown', value: null},
                ],
                defaultValues: defaults
            });
            this.callParent([config]);
        }
    });


    Ext.define("Zenoss.devices.ProductionStateMultiselectMenu", {
        extend:"Zenoss.MultiselectMenu",
        alias: ['widget.multiselect-prodstate'],
        constructor: function(config) {
            var defaults = [];
            if (Ext.isDefined(Zenoss.env.PRODUCTION_STATES)) {
                defaults.Array = Ext.pluck(Zenoss.env.PRODUCTION_STATES, 'value');
            }
            config = Ext.apply(config || {}, {
                text:'...',
                cls: 'x-btn x-btn-default-toolbar-small',
                source: Zenoss.env.productionStates,
                defaultValues: defaults
            });
            this.callParent([config]);
        }
    });


    Ext.define("Zenoss.devices.ManufacturerDataStore", {
        extend:"Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            var router = config.router || Zenoss.remote.DeviceRouter;
            Ext.applyIf(config, {
                root: 'manufacturers',
                totalProperty: 'totalCount',
                initialSortColumn: 'name',
                model: 'Zenoss.model.Name',
                directFn: router.getManufacturerNames
            });
            this.callParent([config]);
        }
    });

    Ext.define("Zenoss.devices.OSProductDataStore", {
        extend:"Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            var router = config.router || Zenoss.remote.DeviceRouter;
            Ext.applyIf(config, {
                root: 'productNames',
                totalProperty: 'totalCount',
                model: 'Zenoss.model.Name',
                initialSortColumn: 'name',
                directFn: router.getOSProductNames
            });
            this.callParent([config]);
        }
    });

    Ext.define("Zenoss.devices.HWProductDataStore", {
        extend:"Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            var router = config.router || Zenoss.remote.DeviceRouter;
            Ext.applyIf(config, {
                root: 'productNames',
                totalProperty: 'totalCount',
                model: 'Zenoss.model.Name',
                initialSortColumn: 'name',
                directFn: router.getHardwareProductNames
            });
            this.callParent([config]);
        }
    });

    Ext.define("Zenoss.devices.ManufacturerCombo", {
        extend:"Zenoss.form.SmartCombo",
        alias: ['widget.manufacturercombo'],
        constructor: function(config) {
            var store = (config||{}).store || new ZD.ManufacturerDataStore();
            config = Ext.applyIf(config||{}, {
                store: store,
                width: 160,
                displayField: 'name',
                valueField: 'name'
            });
            this.callParent([config]);
        }
    });


    Ext.define("Zenoss.devices.ProductCombo", {
        extend:"Zenoss.form.SmartCombo",
        alias: ['widget.productcombo'],
        constructor: function(config) {
            var manufacturer = config.manufacturer || "",
                prodType = config.prodType || 'OS',
                store = (config||{}).store ||
                    prodType==='OS' ? new ZD.OSProductDataStore() : new ZD.HWProductDataStore();
            store.setBaseParam('manufacturer', manufacturer);
            config = Ext.applyIf(config||{}, {
                store: store,
                displayField: 'name',
                valueField: 'name',
                width: 160,
                queryMode: 'remote'
            });
            this.callParent([config]);
        }
    });

}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

var router, dataSourcesId, graphsId, resetCombo,
    addMetricToGraph, showAddToGraphDialog, editDataSourcesId, treeId,
    editingReSelectId;

Ext.ns('Zenoss');

router = Zenoss.remote.TemplateRouter;
dataSourcesId = 'dataSourceTreeGrid';
graphsId = 'graphGrid';
editDataSourcesId = "editDataSource";

// NOTE: this must match the tree id from the template.js file
treeId = 'templateTree';

/**
 *@returns the currently selected Data Source or Data Point, or none if nothing is selected
 **/
function getSelectedDataSourceOrPoint() {
    return Ext.getCmp(dataSourcesId).getSelectionModel().getSelectedNode();
}

resetCombo = function(combo, uid) {
    combo.clearValue();
    combo.getStore().setBaseParam('uid', uid);
    delete combo.lastQuery;
    combo.doQuery(combo.allQuery, true);
};

addMetricToGraph = function(dataPointUid, graphUid) {
    var params, callback;
    params = {dataPointUid: dataPointUid, graphUid: graphUid};
    router.addDataPointToGraph(params, callback);
};

Ext.define('Zenoss.GraphModel', {
    extend: 'Ext.data.Model',
    idProperty: 'uid',
    fields: [
        'uid', 'name', 'graphPoints', 'units', 'height', 'width',
             'sequence'
    ]
});

Ext.define("Zenoss.GraphStore", {
    extend:"Zenoss.NonPaginatedStore",
    alias: ['widget.graphstore'],
    constructor: function(config) {
        Ext.applyIf(config, {
            root: 'data',
            directFn: router.getGraphs,
            model: 'Zenoss.GraphModel'
        });
        this.callParent(arguments);
    }
});


showAddToGraphDialog = function(node, templateUid) {
    var nodeDataSource, metricName, html, combo;
    nodeDataSource = node;
    if ( nodeDataSource && nodeDataSource.isLeaf() ) {
        metricName = nodeDataSource.data.name;
        html = '<div>Data Point</div>';
        html += '<div>' + metricName + '</div><br/>';

        Ext.create('Zenoss.dialog.BaseWindow', {
            id: 'addToGraphDialog',
            width: 400,
            height: 250,
            title: _t('Add Data Point to Graph'),
            items: [
                {
                    xtype: 'panel',
                    id: 'addToGraphMetricPanel'
                }, {
                    xtype: 'combo',
                    id: 'graphCombo',
                    fieldLabel: _t('Graph'),
                    displayField: 'name',
                    valueField: 'uid',
                    width:300,
                    minChars: 999, // only do an all query
                    resizeable: true,
                    editable: false,
                    emptyText: 'Select a graph...',
                    store: new Zenoss.GraphStore({}),
                    listeners: {select: function(){
                        Ext.getCmp('addToGraphDialog').submit.enable();
                    }}
                }],
            buttons: [
                {
                    xtype: 'DialogButton',
                    ref: '../submit',
                    text: _t('Submit'),
                    disabled: true,
                    handler: function() {
                        var node, datapointUid, graphUid;
                        node = Ext.getCmp(dataSourcesId).getSelectionModel().getSelectedNode();
                        datapointUid = node.data.uid;
                        graphUid = Ext.getCmp('graphCombo').getValue();
                        addMetricToGraph(datapointUid, graphUid);
                    }
                }, {
                    xtype: 'DialogButton',
                    text: _t('Cancel')
                }]
        }).show();

        Ext.getCmp('addToGraphMetricPanel').body.update(html);
        combo = Ext.getCmp('graphCombo');
        resetCombo(combo, templateUid);
        Ext.getCmp('addToGraphDialog').submit.disable();
    } else {
        new Zenoss.dialog.ErrorDialog({message: _t('You must select a Data Point.')});
    }
};


/**********************************************************************
 *
 * Add Data Point
 *
 **/

/**
 * Causes the DataSources Grid to refresh from the server
 *
 **/
function refreshDataSourceGrid(selectedId) {
    var grid = Ext.getCmp(dataSourcesId);
    if (selectedId) {
        grid.refresh(function(){
            grid.getRootNode().cascade(function(node){
                if (node.data.id === selectedId) {
                    node.expand();
                    node.select();
                }
            });
        });
    }else{
        grid.refresh();
    }
}


Ext.define('Zenoss.templates.AddDataPointDialog', {
    alias: ['widget.addDpDialog'],
    extend: 'Zenoss.dialog.BaseWindow',
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            title: _t('Add Data Point'),
            height: 160,
            width: 310,
            listeners: {
                validitychange: function(form, isValid) {
                    this.ownerCt.query('DialogButton')[0].setDisabled(!isValid);
                }
            },
            items:{
                xtype: 'form',
                buttonAlign: 'left',
                items: [{
                    xtype: 'idfield',
                    ref: 'metricName',
                    fieldLabel: _t('Name'),
                    allowBlank: false,
                    regex: Zenoss.env.textMasks.allowedNameTextDash,
                    regexText: Zenoss.env.textMasks.allowedNameTextFeedbackDash,
                    blankText: _t('Name is a required field')
                }],
                buttons: [{
                    xtype: 'DialogButton',
                    ref: '../submitNewDataPoint',
                    text: _t('Submit'),
                    formBind: true,
                    handler: function() {
                        var parameters = {
                            name: this.refOwner.metricName.getValue(),
                            dataSourceUid: config.dataSourceUid
                        };
                        return router.addDataPoint(parameters, function() {
                            if (config.selectedId) {
                                refreshDataSourceGrid(config.dataSourceId);
                            } else {
                                refreshDataSourceGrid();
                            }
                        });
                    }
                },{
                    xtype: 'DialogButton',
                    text: _t('Cancel')
                }]
            }
        });
        this.callParent(arguments);
    }
});

/**
 * Displays the Add Data Point dialog and saves the inputted infomation
 * back to the server
 **/
function showAddDataPointDialog() {
    var grid = Ext.getCmp(dataSourcesId),
        selectedNode = grid.getSelectionModel().getSelectedNode();

    // make sure they selected a node
    if (!selectedNode) {
        new Zenoss.dialog.ErrorDialog({message: _t('You must select a data source.')});
        return;
    }

    // display the name dialog
    Ext.create('Zenoss.templates.AddDataPointDialog', {
        dataSourceUid: selectedNode.data.uid,
        dataSourceId: selectedNode.data.id
    }).show();
}

/**********************************************************************
 *
 * Add Data Source
 *
 */

/**
 * @class Zenoss.templates.DataSourceTypeModel
 * @extends Ext.data.Model
 *
 **/
Ext.define('Zenoss.templates.DataSourceTypeModel',  {
    extend: 'Ext.data.Model',
    idProperty: 'type',
    fields: [
        {name: 'type'}
    ]
});

Ext.define('Zenoss.templates.AddDataSourceDialog', {
    alias: ['widget.addDsDialog'],
    extend: 'Zenoss.dialog.BaseWindow',
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            title: _t('Add Data Source'),
            height: 180,
            width: 350,
            items:{
                xtype:'form',
                buttonAlign: 'left',
                listeners: {
                    validitychange: function(form, isValid) {
                        if (isValid) {
                            this.ownerCt.query('button')[0].enable();
                        } else {
                            this.ownerCt.query('button')[0].disable();
                        }
                    }
                },
                items:[{
                    xtype: 'idfield',
                    ref: 'dataSourceName',
                    fieldLabel: _t('Name'),
                    allowBlank: false,
                    regex: Zenoss.env.textMasks.allowedNameTextDash,
                    regexText: Zenoss.env.textMasks.allowedNameTextFeedbackDash,
                    blankText: _t('Name is a required field')
                }, {
                    xtype: 'combo',
                    ref: 'dataSourceTypeCombo',
                    allowBlank: false,
                    displayField: 'type',
                    fieldLabel: _t('Type'),
                    editable: false,
                    value: 'SNMP',
                    triggerAction: 'all',
                    store:  {
                        type: 'directcombo',
                        model: 'Zenoss.templates.DataSourceTypeModel',
                        root: 'data',
                        directFn: router.getDataSourceTypes
                    }
                }],
                buttons:[{
                    xtype: 'DialogButton',
                    ref: '../submitNewDataSource',
                    disabled: true,
                    text: _t('Submit'),
                    handler: function() {
                        var parameters = {
                            name: this.refOwner.dataSourceName.getValue(),
                            type: this.refOwner.dataSourceTypeCombo.getValue(),
                            templateUid: config.templateUid
                        };
                        return router.addDataSource(parameters, refreshDataSourceGrid);
                    }
                },Zenoss.dialog.CANCEL
                ]
            }
        });
        this.callParent(arguments);
    }
});

/**
 * Shows the Add Data Source dialog and saves the inputted information
 * back to the server
 **/
function showAddDataSourceDialog() {
    var cmp = Ext.getCmp(treeId),
        selectedNode = cmp.getSelectionModel().getSelectedNode(),
        templateUid;

    // make sure they selected a node
    if (!selectedNode) {
        new Zenoss.dialog.ErrorDialog({message: _t('You must select a template.')});
        return;
    }
    templateUid = selectedNode.data.uid;
    Ext.create('Zenoss.templates.AddDataSourceDialog', {templateUid: templateUid}).show();
}

/**********************************************************************
 *
 * Delete DataSource
 *
 */

/**
 * Creates the dynamic delete message and shows the dialog
 **/
function showDeleteDataSourceDialog() {
    var msg, name, html, dialog;
    if (getSelectedDataSourceOrPoint()) {
        // set up the custom delete message
        msg = _t("Are you sure you want to remove {0}? There is no undo.");
        name = getSelectedDataSourceOrPoint().data.name;
        html = Ext.String.format(msg, name);

        // show the dialog
        dialog = Ext.getCmp('deleteDataSourceDialog');
        dialog.setText(html);
        dialog.show();
    }else{
        new Zenoss.dialog.ErrorDialog({message: _t('You must select a Data Source or Data Point.')});
    }
}

new Zenoss.MessageDialog({
    id: 'deleteDataSourceDialog',
    title: _t('Delete'),
    // msg is generated dynamically
    okHandler: function(){
        var params, node = getSelectedDataSourceOrPoint(),
        selectedId, callback;
        params = {
            uid: getSelectedDataSourceOrPoint().get("uid")
        };
        callback = function() {
            refreshDataSourceGrid(selectedId);
        };
        // data points are always leafs
        if (getSelectedDataSourceOrPoint().data.leaf) {
            selectedId = node.parentNode.data.id;

            router.deleteDataPoint(params, callback);
        }else {
            router.deleteDataSource(params, refreshDataSourceGrid);
        }
    }
});

/**********************************************************************
 *
 * Edit DataSource/DataPoint
 *
 */

/**
 * Closes the edit dialog and updates the store of the datasources.
 * This is called after the router request to save the edit dialog
 **/
function closeEditDialog(response) {
    if (!response.success) {
        return;
    }
    var dialog = Ext.getCmp(editDataSourcesId);
    refreshDataSourceGrid(editingReSelectId);

    // hide the dialog
    if (dialog) {
        dialog.hide();
    }
}

/**
 * Event handler for when a user wants to test a datasource
 * against a specific device.
 **/
function testDataSource() {
    var cmp = Ext.getCmp(editDataSourcesId),
        values = cmp.editForm.form.getValues(),
        win, testDevice;

    testDevice = values.testDevice;

    win = new Zenoss.CommandWindow({
        uids: testDevice,
        title: _t('Test Data Source'),
        data: values,
        target: values.uid + '/test_datasource'
    });

    win.show();
}

/**
 * Used when we save the data grid, it needs to
 * explicitly get the "Alias" value and turn it into a
 * list before going back to the server
 **/
function submitDataPointForm (values, callback) {
    // will always have only one alias form
    var aliases = Ext.getCmp(editDataSourcesId).query('alias'),
        alias;
    // assert that we have one exactly one alias form
    if (aliases.length < 1) {
        throw "The DataPoint form does not have an alias field, it should have only one";
    }

    alias = aliases[0];
    values.aliases = alias.getValue();
    router.setInfo(values, callback);
}

/**
 * Event handler for editing a specific datasource or
 * datapoint.
 **/
function editDataSourceOrPoint() {
    var cmp = Ext.getCmp(dataSourcesId),
        selectedNode = cmp.getSelectionModel().getSelectedNode(),
        data,
        isDataPoint = false,
        params;

    // make sure they selected something
    if (!selectedNode) {
        new Zenoss.dialog.ErrorDialog({message: _t('You must select a Data Source or Data Point.')});
        return;
    }
    data = selectedNode.data;

    // find out if we are editing a datasource or a datapoint
    if (data.leaf) {
        isDataPoint = true;
        editingReSelectId = selectedNode.parentNode.data.id;
    }else{
        editingReSelectId = data.id;
    }

    // parameters for the router call
    params = {
        uid: data.uid
    };

    function findSubObject(keyObj, array) {
        var p, key, val, ret;
        for (p in keyObj) {
            if (keyObj.hasOwnProperty(p)) {
                key = p;
                val = keyObj[p];
            }
        }
        for (p in array) {
            if (p == key) {
                if (array[p] == val) {
                    return array;
                }
            } else if (array[p] instanceof Object) {
                if (array.hasOwnProperty(p)) {
                    ret = findSubObject(keyObj, array[p]);
                    if (ret) {
                        return ret;
                    }
                }
            }
        }
        return null;
    };

    // callback for the router request
    function displayEditDialog(response) {
        var win, newId, config = {};

        config.record = response.record;
        config.items = response.form;
        config.id = editDataSourcesId;
        config.isDataPoint = isDataPoint;
        config.title = _t('Edit Data Source');
        config.directFn = router.setInfo;
        config.width = 800;

        newId = findSubObject({name:"newId"}, config)
        if (newId) {
            newId.inputAttrTpl = null;
            newId.regex = Zenoss.env.textMasks.allowedNameTextDash;
            newId.regexText = Zenoss.env.textMasks.allowedNameTextFeedbackDash;
        }

        if (isDataPoint) {
            config.title = _t('Edit Data Point');
            config.directFn = submitDataPointForm;
            config.singleColumn = true;
        } else if (config.record.testable &&
                   Zenoss.Security.hasPermission('Change Device')){
            // add the test against device panel
            config.items.items.push({
                xtype:'panel',
                columnWidth: 0.5,
                baseCls: 'test-against-device',
                hidden: Zenoss.Security.doesNotHavePermission('Run Commands'),
                title: _t('Test Against a Device'),
                items:[{
                    xtype: 'textfield',
                    fieldLabel: _t('Device Name'),
                    id: 'testDevice',
                    width: 300,
                    name: 'testDevice'
                },{
                    xtype: 'hidden',
                    name: 'uid',
                    value: response.record.id
                },{
                    xtype: 'button',
                    text: _t('Test'),
                    id: 'testDeviceButton',
                    handler: testDataSource
                }]});

        }

        config.saveHandler = closeEditDialog;
        win = new Zenoss.form.DataSourceEditDialog(config);
        var cmdField = win.editForm.form.findField('commandTemplate');
        if (cmdField !== null) {
            cmdField.addListener('dirtychange', function() {
                Ext.getCmp('testDeviceButton').disable();
                var devField = Ext.getCmp('testDevice');
                devField.setValue(_t("Save and reopen this dialog to test."));
                devField.disable();
            });
        }
        win.show();
    }
    // get the details
    if (isDataPoint) {
        router.getDataPointDetails(params, displayEditDialog);
    }else{
        router.getDataSourceDetails(params, displayEditDialog);
    }
}


/**
 * @class Zenoss.templates.DataSourceModel
 * @extends Ext.data.Model
 * Field definitions for the datasource/datapoint grid
 **/
Ext.define('Zenoss.templates.DataSourceModel',  {
    extend: 'Ext.data.Model',
    idProperty: 'uid',
    fields: [
        {name: 'uid'},
        {name: 'name'},
        {name: 'source'},
        {name: 'enabled'},
        {name: 'type'}
    ]
});

/**
 * @class Zenoss.templates.DataSourceStore
 * @extend Ext.data.TreeStore
 * Direct store for loading datasources and datapoints
 */
Ext.define("Zenoss.templates.DataSourceStore", {
    extend: "Ext.data.TreeStore",
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            model: 'Zenoss.templates.DataSourceModel',
            nodeParam: 'uid',
            remoteSort: false,
            proxy: {
                limitParam: undefined,
                startParam: undefined,
                pageParam: undefined,
                sortParam: undefined,
                type: 'direct',
                directFn: router.getDataSources,
                reader: {
                    root: 'data',
                    totalProperty: 'count'
                }
            }
        });
        this.callParent(arguments);
    }
});

/**
 * @class Zenoss.DataSourceTreeGrid
 * @extends Ext.Tree.Panel
 * @constructor
 */
Ext.define("Zenoss.DataSourceTreeGrid", {
    extend: "Ext.tree.Panel",
    alias: ['widget.DataSourceTreeGrid'],

    constructor: function(config) {
        var tbarItems = config.tbarItems || [],
            dsAddHandler = config.dsAddHandler || showAddDataSourceDialog,
            dsDelHandler = config.dsDelHandler || showDeleteDataSourceDialog,
            addToGraphHandler = config.addToGraphHandler || showAddToGraphDialog,
            dpAddHandler = config.dpAddHandler || showAddDataPointDialog,
            editEitherHandler = config.editEitherHandler || editDataSourceOrPoint,
            me = this;
            //dsDelHandler = config.dsDelHandler || showDeleteDataSourceDialog,
            //dsEditMenu = config.dsEditMenu || dataSourceMenu;
        Ext.applyIf(config, {
            useArrows: true,
            cls: 'x-tree-noicon',
            rootVisible: false,
            id: dataSourcesId,
            title: _t('Data Sources'),
            listeners: {
                // when they doubleclick we will open up the tree and
                // display the dialog
                beforeitemdblclick: editDataSourceOrPoint
            },
            store: Ext.create('Zenoss.templates.DataSourceStore', {}),
            tbar: tbarItems.concat([{
                    xtype: 'button',
                    iconCls: 'add',
                    id:'datasourceAddButton',
                    ref: '../addButton',
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    handler: dsAddHandler,
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor('datasourceAddButton');
                        }
                    }
            }, {
                xtype: 'button',
                iconCls: 'delete',
                ref: '../deleteButton',
                id: 'datasourceDeleteButton',
                disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('datasourceDeleteButton');
                    }
                },
                handler: dsDelHandler
            },{
                xtype: 'button',
                id: 'datasourceEditButton',
                iconCls: 'customize',
                ref: '../customizeButton',
                disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('datasourceEditButton');
                    }
                },
                menu: new Ext.menu.Menu({
                    items: [{
                        xtype: 'menuitem',
                        text: _t('Add Data Point To Graph'),
                        disable: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                        handler: function() {
                            var node = me.getSelectionModel().getSelectedNode();
                            addToGraphHandler(node, me.uid);
                        }
                    },{
                        xtype: 'menuitem',
                        text: _t('Add Data Point'),
                        disable: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                        handler: dpAddHandler
                    },{
                        xtype: 'menuitem',
                        text: _t('View and Edit Details'),
                        disable: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                        handler: editEitherHandler
                    }]
                })
            }]),
            columns: [{
                xtype: 'treecolumn', //this is so we know which column will show the tree
                text: 'Name',
                flex: 2,
                sortable: true,
                dataIndex: 'name'
            }, {
                dataIndex: 'source',
                flex: 1,
                header: 'Source',
                width: 250
            }, {
                dataIndex: 'enabled',
                header: 'Enabled',
                width: 60
            }, {
                dataIndex: 'type',
                header: 'Type',
                width: 90
            }],
            selModel: Ext.create('Zenoss.TreeSelectionModel', {
                mode: 'SINGLE'
            })
        });
        this.callParent(arguments);
    },
    disableToolBarButtons: function(bool) {
        this.addButton.setDisabled(bool && Zenoss.Security.hasPermission('Manage DMD'));
        this.deleteButton.setDisabled(bool && Zenoss.Security.hasPermission('Manage DMD'));
        this.customizeButton.setDisabled(bool && Zenoss.Security.hasPermission('Manage DMD'));
    },
    setContext: function(uid) {
        if (uid !== this.uid){
            this.uid = uid;
            this.refresh();
        }
    },
    refresh: function(callback, scope) {
        var root = this.getRootNode();
        root.setId(this.uid);
        root.data.uid = this.uid;
        root.uid = this.uid;
        if (callback) {
            this.getStore().load({
                callback: callback,
                scope: scope || this
            });
        }else {
            this.getStore().load();
        }

    }

});



})();
/*****************************************************************************
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {
    Ext.namespace('Zenoss.templates');
    /**********************************************************************
     *
     * Variable Declarations
     *
     */
    var router,
        treeId,
        dataSourcesId;

    Zenoss.templates.thresholdsId = 'thresholdGrid';
    dataSourcesId = 'dataSourceTreeGrid';
    router = Zenoss.remote.TemplateRouter;

    // The id of the tree on the left hand side of the screen
    treeId = 'templateTree';

    /**********************************************************************
     *
     * Add Threshold
     *
     */

    function addThreshold(data, grid){
        var uid,
            node,
            dataPoints,
            params,
            callback;
        uid = grid.getTemplateUid();
        if (Ext.getCmp(dataSourcesId)) {
            node = Ext.getCmp(dataSourcesId).getSelectionModel().getSelectedNode();
        }
        if ( node && node.isLeaf() ) {
            dataPoints = [node.data.uid];
        } else {
            dataPoints = [];
        }
        params = {
            uid: uid,
            thresholdType: data.thresholdTypeField,
            thresholdId: data.thresholdIdField,
            dataPoints: dataPoints
        };
        callback = function() {
            grid.refresh();
        };
        Zenoss.remote.TemplateRouter.addThreshold(params, callback);
    }

    function showAddThresholdDialog(grid) {
        if (!grid.getTemplateUid()) {
            return;
        }
        var context = grid.getTemplateUid() + '/thresholds';
        var addThresholdDialog = Ext.create('Zenoss.dialog.BaseWindow', {
            id: 'addThresholdDialog',
            title: _t('Add Threshold'),
            message: _t('Allow the user to add a threshold.'),
            width: 295,
            height: 250,
            modal: true,
            padding: 10,
            listeners:{
                show: function() {
                    this.formPanel.getForm().reset();
                }
            },

            buttons: [{
                ref: '../submitButton',
                text: _t('Add'),
                disabled: true,
                xtype: 'DialogButton',
                handler: function(submitButton) {
                    var dialogWindow, basicForm;
                    dialogWindow = submitButton.refOwner;
                    basicForm = dialogWindow.formPanel.getForm();
                    addThreshold(basicForm.getValues(), grid);
                }
            }, {
                ref: '../cancelButton',
                text: _t('Cancel'),
                xtype: 'DialogButton'
            }],
            items: {
                xtype: 'form',
                ref: 'formPanel',
                height: 75,
                leftAlign: 'top',
                monitorValid: true,
                paramsAsHash: true,
                listeners: {
                    validitychange: function(formPanel, valid) {
                        addThresholdDialog.submitButton.setDisabled( !valid );
                    }
                },
                items: [{
                    name: 'thresholdIdField',
                    xtype: 'idfield',
                    fieldLabel: _t('Name'),
                    allowBlank: false,
                    context: context
                },{
                    name: 'thresholdTypeField',
                    xtype: 'combo',
                    fieldLabel: _t('Type'),
                    displayField: 'type',
                    forceSelection: true,
                    triggerAction: 'all',
                    emptyText: _t('Select a type...'),
                    selectOnFocus: true,
                    allowBlank: false,
                    store: {
                        type: 'directcombo',
                        autoLoad: true,
                        directFn: Zenoss.remote.TemplateRouter.getThresholdTypes,
                        root: 'data',
                        fields: ['type']
                    }
                }]
            }});
        addThresholdDialog.show();
    }


     /**********************************************************************
     *
     * Edit Thresholds
     *
     */

    /**
     *@returns Zenoss.FormDialog Ext Dialog type associated with the
     *          selected threshold type
     **/
    function thresholdEdit(grid) {
        var record = grid.getSelectionModel().getSelected();


        function displayEditDialog(response) {

            var thrtypes = ["PredictiveThreshold", "MinMaxThreshold"]
            if (thrtypes.includes(response.record.type)){
                //test button should always be at the bottom of the form
                response.form.items[response.form.items.length - 1].items.push({
                    xtype:'panel',
                    columnWidth: 0.5,
                    baseCls: 'show-rpn-values',
                    items:[{
                        xtype: 'displayfield',
                        value: _t('Show values after RPN used'),
                        width: 300,
                    },{
                        xtype: 'button',
                        text: _t('Test with RPN'),
                        id: 'testDeviceButton',
                        handler: showRPNValues
                  }]});
            }

            var win = Ext.create( 'Zenoss.form.DataSourceEditDialog', {
                record: response.record,
                items: response.form,
                width: 850,
                singleColumn: response.form.items.length === 1 ? true : false,
                xtype: 'datasourceeditdialog',
                title: _t('Edit Threshold'),
                directFn: router.setInfo,
                id: 'editThresholdDialog',
                saveHandler: function() {
                    grid.refresh();
                    if (win) {
                        win.hide();
                    }
                }
            });

            win.show();
        }
 
        function showRPNValues (){
            //This is for picking up datapoints name in different type of thresholds
            var dpsobj, selectedds;
            dpsobj = Ext.ComponentQuery.query('[fieldLabel=DataPoints]')[0] ||
                     Ext.ComponentQuery.query('[fieldLabel=DataPoint]')[0]
            if (dpsobj.fieldLabel == 'DataPoint'){
                if (dpsobj.value == null){
                    selecteddps = [];
                } else {
                    selecteddps = [dpsobj.value];
                }
            } else {
                selecteddps = dpsobj.value
            }
            var minval = Ext.ComponentQuery.query('[name="minval"]')[0],
                maxval = Ext.ComponentQuery.query('[name="maxval"]')[0];
            router.getDataPointsRPNValues({thuid: record.data.uid, selecteddps: selecteddps,
                                           minval: minval.value, maxval:maxval.value}, addRPNtable)
        }

        function addRPNtable(response){
            Ext.create('Zenoss.stats.RPNValues', {
                        response: response
            }).show();
        };

        // send the request for all of the threshold's info to the server
        router.getThresholdDetails({uid: record.data.uid}, displayEditDialog);
    }


     /**********************************************************************
     *
     * Threshold Data Grid
     *
     */


    /**
     * @class Zenoss.thresholds.Model
     * @extends Ext.data.Model
     * Field definitions for the thresholds
     **/
    Ext.define('Zenoss.thresholds.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'uid',
        fields: ['name', 'type', 'dataPoints', 'severity', 'enabled','type', 'minval', 'maxval', 'uid']
    });

    /**
     * @class Zenoss.thresholds.Store
     * @extend Zenoss.DirectStore
     * Direct store for loading thresholds
     */
    Ext.define("Zenoss.thresholds.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.thresholds.Model',
                directFn: router.getThresholds,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    /**
     * Definition for the Thresholds datagrid. This is used in
     * templates.js in the updateThresholds function.
     **/
    Ext.define("Zenoss.templates.thresholdDataGrid", {
        alias:['widget.thresholddatagrid'],
        extend:"Zenoss.ContextGridPanel",

        constructor: function(config) {
            var listeners = {},
                me = this,
                tbarItems = config.tbarItems || [];

            listeners = Ext.apply(listeners, {
                itemdblclick: function(gridview) {
                    thresholdEdit(gridview.ownerCt);
                }
            });

            config = config || {};
            Ext.applyIf(config, {
                id: Zenoss.templates.thresholdsId,
                selModel:   new Zenoss.SingleRowSelectionModel ({
                    listeners : {
                        /**
                         * If they have permission and they select a row, show the
                         * edit and delete buttons
                         **/
                        select: function () {
                            // enable the "Delete Threshold" button
                            if (Zenoss.Security.hasPermission('Manage DMD')) {
                                me.deleteButton.enable();
                                me.editButton.enable();
                            }
                        }
                    }
                }),
                viewConfig: {
                    plugins: {
                        ptype: 'gridviewdragdrop',
                        dragText: _t('Drag to add to Graph Definition'),
                        dragGroup: 'addtoGraph'
                    }
                },
                title: _t('Thresholds'),
                store: Ext.create('Zenoss.thresholds.Store', { }),
                listeners: listeners,
                tbar: tbarItems.concat([{
                    xtype: 'button',
                    iconCls: 'add',
                    id: 'thresholdAddButton',
                    ref: '../addButton',
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    handler: function(btn) {
                        var templateUid = Ext.getCmp('dataSourceTreeGrid').uid;
                        if (!templateUid) {
                            new Zenoss.dialog.ErrorDialog({message: _t('There is no template to which to add a threshold.')});
                            return;
                        }
                        showAddThresholdDialog(btn.refOwner);
                    },
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor('thresholdAddButton');
                        }
                    }
                }, {
                    ref: '../deleteButton',
                    id: 'thresholdDeleteButton',
                    xtype: 'button',
                    iconCls: 'delete',
                    disabled: true,
                    handler: function() {
                        var row = me.getSelectionModel().getSelected(),
                            uid,
                            params;
                        if (row){
                            uid = row.get("uid");
                            // show a confirmation
                     new Zenoss.dialog.SimpleMessageDialog({
                                title: _t('Delete Threshold'),
                                message: Ext.String.format(_t("Are you sure you want to delete this threshold? There is no undo.")),
                            buttons: [{
                                xtype: 'DialogButton',
                                text: _t('OK'),
                                handler: function() {
                                    params= {
                                        uid:uid
                                    };
                                    router.removeThreshold(params, function(){
                                        me.refresh();
                                        me.deleteButton.disable();
                                        me.editButton.disable();
                                    });
                                }
                            }, {
                                xtype: 'DialogButton',
                                text: _t('Cancel')
                            }]
                        }).show();
                        }
                    },
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor('thresholdDeleteButton');
                        }
                    }

                }, {
                    id: 'thresholdEditButton',
                    ref: '../editButton',
                    xtype: 'button',
                    iconCls: 'customize',
                    disabled: true,
                    handler: function(button) {
                        thresholdEdit(button.refOwner);
                    },
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor('thresholdEditButton');
                        }
                    }
                }]),
                columns: [{
                    dataIndex: 'name',
                    flex: 1,
                    header: _t('Name')
                }, {
                    dataIndex: 'type',
                    header: _t('Type')
                }, {
                    dataIndex: 'minval',
                    header: _t('Min. Value')
                }, {
                    dataIndex: 'maxval',
                    header: _t('Max. Value')
                }]
            });

            Zenoss.templates.thresholdDataGrid.superclass.constructor.apply(
                this, arguments);
        },
        getTemplateUid: function() {
            var tree = Ext.getCmp(treeId),
                node = tree.getSelectionModel().getSelectedNode();
            if (node) {
                return node.data.uid;
            }
        }
    });

    Ext.define('RPNValues', {
        extend: 'Ext.data.Model',
        fields: ['name', 'maxrpn', 'minrpn', 'rpnvalue']
    });

    Ext.define("Zenoss.stats.RPNValues", {
        extend: "Zenoss.dialog.BaseDialog",
        constructor: function(config) {
           var dialogStore = Ext.create('Ext.data.Store', {
               model: 'RPNValues',
               data: config.response.dpRPN
           });
           config = config || {};
           Ext.applyIf(config, {
               title: _t('Values with RPN used'),
               minWidth: 500,
               minHeight: 200,
               height: 400,
               autoScroll: true,
               items: Ext.create('Ext.grid.Panel', {
                   stripeRows: true,
                   store: dialogStore,
                   sortable: false,
                   columns: [{
                       header: "Datapoint Name",
                       sortable: false,
                       dataIndex: 'name',
                       flex: 2,
                   },{
                       header: "RPN formula",
                       dataIndex: 'rpnvalue',
                       flex: 2
                   },{
                       header: "Min value after rpn",
                       dataIndex: 'minrpn',
                       flex: 2
                   },{
                       header: "Max value after rpn",
                       dataIndex: 'maxrpn',
                       flex: 2
                   }]
               }),
           });
           this.callParent([config]);
        }
    });

}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

var router, getSelectedTemplateUid, getSelectedTemplate, getSelectedGraphDefinition;
    //addGraphDefinition, deleteGraphDefinition, addThresholdToGraph;


Ext.ns('Zenoss', 'Zenoss.templates');

router = Zenoss.remote.TemplateRouter;

getSelectedTemplate = function() {
    return Ext.getCmp('templateTree').getSelectionModel().getSelectedNode();
};

getSelectedTemplateUid = function() {
    return getSelectedTemplate().data.uid;
};

getSelectedGraphDefinition = function() {
    return Ext.getCmp('graphGrid').getSelectionModel().getSelected();
};


Ext.define('Zenoss.InstructionTypeModel', {
    extend: 'Ext.data.Model',
    idProperty: 'pythonClassName',
    fields: ['pythonClassName', 'label']
});

/**
 * @class Zenoss.graph.GraphPointModel
 * @extends Ext.data.Model
 * Field definitions for the Graph Point
 **/
Ext.define('Zenoss.graph.GraphPointModel',  {
    extend: 'Ext.data.Model',
    idProperty: 'uid',
    fields: ['uid', 'name', 'type', 'description']
});

Ext.define("Zenoss.GraphPointStore", {
    alias:['widget.graphpointstore'],
    extend:"Zenoss.NonPaginatedStore",
    constructor: function(config){
        Ext.applyIf(config, {
            model: 'Zenoss.graph.GraphPointModel',
            directFn: router.getGraphPoints,
            root: 'data'
        });
        this.callParent(arguments);
    }
});

Ext.define("Zenoss.BaseSequenceGrid", {
    extend:"Ext.grid.GridPanel",
    constructor: function(config){
        Ext.applyIf(config, {
            enableDragDrop   : true,
            viewConfig: {
                forcefit: true,
                plugins: {
                    ptype: 'gridviewdragdrop'
                }
            }
        });
        this.addEvents({'resequence': true});
        Zenoss.BaseSequenceGrid.superclass.constructor.call(this, config);
    }
});

Ext.define("Zenoss.GraphPointGrid", {
    alias:['widget.graphpointgrid'],
    extend:"Zenoss.ContextGridPanel",
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            height: 215,
            viewConfig: {
                plugins: {
                    ptype: 'gridviewdragdrop'
                }
            },
            listeners: {
                /**
                 * The selection model was being ignored at this point so I used the
                 * row click.
                 **/
                itemclick: function() {
                    //var record = getSelectedGraphPoint();
                    var record = me.getSelectionModel().getSelected();
                    if (record) {
                        //Ext.getCmp('deleteGraphPointButton').enable();
                        //Ext.getCmp('editGraphPointButton').enable();
                        this.deleteGraphPointButton.enable();
                        this.editGraphPointButton.enable();
                    }else{
                        //Ext.getCmp('deleteGraphPointButton').disable();
                        //Ext.getCmp('editGraphPointButton').disable();
                        this.deleteGraphPointButton.disable();
                        this.editGraphPointButton.disable();
                    }
                },
                itemdblclick: function() {
                    if (me.getSelectionModel().getSelected()) {
                        displayGraphPointForm(me.getSelectionModel().getSelected(), me);
                    }
                }
            },
            store: Ext.create('Zenoss.GraphPointStore', {}),
            columns: [
                {dataIndex: 'name', header: _t('Name'), width: 150},
                {dataIndex: 'type', header: _t('Type'), width: 150},
                {
                    dataIndex: 'description',
                    header: _t('Description'),
                    //id: 'definition_description',
                    flex: 1
                }
            ],
            tbar: [{
                xtype: 'button',
                //id: 'addGraphPointButton',
                iconCls: 'add',
                //menu: 'graphPointMenu',
                menu: new Ext.menu.Menu({
                    items: [{
                        xtype: 'menuitem',
                        text: _t('Data Point'),
                        handler: function(){
                            //Ext.getCmp('addDataPointToGraphDialog').show();
                            var addDataPointToGraphDialog =
                                new Zenoss.HideFormDialog({
                                    //id: 'addDataPointToGraphDialog',
                                    title: _t('Add Data Point'),
                                    closeAction: 'hide',
                                    items:[{
                                        xtype: 'combo',
                                        ref: 'comboBox',
                                        getInnerTpl: function() {
                                            return '<tpl for="."><div ext:qtip="{name}" class="x-combo-list-item">{name}</div></tpl>';
                                        },
                                        fieldLabel: _t('Data Point'),
                                        valueField: 'uid',
                                        displayField: 'name',
                                        triggerAction: 'all',
                                        forceSelection: true,
                                        editable: false,
                                        allowBlank: false,
                                        listeners: {
                                            validitychange: function(form, isValid){
                                                //var window = Ext.getCmp('addDataPointToGraphDialog');
                                                var window = addDataPointToGraphDialog;
                                                if (window.isVisible()){
                                                    window.submit.setDisabled(!isValid);
                                                }
                                            }
                                        },
                                        store: Ext.create('Zenoss.NonPaginatedStore', {
                                            root: 'data',
                                            model: 'Zenoss.model.Basic',
                                            directFn: router.getDataPoints
                                        })
                                    },{
                                        xtype: 'checkbox',
                                        name: 'include_related_thresholds',
                                        ref: 'includeRelatedThresholds',
                                        fieldLabel: _t('Include Related Thresholds'),
                                        checked: true
                                    }],
                                    listeners: {
                                        show: function() {
                                            var combo, uid;
                                            //combo = Ext.getCmp('addDataPointToGraphDialog').comboBox;
                                            combo = addDataPointToGraphDialog.comboBox;
                                            combo.reset();
                                            //Ext.getCmp('addDataPointToGraphDialog').submit.disable();
                                            addDataPointToGraphDialog.submit.disable();
                                            //uid = getSelectedTemplate().data.uid;
                                            uid = me.templateUid;
                                            combo.store.setContext(uid);
                                        }
                                    },
                                    buttons: [
                                        {
                                            xtype: 'HideDialogButton',
                                            ui: 'dialog-dark',
                                            ref: '../submit',
                                            text: _t('Submit'),
                                            disabled: true,
                                            handler: function() {
                                                //var dataPointUid = Ext.getCmp('addDataPointToGraphDialog').comboBox.getValue(),

                                                var dataPointUid = addDataPointToGraphDialog.comboBox.getValue(),
                                                //graphUid = getSelectedGraphDefinition().get("uid"),
                                                graphUid = me.uid,
                                                //includeThresholds = Ext.getCmp('addDataPointToGraphDialog').includeRelatedThresholds.getValue(),
                                                includeThresholds = addDataPointToGraphDialog.includeRelatedThresholds.getValue(),
                                                params = {
                                                    dataPointUid: dataPointUid,
                                                    graphUid: graphUid,
                                                    includeThresholds: includeThresholds
                                                },
                                                callback = function() {
                                                    //Ext.getCmp('graphPointGrid').refresh();
                                                    me.refresh();
                                                };
                                                router.addDataPointToGraph(params, callback);
                                            }
                                        }, {
                                            xtype: 'HideDialogButton',
                                            ui: 'dialog-dark',
                                            text: _t('Cancel')
                                        }]

                                });
                            addDataPointToGraphDialog.show();
                        }
                    }, {
                        xtype: 'menuitem',
                        text: _t('Threshold'),
                        handler: function(){
                            var win = new Zenoss.HideFormDialog({
                                title: _t('Add Threshold'),
                                items: {
                                    xtype: 'combo',
                                    getInnerTpl: function() {
                                        return '<tpl for="."><div ext:qtip="{name}" class="x-combo-list-item">{name}</div></tpl>';
                                    },
                                    fieldLabel: _t('Threshold'),
                                    valueField: 'uid',
                                    displayField: 'name',
                                    triggerAction: 'all',
                                    forceSelection: true,
                                    editable: false,
                                    allowBlank: false,
                                    store: Ext.create('Zenoss.NonPaginatedStore', {
                                        root: 'data',
                                        model: 'Zenoss.model.Basic',
                                        directFn: router.getThresholds
                                    }),
                                    listeners: {
                                        validitychange: function(form, isValid){
                                            var button = win.down('button[formBind=true]');
                                            if (button.isVisible()){
                                                button.setDisabled(!isValid);
                                            }
                                        }
                                    }
                                },
                                listeners: {
                                    show: function() {
                                        var combo, uid;
                                        combo = win.down('combo');
                                        combo.reset();
                                        uid = me.templateUid;
                                        combo.store.setContext(uid);
                                    }
                                },
                                buttons: [
                                    {
                                        xtype: 'HideDialogButton',
                                        ui: 'dialog-dark',
                                        formBind: true,
                                        disabled:true,
                                        text: _t('Submit'),
                                        handler: function() {
                                            var params, callback;
                                            params = {
                                                graphUid: me.uid,
                                                thresholdUid: win.down('combo').getValue()
                                            };
                                            callback = function() {
                                                me.refresh();
                                            };
                                            router.addThresholdToGraph(params, callback);
                                        }
                                    }, {
                                        xtype: 'HideDialogButton',
                                        ui: 'dialog-dark',
                                        text: _t('Cancel')
                                    }]

                            });
                            win.show();
                        }
                    }, {
                        xtype: 'menuitem',
                        text: _t('Custom Graph Point'),
                        handler: function(){
                            var win = new Zenoss.HideFormDialog({
                                title: _t('Add Custom Graph Point'),
                                listeners: {
                                    show: function(dialog) {
                                        dialog.addForm.idField.reset();
                                        dialog.addForm.typeCombo.reset();
                                        dialog.addForm.typeCombo.store.load();
                                    }
                                },
                                items: [{
                                    xtype: 'form',
                                    ref: 'addForm',
                                    listeners: {
                                        validitychange: function(form, isValid) {
                                            win.down('button[formBind=true]').setDisabled(!isValid);
                                        }
                                    },
                                    items: [{
                                        xtype: 'idfield',
                                        ref: 'idField',
                                        fieldLabel: _t('Name'),
                                        allowBlank: false
                                    }, {
                                        xtype: 'combo',
                                        ref: 'typeCombo',
                                        fieldLabel: _t('Instruction Type'),
                                        valueField: 'pythonClassName',
                                        displayField: 'label',
                                        triggerAction: 'all',
                                        forceSelection: true,
                                        editable: false,
                                        allowBlank: false,
                                        store: Ext.create('Zenoss.NonPaginatedStore', {
                                            root: 'data',
                                            autoLoad: false,
                                            model: 'Zenoss.InstructionTypeModel',
                                            directFn: router.getGraphInstructionTypes
                                        })
                                    }]
                                }],
                                buttons: [{
                                    xtype: 'HideDialogButton',
                                    ui: 'dialog-dark',
                                    disabled: true,
                                    formBind: true,
                                    ref: '../submitButton',
                                    text: _t('Add'),
                                    handler: function() {
                                        var params, callback, form = win.addForm;
                                        params = {
                                            graphUid: me.uid,
                                            customId: form.idField.getValue(),
                                            customType: form.typeCombo.getValue()
                                        };
                                        callback = function() {
                                            me.refresh();
                                        };
                                        router.addCustomToGraph(params, callback);
                                    }
                                }, {
                                    xtype: 'HideDialogButton',
                                    ui: 'dialog-dark',
                                    ref: '../cancelButton',
                                    text: _t('Cancel')
                                }]
                            });
                            win.show();
                        }
                    }]
                })
                //TODO: register tooltip without id?
                /*
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('addGraphPointButton');
                    }
                }
                */
            }, {
                xtype: 'button',
                //id: 'deleteGraphPointButton',
                ref: '../deleteGraphPointButton',
                iconCls: 'delete',
                disabled: true,
                //TODO: register tooltip without id?
                /*
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('deleteGraphPointButton');
                    }
                },
                */
                handler: function() {
                    var html, dialog;
                    // format the confimation message
                    html = _t("Are you sure you want to remove the graph point, {0}? There is no undo.");
                    //html = Ext.String.format(html, getSelectedGraphPoint().data.name);
                    html = Ext.String.format(html, me.getSelectionModel().getSelected().data.name);

                    // show the dialog
                    //dialog = Ext.getCmp('deleteGraphPointDialog');
                    dialog =
                        new Zenoss.MessageDialog({
                            //id: 'deleteGraphPointDialog',
                            title: _t('Delete Graph Point'),
                            // the message is generated dynamically
                            okHandler: function() {
                                var params, callback;
                                params = {
                                    uid: me.getSelectionModel().getSelected().get("uid")
                                };
                                callback = function() {
                                    me.deleteGraphPointButton.disable();
                                    me.editGraphPointButton.disable();
                                    me.refresh();
                                };
                                router.deleteGraphPoint(params, callback);
                            }
                        });

                    dialog.setText(html);
                    dialog.show();
                }
            }, {
                xtype: 'button',
                //id: 'editGraphPointButton',
                ref: '../editGraphPointButton',
                iconCls: 'customize',
                disabled: true,
                //TODO: register tooltip without id?
                /*
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('editGraphPointButton');
                    }
                },
                */
                handler: function() {
                    if (me.getSelectionModel().getSelected()) {
                        displayGraphPointForm(me.getSelectionModel().getSelected(), me);
                    }
                }
            }],
            selModel: Ext.create('Zenoss.SingleRowSelectionModel', {})
        });
        this.callParent(arguments);
    }
});


/**********************************************************************
 *
 * Graph Point Edit Dialog/Grid
 *
 */

/**
 * Call back function from when a user selects a graph point.
 * This shows yet another dialog for editing a graph point
 **/
function displayGraphPointForm(record, grid) {

    function displayEditDialog(response) {
        var win = Ext.create('Zenoss.form.DataSourceEditDialog', {
            record: response.data,
            items: response.form,
            singleColumn: true,
            width: 400,
            title: _t('Edit Graph Point'),
            directFn: router.setInfo,
            id: 'editGraphPointDialog',
            saveHandler: function() {
                grid.refresh();
            }
        });

        win.show();
    }

    // remote call to get the object details
    router.getInfo({uid: record.get("uid")}, displayEditDialog);
}

Ext.create('Zenoss.dialog.BaseWindow', {
    layout: 'fit',
    id: 'viewGraphDefinitionDialog',
    title: _t('View and Edit Graph Definition'),
    closeAction: 'hide',
    buttonAlign: 'left',
    autoScroll: true,
    plain: true,
    modal: true,
    width: 400,
    height: 500,
    padding: 10,
    buttons: [{
        ref: '../submitButton',
        xtype: 'HideDialogButton',
        ui: 'dialog-dark',
        text: _t('Submit'),
        disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
        handler: function(submitButton){
            var dialogWindow, basicForm, params;
            dialogWindow = submitButton.refOwner;
            basicForm = dialogWindow.formPanel.getForm();
            params = Ext.applyIf(basicForm.getValues(), {
                uid: dialogWindow.uid,
                hasSummary: false,
                log: false,
                base: false
            });
            basicForm.api.submit(params, function() {
                Ext.getCmp('graphGrid').refresh();
            });
        }
    },{
        xtype: 'HideDialogButton',
        ref: '../cancelButton',
        ui: 'dialog-dark',
        text: 'Cancel'
    }],
    items: {
        xtype: 'form',
        ref: 'formPanel',
        autoScroll: true,
        paramsAsHash: true,
        api: {
            load: router.getGraphDefinition,
            submit: router.setGraphDefinition
        },
        listeners: {
            validitychange: function(formPanel, valid){
                var dialogWindow = Ext.getCmp('viewGraphDefinitionDialog');
                if (Zenoss.Security.hasPermission('Manage DMD')) {
                    dialogWindow.submitButton.setDisabled( ! valid );
                }
            },
            show: function(formPanel){
                formPanel.getForm().load();
            }
        },
        items: [{
            xtype: 'idfield',
            fieldLabel: _t('Name'),
            name: 'newId',
            allowBlank: false
        },{
            xtype: 'numberfield',
            fieldLabel: _t('Height'),
            name: 'height',
            minValue: 250
        },{
            xtype: 'textfield',
            fieldLabel: _t('Units'),
            name: 'units'
        },{
            xtype: 'checkbox',
            fieldLabel: _t('Logarithmic Scale'),
            name: 'log'
        },{
            xtype: 'checkbox',
            fieldLabel: _t('Base 1024'),
            name: 'base'
        },{
            xtype: 'numberfield',
            fieldLabel: _t('Min Y'),
            name: 'miny'
        },{
            xtype: 'numberfield',
            fieldLabel: _t('Max Y'),
            name: 'maxy'
        },{
            xtype: 'textarea',
            fieldLabel: _t('Description'),
            name: 'description'
        },{
            xtype: 'checkbox',
            fieldLabel: _t('Has Summary'),
            name: 'hasSummary'
        }]
    },
    loadAndShow: function(uid) {
        this.uid = uid;
        this.formPanel.getForm().load({
            params: {uid:uid},
            success: function() {
                this.show();
            },
            scope: this
        });
    }
});

Ext.define("Zenoss.templates.GraphGrid", {
    alias:['widget.graphgrid'],
    extend:"Zenoss.ContextGridPanel",
    constructor: function(config) {
        var me = this,
            tbarItems = config.tbarItems || [];
        Ext.applyIf(config, {
            title: _t('Graph Definitions'),
            store: Ext.create('Zenoss.GraphStore', {}),
            enableDragDrop   : true,
            viewConfig: {
                forcefit: true,
                plugins: {
                    ptype: 'gridviewdragdrop'
                },
                listeners: {
                    /**
                     * Updates the graph order when the user drags and drops them
                     **/
                    drop: function() {
                        var records, uids;
                        records = me.store.getRange();
                        uids = Ext.pluck(Ext.pluck(records, 'data'), 'uid');
                        router.setGraphDefinitionSequence({'uids': uids});
                    }
                }
            },
            listeners: {
                /**
                 * Double click to edit a graph definition
                 **/
                itemdblclick: function(grid, record)  {
                    var dialogWindow, uid;
                    dialogWindow = Ext.getCmp('viewGraphDefinitionDialog');
                    uid = record.data.uid;
                    dialogWindow.loadAndShow(uid);
                }

            },
            selModel: new Zenoss.SingleRowSelectionModel({
                listeners: {
                    select: function() {
                        if (Zenoss.Security.hasPermission('Manage DMD')){
                            Ext.getCmp('deleteGraphDefinitionButton').enable();
                            Ext.getCmp('graphDefinitionMenuButton').enable();
                        }
                    }
                }
            }),
            columns: [{dataIndex: 'name', header: _t('Name'), flex:1, width: 400}],

            tbar: tbarItems.concat([{
                id: 'addGraphDefinitionButton',
                xtype: 'button',
                iconCls: 'add',
                ref: '../addButton',
                disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('addGraphDefinitionButton');
                    }
                },
                handler: function() {
                    var templateUid = Ext.getCmp('dataSourceTreeGrid').uid;
                    if (!templateUid) {
                        new Zenoss.dialog.ErrorDialog({message: _t('There is no template to which to add a graph definition.')});
                        return;
                    }
                    var dialog = Ext.create('Zenoss.dialog.BaseWindow', {
                        title: _t('Add Graph Definition'),
                        buttonAlign: 'left',
                        autoScroll: true,
                        plain: true,
                        width: 300,
                        autoHeight: true,
                        modal: true,
                        padding: 10,
                        items: [{
                            xtype: 'form',
                            listeners: {
                                validitychange: function(formPanel, valid) {
                                    dialog.submitButton.setDisabled( !valid );
                                }
                            },
                            items: [
                                {
                                    xtype: 'idfield',
                                    id: 'graphDefinitionIdTextfield',
                                    fieldLabel: _t('Name'),
                                    allowBlank: false
                                }
                            ]

                        }],
                        buttons: [
                            {
                                xtype: 'DialogButton',
                                ref: '../submitButton',
                                disabled: true,
                                text: _t('Submit'),
                                handler: function() {
                                    var params, callback;
                                    params = {
                                        templateUid: me.getSelectedTemplateUid(),
                                        graphDefinitionId: Ext.getCmp('graphDefinitionIdTextfield').getValue()
                                    };
                                    callback = function() {
                                        me.refresh();
                                    };

                                    router.addGraphDefinition(params, callback);
                                }
                            }, {
                                xtype: 'DialogButton',
                                text: _t('Cancel')
                            }]
                    });
                    dialog.show();
                }
            }, {
                id: 'deleteGraphDefinitionButton',
                ref: 'deleteGraphDefinitionButton',
                xtype: 'button',
                iconCls: 'delete',
                disabled: true,
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('deleteGraphDefinitionButton');
                    }
                },
                handler: function(button) {
                    var msg, name, html, dialog;
                    msg = _t("Are you sure you want to remove {0}? There is no undo.");
                    name = me.getSelectedGraphDefinition().data.name;
                    html = Ext.String.format(msg, name);
                    dialog = new Zenoss.MessageDialog({
                        title: _t('Delete Graph Definition'),
                        // the message is generated dynamically
                        okHandler: function(){
                            var params, callback;
                            params = {
                                uid: me.getSelectedGraphDefinition().get("uid")
                            };
                            callback = function() {
                                button.disable();
                                button.refOwner.graphDefinitionMenuButton.disable();
                                me.refresh();
                            };
                            router.deleteGraphDefinition(params, callback);
                        }
                    });
                    dialog.setText(html);
                    dialog.show();
                }
            }, {
                id: 'graphDefinitionMenuButton',
                ref: 'graphDefinitionMenuButton',
                xtype: 'button',
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('graphDefinitionMenuButton');
                    }
                },
                iconCls: 'customize',
                menu: new Ext.menu.Menu({
                    items: [{
                        xtype: 'menuitem',
                        text: _t('Manage Graph Points'),
                        handler: function(){
                            var uid, dialog;
                            uid = me.getSelectedGraphDefinition().get("uid");
                            dialog = new Zenoss.HideFitDialog({
                                title: _t('Manage Graph Points'),
                                items: [{
                                    xtype: 'graphpointgrid',
                                    templateUid: me.getSelectedTemplateUid(),
                                    ref: 'graphGrid'
                                }],
                                buttons: [{
                                    xtype: 'HideDialogButton',
                                    ui: 'dialog-dark',
                                    text: _t('Save'),
                                    handler: function(){
                                        if (Zenoss.Security.hasPermission('Manage DMD')) {
                                            var records, uids;
                                            //records = grid.getStore().getRange();
                                            records = dialog.graphGrid.getStore().getRange();
                                            uids = Ext.Array.pluck(Ext.Array.pluck(records, 'data'), 'uid');
                                            router.setGraphPointSequence({'uids': uids});
                                        }
                                    }
                                }, {
                                    xtype: 'HideDialogButton',
                                    ui: 'dialog-dark',
                                    text: _t('Cancel')
                                }]
                            });
                            dialog.show();
                            dialog.graphGrid.setContext(uid);
                        }
                    }, {
                        xtype: 'menuitem',
                        text: _t('View and Edit Details'),
                        handler: function(){
                            var dialogWindow, uid;
                            dialogWindow = Ext.getCmp('viewGraphDefinitionDialog');
                            uid = me.getSelectedGraphDefinition().get("uid");
                            dialogWindow.loadAndShow(uid);
                        }
                    }]
                }),
                disabled: true
            }])
        });
        this.callParent(arguments);
    },
    getSelectedTemplateUid: getSelectedTemplateUid,
    getSelectedGraphDefinition: function() {
        return this.getSelectionModel().getSelected();
    }
});


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss', 'Zenoss.templates');

var REMOTE = Zenoss.remote.DeviceRouter;

/**
 * Updates the data store for the template tree. This will select the
 * first template when refreshed.
 **/
function refreshTemplateTree() {
    var cmp = Ext.getCmp('templateTree');
    if (cmp && cmp.isVisible()) {

        cmp.refresh(function() {
            // select the first node
            var root = cmp.getRootNode();

            if (root.firstChild) {
                cmp.getSelectionModel().select(root.firstChild);
            }
        });

    }
}

Ext.define("Zenoss.templates.Container", {
    alias:['widget.templatecontainer'],
    extend:"Ext.Panel",
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'border',
            defaults: {
                split: true
            },
            items: [{
                xtype: 'DataSourceTreeGrid',
                id: 'dataSourceTreeGrid',
                region: 'center',
                ref: 'dataSourceTreeGrid',
                uid: config.uid,
                root: {
                    uid: config.uid,
                    id: config.uid
                }
            }, {
                xtype: 'panel',
                layout: 'border',
                region: 'east',
                width: '35%',
                defaults: {
                    split: true
                },
                items: [{
                    xtype: 'thresholddatagrid',
                    id: 'thresholdGrid',
                    ref: '../thresholdGrid',
                    region: 'north',
                    height: 300
                }, {
                    xtype: 'graphgrid',
                    id: 'graphGrid',
                    ref: '../graphGrid',
                    region: 'center'
                }]
            }]
        });
        Zenoss.templates.Container.superclass.constructor.call(this, config);
    },
    setContext: function(uid){
        this.updateTreeGrid(this.dataSourceTreeGrid, uid);
        this.updateGrid(this.thresholdGrid, uid);
        this.updateGrid(this.graphGrid, uid);
    },
    updateTreeGrid: function(treeGrid, uid){
        treeGrid.setContext(uid);
    },
    updateGrid: function(grid, uid) {
        grid.setContext(uid);
    }
});


Zenoss.BubblingSelectionModel = Ext.extend(Zenoss.TreeSelectionModel, {
    constructor: function(config) {
        Zenoss.BubblingSelectionModel.superclass.constructor.call(this, config);
        this.enableBubble('selectionchange');
        this.bubbleTarget = config.bubbleTarget;
    },
    getBubbleTarget: function() {
        return this.bubbleTarget;
    }
});

Zenoss.MonTemplateSelectionModel = Ext.extend(Zenoss.BubblingSelectionModel, {
    constructor: function(config) {
        Ext.applyIf(config, {
            listeners: {
                beforeselect: function(sm, node) {
                    return node.isLeaf();
                }
            }
        });
        Zenoss.MonTemplateSelectionModel.superclass.constructor.call(this, config);
    }
});


Ext.define('Zenoss.templates.TemplateTreeModel', {
    extend: 'Zenoss.model.Tree',
    fields: [ 'text', 'uid'],
    getId: function() {
        return this.get("uid");
    },
    proxy: {
        simpleSortMode: true,
        type: 'direct',
        directFn: REMOTE.getTemplates,
        paramOrder: ['uid']
    }
});

Ext.define("Zenoss.templates.MonTemplateTreePanel", {
    alias:['widget.montemplatetreepanel'],
    extend:"Ext.tree.TreePanel",
    constructor: function(config){

        // create the model
        Ext.applyIf(config, {
            useArrows: true,
            manageHeight: false,
            cls: 'x-tree-noicon',
            model: 'Zenoss.model.Tree',
            selModel: new Zenoss.MonTemplateSelectionModel({
                bubbleTarget: config.bubbleTarget
            }),
            store: Ext.create('Ext.data.TreeStore', {
                model: 'Zenoss.templates.TemplateTreeModel',
                nodeParam: 'uid'
            }),
            hideHeaders: true,
            columns: [{
                xtype: 'treecolumn',
                flex: 1,
                dataIndex: 'text'
            }],
            root: {
                text: _t('Monitoring Templates')
            }
        });

        this.callParent([config]);
    },
    initComponent: function(){
        this.callParent(arguments);
        this.getStore().on('load', function() {
            this.getRootNode().expand();
        }, this);
    },
    setContext: function(uid, callback, scope) {
        this.uid = uid;
        if ( uid.match('^/zport/dmd/Devices') ) {
            this.show();
            var root = this.getRootNode();
            if (root) {
                root.data.uid = uid;
                this.getStore().load({
                    callback: callback,
                    scope: scope
                });
            }
        } else {
            this.hide();
        }
    },
    onSelectionChange: function(nodes) {
        var detail, node, uid;
        if (nodes && nodes.length) {
            node = nodes[0];
            uid = node.get("id");
            detail = Ext.getCmp(this.initialConfig.detailPanelId);
            if ( ! detail.items.containsKey('montemplate') ) {
                detail.add({
                    xtype: 'templatecontainer',
                    id: 'montemplate',
                    ref: 'montemplate',
                    uid: uid
                });
            }
            detail.montemplate.setContext(uid);
            detail.getLayout().setActiveItem('montemplate');
        }
    },
    refresh: function(callback, scope) {
        this.setContext(this.uid, callback, scope);
    }
});


Ext.define("Zenoss.BindTemplatesItemSelector", {
    alias:['widget.bindtemplatesitemselector'],
    extend:"Ext.ux.form.ItemSelector",
        constructor: function(config) {
        Ext.applyIf(config, {
            imagePath: "/++resource++zenui/img/xtheme-zenoss/icon",
            drawUpIcon: false,
            drawDownIcon: false,
            drawTopIcon: false,
            drawBotIcon: false,
            displayField: 'name',
            width: 380,
            valueField: 'id',
            store:  Ext.create('Ext.data.ArrayStore', {
                data: [],
                model: 'Zenoss.model.IdName',
                sorters: [{
                    property: 'value'
                }]
            })
        });
        Zenoss.BindTemplatesItemSelector.superclass.constructor.apply(this, arguments);
    },
    setContext: function(uid) {
        REMOTE.getUnboundTemplates({uid: uid}, function(provider, response){
            var data = response.result.data;
            // stack the calls so we can make sure the store is setup correctly first
            REMOTE.getBoundTemplates({uid: uid}, function(provider, response){
                var results = [];
                Ext.each(response.result.data, function(row){
                    results.push(row[0]);
                    data.push(row);
                });
                this.store.loadData(data);
                this.bindStore(this.store);
                this.setValue(results);
            }, this);
        }, this);

    }
});


Ext.define("Zenoss.AddLocalTemplatesDialog", {
    alias:['widget.addlocaltemplatesdialog'],
    extend:"Zenoss.HideFitDialog",
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            title: _t('Add Local Template'),
            layout: 'anchor',
            items: [{
                xtype: 'form',
                ref: 'formPanel',
                listeners: {
                    validitychange: function(formPanel, valid) {
                        me.submitButton.setDisabled( ! valid );
                    }
                },
                items: [{
                    xtype: 'idfield',
                    fieldLabel: _t('Name'),
                    ref: 'templateName',
                    context: config.context
                }]
            }],
            listeners: {
                show: function() {
                    this.formPanel.templateName.setValue('');
                }

            },
            buttons: [
            {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                ref: '../submitButton',
                text: _t('Submit'),
                handler: function() {
                    var templateId = me.formPanel.templateName.getValue();

                    REMOTE.addLocalTemplate({
                       deviceUid: me.context,
                       templateId: templateId
                    }, refreshTemplateTree);
                }
            }, {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                text: _t('Cancel')
            }]
        });
        Zenoss.AddLocalTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});


Ext.define("Zenoss.BindTemplatesDialog", {
    alias:['widget.bindtemplatesdialog'],
    extend:"Zenoss.HideFitDialog",
    constructor: function(config){
        var me = this;
        var itemId = Ext.id();

        Ext.applyIf(config, {
            width: 600,
            height: 400,
            title: _t('Bind Templates'),
            items: [{
                xtype: 'panel',
                width:  550,
                layout: 'column',
                defaults: {
                    columnWidth: 0.5
                },
                items: [{
                    xtype: 'label',
                    style: {'padding':'0 0 5px 7px'},
                    text: 'Available'
                },{
                    xtype: 'label',
                    style: {'padding':'0 0 5px 0'},
                    text: 'Selected'
                }]
            },{
                xtype: 'bindtemplatesitemselector',
                ref: 'itemselector',
                width:500,
                height:200,
                id: itemId,
                context: config.context
            }],
            listeners: {
                show: function() {
                    Ext.getCmp(itemId).setContext(this.context);
                }
            },
            buttons: [
            {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                text: _t('Save'),
                handler: function(){
                    var templateIds;
                    if (Zenoss.Security.hasPermission('Manage DMD')) {
                        templateIds = Ext.getCmp(itemId).getValue();
                        REMOTE.setBoundTemplates({
                            uid: me.context,
                            templateIds: templateIds
                        }, refreshTemplateTree);
                    }
                }
            }, {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                text: _t('Cancel')
            }]
        });
        Zenoss.BindTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});


Ext.define("Zenoss.ResetTemplatesDialog", {
    alias:['widget.resettemplatesdialog'],
    extend:"Zenoss.MessageDialog",
    constructor: function(config) {
        var me = this;
        Ext.applyIf(config, {
            title: _t('Reset Template Bindings'),
            message: _t('Are you sure you want to delete all local template bindings and use default values?'),
            buttons: [
                {
                    xtype: 'HideDialogButton',
                    ui: 'dialog-dark',
                    text: _t('Reset Bindings'),
                    handler: function() {
                        if (Zenoss.Security.hasPermission('Manage DMD')) {
                            REMOTE.resetBoundTemplates(
                                { uid: me.context },
                                refreshTemplateTree);
                        }
                    }
                }, {
                    xtype: 'HideDialogButton',
                    ui: 'dialog-dark',
                    text: _t('Cancel')
                }
            ]
        });
        Zenoss.ResetTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});



Ext.define("Zenoss.OverrideTemplatesDialog", {
    alias:['widget.overridetemplatesdialog'],
    extend:"Zenoss.HideFitDialog",
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            height: 200,
            width: 300,
            title: _t('Override Templates'),
            listeners: {
                show: function() {
                    // completely reload the combobox every time
                    // we show the dialog
                    me.submit.setDisabled(true);
                    me.comboBox.setValue(null);
                    me.comboBox.store.setBaseParam('query', me.context);
                    me.comboBox.store.setBaseParam('uid', me.context);

                }
            },
            items: [{
                xtype: 'label',
                html: _t('Select the bound template you wish to override.')
            },{
                xtype: 'combo',
                forceSelection: true,
                emptyText: _t('Select a template...'),
                minChars: 0,
                ref: 'comboBox',
                selectOnFocus: true,
                typeAhead: true,
                valueField: 'uid',
                displayField: 'label',
                listConfig: {
                    resizable: true
                },
                store: Ext.create('Zenoss.NonPaginatedStore', {
                    root: 'data',
                    model: 'Zenoss.model.Label',
                    directFn: REMOTE.getOverridableTemplates
                }),
                listeners: {
                    select: function(){
                        // disable submit if nothing is selected
                        me.submit.setDisabled(!me.comboBox.getValue());
                    }
                }
            }],
            buttons: [
            {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                ref: '../submit',
                disabled: true,
                text: _t('Submit'),
                handler: function(){
                    if (Zenoss.Security.hasPermission('Manage DMD')) {
                        var templateUid = me.comboBox.getValue();
                        Zenoss.remote.TemplateRouter.copyTemplate({
                            uid: templateUid,
                            targetUid: me.context
                        }, refreshTemplateTree);
                    }
                }
            }, {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                text: _t('Cancel')
            }]
        });
        Zenoss.OverrideTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});


Ext.define("Zenoss.removeLocalTemplateDialog", {
    alias:['widget.removelocaltemplatesdialog'],
    extend:"Zenoss.HideFitDialog",
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            height: 200,
            width: 300,
            title: _t('Remove Local Template'),
            listeners: {
                show: function() {
                    // completely reload the combobox every time
                    // we show the dialog
                    me.submit.setDisabled(true);
                    me.comboBox.setValue(null);
                    me.comboBox.store.setBaseParam('query', me.context);
                    me.comboBox.store.setBaseParam('uid', me.context);
                }
            },
            items: [{
                xtype: 'label',
                html: _t('Select the locally defined template you wish to remove.')
            },{
                xtype: 'combo',
                forceSelection: true,
                width: 200,
                emptyText: _t('Select a template...'),
                minChars: 0,
                ref: 'comboBox',
                selectOnFocus: true,
                valueField: 'uid',
                displayField: 'label',
                typeAhead: true,
                store: Ext.create('Zenoss.NonPaginatedStore', {
                    root: 'data',
                    model: 'Zenoss.model.Label',
                    directFn: REMOTE.getLocalTemplates
                }),
                listeners: {
                    select: function(){
                        // disable submit if nothing is selected
                        me.submit.setDisabled(!me.comboBox.getValue());
                    }
                }
            }],
            buttons: [
            {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                ref: '../submit',
                disabled: true,
                text: _t('Submit'),
                handler: function(){
                    if (Zenoss.Security.hasPermission('Manage DMD')) {
                        var templateUid = me.comboBox.getValue();
                        REMOTE.removeLocalTemplate({
                            deviceUid: me.context,
                            templateUid: templateUid
                        }, refreshTemplateTree);
                    }
                }
            }, {
                xtype: 'HideDialogButton',
                ui: 'dialog-dark',
                text: _t('Cancel')
            }]
        });
        Zenoss.OverrideTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2009, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');


// options comprises the following:
//      addToZenPack: true puts the Add To ZenPack icon in, default true
//      hasOrganizers: true puts the Add ___  Organizer in, default true
//      customAddDialog: config for a SmartFormDialog to override the default
//      deleteMenu: needs separate menu items for deleting organizers and items,
//                  works in conjunction with hasOrganizers
//      contextGetter: fetches the context UIDs for the specific page

Zenoss.footerHelper = function(itemName, footerBar, options) {
    var items;

    options = Ext.applyIf(options || {}, {
        addToZenPack: true,
        hasOrganizers: true,
        deleteMenu: true,
        hasContextMenu: true,
        customAddDialog: {},
        buttonContextMenu: {},
        contextGetter: null,
        onGetDeleteMessage: function (itemName) {
            return Ext.String.format(_t('The selected {0} will be deleted.'), itemName.toLowerCase());
        },
        onGetAddDialogItems: function () {
            return [{
                xtype: 'textfield',
                name: 'id',
                fieldLabel: _t('Name'),
                anchor: '80%',
                allowBlank: false
            }];
        },
        onGetItemName: function() {
            return itemName;
        }
    });

    Ext.applyIf(options.buttonContextMenu, {
        xtype: 'ContextMenu',
        tooltip: _t('Context-sensitive actions'),
        menu: {
            items: []
        },
        ref: 'buttonContextMenu'
    });

    footerBar = footerBar || Ext.getCmp('footer_bar');

    // For now, we will monkey-patch a setContext onto it.
    footerBar.setContext = function(contextUid) {
        Ext.each(this.items.items, function(i) {
            if (i.setContext) { i.setContext(contextUid); }
        });
    };


    function showAddDialog(template, event) {
        var handler, dialog, addDialogConfig;

        // Shallow copy config to avoid mangling the original config
        addDialogConfig = Ext.apply({}, options.customAddDialog);

        handler = function(values) {
            footerBar.fireEvent('buttonClick', event, values.id, values);
        };

        addDialogConfig = Ext.applyIf(addDialogConfig, {
            submitHandler: handler,
            items: options.onGetAddDialogItems(),
            title: Ext.String.format(template, options.onGetItemName())
        });

        dialog = new Zenoss.SmartFormDialog(addDialogConfig);
        dialog.show();

    }

    items = [
        {
            xtype: 'FlexButton',
            id: 'footer_add_button',
            iconCls: 'add',
            hidden: Zenoss.Security.doesNotHavePermission('Add DMD Objects'),
            tooltip: _t('Add a child to the selected organizer'),
            menu: {
                items: [
                    {
                        text: Ext.String.format(_t('Add {0}'), itemName),
                        listeners: {
                            click: Ext.pass(showAddDialog, ['Add {0}', 'addClass'])
                        }
                    }
                ]
            },
            ref: 'buttonAdd'
        },
        {
            xtype: 'FlexButton',
            id: 'footer_delete_button',
            iconCls: 'delete',
            hidden: Zenoss.Security.doesNotHavePermission('Delete objects'),
            tooltip: Ext.String.format(_t('Delete {0}'), itemName),
            menu: {
                items: [{

                    text: Ext.String.format(_t('Delete {0}'), options.onGetItemName()),
                    listeners: {
                        click: function() {
                            var itemName = options.onGetItemName();
                            new Zenoss.dialog.SimpleMessageDialog({
                                message: options.onGetDeleteMessage(itemName),
                                buttonAlign: 'center',
                                title: Ext.String.format(_t('Delete {0}'), options.onGetItemName()),
                                buttons: [{
                                    xtype: 'DialogButton',
                                    text: _t('OK'),
                                    handler: function(){
                                        footerBar.fireEvent('buttonClick', 'delete');
                                    }
                                }, {
                                    xtype: 'DialogButton',
                                    text: _t('Cancel')
                                }]
                            }).show();
                        }
                    }
                }]

            },
            ref: 'buttonDelete'
        }
    ];





    if (options.hasOrganizers)
    {
        // add button
        items[0].menu.items.push({
            text: Ext.String.format(_t('Add {0} Organizer'), itemName),
            param: 'addOrganizer',
            listeners: {
                click: Ext.pass(showAddDialog, [_t('Add {0} Organizer'), 'addOrganizer'])
            },
            ref: 'buttonAddOrganizer'
        });

        if (options.deleteMenu)
        {

            items[1].menu.items.push({
                text: Ext.String.format(_t('Delete {0} Organizer'), itemName),
                ref: 'buttonDeleteOrganizer',
                listeners: {
                    click: function() {
                        var itemName = options.onGetItemName();
                        Ext.MessageBox.show({
                            title: Ext.String.format(_t('Delete {0} Organizer'), itemName),
                            msg: Ext.String.format(_t('The selected {0} organizer will be deleted.'),
                                    itemName.toLowerCase()),
                            fn: function(buttonid){
                                if (buttonid==='ok') {
                                    footerBar.fireEvent('buttonClick', 'deleteOrganizer');
                                }
                            },
                            buttons: Ext.MessageBox.OKCANCEL
                        });
                    }
                }
            });
        }
    }

    if (options.addToZenPack) {
        options.buttonContextMenu.menu.items.push({
            ref: 'buttonAddToZenPack',
            text: Ext.String.format(_t('Add {0} to ZenPack'), itemName),
            listeners: {
                click: function() {
                    var addToZenPackDialog = new Zenoss.AddToZenPackWindow({});
                    var target = options.contextGetter.getUid();
                    if ( ! target ) {
                        return;
                    }
                    addToZenPackDialog.setTarget(target);
                    addToZenPackDialog.show();
                }
            }
        });

        if ( options.contextGetter.hasTwoControls() ) {

            options.buttonContextMenu.menu.items.push({
                ref: 'buttonAddOrganizerToZenPack',
                text: Ext.String.format(_t('Add {0} Organizer to ZenPack'), itemName),
                listeners: {
                    click: function() {
                        var addToZenPackDialog = new Zenoss.AddToZenPackWindow({});
                        var target = options.contextGetter.getOrganizerUid();
                        if ( ! target ) {
                            return;
                        }
                        addToZenPackDialog.setTarget(target);
                        addToZenPackDialog.show();
                    }
                }
            });

        }

    }

    if ( options.hasContextMenu || options.addToZenPack ) {
        items.push(' ');
        items.push(options.buttonContextMenu);
    }

    items.push('-');
    footerBar.add(items);

};

})();
(function(){

Ext.ns('Zenoss');

function makeIpAddress(val) {
    var octets = val.split('.');
    if(octets.length>4) {
        return false;
    }
    while(octets.length < 4) {
        octets.push('0');
    }
    for(var i=0;i<octets.length;i++) {
        var octet=parseInt(octets[i], 10);
        if (!octet && octet!==0) {
            return false;
        }
        try {
            if (octet>255) {
                return false;
            }
        } catch(e) {
            return false;
        }
        octets[i] = octet.toString();
    }
    return octets.join('.');
}


/**
 * @class Zenoss.IpAddressField
 * @extends Ext.form.TextField
 * @constructor
 */
Ext.define("Zenoss.IpAddressField", {
    alias:['widget.ipaddressfield'],
    extend:"Ext.form.TextField",
    constructor: function(config){
        config.maskRe = true;
        Zenoss.IpAddressField.superclass.constructor.call(this, config);
    },
    filterKeys: function(e, dom) {
        if(e.ctrlKey || e.isSpecialKey()){
            return;
        }
        e.stopEvent();
        var result, newoctet,
            cursor = dom.selectionStart,
            selend = dom.selectionEnd,
            beg = dom.value.substring(0, cursor),
            end = dom.value.substring(selend),
            s = String.fromCharCode(e.getCharCode());
        if (s==='.') {
            result = beg + end;
            cursor += end.indexOf('.');
            newoctet = end.split('.')[1];
            if (selend===cursor+1) {
                cursor++;
            }
            if(newoctet) {
                dom.setSelectionRange(cursor+1, cursor+newoctet.length+1);
            }
        } else {
            result = makeIpAddress(beg + s + end);
            if (result) {
                cursor++;
                dom.value = result;
                dom.setSelectionRange(cursor, cursor);
            }
        }
    }

}); // Ext.extend



})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function () {

    Ext.define("Zenoss.SlidingCardLayout", {
        extend:"Ext.layout.CardLayout",
        alias:['layout.slide'],
        sizeAllCards:true,
        setActiveItem:function (index) {
            var C = this.owner,
                B = C.body,
                card = C.getComponent(index),
                active = this.activeItem,
                activeIndex = Ext.Array.indexOf(C.items.items, active), shiftsCallback;
            if (card !== active) {
                if (active) {
                    if (card) {
                        C.fireEvent('beforecardchange', C, card, index, active, activeIndex);
                        if (!card.rendered) {
                            this.renderItem(card, index, C.getLayoutTarget());
                        }
                        card.show();
                        if (card.doLayout && (this.layoutOnCardChange ||
                            !card.rendered)) {
                            card.doLayout();
                        }
                        var _done = 0;

                        shiftsCallback = function() {
                            _done++;
                            if (_done === 2) {
                                C.fireEvent('cardchange', C, card, index, active,
                                    activeIndex);
                            }
                        };

                        var x = B.getX(),
                            w = B.getWidth(),
                            s = [x - w, x + w],
                            cfg = {
                                duration:250,
                                easing:'ease',
                                opacity:0,
                                callback:shiftsCallback
                            };
                        card.el.setY(B.getY());
                        card.el.setX((activeIndex < index ? s[1] : s[0]));
                        active.el.shift(Ext.applyIf({
                            x:activeIndex < index ? s[0] : s[1]
                        }, cfg));
                        card.el.shift(Ext.applyIf({
                            x:x,
                            opacity:1
                        }, cfg));
                    }
                }
                this.activeItem = card;
                this.initLayout();
            }

        }

    });

    Ext.layout.container['slide'] = Zenoss.SlidingCardLayout;


    Ext.define("Zenoss.HorizontalSlidePanel", {
        alias:['widget.horizontalslide'],
        extend:"Ext.Panel",
        constructor:function (config) {
            this.headerText = Ext.create('Ext.toolbar.TextItem', {
                html:(config && config.text) ? config.text : ''
            });
            config = Ext.applyIf(config || {}, {
                cls:'subselect',
                layout:'slide',
                activeItem:0
            });
            var items = [];
            Ext.each(config.items, function (item, index) {
                var headerText = Ext.create('Ext.toolbar.TextItem', {
                    html:(item && item.text) ? item.text : ''
                });
                var navButton = Ext.create('Ext.Button', {
                    text:(item && item.buttonText) ? item.buttonText : '',
                    // make the button belong to the owner panel
                    ref:(item.buttonRef || 'navButton'),
                    cls:index ? 'toleft' : 'toright',
                    ui:'arrowslide',
                    handler:function () {
                        this.layout.setActiveItem(index ? 0 : 1);

                    },
                    scope:this
                }, this);
                this[item.buttonRef] = navButton;
                navButton.hide();
                items.push({
                    layout:'fit',
                    tbar:{
                        cls:'subselect-head',
                        height:37,
                        items:[headerText, '->', navButton]
                    },
                    items:[item],
                    listeners:{
                        render:function (card) {
                            card.card = card.items.items[0];
                            card.card.parentCard = card;
                            card.headerText = headerText;
                            card.navButton = navButton;
                            var setHeaderText = function (text, qtip) {
                                this.headerText.setText(text);
                                Ext.QuickTips.unregister(this.headerText);

                                if (qtip) {
                                    Ext.QuickTips.register({
                                        target:this.headerText,
                                        text:qtip
                                    });
                                }
                            };
                            var setButtonText = function (text) {
                                this.navButton.setText(text);
                            };
                            card.setHeaderText = Ext.bind(setHeaderText, card);
                            card.setButtonText = Ext.bind(setButtonText, card);
                        }
                    }
                });
            }, this);
            config.items = items;
            Zenoss.HorizontalSlidePanel.superclass.constructor.call(this, config);
        },
        initEvents:function () {
            this.addEvents('beforecardchange');
            this.addEvents('cardchange');
            Zenoss.HorizontalSlidePanel.superclass.initEvents.call(this);
        }
    });


    /**
     * A MixedCollection for nav configs.
     *
     * Each item in the config is a normal dictionary with an xtype. The xtype specified will be used for the display
     * when the item is clicked.
     *
     * Example item structure:
     *
     * {
     *     text: 'Menu label',
     *     id: 'View id',
     *     xtype: 'somextype', // optional, defaults to Panel
     *     filterNav: function(navTree) {
     *         // An optional function that is called to determine if this item should be shown
     *         // Use navTree.contextUid for the context.
     *         // return true to display the item or false to remove it
     *     }
     * }
     */
    Zenoss.NavConfig = Ext.extend(Ext.util.MixedCollection, {
        constructor:function () {
            Zenoss.NavConfig.superclass.constructor.call(this, true, function (item) {
                return item.id;
            });
        }
    });

    Zenoss.NavManager = Ext.extend(Object, {
        all:null,
        constructor:function () {
            this.all = new Zenoss.NavConfig();
            this.all.addEvents('navready');
        },
        /**
         * Register a nav item tree. See `Zenoss.NavConfig` for item structure.
         *
         * Zenoss.nav.register({
         *     DeviceGroup: [
         *         {
         *             id: 'device_grid',
         *             text: 'Devices',
         *             listeners: {
         *                 render: updateNavTextWithCount
         *             }
         *         },
         *         {
         *             id: 'events_grid',
         *             text: _t('Events')
         *         }
         *     ]
         * });
         *
         * @param navSpec
         */
        register:function (navSpec) {
            for (var type in navSpec) {
                this.add(type, navSpec[type]);
            }
        },
        /**
         * Adds a nav of `type` with its `items`. If a nav of `type` already exists, the items are appended.
         *
         * @param type Type of nav menu to add (ex: Device, DeviceGroup)
         * @param items An array of nav items. See `Zenoss.NavConfig` for item structure.
         */
        add:function (type, items) {
            if (!this.all.containsKey(type)) {
                // Create an empty nav container
                var newNav = new Zenoss.NavConfig();
                newNav.id = type;
                this.all.add(type, newNav);

                this.appendTo(type, items);

                this.all.fireEvent('navready', newNav);
            }
            else {
                this.appendTo(type, items);
            }
        },
        /**
         * Get a nav config if it exists.
         *
         * @param type
         */
        get:function (type) {
            return this.all.getByKey(type);
        },
        /**
         * Add menu nodes to the end of a nav config
         *
         * @param type Nav config type
         * @param items Array of items. See `Zenoss.NavConfig` for item structure.
         */
        appendTo:function (type, items) {
            if (this.all.containsKey(type)) {
                var nav = this.all.getByKey(type);
                nav.addAll(items);
            }
            else {
                this.onAvailable(type, function (item) {
                    item.addAll(items);
                });
            }
        },
        /**
         * Registers a callback to be called as soon as a nav tree matching navId is added.
         *
         * @param navId
         * @param callback
         */
        onAvailable:function (navId, callback, scope) {
            function onAdd(item) {
                if (item.id === navId) {
                    callback.call(scope || item, item);
                    this.all.un('navready', onAdd, scope);
                }
            }

            this.all.on('navready', onAdd, this);
        }
    });

    Zenoss.nav = new Zenoss.NavManager();

// Zenoss.SubselectionNodeUI = Ext.extend(Ext.tree.TreeNodeUI, {
//     render: function() {
//         Zenoss.SubselectionNodeUI.superclass.render.call(this);
//         Ext.removeNode(this.getIconEl());
//     }
// });

// Zenoss.SubselectionNode = Ext.extend(Ext.tree.TreeNode, {
//     constructor: function(config) {
//         Ext.applyIf(config, {
//             leaf: true,
//             uiProvider: Zenoss.SubselectionNodeUI
//         });
//         Zenoss.SubselectionNode.superclass.constructor.call(this, config);
//         this.addEvents('render');
//     },
//     render: function(bulkRender) {
//         Zenoss.SubselectionNode.superclass.render.call(this, bulkRender);
//         this.fireEvent('render', this);
//     }

// });

// Ext.tree.TreePanel.nodeTypes.subselect = Zenoss.SubselectionNode;


    Ext.define("Zenoss.SubselectionPanel", {
        alias:['widget.subselection'],
        extend:"Ext.Panel",
        constructor:function (config) {
            var id = config.id || Ext.id();
            Ext.applyIf(config, {
                id:id,
                layout:'fit',
                bodyStyle:{ 'margin-top':10 },
                items:[
                    {
                        xtype:'treepanel',
                        ref:'treepanel',
                        hideHeaders: true,
                        header: false,
                        selModel:new Zenoss.TreeSelectionModel({
                            listeners:{
                                selectionchange:function (sm, node) {
                                    if (node) {
                                        var action = node.data.action;
                                        if (action) {
                                            if (Ext.isString(this.target)) {
                                                this.target = Ext.getCmp(this.target);
                                            }
                                            action.call(node, node, this.target);
                                        }
                                    }
                                },
                                scope:this
                            }
                        }),
                        id:'subselecttreepanel' + id,
                        rootVisible:false,
                        root:{nodeType:'node'}
                    }
                ]
            });
            Zenoss.SubselectionPanel.superclass.constructor.call(this, config);
        },
        initComponent:function () {
            Zenoss.SubselectionPanel.superclass.initComponent.call(this);
            this.treepanel = Ext.getCmp('subselecttreepanel' + this.id);
        },
        setContext:function (uid) {
            var type = Zenoss.types.type(uid),
                nodes = Zenoss.nav.get(type);
            if (nodes) {
                Zenoss.util.each(nodes, function (node) {
                    Ext.applyIf(node, {
                        nodeType:'subselect'
                    });
                });
                var root = new Ext.tree.AsyncTreeNode({
                    children:nodes,
                    listeners:{
                        load:function (node) {
                            var toselect = node.firstChild;
                            if (toselect) {
                                if (toselect.rendered) {
                                    toselect.select();
                                } else {
                                    toselect.on('render', function (node) {
                                        node.select();
                                    });
                                }
                            }
                        },
                        scope:this
                    }
                });
                this.treepanel.setRootNode(root);
            }
        }
    });

    Ext.define("Zenoss.DetailNavTreeModel", {
        extend:'Ext.data.Model',
        fields:Zenoss.model.BASE_TREE_FIELDS.concat([
            {
                name:'action',
                type:'function'
            }
        ])
    });

    Ext.define("Zenoss.DetailNavTreePanel", {
        alias:['widget.detailnavtreepanel'],
        extend:"Ext.tree.Panel",
        constructor:function (config) {
            Ext.applyIf(config, {
                store:Ext.create('Ext.data.TreeStore', {
                    model:'Zenoss.DetailNavTreeModel',
                    proxy:{
                        type:'memory',
                        reader:{
                            type:'json'
                        }
                    },
                    autoLoad:false
                }),
                useArrows:true,
                manageHeight: false,
                hideHeaders:true,
                columns:[
                    {
                        xtype:'treecolumn',
                        flex:1,
                        dataIndex:'text'
                    }
                ],
                selModel:new Zenoss.BubblingSelectionModel({
                    bubbleTarget:config.bubbleTarget
                }),
                id:'subselecttreepanel' + config.idSuffix,
                ref:'subselecttreepanel',
                rootVisible:false,
                iconCls:'x-tree-noicon',
                header: false,
                root:{nodeType:'node'}
            });
            this.callParent([config]);
        },
        setNodeVisible:function (node, visible) {
            if (Ext.isString(node)) {
                node = this.getNodeById(node);
            }
            var view = this.getView(),
                el = Ext.fly(view.getNodeByRecord(node));
            if (el) {
                el.setVisibilityMode(Ext.Element.DISPLAY);
                el.setVisible(visible);
            }
        }
    });


    /**
     * Used to manage and display detail navigation tree for a contextId
     *
     * @class Zenoss.DetailNavPanel
     * @extends Zenoss.SubselectionPanel
     */
    Ext.define("Zenoss.DetailNavPanel", {
        alias:['widget.detailnav'],
        extend:"Zenoss.SubselectionPanel",
        /**
         * @cfg {function} onGetNavConfig abstract function; hook to provide more nav items
         * @param {string} uid; item to get nav items for
         */
        onGetNavConfig:function () {
            return [];
        },
        /**
         * Called when an item in detail nav is selected
         * @param {object} this, the DetailNavPanel
         * @param {object} the navigation node selected
         */
        onSelectionChange:Ext.emptyFn,
        /**
         * Filter out from being in the detail nav; used to filter out nav nodes
         * and the content panels. Return true if it should be kept, false otherwise
         * @param {DetailNavPanel}
            * @param {Object} config
         */
        filterNav:function () {
            return true;
        },
        /**
         * The object id for which the detail navigation belongs to
         */
        contextId:null,
        /**
         * Menu ids used to get non dialog menus to be used in navigation;
         * Empty or null if no nav from old menu items is desired
         */
        menuIds:['More', 'Manage', 'Edit', 'Actions', 'Add', 'TopLevel'],
        /**
         * map of nav id to panel configuration
         */
        loaded:false,
        panelConfigMap:null,
        selectFirstNode:true,
        constructor:function (config) {
            Ext.applyIf(config, {
                id:Ext.id(),
                bodyCls:'detailnav',
                layout:'fit',
                bodyStyle:{ 'margin-top':10 }
            });
            // call second applyIf so config.id is set correctly
            Ext.applyIf(config, {
                items:{
                    xtype:'detailnavtreepanel',
                    ref:'navtreepanel',
                    idSuffix:config.id,
                    bubbleTarget:config.bubbleTarget
                }
            });
            Zenoss.DetailNavPanel.superclass.constructor.call(this, config);
        },
        initEvents:function () {
            this.addEvents(
                /**
                 * @event navloaded
                 * Fires after the navigation has been loaded
                 * @param {DetailNavPanel} this The DetailNavPanel
                 * @param {AsyncTreeNode} root The root node
                 */
                'navloaded',
                /*
                 * @event nodeloaded
                 * Fires after each navigation node has been loaded
                 * @param {DetailNavPanel} this The DetailNavPanel
                 * @param {Object} The Navigation config loaded
                 */
                'nodeloaded',
                /**
                 * @event contextchange
                 * Fires after the navigation has been loaded
                 * @param {DetailNavPanel} this The DetailNavPanel
                 */
                'contextchange'
            );
            Zenoss.DetailNavPanel.superclass.initEvents.call(this);
            if (this.selectFirstNode) {
                this.on('navloaded', this.selectFirst, this);
            }
        },
        setContext:function (uid) {
            //called to load the nav tree
            this.loaded = false;
            this.contextId = uid;
            this.treepanel.setRootNode([]);
            this.getNavConfig(uid);
            this.fireEvent('contextchange', this);
        },
        getSelectionModel:function () {
            return this.treepanel.getSelectionModel();
        },
        getNavConfig:function (uid) {
            //Direct call to get nav configs from server
            var me = this;
            var myCallback = function (provider, response) {
                var detailConfigs = response.result.detailConfigs;

                var filterFn = function (val) {
                    var show = true;
                    if (Ext.isFunction(val.filterNav)) {
                        show = val.filterNav(me);
                    }

                    return show && me.filterNav(me, val);
                };

                detailConfigs = Zenoss.util.filter(detailConfigs, filterFn, me);
                var nodes = me.onGetNavConfig(me.contextId);
                if (!Ext.isDefined(nodes) || nodes === null) {
                    nodes = [];
                }

                nodes = Zenoss.util.filter(nodes, filterFn, me);

                if (detailConfigs) {
                    nodes = nodes.concat(detailConfigs);
                }

                me.panelConfigMap = [];
                Zenoss.util.each(nodes, function (val) {
                    me.panelConfigMap[val.id] = val;
                });

                me.setNavTree(nodes);
            };
            var args = {
                'uid':uid
            };
            if (this.menuIds !== null && this.menuIds.length >= 1) {
                args['menuIds'] = this.menuIds;
            }
            if (Zenoss.env.lefthandnav) {
                myCallback(null, {
                    result:Zenoss.env.lefthandnav
                });
                delete Zenoss.env.lefthandnav;
            } else {
                Zenoss.remote.DetailNavRouter.getDetailNavConfigs(args, myCallback, this);
            }

        },
        reset:function () {
            return this.treepanel.setRootNode({});
        },
        selectFirst:function (me, root) {
            var sel = me.getSelectionModel().getSelectedNode(),
                token = Ext.History.getToken(),
                firstToken = me.id + Ext.History.DELIMITER + root.firstChild.id;
            if (!sel) {
                if (!token || (token && token === firstToken)) {
                    me.getSelectionModel().select(root.firstChild);
                }
            }
        },
        /**
         * Set the nodes to display in the nav tree.
         * @param nodes Zenoss.NavConfig Nodes to set the nav to
         */
        setNavTree:function (nodes) {
            //get any configs registered by the page
            var root;
            if (nodes) {
                root = this.treepanel.getRootNode();
                Zenoss.util.each(nodes, function (node) {
                    Ext.applyIf(node, {
                        nodeType:'subselect',
                        leaf:true
                    });
                    root.appendChild(node);
                });

                // Send an alert for all nodes after all nodes have been loaded
                Zenoss.util.each(nodes, function (navConfig) {
                    this.fireEvent('nodeloaded', this, navConfig);
                }, this);

            }
            this.loaded = true;
            this.fireEvent('navloaded', this, root);

            root.expand(false, function(){
                if (this.manualAdjustHeight) {
                    //HACK: cheat to make sure that all nodes are visible
                    var view = this.treepanel.getView();
                    var height = Ext.fly(view.getNode(0)).getHeight();
                    this.setHeight((height * nodes.length) + 35);
                }
            }, this);
            this.doLayout();


        }
    });


    Ext.define("Zenoss.DetailNavCombo", {
        alias:['widget.detailnavcombo'],
        extend:"Ext.form.ComboBox",
        target:null,
        contextUid:null,
        lastSelItem:null,
        panelConfigMap:null,
        queryMode:'local',
        editable:false,
        forceSelection:true,
        typeAhead:false,
        triggerAction:'all',
        filterNav:function () {
            return true;
        },
        menuIds:['More', 'Manage', 'Edit', 'Actions', 'Add', 'TopLevel'],
        onSelectionChange:Ext.emptyFn,
        onGetNavConfig:function () {
            return [];
        },
        constructor:function (config) {
            Ext.applyIf(config, {
                store:new Ext.data.ArrayStore({
                    'id':0,
                    model: 'Zenoss.model.ValueText',
                    autoDestroy:true
                })
            });
            this.callParent(arguments);
        },
        getTarget:function () {
            var target = this.target;
            return Ext.isString(target) ? Ext.getCmp(target) : target;
        },
        initEvents:function () {
            Zenoss.DetailNavCombo.superclass.initEvents.call(this);
            this.on('select', this.onItemSelected, this);
        },
        onItemSelected:function (me, item) {
            var item = item[0],
                target = this.getTarget(),
                id = item.get('value'),
                config = this.panelConfigMap[id],
                action = config.action || function (node, target) {
                    if (!(id in target.items.map)) {
                        if (config) {
                            target.add(config);
                            target.doLayout();
                        }
                    }
                    target.items.map[id].setContext(this.contextUid);
                    target.layout.setActiveItem(id);
                };
            this.lastSelItem = item;
            action.call(this, item, target, this);
        },
        selectAt:function (idx) {
            var record = this.store.getAt(idx || 0);
            this.selectByItem(record);
        },
        selectByItem:function (item) {
            this.lastSelItem = item || this.store.getAt(0);
            this.select(this.lastSelItem);
            this.fireEvent('select', this, [this.lastSelItem]);
        },
        setContext:function (uid) {
            // make sure the context actually changed.
            if (uid === this.contextUid) {
                return;
            }
            this.contextUid = uid;
            var args = {uid:uid};
            if (!Ext.isEmpty(this.menuIds)) {
                args.menuIds = this.menuIds;
            }

            Zenoss.remote.DetailNavRouter.getDetailNavConfigs(args, function (r) {
                var detailConfigs = r.detailConfigs,
                    items = [],
                    nodes = [],
                    lastSelItem = this.lastSelItem,
                    hasItem = false,
                    panelMap = {};
                var filterFn = function (val) {
                    var show = true;
                    if (Ext.isFunction(val.filterNav)) {
                        show = val.filterNav(this);
                    }

                    return show && this.filterNav(val);
                };

                nodes = this.onGetNavConfig(uid);
                nodes = Zenoss.util.filter(nodes, filterFn, this);

                detailConfigs = Zenoss.util.filter(detailConfigs, filterFn, this);

                nodes = nodes.concat(detailConfigs);

                Ext.each(nodes, function (cfg) {
                    items.push([cfg.id, cfg.text]);
                    panelMap[cfg.id] = cfg;
                    // when switching component types we need to make sure
                    // that they share a common menu item
                    if (lastSelItem && cfg.id === lastSelItem.getId()) {
                        hasItem = true;
                    }
                });

                this.panelConfigMap = panelMap;
                this.store = new Ext.data.ArrayStore({
                    'id':0,
                    model: 'Zenoss.model.ValueText',
                    data:items,
                    autoDestroy:true
                });
                this.valueField = 'value';
                this.displayField = 'text';
                this.list = null;
                this.bindStore(this.store);
                this.doComponentLayout();
                // "sticky" menu selection, show same item as was shown for last context
                if (hasItem) {
                    this.selectByItem(this.lastSelItem);
                } else {
                    this.selectAt(0);
                }
            }, this);
        }
    });


    Ext.define("Zenoss.DetailContainer", {
        alias:['widget.detailcontainer'],
        extend:"Ext.Container",
        constructor:function (config) {
            Ext.applyIf(config, {
                autoScroll:true,
                listeners:{
                    selectionchange:this.onSelectionChange,
                    scope:this
                }
            });
            Ext.each(config.items, function (item) {
                item.bubbleTarget = this;
            }, this);
            this.callParent(arguments);
        },
        setContext:function (uid) {
            this.items.each(function (item) {
                item.setContext(uid);
            });
        },

        /**
         *  When used in devdetail.js, there will be two separate trees in here.
         *  So we need to handle the times that the user first selected
         *  something from one tree and then selected something from the other tree.
         *  We need to deselect the other tree nicely.
         * @param eventSelModel
         * @param node    should be the node that just got SELECTED.
         */
        onSelectionChange:function (eventSelModel, node) {
            var itemSelModel;
            // if node is empty then stuff just got deselected
            // from a tree, and there is nothing to do here when that happens
            if (node.length > 0) {
                this.items.each(function (item) {
                    itemSelModel = item.getSelectionModel();
                    if (itemSelModel === eventSelModel) {
                        item.onSelectionChange(node);
                    } else {
                        itemSelModel.deselectAll();
                    }
                });
            }
        }
    });


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.define("Zenoss.IFramePanel", {
    alias:['widget.iframe'],
    extend:"Ext.ux.IFrame",
    frameLoaded: false,
    testEarlyReadiness: false,
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            timeout: 5000, // Wait 5s for iframe to initialize before failing
            pollInterval: 50,
            processed: true,
            loadMask: _t('Loading...'),
            src: config.url || 'about:blank',
            ignoreClassName: false
        });
        Zenoss.IFramePanel.superclass.constructor.call(this, config);

    },
    initComponent: function(){
        this.callParent(arguments);
        this.addEvents('frameload', 'framefailed', 'isReady');
        this.on('frameload', function() {
            // Load any messages that may have been created by the frame
            Zenoss.messenger.checkMessages();
        }, this);
    },
    onRender: function() {
        Zenoss.IFramePanel.superclass.onRender.apply(this, arguments);
        // Hook up load events
        this.frame = this.getEl();
        this.waitForLoad();
    },
    afterRender: function() {
        Zenoss.IFramePanel.superclass.afterRender.apply(this, arguments);
        if (!this.ownerCt) {
            var pos = this.getPosition(),
                size = this.frame.parent().getViewSize();
            this.setSize(size.width - pos[0], size.height-pos[1]);
        }
    },

    waitForLoad: function() {
        var doc = this.getDocument(),
            currentUrl = doc ? doc.location.href : null,
            ready = false,
            readyTooEarly = this.testEarlyReadiness,
            body, dom, href,
            i = 0,
            timestocheck = this.timeout / this.pollInterval;
        Ext.bind(function do_check() {
            if (this.frameLoaded) {
                return;
            }
            body = this.getBody();
            if (currentUrl === 'about:blank' || currentUrl === '') {
                // if an iframe is reused, it could have a body and
                // className immediately, but not the desired ones.
                // in that case, poll until the ready test fails,
                // then again until it succeeds.
                if (readyTooEarly) {
                    readyTooEarly = !!body && (this.ignoreClassName || !!body.className);
                } else {
                    ready = !!body && (this.ignoreClassName || !!body.className);

                    // Allow subclasses and clients defined when the panel is ready
                    ready = ready && this.fireEvent('isReady', this.getWindow());
                }
            } else {
                dom = body ? body.dom : null;
                href = this.getDocument().location.href;
                ready = href !== currentUrl || (dom && dom.innerHTML);

                // Allow subclasses and clients defined when the panel is ready
                ready = ready && this.fireEvent('isReady', this.getWindow());
            }
            if (ready  || i++ > timestocheck) {
                this.frameLoaded = ready;
                    this.fireEvent(ready ? 'frameload' : 'framefailed',
                               this.getWindow());
            } else {
                Ext.defer(do_check, this.pollInterval, this);
            }
        }, this)();
    },
    getDocument: function() {
        return this.getDoc();
    },
    getWindow: function() {
        return this.getWin();
    },
    setSrc: function(url) {
        this.frameLoaded = false;
        if (url === 'about:blank' || url === '') {
            this.load('about:blank');
        } else {
            this.load(Ext.urlAppend(url,
                    '_dc=' + new Date().getTime()));
        }
        this.waitForLoad();
    }
});




/**
 * Panel used for displaying old zenoss ui pages in an iframe. Set Context
 * should be called by page to initialze panel for viewing.
 *
 * @class Zenoss.BackCompatPanel
 * @extends Zenoss.ContextualIFrame
 */
Ext.define("Zenoss.BackCompatPanel", {
    alias:['widget.backcompat'],
    extend:"Zenoss.IFramePanel",
    contextUid: null,
    constructor: function(config) {
        Ext.apply(config || {}, {
            testEarlyReadiness: true
        });
        Zenoss.BackCompatPanel.superclass.constructor.call(this, config);
        this.addEvents('frameloadfinished');
        this.on('frameload', function(win) {
            if (Ext.isDefined(win.Ext) && Ext.isDefined(win.Ext.onReady)) {
                var me = this;
                win.Ext.onReady(function(){
                    me.fireEvent('frameloadfinished', win);
                });
            }else if (win.document && win.document.body) {
                this.fireEvent('frameloadfinished', win);
            } else {
                win.onload = Ext.bind(function() {
                    this.fireEvent('frameloadfinished', win);
                }, this);
            }
        }, this);
        this.on('load', function(){
            this.processed = true;
        }); 
        // the frame is not finished loading until Ext is ready
        this.on('isReady', function(win){
            return Ext.isDefined(win.Ext) && Ext.isDefined(win.Ext.onReady);
        });
    },
    setContext: function(uid) {
        this.contextUid = uid;
        var url = uid;
        if (Ext.isDefined(this.viewName) && this.viewName !== null) {
            url = uid + '/' + this.viewName;
        }
        // make sure we are rendered before we set our source
        if (this.rendered) {
            this.setSrc(url);
        } else {
            this.on('afterrender', function() { this.setSrc(url);}, this, {single:true});
        }
    }
});





Zenoss.util.registerBackCompatMenu = function(menu, btn, align, offsets){

    align = align || 'bl';
    offsets = offsets || [0, 0];

    var layer = new Ext.Panel({
        floating: true,
        contentEl: menu,
        border: false,
        shadow: !Ext.isIE,
        bodyCls: menu.id==='contextmenu_items' ? 'z-bc-z-menu z-bc-page-menu' : 'z-bc-z-menu'
    });

    layer.render(Ext.getBody());

    function showMenu() {
        var xy = layer.getEl().getAlignToXY(btn.getEl(), align, offsets);
        layer.setPagePosition(xy[0], xy[1]);
        menu.dom.style.display = 'block';
        layer.show();
    }

    function hideMenu() {
        layer.hide();
    }

    function menuClicked(e) {
        var link = e.getTarget('a');
        if (link) {
            // Fake a click
            location.href = link.href;
        }
    }
    btn.on('click', function(){
        btn.fireEvent('menushow', btn, btn.menu);
    });
    btn.on('menushow', showMenu);
    btn.on('menuhide', hideMenu);
    menu.on('mousedown', menuClicked);

};

})();
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


(function(){

/**********************************************************************
 *
 * Command Panel
 *
 */
var formTpl = new Ext.Template(
    '<form name="commandform" method="POST" action="{target}">',
    '<textarea style="visibility:hidden" name="data">',
    '{data}',
    '</textarea>',
    '</form>');
formTpl.compile();

Ext.define("Zenoss.CommandPanel", {
    alias:['widget.commandpanel'],
    extend:"Zenoss.IFramePanel",
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            ignoreClassName: true
        });
        Zenoss.CommandPanel.superclass.constructor.call(this, config);
        this.on('frameload', this.injectForm, this, {single:true});
    },
    injectForm: function(win){
        var doc = this.getDocument(),
            form = formTpl.apply({
                data: Ext.encode(this.data),
                target: this.target
            });
        this.getBody().innerHTML = form;
        this.getFrame().setAttribute("allowtransparency", "true");   
        doc.commandform.submit();
        this.parentWindow.setSize(this.parentWindow.getSize());
    }
});



/**********************************************************************
 *
 *  Command Window
 *
 */
Ext.define("Zenoss.CommandWindow", {
    alias:['widget.commandwindow'],
    extend:"Ext.Window",
    constructor: function(config) {
        this.cpanel = Ext.id();
        this.commandData = config.data ||
            { uids: config.uids, command: config.command };
        this.target = config.target;
        config = Ext.applyIf(config || {}, {
            layout: 'fit',
            title: config.command || config.title,
            cls: 'streaming-window',
            constrain: true,
            closable:true,
            plain: true,
            items: {
                id: this.cpanel,
                xtype: config.panel || 'commandpanel', //default to commandpanel
                data: this.commandData,
                target: config.target,
                autoLoad: config.autoLoad,
                parentWindow: this
            },
            fbar: {
                buttonAlign: 'left',
                id:'window_footer_toolbar',
                items: {
                    xtype: 'checkbox',
                    checked: true,
                    boxLabel: '<span style="color:white">Autoscroll</span>',
                    handler: Ext.bind(function(c){
                        if (c.checked) {
                            this.startScrolling();
                        } else {
                            this.stopScrolling();
                        }
                    }, this)
                }
            }
        });
        Zenoss.CommandWindow.superclass.constructor.call(this, config);
        this.task = new Ext.util.DelayedTask(this.scrollToBottom, this);
        this.on('afterrender', this.startScrolling, this);
        this.on('afterrender', this.resizeOnRender, this);
        this.on('afterlayout', function(){this.center();}, this, {single:true});
        this.on('close', this.stopScrolling, this);
            this.on('close', function() {
                if(Ext.isDefined(config.redirectTarget)){
                        this.closeAndRedirect();
                }
            });
    },
    resizeOnRender: function() {
        var vsize = Ext.getBody().getViewSize();
        this.setSize({width:vsize.width*0.95, height:vsize.height*0.95});
    },
    getCommandPanel: function() {
        if (Ext.isString(this.cpanel)) {
            this.cpanel = Ext.getCmp(this.cpanel);
        }
        return this.cpanel;
    },
    startScrolling: function() {
        this.task.delay(250);
    },
    stopScrolling: function() {
        this.task.cancel();
    },
    scrollToBottom: function() {
        try {
            var win = this.getCommandPanel().getWindow(),
                body = this.getCommandPanel().getBody();
            Zenoss.env.BODY = body;
            win.scrollBy(0, body.scrollHeight);
        } catch(e) {
            Ext.emptyFn();
        }

        if (Ext.get('window_footer_toolbar')) {
            Ext.get('window_footer_toolbar').focus();
            this.task.delay(250);
        }
    },
    closeAndRedirect: function() {
        window.top.location = this.redirectTarget;
        this.destroy();
    }
});



})();
(function(){
    var REMOTE = Zenoss.remote.DeviceRouter;

    var resetCombo = function(combo, manufacturer) {
        combo.clearValue();
        combo.store.setBaseParam('manufacturer', manufacturer);
        delete combo.lastQuery;
        //combo.doQuery(combo.allQuery, true);
    };

    var clickToEditConfig = function(obj) {
        return {
            constructor: function(config) {
                var title = _t('Click to edit this field');
                var editLink = '<a href="javascript:" title="' + title  + '" class="manu-edit-link">'+
                                _t('Edit') + '</a>',
                    hasPermission = true;
                // do not render the edit link if we don't have permission
                if (config.permission && !Zenoss.Security.hasPermission(config.permission)) {
                    hasPermission = false;
                    editLink = '';
                }
                config.fieldLabel += editLink;
                config.listeners = Ext.apply(config.listeners||{}, {
                    render: function(p) {
                        if (hasPermission) {
                            p.editlink = p.labelEl.select('a.manu-edit-link');
                            p.editlink.on('click', function(){
                                p.fireEvent('labelclick', p);
                            }, p);
                        }
                    }
                });

                obj.superclass.constructor.call(this, config);
                this.addEvents('labelclick');
            },
            /**
             * Sets the display value for the "edit link" fields.
             * Special case if there it is a manufacturer to edit the manufacturer object in place.
             * TODO: Refactor this method so separate types for the different link fields as this method has gotten too complicated.
             **/
            setValue: function(value) {
                var origValue = value;
                if (Ext.isEmpty(value)) {
                    value = _t('None');
                } else {
                    if (Ext.isArray(value)){
                        var items = [];
                        Ext.each(value, function(v){
                            items.push(Zenoss.render.link(v));
                        });
                        value = items.join('<br/>');
                    } else {
                        if(value.uid && value.uid.match(/Manufacturers/)){
                            var grid = null;
                            var data = value;
                            if(value.uid.match(/products/)){
                                Zenoss.manufacturers.launchProductDialog = function(){
                                    Zenoss.manufacturers.productsDialog(grid, data);
                                };
                                value =  '<a title="Edit this Product details in place" href="javascript:void(0);" onClick="Zenoss.manufacturers.launchProductDialog()">'+value.name+'</a>';
                            }else{
                                value = '<a title="Go to the grid for this Manufacturer" href="/zport/dmd/manufacturers#manufacturers_tree:.zport.dmd.Manufacturers.'+value.name+'" >'+value.name+'</a>';
                            }
                        } else if (Ext.isObject(value)) {
                            value = Zenoss.render.link(value);
                        } else{
                            // if it is a UID then render as a link to that item
                            if (Ext.isString(value) && value.startswith('/zport/dmd/')) {
                                value = Zenoss.render.link(value);
                            }
                        }
                    }
                }
                this.setRawValue(value);
                this.rawValue = origValue;
            }


        };
    };

    Zenoss.ClickToEditField = Ext.extend(Zenoss.form.LinkField, {});
    Zenoss.ClickToEditField = Ext.extend(Zenoss.form.LinkField, clickToEditConfig(Zenoss.ClickToEditField));
    Ext.reg('clicktoedit', "Zenoss.ClickToEditField");

    Zenoss.ClickToEditNoLink = Ext.extend(Ext.form.DisplayField, {});
    Zenoss.ClickToEditNoLink = Ext.extend(Ext.form.DisplayField,
                                   clickToEditConfig(Zenoss.ClickToEditNoLink));
    Ext.reg('clicktoeditnolink', "Zenoss.ClickToEditNoLink");

    // waits till user clicks to fetch and display
    // link info
    Ext.define("Zenoss.form.ClickToDisplayField", {
        extend: "Zenoss.form.LinkField",
        alias: ['widget.clicktodisplay'],
        constructor: function(config){
            this.getContext = config.getContext;
            this.callParent([config]);
        },
        initComponent: function() {
            this.callParent(arguments);

            // set up the "Show Links" link
            var a = document.createElement("a");
            a.href = "#";
            a.innerText = "Show Links";
            this.displayLinkEl = new Ext.Element(a);
            this.displayLinkEl.on("click", this.fetchLinks.bind(this));

            // set up loading spinner
            var spinner = document.createElement("div");
            spinner.innerHTML = "<img style='height: 20px;' src='img/spinning_wheel_throbber.gif'>";
            this.spinnerEl = new Ext.Element(spinner);

            this.on("afterrender", function(){
                this.bodyEl.appendChild(this.displayLinkEl);
                this.bodyEl.appendChild(this.spinnerEl);
                this.hideLinkList();
                this.hideSpinner();
            });

            // get a reference to getInfo, and make sure
            // it doesnt get batched
            this.getInfo = Zenoss.util.isolatedRequest(REMOTE.getInfo);
        },

        // show and hide the "Show Links" link
        showFetchLink: function(){
            this.displayLinkEl.show();
        },
        hideFetchLink: function(){
            this.displayLinkEl.setVisibilityMode(Ext.Element.DISPLAY);
            this.displayLinkEl.hide();
        },

        showSpinner: function(){
            this.spinnerEl.show();
        },
        hideSpinner: function(){
            this.spinnerEl.setVisibilityMode(Ext.Element.DISPLAY);
            this.spinnerEl.hide();
        },

        // show and hide the list of links
        showLinkList: function(){
            this.inputEl.show();
        },
        hideLinkList: function(){
            // make inputEl hide using display
            // rather than visibility
            this.inputEl.setVisibilityMode(Ext.Element.DISPLAY);
            this.inputEl.hide();
        },

        // fetch the list of links
        fetchLinks: function(){
            var contextUid = this.getContext();
            if(!contextUid){
                Zenoss.message.error("Unable to fetch links: cannot determine context uid");
                return;
            }

            this.showSpinner();
            this.hideFetchLink();

            this.getInfo({keys: ["links"], uid: contextUid}, function(results){
                this.hideSpinner();

                if(!results.success){
                    Zenoss.message.error("Unable to fetch links: "+ results.msg);
                    this.showFetchLink();
                    return;
                }

                var links = results.data ? results.data.links : "";
                this.setValue(links);
                this.showLinkList();
            }.bind(this));
        }
    });

    function editManuInfo (vals, uid) {
        function name(uid) {
            if (!uid){
                return 'Unknown';
            }
            if (!Ext.isString(uid)) {
                uid = uid.uid;
            }
            return uid.split('/').reverse()[0];
        }

        var FIELDWIDTH = 300;

        var hwManufacturers = {
            xtype: 'manufacturercombo',
            width: FIELDWIDTH,
            name: 'hwManufacturer',
            id: 'hwmanufacturercombo',
            fieldLabel: _t('Hardware Manufacturer'),
            listConfig: {
                resizable: true, resizeHandles: 'e'
            },
            listeners: {'select': function(combo, record){
                record = record[0];
                var productCombo = Ext.getCmp('hwproductcombo');
                resetCombo(productCombo, record.data.name);
            }}
        };

        var hwProduct = {
            xtype: 'productcombo',
            prodType: 'HW',
            width: FIELDWIDTH,
            listConfig: {
                resizable: true, resizeHandles: 'e'
            },
            name: 'hwProductName',
            fieldLabel: _t('Hardware Model'),
            id: 'hwproductcombo',
            manufacturer: name(vals.hwManufacturer)
        };

        var osManufacturers = {
            xtype: 'manufacturercombo',
            width: FIELDWIDTH,
            name: 'osManufacturer',
            id: 'osmanufacturercombo',
            fieldLabel: _t('OS Manufacturer'),
            listConfig: {
                resizable: true, resizeHandles: 'e'
            },
            listeners: {'select': function(combo, record){
                record = record[0];
                var productCombo = Ext.getCmp('osproductcombo');
                resetCombo(productCombo, record.data.name);
            }}
        };

        var osProduct = {
            xtype: 'productcombo',
            prodType: 'OS',
            width: FIELDWIDTH,
            listConfig: {
                resizable: true, resizeHandles: 'e'
            },
            name: 'osProductName',
            id: 'osproductcombo',
            fieldLabel: _t('OS Model'),
            manufacturer: name(vals.osManufacturer)
        };

        var win = new Zenoss.FormDialog({
            autoHeight: true,
            width: 390,
            title: _t('Edit Manufacturer Info'),
            items: [{
                xtype: 'container',
                layout: 'anchor',
                autoHeight: true,
                style: 'padding-bottom:5px;margin-bottom:5px;border-bottom:1px solid #555;',
                items: [hwManufacturers, hwProduct]
            },{
                xtype: 'container',
                layout: 'anchor',
                autoHeight: true,
                items: [osManufacturers, osProduct]
            }],
            buttons: [{
                text: _t('Save'),
                ref: '../savebtn',
                xtype: 'DialogButton',
                id: 'win-save-button',
                disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                handler: function(btn){
                    var form = btn.refOwner.editForm.getForm(),
                        vals = form.getValues();
                    Ext.apply(vals, {uid:uid});
                    REMOTE.setProductInfo(vals, function() {
                        Ext.getCmp('device_overview').load();
                        win.destroy();
                    });
                }
            },{
                text: _t('Cancel'),
                xtype: 'DialogButton',
                id: 'win-cancel-button',
                handler: function(){
                    win.destroy();
                }
            }]
        });
        win.show();
        win.doLayout();
        Ext.getCmp('hwmanufacturercombo').getStore().addListener('load', function fn(){
            var manufacturerName = name(vals.hwManufacturer);
            Ext.getCmp('hwmanufacturercombo').setValue(manufacturerName);
        });
        Ext.getCmp('hwproductcombo').getStore().addListener('load', function fn(){
            var modelName = name(vals.hwModel);
            Ext.getCmp('hwproductcombo').setValue(modelName);
        });
        Ext.getCmp('osmanufacturercombo').getStore().addListener('load', function fn(){
            var manufacturerName = name(vals.osManufacturer);
            Ext.getCmp('osmanufacturercombo').setValue(manufacturerName);
        });
        Ext.getCmp('osproductcombo').getStore().addListener('load', function fn(){
            var modelName = name(vals.osModel);
            Ext.getCmp('osproductcombo').setValue(modelName);
        });
    }


    /**
     * Displays a dialog allowing the user to edit those zproperties
     * which define the credentials for a server.
     **/
    var editConnectionInfo = function(uid) {
        REMOTE.getConnectionInfo({
            uid: uid
        }, function (response) {
            if (!response.success) {
                return;
            }
            // build the form elements
            var items = [], data = response.data, item, config, dialog, handler,i, id;
            for (i=0;i<data.length;i++) {
                id = data[i].id;
                item = Zenoss.zproperties.createZPropertyField(data[i]);
                item.name = id;

                // make sure we always display the correct value
                item.value = item.value || data[i].valueAsString;
                items.push(item);
            }

            // this is executed after we save the zproperties
            handler = function(values, remodel) {
                // need to save the zproperty values
                Zenoss.remote.PropertiesRouter.setZenProperty({
                    uid: uid,
                    zProperty: values
                }, function(response){
                    if (response.success){
                        // refresh the overview page
                        Ext.getCmp('device_overview').setContext(uid);
                        if (remodel) {
                            var win = new Zenoss.CommandWindow({
                                uids: [uid],
                                target: 'run_model',
                                listeners: {
                                    close: function(){
                                        Ext.defer(function() {
                                            window.top.location.reload();
                                        }, 1000);
                                    }
                                },
                                title: _t('Model Device')
                            });
                            win.show();
                        }
                    }
                });
            };

            // display the dialog
            config = {
                submitHandler: handler,
                minHeight: 250,
                autoHeight: true,
                width: 500,
                title: _t('Edit Connection Information'),
                listeners: {
                    show: function() {
                        dialog.getForm().query("field[ref='editConfig']")[0].focus(true, 500);
                    }
                },
                items: items,
                // explicitly do not allow enter to submit the dialog
                keys: {

                },
                buttons:[{
                    xtype: 'DialogButton',
                    text: _t('Submit'),
                    disabled: true,
                    type: 'submit',
                    ref: 'buttonSubmit'
                }, {
                    xtype: 'DialogButton',
                    ui:'dialog-dark',
                    ref: 'btnRemodel',
                    text: _t('Submit and Remodel'),
                    handler: function(btn) {
                        // traverse back up to the dialog window to get the form
                        var values = btn.up('window').getForm().getForm().getValues();
                        handler(values, true);
                    }
                },{
                    xtype: 'DialogButton',
                    ref: 'buttonCancel',
                    text: _t('Cancel')
                }]
            };
            dialog = new Zenoss.SmartFormDialog(config);
            dialog.show();
        });
    };

    var editCollector = function(values, uid) {
        var win = new Zenoss.FormDialog({
            autoHeight: true,
            width: 300,

            title: _t('Set Collector'),
            items: [{
                xtype: 'combo',
                name: 'collector',
                listConfig: {
                    resizable: true, resizeHandles: 'e'
                },
                fieldLabel: _t('Select a collector'),
                queryMode: 'local',
                store: new Ext.data.ArrayStore({
                    data: Zenoss.env.COLLECTORS,
                    fields: ['name']
                }),
                valueField: 'name',
                displayField: 'name',
                value: values.collector,
                forceSelection: true,
                editable: false,
                autoSelect: true,
                triggerAction: 'all'
            }],
            buttons: [{
                text: _t('Save'),
                ref: '../savebtn',
                xtype: 'DialogButton',
                id: 'editcollector-save-button',
                disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                handler: function(btn) {
                    var vals = btn.refOwner.editForm.getForm().getValues();
                    var submitVals = {
                        uids: [uid],
                        asynchronous: Zenoss.settings.deviceMoveIsAsync([uid]),
                        collector: vals.collector,
                        hashcheck: ''
                    };
                    Zenoss.remote.DeviceRouter.setCollector(submitVals, function() {
                        Ext.getCmp('device_overview').load();
                    });
                    win.destroy();
                }
            }, {
                text: _t('Cancel'),
                xtype: 'DialogButton',
                id: 'editcollector-cancel-button',
                handler: function() {
                    win.destroy();
                }
            }]
        });
        win.show();
        win.doLayout();
    };

    var editGroups = function(currentGroups, uid, config) {
        var win = new Zenoss.FormDialog({
            width: 500,
            autoHeight: true,
            title: config.title,
            items: [{
                xtype: 'panel',
                html: config.instructions
            }, {
                xtype: 'tbspacer',
                height: 5
            }, {
                xtype: 'panel',
                layout: 'vbox',
                width: '100%',
                items: [{
                    xtype: 'combo',
                    ref: '../../selectgroup',
                    name: 'group',
                    width: 250,
                    store: new Ext.data.DirectStore({
                        directFn: config.getGroupFn,
                        root: config.getGroupRoot,
                        fields: ['name']
                    }),
                    valueField: 'name',
                    displayField: 'name',
                    id: 'editgroups-combo',
                    forceSelection: true,
                    listConfig: {
                        resizable: true, resizeHandles: 'e'
                    },
                    editable: false,
                    autoSelect: true,
                    triggerAction: 'all',
                    flex: 4
                }, {
                    xtype: 'button',
                    ref: '../../addgroupbutton',
                    ui: 'dialog-dark',
                    text: _t('Add'),
                    id: 'addgroup-button',
                    handler: function(btn) {
                        var selectedGroup = btn.refOwner.selectgroup.getValue();
                        if (selectedGroup) {
                            btn.refOwner.grouplist.addGroup(selectedGroup);
                        }
                    },
                    flex: 1
                }]
            }, {
                xtype: 'panel',
                ref: '../grouplist',
                addGroup: function(group, displayOnly) {
                    if (group in this.groups) {
                        if (this.groups[group] === 'del') {
                            this.groups[group] = '';
                        } else {
                            return;
                        }
                    }
                    else {
                        this.groups[group] = displayOnly ? '' : 'add';
                    }

                    var grouplist = this;
                    var oldHeight = this.getHeight();
                    this.add({xtype: 'tbspacer', height: 5});
                    this.add({
                        xtype: 'panel',
                        layout: 'hbox',
                        width: '100%',
                        layoutConfig: {
                            align:'middle'
                        },
                        items: [{
                            xtype: 'panel',
                            html: group
                        }, {
                            xtype: 'tbspacer',
                            flex: 1
                        }, {
                            xtype: 'button',
                            ui: 'dialog-dark',
                            text: _t('Remove'),
                            ref: 'delbutton',
                            group: group,
                            handler: function(btn) {
                                grouplist.delGroup(group, btn.refOwner);
                            }
                        }]
                    });
                    this.bubble(function() {this.doLayout();});

                    if (displayOnly) {
                        return;
                    }
                    win.setHeight(win.getHeight() + this.getHeight() - oldHeight);
                },
                delGroup: function(group, panel) {
                    if (this.groups[group] === 'add') {
                        delete this.groups[group];
                    } else {
                        this.groups[group] = 'del';
                    }
                    var oldHeight = this.getHeight();
                    panel.destroy();
                    this.bubble(function() {this.doLayout();});
                    win.setHeight(win.getHeight() + this.getHeight() - oldHeight);
                },
                groups: {},
                listeners: {
                    render: function(thisPanel) {
                        Ext.each(currentGroups, function(group){
                            thisPanel.addGroup(group.uid.slice(config.dmdPrefix.length), true);
                        });
                    }
                }
            }],
            buttons: [{
                text: _t('Save'),
                ref: '../savebtn',
                xtype: 'DialogButton',
                id: 'editgroups-save-button',
                disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                handler: function(btn) {
                    Ext.iterate(btn.refOwner.grouplist.groups, function(group, op) {
                        var submitVals = {
                            uids: [uid],
                            hashcheck: ''
                        };

                        if (op === 'del') {
                            submitVals.uid = config.dmdPrefix + group;
                            Zenoss.remote.DeviceRouter.removeDevices(submitVals, function() {
                                Ext.getCmp('device_overview').load();
                            });
                        }
                        if (op === 'add') {
                            submitVals.target = config.dmdPrefix + group;
                            submitVals.asynchronous = Zenoss.settings.deviceMoveIsAsync(submitVals.uids);
                            Zenoss.remote.DeviceRouter.moveDevices(submitVals, function() {
                                Ext.getCmp('device_overview').load();
                            });
                        }
                    });
                    win.destroy();
                }
            }, {
                text: _t('Cancel'),
                xtype: 'DialogButton',
                id: 'editgroups-cancel-button',
                handler: function() {
                    win.destroy();
                }
            }]
        });
        win.show();
        win.doLayout();
        win.setHeight(win.getHeight() + win.grouplist.getHeight());
    };

    var editLocation = function(values, uid) {
        var win = new Zenoss.FormDialog({
            autoHeight: true,
            width: 500,
            title: _t('Set Location'),
            items: [{
                xtype: 'combo',
                name: 'location',
                fieldLabel: _t('Select a location'),
                store: new Ext.data.DirectStore({
                    autoload: true,
                    directFn: Zenoss.remote.DeviceRouter.getLocations,
                    root: 'locations',
                    fields: ['name']
                }),
                valueField: 'name',
                displayField: 'name',
                id: 'editlocation-name-combo',
                value: values.location ? values.location.uid.slice(20) : '',
                listConfig: {
                    resizable: true, resizeHandles: 'e'
                },
                width: 250,
                triggerAction: 'all'
            }],
            buttons: [{
                text: _t('Save'),
                ref: '../savebtn',
                xtype: 'DialogButton',
                id: 'editlocation-save-button',
                disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                handler: function(btn) {
                    var vals = btn.refOwner.editForm.getForm().getValues();
                    if (vals.location) {
                        var submitVals = {
                            uids: [uid],
                            asynchronous: Zenoss.settings.deviceMoveIsAsync([uid]),
                            target: '/zport/dmd/Locations' + vals.location,
                            hashcheck: ''
                        };
                        Zenoss.remote.DeviceRouter.moveDevices(submitVals, function(data) {
                            if (data.success) {
                                Ext.getCmp('device_overview').load();
                            }
                        });
                    }
                    win.destroy();
                }
            }, {
                text: _t('Cancel'),
                xtype: 'DialogButton',
                id: 'editlocation-cancel-button',
                handler: function() {
                    win.destroy();
                }
            }]
        });
        win.show();
        win.doLayout();
    };


    function isField(c) {
        return !!c.setValue && !!c.getValue && !!c.markInvalid && !!c.clearInvalid;
    }

    Ext.define("Zenoss.DeviceOverviewForm", {
        alias:['widget.devformpanel'],
        extend:"Ext.form.FormPanel",
        fieldDefaults: {
            labelAlign: 'top'
        },
        paramsAsHash: true,
        frame: false,
        defaults: {
            anchor: '95%',
            labelStyle: 'font-size: 13px; color: #5a5a5a'
        },
        buttonAlign: 'left',
        buttons: [{
            text: _t('Save'),
            xtype:'button',
            ref: '../savebtn',
            disabled: Zenoss.Security.doesNotHavePermission('Manage Device') &&
                      Zenoss.Security.doesNotHavePermission('Change Device Production State'),
            hidden: true,
            handler: function(){
                this.refOwner.getForm().submit();
            }
        },{
            text: _t('Cancel'),
            xtype: 'button',
            ref: '../cancelbtn',
            disabled: true,
            hidden: true,
            handler: function() {
                this.refOwner.getForm().reset();
            }
        }],
        cls: 'device-overview-form-wrapper',
        bodyCls: 'device-overview-form',
        style:{'background-color':'#fafafa'},
        listeners: {
            add: function(me, field){
                if (isField(field)) {
                    this.onFieldAdd.call(this, field);
                }
            },
            validitychange: function(me, isValid) {
                this.setButtonsDisabled(!isValid);
            }
        },
        constructor: function(config) {
            config = Ext.applyIf(config || {}, {
                trackResetOnLoad: true
            });
            config.listeners = Ext.applyIf(config.listeners||{}, this.listeners);
            Zenoss.DeviceOverviewForm.superclass.constructor.call(this, config);
        },
        showButtons: function() {
            if (!this.rendered) {
                this.on('render', this.showButtons, this);
            } else {
                Ext.getCmp(this.savebtn.id).addCls("savebtn-button"+this.id);
                Ext.getCmp(this.cancelbtn.id).addCls("cancelbtn-button"+this.id);
                this.savebtn.show();
                this.cancelbtn.show();
            }
        },
        doButtons: function() {
            this.setButtonsDisabled(!this.form.isDirty());
        },
        setButtonsDisabled: function(b) {
            if (Zenoss.Security.hasPermission('Manage Device')) {
                this.savebtn.setDisabled(b);
            }
            this.cancelbtn.setDisabled(b);
        },
        onFieldAdd: function(field) {
            if (!field.isXType('displayfield')) {
                this.showButtons();
                this.mon(field, 'dirtychange', this.doButtons, this);
            }
        },
        hideFooter: function() {
            this.footer.hide();
        },
        showFooter: function() {
            this.footer.show();
        },
        addField: function(field) {
            // TODO: remove this after we upgrade extjs
            // there is a bug in extjs 4.1.3 where when we try
            // to add a field before we are rendered there is an exception
            // registering listners
            if (this.rendered) {
                this.add(field);
            } else {
                this.on('afterrender', function(){
                    this.add(field);
                }, this, {single:true});
            }
        },
        addFieldAfter: function(field, afterFieldName) {
            var position = this._indexOfFieldName(afterFieldName) +1;
            this.insert(position, field);
        },
        _indexOfFieldName: function(name) {
            var idx = -1, items = this.getItems(), i;
            for ( i = 0; i < items.length; i++ ){
                if (items[i].name === name){
                    idx = i;
                    break;
                }
            }
        return idx;
        },
        replaceField: function(name, field) {
            this.removeField(name);
            this.addField(field);
        },
        removeField: function(name) {
            var field = this.getField(name);

            if (field) {
                this.remove(field);
            }
        },
        getField: function(name) {
            return this.getItems()[this._indexOfFieldName(name)];
        },
        getItems: function(){
            return this.items.items;
        }
    });



    Ext.define("Zenoss.DeviceOverviewPanel", {
        alias:['widget.deviceoverview'],
        extend:"Ext.Panel",
        constructor: function(config) {
            config = Ext.applyIf(config||{}, {
                autoScroll: true,
                bodyCls: 'device-overview-panel',
                padding: '10',
                frame: false,
                forms: [],
                listeners: {
                    add: function(me, container) {
                        if (!container.items) {
                            return;
                        }
                        Ext.each(container.items.items, function(item) {
                            if (item.isXType('form')) {
                                var f = item.getForm();
                                f.api = this.api;
                                f.baseParams = this.baseParams;
                                this.forms.push(item);
                            }
                        }, this);
                    }
                },
                items: [{
                    layout: {
                        type: 'hbox'
                    },
                    id: 'deviceoverviewpanel_main',
                    defaults: {
                        bodyStyle: 'background-color:#fafafa;',
                        minHeight: 350,
                        margin:'0 10 10 0',
                        flex: 1
                    },
                    defaultType: 'devformpanel',
                    items: [{
                        id:'deviceoverviewpanel_summary',
                        defaultType: 'displayfield',
                        frame:false,
                        items: [{
                            fieldLabel: _t('Device ID'),
                            id: 'device-id-label',
                            name: 'device'
                        },{
                            xtype: 'clicktoeditnolink',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editConnectionInfo(this.contextUid);
                                },
                                scope: this
                            },
                            fieldLabel: _t('Connection Information'),
                            name: 'deviceConnectionInfo',
                            id: 'device-connection-editlink',
                            hidden: Zenoss.Security.doesNotHavePermission('Manage Device')
                        },{
                            fieldLabel: _t('Uptime'),
                            id: 'uptime-label',
                            name: 'uptime'
                        },{
                            fieldLabel: _t('First Seen'),
                            id: 'first-seen-label',
                            name: 'firstSeen',
                            xtype: "datedisplayfield"
                        },{
                            fieldLabel: _t('Last Change'),
                            id: 'last-change-label',
                            name: 'lastChanged',
                            xtype: "datedisplayfield"
                        },{
                            fieldLabel: _t('Model Time'),
                            id: 'model-time-label',
                            name: 'lastCollected',
                            xtype: "datedisplayfield"
                        },{
                            fieldLabel: _t('Locking'),
                            id: 'locking-label',
                            name: 'locking'
                        },{
                            xtype: 'displayfield',
                            id: 'memory-displayfield',
                            name: 'memory',
                            fieldLabel: _t('Memory/Swap')
                        }, {
                            xtype: 'displayfield',
                            id: 'ssh-link',
                            name: 'sshLink',
                            hidden: Zenoss.Security.doesNotHavePermission('Manage Device'),
                            setValue: function(val) {
                                if (val) {
                                    val = Ext.String.format("<a href='{0}'>{1}</a>", val, _t('Connect to this device'));
                                    Ext.form.field.Display.prototype.setValue.apply(this, [val]);
                                }
                            }
                        }]
                    },{
                        id:'deviceoverviewpanel_idsummary',
                        defaultType: 'displayfield',
                        frame:false,

                        listeners: {
                            actioncomplete: function(form, action) {
                                if (action.type==='directsubmit') {
                                    var bar = Ext.getCmp('devdetailbar');
                                    if (bar) {
                                        bar.refresh();
                                    }
                                }
                            }
                        },
                        items: [{
                            xtype: 'textfield',
                            name: 'name',
                            fieldLabel: _t('Device Name'),
                            id: 'device-name-textfield',
                            allowBlank: false
                        },{
                            xtype: 'ProductionStateCombo',
                            fieldLabel: _t('Production State'),
                            id: 'production-state-combo',
                            name: 'productionState'
                        },{
                            xtype: 'PriorityCombo',
                            fieldLabel: _t('Priority'),
                            id: 'priority-combo',
                            name: 'priority'
                        },{
                            fieldLabel: _t('Tag'),
                            name: 'tagNumber',
                            id: 'tagnumber-textfield',
                            xtype: 'textfield'
                        },{
                            fieldLabel: _t('Serial Number'),
                            name: 'serialNumber',
                            id: 'serialnumber-textfield',
                            xtype: 'textfield'
                        },{
                            fieldLabel: _t('Rack Slot'),
                            name: 'rackSlot',
                            id: 'rackslot-textfield',
                            xtype: 'textfield'
                        }]
                    },{
                        id:'deviceoverviewpanel_descriptionsummary',
                        defaultType: 'textfield',
                        frame:false,

                        items: [{
                            xtype: 'clicktoeditnolink',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editCollector(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            fieldLabel: _t('Collector'),
                            name: 'collector',
                            id: 'collector-editnolink'
                        },{
                            xtype: 'clicktoedit',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'hwManufacturer',
                            id: 'hwmanufacturer-editlink',
                            fieldLabel: _t('Hardware Manufacturer')
                        },{
                            xtype: 'clicktoedit',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'hwModel',
                            id: 'hwmodel-editlink',
                            fieldLabel: _t('Hardware Model')
                        },{
                            xtype: 'clicktoedit',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'osManufacturer',
                            id: 'osmanufacturer-editlink',
                            fieldLabel: _t('OS Manufacturer')
                        },{
                            xtype: 'clicktoedit',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'osModel',
                            id: 'osmodel-editlink',
                            fieldLabel: _t('OS Model')
                        }]
                    }]
                },{
                    id:'deviceoverviewpanel_customsummary',
                    defaultType: 'devformpanel',
                    frame:false,

                    layout: 'hbox',
                    defaults: {
                        bodyStyle: 'background-color:#fafafa;',
                        minHeight: 400,
                        margin:'0 10 10 0'
                    },
                    layoutConfig: {
                        align: 'stretchmax'
                    },
                    items: [{
                        defaultType: 'displayfield',
                        flex: 2,
                        minHeight: 400,
                        frame:false,
                        id: 'deviceoverviewpanel_systemsummary',
                        items: [{
                            xtype: 'clicktoedit',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editGroups(this.getValues().systems, this.contextUid, {
                                        title: _t('Set Systems'),
                                        instructions: _t('Add/Remove systems'),
                                        getGroupFn: Zenoss.remote.DeviceRouter.getSystems,
                                        getGroupRoot: 'systems',
                                        dmdPrefix: '/zport/dmd/Systems'
                                    });
                                },
                                scope: this
                            },
                            fieldLabel: _t('Systems'),
                            name: 'systems',
                            id: 'systems-editlink'
                        },{
                            xtype: 'clicktoedit',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editGroups(this.getValues().groups, this.contextUid, {
                                        title: _t('Set Groups'),
                                        instructions: _t('Add/Remove groups'),
                                        getGroupFn: Zenoss.remote.DeviceRouter.getGroups,
                                        getGroupRoot: 'groups',
                                        dmdPrefix: '/zport/dmd/Groups'
                                    });
                                },
                                scope: this
                            },
                            fieldLabel: _t('Groups'),
                            name: 'groups',
                            id: 'groups-editlink'
                        },{
                            xtype: 'clicktoedit',
                            permission: 'Manage Device',
                            listeners: {
                                labelclick: function(){
                                    editLocation(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            fieldLabel: _t('Location'),
                            name: 'location',
                            id: 'location-editlink'
                        },{
                            xtype: 'clicktodisplay',
                            fieldLabel: _t('Links'),
                            name: 'links',
                            id: 'links-label',
                            getContext: function(){ return this.contextUid; }.bind(this)
                        },{
                            xtype: 'textarea',
                            grow: true,
                            fieldLabel: _t('Comments'),
                            name: 'comments',
                            id: 'comments-textarea'
                        }]
                    },{
                        id:'deviceoverviewpanel_snmpsummary',
                        defaultType: 'displayfield',
                        frame:false,

                        flex: 1,
                        bodyStyle: 'background-color:#fafafa;',
                        minHeight: 400,
                        items: [{
                            fieldLabel: _t('SNMP SysName'),
                            name: 'snmpSysName',
                            id: 'snmpsysname-label'
                        },{
                            fieldLabel: _t('SNMP Location'),
                            name: 'snmpLocation',
                            id: 'snmplocation-label'
                        },{
                            fieldLabel: _t('SNMP Contact'),
                            name: 'snmpContact',
                            id: 'snmpcontact-label'
                        },{
                            fieldLabel: _t('SNMP Description'),
                            autoWidth: true,
                            name: 'snmpDescr',
                            id: 'snmpdescr-label'
                        },{
                            fieldLabel: _t('SNMP Community'),
                            name: 'snmpCommunity',
                            id: 'snmpcommunity-label',
                            hidden: Zenoss.Security.doesNotHavePermission('Manage Device')
                        },{
                            fieldLabel: _t('SNMP Version'),
                            name: 'snmpVersion',
                            id: 'snmpversion-label'
                        }]
                    }]
                }]
            });
            Zenoss.DeviceOverviewPanel.superclass.constructor.call(this, config);
        },
        api: {
            load: Zenoss.util.isolatedRequest(REMOTE.getInfo),
            submit: function(form, success, scope) {
                var o = {},
                vals = scope.form.getValues(false, true);
                Ext.apply(o, vals, scope.form.baseParams);
                REMOTE.setInfo(o, function(){
                    this.form.clearInvalid();
                    this.form.setValues(vals);
                    this.form.afterAction(this, true);
                    this.form.reset();
                }, scope);
            }
        },
        baseParams: {},
        setContext: function(uid) {
            this.contextUid = uid;
            this.baseParams.uid = uid;
            // if we havne't rendered yet wait until we have rendered
            if (!this.getEl()) {
                this.on('afterrender', this.load, this, {single: true});
            } else {
                this.load();
            }

        },
        getFieldNames: function() {
            var keys = [], key;
            Ext.each(this.forms, function(f){
                Ext.each(f.getForm().getFields().items, function(field) {
                    key = field.name;
                    if (Ext.Array.indexOf(keys, key)===-1 && (key !== 'links') && (key !== 'uptime')) {
                        keys.push(key);
                    }
                });
            });
            return keys;
        },
        load: function() {
            var o = Ext.apply({keys:this.getFieldNames()}, this.baseParams);
            var callback = function(result) {
                this.loadedData = result.data;
                var D = result.data;
                if (D.locking) {
                    D.locking = Zenoss.render.locking(D.locking);
                }
                if (D.memory) {
                    D.memory = D.memory.ram + '/' + D.memory.swap;
                } else {
                    D.memory = 'Unknown/Unknown';
                }
                if (D.snmpCommunity) {
                   D.snmpCommunity = Ext.String.htmlEncode(D.snmpCommunity);
                }
                D.comments = Ext.htmlDecode(D.comments);
                D.tagNumber = Ext.htmlDecode(D.tagNumber);
                D.serialNumber = Ext.htmlDecode(D.serialNumber);
                D.rackSlot = Ext.htmlDecode(D.rackSlot);
                D.name = Ext.htmlDecode(D.name);
                if (D.deviceConnectionInfo === false) {
                    Ext.getCmp('device-connection-editlink').hide();
                } else {
                    Ext.getCmp('device-connection-editlink').show();
                }
                D.deviceConnectionInfo = Ext.String.htmlEncode(D.deviceConnectionInfo);
                this.setValues(D);

                // load uptime in a separate request since it
                // can be very expensive
                var opts = Ext.apply({keys:['uptime']}, this.baseParams);
                this.api.load(opts, function(results){
                    this.setValues(results.data);
                }, this);
            };


            if (Zenoss.env.infoObject) {
                Ext.bind(callback, this, [Zenoss.env.infoObject])();
                delete Zenoss.env.infoObject;
            } else {
                this.api.load(o, callback, this);
            }
        },
        getValues: function() {
            var o = {};
            Ext.each(this.forms, function(form){
                Ext.apply(o, form.getForm().getValues(false, false, true, true));
            }, this);
            return o;
        },
        setValues: function(d) {
            this.suspendLayouts();
            Ext.each(this.forms, function(form){
                form.getForm().setValues(d);
            });
            this.resumeLayouts(true);
        }
    });

})();
(function(){

Ext.define("Zenoss.DeviceDetailItem", {
    alias:['widget.devdetailitem'],
    extend:"Ext.Container",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            hideParent: true,
            cls: 'devdetailitem',
            items: [{
                cls: 'devdetail-textitem ' + (config.textCls||''),
                ref: 'textitem',
                xtype: 'tbtext',
                text: config.text || ''
            },{
                cls: 'devdetail-labelitem ' + (config.labelCls||''),
                ref: 'labelitem',
                xtype: 'tbtext',
                text: config.label || ''
            }]
        });
        Zenoss.DeviceDetailItem.superclass.constructor.call(this, config);
    },
    setText: function(t) {
        this.textitem.setText(t);
    },
    setLabel: function(t) {
        this.labelitem.setText(t);
    }
});



Ext.define("Zenoss.DeviceNameItem", {
    alias:['widget.devnameitem'],
    extend:"Ext.Container",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            defaults: {
                xtype: 'tbtext'
            },
            items: [{
                cls: 'devdetail-devname',
                ref: 'devname'
            },{
                ref: 'devclass',
                cls: 'devdetail-devclass'
            },{
                ref: 'ipAddress',
                cls: 'devdetail-ipaddress'
            }]
        });
        Zenoss.DeviceNameItem.superclass.constructor.call(this, config);
    }
});



Ext.define("Zenoss.DeviceDetailBar", {
    alias:['widget.devdetailbar'],
    extend:"Zenoss.LargeToolbar",
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            cls: 'largetoolbar devdetailbar',
            height: 55,
            directFn: Zenoss.remote.DeviceRouter.getInfo,
            defaultType: 'devdetailitem',
            items: [{
                ref: 'iconitem',
                cls: 'devdetail-icon'
            },{
                cls: 'evdetail-sep'
            },{
                xtype: 'devnameitem',
                height: 45,
                ref: 'deviditem'
            },'-',{
                xtype: "eventrainbow",
                width:202,
                ref: 'eventsitem',
                id: 'detailrainbow',
                label: _t('Events'),
                listeners: {
                    render: function(me) {
                        me.getEl().on('click', function(){
                            Ext.History.add('#deviceDetailNav:device_events');
                        });
                    }
                },
                count: 4
            },'-',{
                ref: 'statusitem',
                width:98,
                label: _t('Device Status'),
                id: 'statusitem'
            },'-',{
                ref: 'prodstateitem',
                width:120,
                label: _t('Production State'),
                id: 'prodstateitem'
            },'-',{
                ref: 'priorityitem',
                width:100,
                label: _t('Priority'),
                id: 'priorityitem'
            }]
        });
        this.contextKeys = [
            'ipAddressString',
            'deviceClass',
            'name',
            'icon',
            'status',
            'productionState',
            'priority'
        ];
        Zenoss.DeviceDetailBar.superclass.constructor.call(this, config);
    },
    contextCallbacks: [],
    addDeviceDetailBarItem: function(item, fn, added_keys) {
      this.add('-');
      this.add(item);
      this.on('contextchange', fn, this);
      for (var i = 0; i < added_keys.length; i++) {
        this.contextKeys.push(added_keys[i]);
      }
    },
    refresh: function() {
        this.setContext(this.contextUid);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.directFn({uid:uid, keys:this.contextKeys}, function(result){
            this.suspendLayouts();
            this.layout.targetEl.setWidth(this.getWidth());
            var ZR = Zenoss.render,
                data = result.data;
            Zenoss.env.icon = this.iconitem;
            this.iconitem.getEl().setStyle({
                'background-image' : 'url(' + data.icon + ')'
            });
            this.deviditem.devname.setText(Ext.htmlEncode(data.name));
            var ipAddress = data.ipAddressString;
            this.deviditem.ipAddress.setText(ipAddress);
            this.deviditem.devclass.setText(ZR.DeviceClass(data.deviceClass.uid));
            this.eventsitem.setContext(uid);
            this.statusitem.setText(
                ZR.pingStatusLarge(data.status));
            /* reformat the production state name so that it doesn't mess up the UI when too long */
            var pstatetxt = Zenoss.env.PRODUCTION_STATES_MAP[data.productionState];
            var pstate =  "<span title='"+pstatetxt+"'>";
                pstate += pstatetxt.length > 14 ? pstatetxt.substring(0, 12)+"..." : pstatetxt;
                pstate += "</span>";
            this.prodstateitem.setText(pstate);
            this.priorityitem.setText(Zenoss.env.PRIORITIES_MAP[data.priority]);

            // reset the positions based on text width and what not:
            this.iconitem.setPosition(0, 0);
            Ext.getCmp(Ext.query('.evdetail-sep')[0].id).setPosition(this.iconitem.getWidth()+this.iconitem.x, 0);
            var devitem_y = Ext.isEmpty(ipAddress) ? 7 : -2;
            this.deviditem.setPosition(this.iconitem.getWidth() +this.iconitem.x + 30, devitem_y);
            Ext.getCmp('detailrainbow').setPosition(this.deviditem.devname.getWidth() +this.deviditem.x + 30, 3);
            this.statusitem.setPosition(Ext.getCmp('detailrainbow').getWidth() +Ext.getCmp('detailrainbow').x + 30, -2);
            Ext.getCmp(Ext.query('.x-toolbar-separator')[0].id).setPosition(this.statusitem.getWidth()+this.statusitem.x+10, 14);
            this.prodstateitem.setPosition(this.statusitem.getWidth() +this.statusitem.x + 30, -2);
            Ext.getCmp(Ext.query('.x-toolbar-separator')[1].id).setPosition(this.prodstateitem.getWidth()+this.prodstateitem.x+10, 14);
            this.priorityitem.setPosition(this.prodstateitem.getWidth() +this.prodstateitem.x + 30, -2);

            this.fireEvent('contextchange', this, data);
            this.resumeLayouts();
        }, this);
    }
});



})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');

Ext.define("Zenoss.ConsoleBar", {
    alias:['widget.consolebar'],
    extend:"Zenoss.LargeToolbar",
    constructor: function(config) {
        var title = config.title || 'Title';
        delete config.title;
        config = Ext.apply(config||{}, {
            cls: 'largetoolbar consolebar',
            height: 35,
            collapseTitle: 'Instances',

            items: [{
                xtype: 'tbtext',
                text: title
            }].concat(config.leftItems||[]).concat([{
                xtype: 'tbfill'
            }])

        });

        this.callParent(arguments);

    },
    /**
     * Because in our UI we want the console bar visible when the
     * panel is collapse we add ourselves to the "dockedItems" when
     * our expand and collapse button is pressed.
     **/
    toggleDockedItemPosition: function() {
        this.up('panel').removeDocked(this, false);
        if (this.parentPanel.collapsed) {
            this.moveToSouthRegion();
        } else {
            this.moveToCenterRegion();
        }
    },
    _getRegion: function(region) {
        var reg = null;
        Ext.getCmp(this.centerPanel).items.each(function(item){
            if (item.region === region) {
                reg = item;
            }
        });
        return reg;
    },
    moveToSouthRegion: function() {
        this.dock = "top";
        this._getRegion("south").addDocked(this);
        this.parentPanel.setHeight(this.oldHeight);
        this.parentPanel.expand();
    },
    moveToCenterRegion: function() {
        this.dock = "bottom";
        this._getRegion("center").addDocked(this, 0);
        // remember the old height for when we expand
        this.oldHeight = Math.max(this.parentPanel.getEl().getComputedHeight(), this.parentPanel.initialConfig.height);
        this.parentPanel.collapse();
        // set height to 0 so the header bar disappears
        this.parentPanel.setHeight(0);
    }

});

})(); // End local namespace
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');



/**
 * @class Zenoss.AddToZenPackWindow
 * @extends Zenoss.dialog.BaseWindow
 */
Ext.define("Zenoss.AddToZenPackWindow", {
    alias:['widget.AddToZenPackWindow'],
    extend:"Zenoss.dialog.BaseWindow",
    constructor: function(config) {
        var me = this;
        config = Ext.applyIf(config || {}, {
            title: _t('Add to ZenPack'),
            layout: 'fit',
            modal: true,
            autoHeight: true,
            width: 310,
            plain: true,
            items: [{
                id: 'addzenpackform',
                xtype: 'form',
                listeners: {
                    validitychange: function(form, isValid) {
                        me.query('DialogButton')[0].setDisabled(!isValid);
                    }
                },
                defaults: {width: 250},
                autoHeight: true,
                frame: false,
                fieldDefaults: {
                    labelWidth: 100,
                    labelAlign: 'top'
                },
                buttonAlign: 'left',
                items: [{
                    fieldLabel: _t('ZenPack'),
                    name: 'zpname',
                    xtype: 'combo',
                    emptyText: _t('Please select a ZenPack...'),
                    listEmptyText: _t('No ZenPacks in development mode'),
                    allowBlank: false,
                    listConfig:{
                        resizable: true
                    },
                    store: new Zenoss.NonPaginatedStore({
                        id: 'myzpstore',
                        root: 'packs',
                        model: 'Zenoss.model.Name',
                        totalProperty: 'totalCount',
                        directFn: Zenoss.remote.ZenPackRouter.getEligiblePacks
                    }),
                    valueField: 'name',
                    displayField: 'name',
                    forceSelection: true,
                    triggerAction: 'all',
                    selectOnFocus: true,
                    id: 'zpcombobox'
                }],
                buttons: [{
                    text: _t('Submit'),
                    xtype: 'DialogButton',
                    disabled: true,
                    handler: function () {
                        var form = Ext.getCmp('addzenpackform'),
                            chosenzenpack = form.getForm().findField('zpname').getValue(), i, targets = [], callback;
                        if (!Ext.isArray(me.target)) {
                            targets.push(me.target);
                        } else {
                            targets = me.target;
                        }
                        callback = function() {
                            Zenoss.message.info(_t("The item was added to {0}"), chosenzenpack);
                        };
                        for (i=0; i<targets.length; i++ ) {
                            Zenoss.remote.ZenPackRouter.addToZenPack({
                                topack: targets[i],
                                zenpack: chosenzenpack
                            }, callback);
                        }
                    }
                },{
                    text: _t('Cancel'),
                    xtype: 'DialogButton'
                }]
            }]
        });
        Zenoss.AddToZenPackWindow.superclass.constructor.call(this, config);
    },
    setTarget: function (target) {
        this.target = target;
    }
});



})();
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


(function(){

Ext.ns('Zenoss');

Zenoss.VerticalBrowsePanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            layout: 'hbox',
            border: false,
            layoutConfig: {
                align: 'stretch'
            },
            defaults: {
                flex: 1,
                autoScroll: true
            }
        });
        Zenoss.VerticalBrowsePanel.superclass.constructor.call(this, config);
    }
});

})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010-2013, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

var ZC = Ext.ns('Zenoss.component'),
    NM = ZC.nameMap = {};

ZC.registerName = function(meta_type, name, plural) {
    NM[meta_type] = [name, plural];
};

ZC.displayName = function(meta_type) {
    return NM[meta_type] || [meta_type, meta_type];
};

ZC.displayNames = function() {
    return NM;
};

var router = Zenoss.remote.TemplateRouter;

function render_link(ob) {
    if (ob && ob.uid) {
        return Zenoss.render.link(ob.uid);
    } else {
        return ob;
    }
}
function getComponentEventPanelColumnDefinitions() {
    var fields = ['eventState',
                  'severity',
                  'eventClass',
                  'summary',
                  'firstTime',
                  'lastTime',
                  'count'],
        cols;
    cols = Zenoss.util.filter(Zenoss.env.COLUMN_DEFINITIONS, function(col){
        return Ext.Array.indexOf(fields, col.dataIndex) !== -1;
    });

    // delete the ids to make sure we do not have duplicates,
    // they are not needed
    Ext.Array.each(cols, function(col){
        if (col.id) {
            delete col.id;
        }
    });

    return cols;
}

function tbarButtoner(target, buttonDefs, combo, cardId, that) {
    var tbar = target.getDockedItems()[0];
    if (tbar._btns) {
        Ext.each(tbar._btns, tbar.remove, tbar);
    }
    var btns = tbar.add(buttonDefs);
    tbar.doLayout();
    tbar._btns = btns;
    combo.on('select', function(c){
        if (c.value!==cardId) {
            Ext.each(btns, tbar.remove, tbar);
        }
    }, that, {single:true});
}

Ext.define('Zenoss.component.TplUidNameModel', {
    extend: 'Ext.data.Model',
    idProperty: 'uid',
    fields: ['qtip', 'definition', 'hidden', 'leaf', 'description', 'name', 'text', 'id', 'meta_type', 'targetPythonClass', 'inspector_type', 'icon_cls', 'children', 'uid']
});

// getGraphDef can fetch graph defs and automatically
// caches them
var getGraphDef = (function(){
    var graphDefsCache = {};
    return function getGraphDef(data, cb, scope){
        var name = data.uid,
            cached = graphDefsCache[name];

        // use cached result if available
        if(cached){
            // NOTE - result must be cloned because the receiver
            // can do whatever they want with the referenced object
            cb.call(scope, Ext.clone(cached.result), cached.event, cached.success);
            return;
        }

        // create a new callback that will cache the result,
        // then call the original callback
        var callback = function(result, event, success){
            if(!success){
                cb.call(scope, result, event, success);
                return;
            }

            // cache result
            graphDefsCache[name] = {
                result: result,
                event: event,
                success: success
            };
            cb.call(scope, Ext.clone(result), event, success);
        };

        Zenoss.remote.DeviceRouter.getGraphDefs(data, callback, scope);
    };
})();

Zenoss.nav.register({
    Component: [{
        nodeType: 'subselect',
        id: 'Graphs',
        text: _t('Graphs'),
        action: function(node, target, combo) {
            var uid = combo.contextUid,
                cardid = 'graph_panel',
                graphs = {
                    id: cardid,
                    xtype: 'graphpanel',
                    viewName: 'graphs',
                    text: _t('Graphs'),
                    directFn: getGraphDef
                };

            if (!Ext.get('graph_panel')) {
                target.add(graphs);
            }
            target.layout.setActiveItem(cardid);
            target.layout.activeItem.setContext(uid);
        }
    },{
        nodeType: 'subselect',
        id: 'Events',
        text: _t('Events'),
        action: function(node, target, combo) {
            var uid = combo.contextUid,
                cardid = 'event_panel',
                showPanel = function() {
                    target.layout.setActiveItem(cardid);
                    target.layout.activeItem.setContext(uid);
                };
            if (!Ext.get('event_panel')) {
                target.add({
                    id: cardid,
                    xtype: 'SimpleEventGridPanel',
                    stateId: 'component-event-console',
                    columns: getComponentEventPanelColumnDefinitions()
                });
            }
            var buttonDefs = [
                '-',
                new Zenoss.ActionButton({
                    iconCls: 'acknowledge',
                    tooltip: _t('Acknowledge events'),
                    permission: 'Manage Events',
                    handler: function() {
                        Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.acknowledge);
                    }
                }),
                new Zenoss.ActionButton({
                    iconCls: 'close',
                    tooltip: _t('Close events'),
                    permission: 'Manage Events',
                    handler: function() {
                        Zenoss.EventActionManager.execute(Zenoss.remote.EventsRouter.close);
                    }
                }),
                new Zenoss.ActionButton({
                    iconCls: 'refresh',
                    permission: 'View',
                    tooltip: _t('Refresh events'),
                    handler: function(btn) {
                        var grid = btn.grid || this.ownerCt.ownerCt;
                        if(grid.getComponent("event_panel")) {
                            grid = grid.getComponent("event_panel");
                        }
                        grid.refresh();
                    }
                }),
                '-',
                new Zenoss.ActionButton({
                    iconCls: 'newwindow',
                    permission: 'View',
                    tooltip: _t('Go to event console'),
                    handler: function() {
                        var curState = Ext.state.Manager.get('evconsole') || {},
                        filters = curState.filters || {},
                        pat = /devices\/([^\/]+)(\/.*\/([^\/]+)$)?/,
                        st, url, matches = Ext.getCmp('event_panel').uid.match(pat);
                        if (matches){
                            filters.device = matches[1];
                            // using "name" from the parent grid here as the UID doesn't contain the component as was expected
                            filters.component = Ext.getCmp('component_card').componentgrid.getView().getSelectionModel().getSelected().get("name");
                        }
                        curState.filters = filters;
                        st = encodeURIComponent(Zenoss.util.base64.encode(Ext.encode(curState)));
                        url = '/zport/dmd/Events/evconsole?state=' + st;
                        window.open(url, '_newtab', "");
                    }
                })
            ];
            tbarButtoner(target, buttonDefs, combo, "Events", this);
            showPanel();
        }
    },{
        nodeType: 'subselect',
        id: 'Edit',
        text: _t('Details'),
        action: function(node, target, combo) {
            var uid = combo.contextUid;
            if (!Ext.get('edit_panel')) {
                Zenoss.form.getGeneratedForm(uid, function(config){
                    target.add(Ext.apply({id:'edit_panel'}, config));
                    target.layout.setActiveItem('edit_panel');
                });
            } else {
                target.layout.setActiveItem('edit_panel');
                target.layout.activeItem.setContext(uid);
            }
        }
    },{
        nodeType: 'subselect',
        id: 'ComponentTemplate',
        text: _t('Templates'),
        action: function(node, target, combo) {
            var cardid = 'templates_panel',
                contextUid = combo.contextUid;
            if (!Ext.get(cardid)) {
                target.add(Ext.create('Zenoss.ComponentTemplatePanel',{
                    ref: 'componentTemplatePanel',
                    id: cardid,
                    contextUid: contextUid
                }));
            }
            var tplCombo = Ext.create('Ext.form.field.ComboBox', {
                xtype: 'combo',
                ref: '../templateCombo',
                displayField: 'name',
                valueField: 'uid',
                initialSortColumn: 'name',
                width: 200,
                editable: false,
                forceSelection: true,
                autoSelect: true,
                store: new Zenoss.NonPaginatedStore({
                    model: 'Zenoss.component.TplUidNameModel',
                    directFn: router.getObjTemplates,
                    autoLoad: true,
                    listeners: {
                        beforeload: function(store, operation){
                            if (!operation.params) {
                                operation.params = {};
                                operation.params.uid = contextUid;
                            } else {
                                operation.params.uid = contextUid;
                                delete operation.params.query;
                            }
                        },
                        load: function(store, records) {
                            if (records.length === 0) {
                                return;
                            }
                            if (!target.componentTemplatePanel) {
                                return;
                            }
                            var tplUid = records[0].data.uid;
                            target.componentTemplatePanel.setContext(tplUid);
                            if (tplCombo.store) {
                                tplCombo.setValue(tplUid);
                            }
                            if (tplCombo.refOwner) {
                                if (tplUid.startswith(contextUid)) {
                                    tplCombo.refOwner.createLocalCopyButton.disable();
                                    tplCombo.refOwner.deleteLocalCopyButton.enable();
                                } else {
                                    tplCombo.refOwner.createLocalCopyButton.enable();
                                    tplCombo.refOwner.deleteLocalCopyButton.disable();
                                }
                            }
                        }
                    }
                }),
                listeners: {
                    select: function(combo, records){
                        var tplUid = records[0].data.uid;
                        target.componentTemplatePanel.setContext(tplUid);
                        if (tplUid.startswith(contextUid)) {
                            this.refOwner.createLocalCopyButton.disable();
                            this.refOwner.deleteLocalCopyButton.enable();
                        } else {
                            this.refOwner.createLocalCopyButton.enable();
                            this.refOwner.deleteLocalCopyButton.disable();
                        }
                    }
                }
            });
            var buttonDefs = [
                '->',
                {
                    xtype: 'label',
                    text: _t('Template:'),
                    margin: '0 10 0 0'
                },
                tplCombo,
                '-',
                {
                    ref: '../createLocalCopyButton',
                    xtype: 'button',
                    disabled: true,
                    text: _t('Create Local Copy'),
                    handler: function() {
                        var tplTbar = this.refOwner,
                            templateName = tplTbar.templateCombo.getValue(),
                            createLocalArgs;
                        if (templateName) {
                            createLocalArgs = {
                                uid: contextUid,
                                templateName: templateName
                            };
                            router.makeLocalRRDTemplate(createLocalArgs, function(response) {
                                target.componentTemplatePanel.setContext(response.tplUid);
                                tplTbar.createLocalCopyButton.disable();
                                tplTbar.deleteLocalCopyButton.enable();
                            });
                        }
                    }
                },{
                    ref: '../deleteLocalCopyButton',
                    xtype: 'button',
                    text: _t('Delete Local Copy'),
                    disabled: true,
                    tooltip: _t('Delete the local copy of this template'),
                    handler: function() {
                        var tplTbar = this.refOwner,
                            templateName = tplTbar.templateCombo.getValue(),
                            removeLocalArgs;
                        if (templateName) {
                            // show a confirmation
                            new Zenoss.dialog.SimpleMessageDialog({
                                title: _t('Delete Copy'),
                                message: Ext.String.format(_t("Are you sure you want to delete the local copy of this template? There is no undo.")),
                                buttons: [{
                                    xtype: 'DialogButton',
                                    text: _t('OK'),
                                    handler: function() {
                                        removeLocalArgs = {
                                            uid: contextUid,
                                            templateName: templateName
                                        };
                                        router.removeLocalRRDTemplate(removeLocalArgs, function(response) {
                                            target.componentTemplatePanel.setContext(response.tplUid);
                                            tplTbar.createLocalCopyButton.enable();
                                            tplTbar.deleteLocalCopyButton.disable();
                                        });
                                    }
                                }, {
                                    xtype: 'DialogButton',
                                    text: _t('Cancel')
                                }]
                            }).show();
                        }
                    }
                }
            ];
            tbarButtoner(target, buttonDefs, combo, "Templates", this);
            target.layout.setActiveItem('templates_panel');
        }
    }]
});

Ext.define("Zenoss.component.ComponentDetailNav", {
    alias:['widget.componentnav'],
    extend:"Zenoss.DetailNavPanel",
    constructor: function(config) {
        Ext.applyIf(config, {
            autoHeight: true,
            autoScroll: true,
            containerScroll: true,
            menuIds: []
        });
        ZC.ComponentDetailNav.superclass.constructor.call(this, config);
        this.relayEvents(this.getSelectionModel(), ['selectionchange']);
        this.on('selectionchange', this.onSelectionChange);
    },
    onGetNavConfig: function() {
        var grid = this.ownerCt.ownerCt.ownerCt.componentgrid,
            items = [],
            monitor = false;
        Zenoss.env.GRID = grid;
        Ext.each(grid.store.data.items, function(record){
            if (record.data.monitor) { monitor = true; }
        });
        Zenoss.util.each(Zenoss.nav.get('Component'), function(item){
            if (!(item.id==='Graphs' && !monitor)) {
                items.push(item);
            }
        });
        return items;
    },
    filterNav: function(navpanel, config){
        //nav items to be excluded
        var excluded = [
            'status',
            'events',
            'resetcommunity',
            'pushconfig',
            'objtemplates',
            'modeldevice',
            'historyevents'
        ];
        return (Ext.Array.indexOf(excluded, config.id)===-1);
    },
    onSelectionChange: function(sm, node) {
        var target = this.target || Ext.getCmp('component_detail_panel'),
            action = node.data.action || Ext.bind(function(node, target) {
                var id = node.get('id');
                if (!(id in target.items.map)) {
                    var config = this.panelConfigMap[id];
                    if(config) {
                        target.add(config);
                        target.doLayout();
                    }
                }
                target.items.map[id].setContext(this.contextId);
                target.layout.setActiveItem(id);
            }, this);
        action(node, target);
    }
});


/**
 *@class Zenoss.component.ComponentPanel
 *@extends Ext.Panel
 **/
Ext.define("Zenoss.component.ComponentPanel", {
    alias:['widget.componentpanel'],
    extend:"Ext.Panel",
    constructor: function(config) {
        var tbar = config.gridtbar,
            tbarid = Ext.id();
        if (tbar) {
            if (Ext.isArray(tbar)) {
                tbar = {items:tbar};
            }
            Ext.apply(tbar, {
                id: tbarid
            });
        }
        config = Ext.applyIf(config||{}, {
            tbarid: tbarid,
            layout: 'border',
            token: null,
            items: [{
                region: 'north',
                height: 250,
                split: true,
                ref: 'gridcontainer',
                tbar: tbar,
                layout: 'fit'
            },{
                xtype: 'contextcardpanel',
                region: 'center',
                ref: 'detailcontainer',
                split: true,
                tbar: {
                    cls: 'largetoolbar componenttbar',
                    height: 32,
                    items: [{
                        xtype: 'tbtext',
                        text: _t("Display: ")
                    },{
                        xtype: 'detailnavcombo',
                        menuIds: [],
                        onGetNavConfig: Ext.bind(function(uid) {
                            var grid = this.componentgrid,
                                items = [],
                                monitor = false;
                            Ext.each(grid.store.data.items, function(record){
                                if (record.data.uid===uid && record.data.monitor) {
                                    monitor = true;
                                }
                            });
                            Zenoss.util.each(Zenoss.nav.get('Component'), function(item){
                                items.push(item);
                            });
                            return items;
                        }, this),
                        filterNav: function(cfg) {
                            var excluded = [
                                'status',
                                'events',
                                'resetcommunity',
                                'pushconfig',
                                'objtemplates',
                                'template',
                                'modeldevice',
                                'historyevents'
                            ];
                            return (Ext.Array.indexOf(excluded, cfg.id)===-1);
                        },
                        ref: '../../componentnavcombo',
                        getTarget: Ext.bind(function() {
                            return this.detailcontainer;
                        }, this)
                    }]
                }
            }]
        });
        ZC.ComponentPanel.superclass.constructor.call(this, config);
        this.addEvents('contextchange');
    },
    getGridToolbar: function(){
        return Ext.getCmp(this.tbarid);
    },
    selectByToken: function(token) {
        if (token) {
            if (this.componentgrid) {
                var grid = this.componentgrid;
                grid.selectByToken(token);
            } else {
                this.token = token;
            }
        }
    },
    setContext: function(uid, type) {
        this.contextUid = uid;
        if (type!==this.componentType) {
            this.componentType = type;

            var compType = this.componentType + 'Panel',
                xtype = Ext.ClassManager.getByAlias('widget.' + compType) ? compType : 'ComponentGridPanel';
            this.gridcontainer.removeAll();
            this.gridcontainer.add({
                xtype: xtype,
                componentType: this.componentType,
                ref: '../componentgrid',
                listeners: {
                    render: function(grid) {
                        grid.setContext(uid);
                    },
                    rangeselect: function() {
                        this.detailcontainer.removeAll();
                        this.componentnavcombo.reset();
                    },
                    select: function(sm, row) {
                        // top grid selection change
                        if (row) {
                            // When infinite grids are resized the "selectionchange" event can be fired
                            // even though the selected row hasn't changed.. This can be very expensive and cause rendering
                            // errors when the layout manager tries to render the layouts of things that have already
                            // been removed.
                            if (this.previousRow && this.previousRow.id === row.id) {
                                return;
                            }
                            this.previousRow = row;
                            this.detailcontainer.removeAll();
                            this.componentnavcombo.reset();
                            Zenoss.env.compUUID = row.data.uuid;
                            this.componentnavcombo.setContext(row.data.uid);
                            var delimiter = Ext.History.DELIMITER,
                                token = Ext.History.getToken().split(delimiter, 1);
                            token = token + delimiter + this.componentType + delimiter + row.data.uid;
                            // set the currenttoken so the "change" event isn't fired. Events are not able
                            // to be suspended because we don't know when the change event will be fired.
                            Ext.util.History.currentToken = token;
                            Ext.util.History.setHash(token);
                            Ext.getCmp('component_monitor_menu_item').setDisabled(!row.data.usesMonitorAttribute);
                        } else {
                            this.detailcontainer.removeAll();
                            this.componentnavcombo.reset();
                        }
                    },
                    scope: this
                }
            });
            this.gridcontainer.doLayout();
            if (this.token) {
                this.componentgrid.selectByToken(this.token);
            }
        } else {
            this.componentgrid.setContext(uid);
        }
        this.fireEvent('contextchange', this, uid, type);

    }
});

/**
 *@class Zenoss.component.ComponentGridPanel
 *@extends Zenoss.BaseGridPanel
 * Base class for all of the component grids including the custom
 * grids extended by zenpacks.
 **/
Ext.define("Zenoss.component.ComponentGridPanel", {
    alias:['widget.ComponentGridPanel'],
    extend:"Zenoss.BaseGridPanel",
    lastHash: null,
    constructor: function(config) {
        config = config || {};
        config.fields = config.fields || [
            {name: 'severity'},
            {name: 'name'},
            {name: 'monitored'},
            {name: 'locking'}
        ];
        config.fields.push({name: 'uuid'});
        config.fields.push({name: 'uid'});
        config.fields.push({name: 'meta_type'});
        config.fields.push({name: 'monitor'});

        // compat for autoExpandColumn
        var expandColumn = config.autoExpandColumn;
        if (expandColumn && config.columns) {
            Ext.each(config.columns, function(col){
                if (expandColumn === col.id) {
                    col.flex = 1;
                }
            });
        }
        // delete the id fields so there are no duplicate ids
        Ext.each(config.columns, function(col){
            delete col.id;
        });
        var modelId = Ext.id();
        Ext.define(modelId, {
            extend: 'Ext.data.Model',
            idProperty: 'uuid',
            fields: config.fields
        });
        config.sortInfo = config.sortInfo || {};

        var store = new ZC.BaseComponentStore({
            model: modelId,
            initialSortColumn: config.sortInfo.field || 'name',
            initialSortDirection: config.sortInfo.direction || 'ASC',
            directFn: config.directFn || Zenoss.remote.DeviceRouter.getComponents,
            buffered: Zenoss.settings.enableInfiniteGridForComponents
        });

        config = Ext.applyIf(config, {
            autoExpandColumn: 'name',
            store: store,
            columns: [{
                id: 'component_severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 50
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name'),
                flex: 1
            }, {
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                renderer: Zenoss.render.checkbox,
                width: 65,
                sortable: true
            }, {
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons,
                width: 65
            }],
            selModel: new Ext.selection.RowModel({
                mode: 'MULTI',
                getSelected: function() {
                    var rows = this.getSelection();
                    if (!rows.length) {
                        return null;
                    }
                    return rows[0];
                }
            })
        });
        ZC.ComponentGridPanel.superclass.constructor.call(this, config);
        this.relayEvents(this.getSelectionModel(), ['rangeselect']);
        this.store.proxy.on('load',
            function(proxy, o) {
                this.lastHash = o.result.hash || this.lastHash;
            },
            this);
    },
    applyOptions: function(){
        // apply options to all future parameters, not just this operation.
        var params = this.getStore().getProxy().extraParams;

        Ext.apply(params, {
            uid: this.contextUid,
            keys: Ext.Array.pluck(this.fields, 'name'),
            meta_type: this.componentType,
            name: this.componentName
        });
    },
    filter: function(name) {
        this.componentName = name;
        this.refresh();
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.getStore().on('guaranteedrange', function(){
            var token = Ext.History.getToken();
            if (token.split(Ext.History.DELIMITER).length!==3) {
                this.getSelectionModel().selectRange(0, 0);
            }
        }, this, {single:true});
        this.callParent(arguments);
    },
    selectByToken: function(uid) {
        var store = this.getStore(),
            selectionModel = this.getSelectionModel(),
            view = this.getView(),
            me = this,
            selectToken = function() {
                function gridSelect() {
                    var found = false, i=0;
                    store.each(function(r){
                        if (r.get('uid') === uid) {
                            if (!selectionModel.isSelected(r)) {
                                selectionModel.select(r);
                            }
                            view.focusRow(r.index);
                            found = true;
                            store.un('guaranteedrange', gridSelect, me);
                            return false;
                        }
                        i++;
                        return true;
                    });
                    return found;
                }

                // see if it is in the current buffer
                var found = gridSelect();
                if (!found) {

                    // find the index, scroll to that position
                    // and then select the component
                    var o = {componentUid:uid};
                    Ext.apply(o, store.getProxy().extraParams);
                    // make sure we have the sort and the direction.
                    // since this only happens on initia
                    Ext.applyIf(o, {
                        sort: me.getStore().sorters.first().property,
                        dir: me.getStore().sorters.first().direction
                    });
                    Zenoss.remote.DeviceRouter.findComponentIndex(o, function(r){
                        // will return a null if not found
                        if (Ext.isNumeric(r.index)) {
                            if (store.buffered){
                                // infinite grids fire guaranteedrange event
                                store.on('guaranteedrange', gridSelect, me);
                                var scroller = me.verticalScroller;
                                me.getView().scrollBy({ x: 0, y: scroller.rowHeight * r.index }, true);
                            } else {
                                // paginated grids fire load event
                                store.loadPage(store.getPageFromRecordIndex(r.index));
                                store.on('load', gridSelect, me);
                            }
                        } else {
                            // We can't find the index, it might be an invalid UID so
                            // select the first item so the details section isn't blank.
                            if (!selectionModel.getSelection()) {
                                selectionModel.select(0);
                            }

                        }
                    });
                }
        };

        if (!store.loaded) {
            store.on('guaranteedrange', selectToken, this, {single: true});
        } else {
            selectToken();
        }

    }
});



Ext.define("Zenoss.component.BaseComponentStore", {
    extend:"Zenoss.DirectStore",
    constructor: function(config) {
        var bufferSize = Zenoss.settings.componentGridBufferSize;
        // work around a bug in ExtJs 4.1.3. TODO: remove this after
        // we update the library.
        if (config.buffered) {
            if (bufferSize < 100) {
                bufferSize = 100;
            }
        }
        Ext.applyIf(config, {
            pageSize: bufferSize,
            directFn: config.directFn
        });
        ZC.BaseComponentStore.superclass.constructor.call(this, config);
        this.on('guaranteedrange', function(){
            this.loaded = true;
        }, this);
        // paginated grid fires load instead of guaranteedrange
        this.on('load', function(){
            if(!this.loaded){
                this.fireEvent('guaranteedrange', this);
            }
        }, this);
    }
});


Ext.define("Zenoss.component.IpInterfacePanel", {
    alias:['widget.IpInterfacePanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'IpInterface',
            autoExpandColumn: 'description',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'description'},
                {name: 'ipAddressObjs'},
                {name: 'network'},//, mapping:'network.uid'},
                {name: 'macaddress'},
                {name: 'usesMonitorAttribute'},
                {name: 'operStatus'},
                {name: 'adminStatus'},
                {name: 'status'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'duplex'},
                {name: 'netmask'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 50
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('IP Interface'),
                width: 250
            },{
                id: 'ipAddresses',
                dataIndex: 'ipAddressObjs',
                header: _t('IP Addresses'),
                renderer: function(ipaddresses) {
                    var returnString = '';
                    Ext.each(ipaddresses, function(ipaddress, index) {
                        if (index > 0) {
                            returnString += ', ';
                        }
                        if (ipaddress && Ext.isObject(ipaddress) && ipaddress.netmask) {
                            var name = ipaddress.name + '/' + ipaddress.netmask;
                            returnString += Zenoss.render.link(ipaddress.uid, undefined, name);
                        }
                        else if (Ext.isString(ipaddress)) {
                            returnString += ipaddress;
                        }
                    });
                    return returnString;
                }
            },{
                id: 'description',
                dataIndex: 'description',
                header: _t('Description')
            },{
                id: 'macaddress',
                dataIndex: 'macaddress',
                header: _t('MAC Address'),
                width: 120
            },{
                id: 'status',
                dataIndex: 'status',
                header: _t('Monitored'),
                renderer: Zenoss.render.pingStatus,
                width: 80
            },{
                id: 'operStatus',
                dataIndex: 'operStatus',
                header: _t('Operational Status'),
                width: 110
            },{
                id: 'adminStatus',
                dataIndex: 'adminStatus',
                header: _t('Admin Status'),
                width: 80
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                renderer: Zenoss.render.checkbox,
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                width: 72,
                renderer: Zenoss.render.locking_icons
            }]

        });
        ZC.IpInterfacePanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('IpInterface', _t('Interface'), _t('Interfaces'));

Ext.define("Zenoss.component.WinServicePanel", {
    alias:['widget.WinServicePanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'WinService',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'status'},
                {name: 'name'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'caption'},
                {name: 'startMode'},
                {name: 'startName'},
                {name: 'serviceClassUid'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                flex: 1,
                header: _t('Service Name'),
                renderer: Zenoss.render.WinServiceClass
            },{
                id: 'caption',
                dataIndex: 'caption',
                header: _t('Caption')
            },{
                id: 'startMode',
                dataIndex: 'startMode',
                header: _t('Start Mode')
            },{
                id: 'startName',
                dataIndex: 'startName',
                header: _t('Start Name')
            },{
                id: 'status',
                dataIndex: 'status',
                header: _t('Status'),
                renderer: Zenoss.render.pingStatus,
                width: 60
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                renderer: Zenoss.render.checkbox,
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.WinServicePanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('WinService', _t('Windows Service'), _t('Windows Services'));


Ext.define("Zenoss.component.IpRouteEntryPanel", {
    alias:['widget.IpRouteEntryPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'IpRouteEntry',
            autoExpandColumn: 'destination',
            fields: [
                {name: 'uid'},
                {name: 'destination'},
                {name: 'nextHop'},
                {name: 'id'}, // needed for nextHop link
                {name: 'device'}, // needed for nextHop link
                {name: 'interface'},
                {name: 'protocol'},
                {name: 'type'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'destination',
                dataIndex: 'destination',
                header: _t('Destination'),
                renderer: render_link
            },{
                id: 'nextHop',
                dataIndex: 'nextHop',
                header: _t('Next Hop'),
                renderer: Zenoss.render.nextHop,
                width: 250
            },{
                id: 'interface',
                dataIndex: 'interface',
                header: _t('Interface'),
                renderer: Zenoss.render.default_uid_renderer
            },{
                id: 'protocol',
                dataIndex: 'protocol',
                header: _t('Protocol')
            },{
                id: 'type',
                dataIndex: 'type',
                header: _t('Type'),
                width: 50
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.IpRouteEntryPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('IpRouteEntry', _t('Network Route'), _t('Network Routes'));

Ext.define("Zenoss.component.IpServicePanel", {
    alias:['widget.IpServicePanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'IpService',
            fields: [
                {name: 'uid'},
                {name: 'name'},
                {name: 'status'},
                {name: 'severity'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'protocol'},
                {name: 'description'},
                {name: 'ipaddresses'},
                {name: 'port'},
                {name: 'serviceClassUid'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name'),
                flex: 1,
                renderer: Zenoss.render.IpServiceClass
            },{
                id: 'protocol',
                dataIndex: 'protocol',
                header: _t('Protocol')
            },{
                id: 'port',
                dataIndex: 'port',
                header: _t('Port')
            },{
                id: 'ipaddresses',
                dataIndex: 'ipaddresses',
                header: _t('IPs'),
                renderer: function(ips) {
                  return Ext.isEmpty(ips) ? '' : Ext.isArray(ips) ? ips.join(', ') : ips.toString();
                }
            },{
                id: 'description',
                dataIndex: 'description',
                header: _t('Description')
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                renderer: Zenoss.render.checkbox,
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.IpServicePanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('IpService', _t('IP Service'), _t('IP Services'));


Ext.define("Zenoss.component.OSProcessPanel", {
    alias:['widget.OSProcessPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'OSProcess',
            autoExpandColumn: 'processName',
            fields: [
                {name: 'uid'},
                {name: 'processName'},
                {name: 'status'},
                {name: 'severity'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'processClass'},
                {name: 'processClassName'},
                {name: 'alertOnRestart'},
                {name: 'failSeverity'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'processClass',
                dataIndex: 'processClass',
                header: _t('Process Class'),
                renderer: function(cls, meta, record) {
                    if (cls && cls.uid) {
                        return Zenoss.render.link(cls.uid, null, record.raw.processClassName);
                    } else {
                        return cls;
                    }
                }
            },{
                id: 'processName',
                dataIndex: 'processName',
                header: _t('Process Set')
            },{
                id: 'alertOnRestart',
                dataIndex: 'alertOnRestart',
                renderer: Zenoss.render.checkbox,
                width: 85,
                header: _t('Restart Alert?')
            },{
                id: 'failSeverity',
                dataIndex: 'failSeverity',
                renderer: Zenoss.render.severity,
                width: 70,
                header: _t('Fail Severity')
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                renderer: Zenoss.render.checkbox,
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                width: 55,
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.OSProcessPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('OSProcess', _t('OS Process'), _t('OS Processes'));


Ext.define("Zenoss.component.FileSystemPanel", {
    alias:['widget.FileSystemPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            autoExpandColumn: 'mount',
            componentType: 'FileSystem',
            fields: [
                {name: 'uid'},
                {name: 'name'},
                {name: 'status'},
                {name: 'severity'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'mount'},
                {name: 'storageDevice'},
                {name: 'totalBytes'},
                {name: 'availableBytes'},
                {name: 'usedBytes'},
                {name: 'capacityBytes'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'mount',
                dataIndex: 'mount',
                header: _t('Mount Point')
            },{
                id: 'storageDevice',
                dataIndex: 'storageDevice',
                header: _t('Storage Device')
            },{
                id: 'totalBytes',
                dataIndex: 'totalBytes',
                header: _t('Total Bytes'),
                renderer: Zenoss.render.bytesString
            },{
                id: 'usedBytes',
                dataIndex: 'usedBytes',
                header: _t('Used Bytes'),
                renderer: Zenoss.render.bytesString
            },{
                id: 'availableBytes',
                dataIndex: 'availableBytes',
                header: _t('Free Bytes'),
                renderer: function(n){
                    if (n<0) {
                        return _t('Unknown');
                    } else {
                        return Zenoss.render.bytesString(n);
                    }

                }
            },{
                id: 'capacityBytes',
                dataIndex: 'capacityBytes',
                header: _t('% Util'),
                renderer: function(n) {
                    if (n==='unknown' || n<0) {
                        return _t('Unknown');
                    } else {
                        return n + '%';
                    }
                }

            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                renderer: Zenoss.render.checkbox,
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.FileSystemPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('FileSystem', _t('File System'), _t('File Systems'));


Ext.define("Zenoss.component.CPUPanel", {
    alias:['widget.CPUPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'CPU',
            autoExpandColumn: 'product',
            fields: [
                {name: 'uid'},
                {name: 'socket'},
                {name: 'manufacturer'},
                {name: 'product'},
                {name: 'clockspeed'},
                {name: 'extspeed'},
                {name: 'cacheSizeL1'},
                {name: 'cacheSizeL2'},
                {name: 'voltage'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'socket',
                dataIndex: 'socket',
                header: _t('Socket'),
                width: 45
            },{
                id: 'manufacturer',
                dataIndex: 'manufacturer',
                header: _t('Manufacturer'),
                renderer: render_link
            },{
                id: 'product',
                dataIndex: 'product',
                header: _t('Model'),
                renderer: render_link
            },{
                id: 'clockspeed',
                dataIndex: 'clockspeed',
                header: _t('Speed'),
                width: 70,
                renderer: function(x){ return x + ' MHz';}
            },{
                id: 'extspeed',
                dataIndex: 'extspeed',
                header: _t('Ext Speed'),
                width: 70,
                renderer: function(x){ return x + ' MHz';}
            },{
                id: 'cacheSizeL1',
                dataIndex: 'cacheSizeL1',
                header: _t('L1'),
                width: 70,
                renderer: function(x){ return x + ' KB';}
            },{
                id: 'cacheSizeL2',
                dataIndex: 'cacheSizeL2',
                header: _t('L2'),
                width: 70,
                renderer: function(x){ return x + ' KB';}
            },{
                id: 'voltage',
                dataIndex: 'voltage',
                header: _t('Voltage'),
                width: 70,
                renderer: function(x){ return x + ' mV';}
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.CPUPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('CPU', _t('Processor'), _t('Processors'));


Ext.define("Zenoss.component.ExpansionCardPanel", {
    alias:['widget.ExpansionCardPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'ExpansionCard',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'slot'},
                {name: 'name'},
                {name: 'serialNumber'},
                {name: 'manufacturer'},
                {name: 'product'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'slot',
                dataIndex: 'slot',
                header: _t('Slot'),
                width: 80
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'serialNumber',
                dataIndex: 'serialNumber',
                header: _t('Serial Number'),
                width: 110
            },{
                id: 'manufacturer',
                dataIndex: 'manufacturer',
                header: _t('Manufacturer'),
                renderer: render_link,
                width: 110
            },{
                id: 'product',
                dataIndex: 'product',
                header: _t('Model'),
                renderer: render_link,
                width: 130
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.ExpansionCardPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('ExpansionCard', _t('Card'), _t('Cards'));


Ext.define("Zenoss.component.PowerSupplyPanel", {
    alias:['widget.PowerSupplyPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'PowerSupply',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'watts'},
                {name: 'type'},
                {name: 'state'},
                {name: 'millivolts'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'watts',
                dataIndex: 'watts',
                header: _t('Watts')
            },{
                id: 'type',
                dataIndex: 'type',
                header: _t('Type')
            },{
                id: 'state',
                dataIndex: 'state',
                header: _t('State')
            },{
                id: 'millivolts',
                dataIndex: 'millivolts',
                header: _t('Millivolts')
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.PowerSupplyPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('PowerSupply', _t('Power Supply'), _t('Power Supplies'));


Ext.define("Zenoss.component.TemperatureSensorPanel", {
    alias:['widget.TemperatureSensorPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'TemperatureSensor',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'state'},
                {name: 'temperature'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'state',
                dataIndex: 'state',
                header: _t('State')
            },{
                id: 'temperature',
                dataIndex: 'temperature',
                header: _t('Temperature'),
                renderer: function(x) {
                    if (x === null) {
                        return "";
                    } else {
                        return x + " F";
                    }
                }
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.TemperatureSensorPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('TemperatureSensor', _t('Temperature Sensor'), _t('Temperature Sensors'));


Ext.define("Zenoss.component.FanPanel", {
    alias:['widget.FanPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'Fan',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'state'},
                {name: 'type'},
                {name: 'rpm'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'state',
                dataIndex: 'state',
                header: _t('State')
            },{
                id: 'type',
                dataIndex: 'type',
                header: _t('Type')
            },{
                id: 'rpm',
                dataIndex: 'rpm',
                header: _t('RPM')
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.FanPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('Fan', _t('Fan'), _t('Fans'));


Ext.define("Zenoss.component.HardDiskPanel", {
    alias:['widget.HardDiskPanel'],
    extend:"Zenoss.component.ComponentGridPanel",
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'HardDisk',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'serialNumber'},
                {name: 'state'},
                {name: 'type'},
                {name: 'rpm'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'serialNumber',
                dataIndex: 'serialNumber',
                header: _t('Serial Number')
            },{
                id: 'state',
                dataIndex: 'state',
                header: _t('State')
            },{
                id: 'type',
                dataIndex: 'type',
                header: _t('Type')
            },{
                id: 'rpm',
                dataIndex: 'rpm',
                header: _t('RPM')
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.HardDiskPanel.superclass.constructor.call(this, config);
    }
});


ZC.registerName('HardDisk', _t('HardDisk'), _t('HardDisks'));


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2017, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {
    Ext.ns('Zenoss.form');

    Ext.define("Zenoss.form.EventAction", {
        alias:['widget.eventaction'],
        extend:"Ext.form.ComboBox",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                fieldLabel: _t('Action'),
                name: 'action',
                editable: false,
                forceSelection: true,
                autoSelect: true,
                triggerAction: 'all',
                queryMode: 'local',
                store: Zenoss.env.ACTIONS
            });
            this.callParent(arguments);
        }
    });
}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {
    Ext.define("Severity", {
        alias:['widget.severity'],
        extend:"Ext.form.ComboBox",
        constructor: function(config) {
            config = config || {};

            Ext.applyIf(config, {
                fieldLabel: _t('Severity'),
                name: 'severity',
                editable: false,
                forceSelection: true,
                autoSelect: true,
                triggerAction: 'all',
                queryMode: 'local',
                // this is defined in zenoss.js so should always be present
                store: Zenoss.env.SEVERITIES
            });
            this.callParent(arguments);
        }
    });


    Ext.define("ReverseSeverity", {
        alias:['widget.reverseseverity'],
        extend:"Severity",
        constructor: function(config) {
            var severities = [[0, "Critical"], [1, "Error"], [2, "Warning"], [3, "Info"], [4, "Debug"], [5, "Clear"]];
            config = config || {};
            Ext.applyIf(config, {
                store: severities
            });
            this.callParent(arguments);
        }
    });

    Ext.define("DefaultSeverity", {
        alias:['widget.defaultseverity'],
        extend:"Severity",
        constructor: function(config) {
            var severities = [[-1, "Default"]].concat(Zenoss.env.SEVERITIES);
            config = config || {};
            Ext.applyIf(config, {
                store: severities
            });
            this.callParent(arguments);
        }
    });

}());
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {
     Ext.define("Zenoss.form.EventClass", {
         alias:['widget.eventclass'],
         extend:"Ext.form.ComboBox",
         constructor: function(config) {
             config = config || {};
             if (!Zenoss.env.EVENT_CLASSES) {
                 throw "You must include the js-snippets viewlet before you use the eventClass control";
             }
             Ext.applyIf(config, {
                 name: 'eventClass',
                 typeAhead: true,
                 editable: true,
                 forceSelection: true,
                 autoSelect: true,
                 triggerAction: 'all',
                 listConfig: {
                     maxWidth:250,
                     maxHeight: 250,
                     resizable: true
                 },
                 queryMode: 'local',
                 store: Zenoss.env.EVENT_CLASSES
             });
             this.callParent([config]);
         }
     });

}());
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {
Ext.define("Zenoss.form.DataPointItemSelector", {
    alias:['widget.datapointitemselector'],
    extend:"Ext.ux.form.ItemSelector",
    constructor: function(config) {
        var record = config.record;

        this.value = config.value;

        Ext.applyIf(config, {
            name: 'dataPoints',
            fieldLabel: _t('Data Points'),
            id: 'thresholdItemSelector',
            imagePath: "/++resource++zenui/img/xtheme-zenoss/icon",
            drawUpIcon: false,
            drawDownIcon: false,
            drawTopIcon: false,
            drawBotIcon: false,
            store: record.allDataPoints
        });
        this.callParent(arguments);
    },
    /**
     * In ExtJs 4.1 The ItemsSelect getValue doesn't work at all.
     *
     * To work around this we can just grab everything in the toField store.
     *
     */
    getValue: function(){
        var store = this.toField.getStore();
        return Ext.pluck(Ext.pluck(store.data.items, 'data'), 'field1');
    }
});

}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function () {
Ext.ns('Zenoss');

Ext.define("Zenoss.ContextMenu", {
    alias:['widget.ContextMenu'],
    extend:"Ext.Button",
    /**
     * onSetContext abstract function called when the context for the menu is
     * potentially changed
     */
    onSetContext: Ext.emptyFn,
    /**
     * The object id for which the context menu belongs to
     */
    contextUid: null,
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            id: 'context-configure-menu',
            iconCls: 'customize'
        });
        Zenoss.ContextMenu.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.contextUid = uid;
    }
});




Ext.define("Zenoss.ContextConfigureMenu", {
    alias:['widget.ContextConfigureMenu'],
    extend:"Zenoss.ContextMenu",
    /**
     * onGetMenuItems abstract function; hook to provide
     * more menu items based on context
     * @param {string} uid; item to get menu items for
     */
    onGetMenuItems: function(uid){ return []; },
    /**
     * Called when an item in the menu is selected
     * @param {object} this, the ContextConfigureMenu
     * @param {object} the menu item selection
     */
    onSelectionChange: Ext.emptyFn,
    /**
     * Filter out menu items;  Return true if it should be kept, false otherwise
     * @param {ContextConfigureMenu}
     * @param {Object} itemConfig
     */
    filterMenu: function(contextMenu, config) {return true;},
     /**
     * Menu ids used to get dialog menus that will be used in context menu;
     * Empty or null if no items from old menu items is desired
     */
    menuIds: ['More','Manage','Edit', 'Actions','Add','TopLevel'],
    menu: {items:[]},
    disabled: true,
    /**
     * @cfg {array} menutItems
     * items that should always be included in the menu
     */
    menuItems: [],
    /**
     * set the UID, menus for the corresponding object will be determined and
     * set
     * @param {Object} uid
     */
    setContext: function(uid) {
        this.disable();
        this.onSetContext(uid);
        this.contextUid = uid;
        var menu = this.menu;
        menu.removeAll();
        this.getMenuItems(uid);
    },
    /**
     * private
     * handler for add to zenpack menu item; displays dialog
     * @param {Object} uid
     */
     addToZenPackHandler: function(e) {
        var dialog = Ext.create('Zenoss.AddToZenPackWindow', {});
        dialog.setTarget(this.contextUid);
        dialog.show();
    },
    /**
     * private
     * gets the menu items to be displayed
     * @param {Object} uid
     */
    getMenuItems: function(uid){
        var callback = function(provider, response){
            var menuItems = [], visibleMenuCount = 0;
            //get statically defined menu items
            if (this.menuItems.length !== 0) {
                menuItems = menuItems.concat(this.menuItems);
            }
            //get any context specific menus if defined
            var moreMenuItems = this.onGetMenuItems(this.contextUid);
            if (moreMenuItems !== null || moreMenuItems.length > 0){
                menuItems = menuItems.concat(moreMenuItems);
            }
            //menus from router
            var itemConfigs = response.result.menuItems;
            var filterFn = function(val) {
                return this.filterMenu(this, val);
            };
            itemConfigs = Zenoss.util.filter(itemConfigs, filterFn, this);
            if (itemConfigs){
                menuItems = menuItems.concat(itemConfigs);
            }
            //add all menus and set handlers if needed
            Ext.each(menuItems, function(item){
                //add to zenpack has a different/new handler
                if (item.id === 'addtozenpack'){
                    item.handler = Ext.bind(this.addToZenPackHandler, this);
                }
                if (!Ext.isDefined(item.handler)) {
                    item.handler = Ext.bind(this.defaultHandler, this);
                }

                // do now show as enabled if we only have hidden items (or spacers)
                if (!item.hidden && item !== '-') {
                    visibleMenuCount += 1;
                }
                this.menu.add(item);
            }, this);

            // if we have stuff then enable this control
            if(this.menu.items.length !== 0 && visibleMenuCount){
                this.enable();
            }
        };
        var args = {
            'uid': uid
        };
        if (this.menuIds !== null && this.menuIds.length >= 1){
            args.menuIds = this.menuIds;
        }

        Zenoss.remote.DetailNavRouter.getContextMenus(args, callback, this);
    },

    /**
     * private
     * handler used if a menu item does not have a handler defined
     * @param {Object} button
     */
    defaultHandler: function(button) {
        var dialog = new Zenoss.dialog.DynamicDialog();
        dialog.setTitle(_t(button.text.replace(/\.\.\./g, '')));
        dialog.show();
        dialog.body.load({
            scripts: true,
            url: this.contextUid + '/' + button.viewName
        });
    }
});



})();
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {
     Ext.define("Zenoss.form.parser", {
         alias:['widget.parser'],
         extend:"Ext.form.ComboBox",
         constructor: function(config) {
             var record = config.record;
             config = config || {};
             Ext.applyIf(config, {
                 fieldLabel: _t('Parser'),
                 name: 'parser',
                 editable: false,
                 forceSelection: true,
                 autoSelect: true,
                 triggerAction: 'all',
                 matchFieldWidth: false,
                 listConfig: {
                     resizable: true,
                     minWidth: 250
                 },
                 queryMode: 'local',
                 store: record.availableParsers
             });
             this.callParent(arguments);
         }
     });

}());
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {
     Ext.define("Zenoss.form.rrdtype", {
         alias:['widget.rrdtype'],
         extend:"Ext.form.ComboBox",
         constructor: function(config) {
             var record = config.record;
             config = config || {};
             var store = [];
             Ext.each(record.availableRRDTypes, function(item) {
                 store.push([item]);
             });
             Ext.apply(config, {
                 fieldLabel: _t('RRD Type'),
                 name: 'rrdtype',
                 editable: false,
                 forceSelection: true,
                 autoSelect: true,
                 triggerAction: 'all',
                 queryMode: 'local',
                 displayField: 'name',
                 valueField: 'name',
                 store:  Ext.create('Ext.data.ArrayStore', {
                     model: 'Zenoss.model.Name',
                     data: store
                 })
             });
             this.callParent(arguments);
         }
     });

}());
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {

Ext.ns('Zenoss.form');

Ext.define("Zenoss.form.Password", {
    alias:['widget.password'],
    extend:"Ext.form.TextField",
    constructor: function(config) {
         config.inputType = 'password';
         Zenoss.form.Password.superclass.constructor.apply(this, arguments);
     }
 });


})();
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


/* package level */
(function() {

Ext.ns('Zenoss.form');

Ext.define("Zenoss.form.UpDownField", {
    alias:['widget.updownfield'],
    extend:"Ext.form.ComboBox",
     constructor: function(config) {
         config = Ext.applyIf(config||{}, {
             editable: false,
             forceSelection: true,
             autoSelect: true,
             triggerAction: 'all',
             queryMode: 'local',
             store: [
                 [1, 'Up'],
                 [0, 'Down']
             ]
         });
         Zenoss.form.UpDownField.superclass.constructor.call(this, config);
     }
 });


})();
(function() {
     Ext.namespace('Zenoss.form.Alias');

     /**
      * When you press the "Add" button this will add a new
      * alias row to the panel.
      **/
     function addAliasRow() {
         var cmp = that,
             button = Ext.getCmp('add_alias_button');

         cmp.remove(button);
         cmp.add(getRowTemplate());
         cmp.add(addAliasButton);
         cmp.doLayout();
     }

     /**
      * Variable Definitions
      **/
     var addAliasButton = {
         id: 'add_alias_button',
         xtype: 'button',
         ui: 'dialog-dark',
         text: 'Add',
         handler: addAliasRow
     },
     that;

     /**
      * Returns the panel definition for the id/formula row
      **/
     function getRowTemplate(id, formula) {
         return {
             xtype: 'container',
             bodyCls: 'alias',
             layout: 'hbox',
             items: [{
                 aliasType: 'id',
                 xtype: 'textfield',
                 width:'150',
                 value: id,
                 regex: /^[^\s]+$/,
                 maxLength: 30
             },{
                 aliasType: 'formula',
                 xtype: 'textfield',
                 width:'150',
                 style: {'margin-left':'10px;'},
                 value: formula
             },{
                 xtype: 'button',
                 ui: 'dialog-dark',
                 text: 'Delete',
                 width: '10',
                 style: {'margin-left':'10px;'},
                 handler: function() {
                     var container = this.findParentByType('container'),
                         cmp = that;
                     cmp.remove(container);
                     cmp.doLayout();
                 }
             }]
        };
     }

     /**
      * Alias Panel. NOTE: This control is designed to have
      * only one instance per page.
      **/
     Ext.define("Zenoss.form.Alias", {
         alias:['widget.alias'],
         extend:"Ext.Panel",
         constructor: function(config) {
             var aliases = config.record.aliases,
             items = [],
             i,
             alias;
             // use "that" as a closure so we have a reference to it
             that = this;
             items.push({
                xtype: 'panel',
                layout: 'anchor',
                html: _t('Alias:<br>ID / FORMULA')
             });
             config = config || {};
             // add a row for each alias defined
             for (i=0; i < aliases.length; i++) {
                 alias = aliases[i];
                 items.push(getRowTemplate(alias.name, alias.formula));
             }
             // always show an extra blank row
             items.push(getRowTemplate());

             // add the button
             items.push(addAliasButton);
             Ext.applyIf(config, {
                 items:items
             });

             Zenoss.form.Alias.superclass.constructor.apply(this, arguments);
         },

         /**
          * This returns a list of all of the  aliases in object form.
          * Since this is a Panel it must explicitly be called.
          * Ex:
          *    Ext.getCmp('aliasId').getValue();
          **/
         getValue: function(){
             var cmp = that,
                 textfields = cmp.query('textfield'),
                 results = [],
                 i, field;

             // initialize the return structure. We want an array of object literals
             // (will be dicts in python)
             for (i = 0; i < textfields.length / 2; i++ ) {
                 results[i] = {};
             }

             // turn each entry into an object literal with the properties
             // id and formula
             for (i = 0; i < textfields.length; i++ ) {
                 field = textfields[i];

                 // aliasType was defined on the dynamically created rows above
                 results[Math.floor(i/2)][field.aliasType] = field.getValue();
             }

             return results;
         }
     });

}());
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


(function() {

Ext.ns('Zenoss.form');
/**
 * This is a special case of a text area. It is designed to take up the entire column
 * on a two column layout.
 **/
Ext.define("Zenoss.form.TwoColumnTextArea", {
    alias:['widget.twocolumntextarea'],
    extend:"Ext.form.TextArea",
     constructor: function(config) {
         config.width = 500;
         config.height = 220;
         Zenoss.form.TwoColumnTextArea.superclass.constructor.apply(this, arguments);
     }
 });

})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {

    Ext.ns('Zenoss.form');
    /**
     * This is a display field that takes an array as its value and
     * renders them with one element per line
     **/
    Ext.define("Zenoss.form.MultiLineDisplayField", {
        alias:['widget.multilinedisplayfield'],
        extend:"Ext.form.DisplayField",
        constructor: function(config) {
            // if a string was passed in then defer to the parents behavior
            if ((typeof config.value) !== 'string') {
                config.value = config.value.join('<br />');
            }

            Zenoss.form.MultiLineDisplayField.superclass.constructor.apply(this, arguments);
        }
    });

}());
(function() { // Local scope

Ext.namespace('Zenoss.inspector');


/**
 * Manages inspector windows to ensure that only one window matching
 * `uid` is active at a time.
 */
Zenoss.inspector.SingleInstanceManager = Ext.extend(Object, {
    _instances: null,
    constructor: function() {
        this._instances = {};
    },
    register: function(uid, instance) {
        this.remove(uid);
        this._instances[uid] = instance;
        instance.on('destroy', function() { delete
        this._instances[uid]; }, this);
    },
    get: function(uid) {
        return this._instances[uid];
    },
    remove: function(uid) {
        Ext.destroyMembers(this._instances, uid);
    }
});


/**
 * Represents a single item in the inspector panel.
 *
 * config:
 * - valueTpl An XTemplate that can be used to render the field. Will
 *            be passed the data that is passed to the inspector.
 */
Ext.define('Zenoss.inspector.InspectorProperty', {
    alias: ['widget.inspectorprop'],
    extend: 'Ext.Container',
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            cls: 'inspector-property',
            layout: 'anchor',
            items: [
                {
                    cls: 'inspector-property-label',
                    ref: 'labelItem',
                    xtype: 'label',
                    text: config.label ? config.label + ':' : ''
                },
                {
                    cls: 'inspector-property-value',
                    ref: 'valueItem',
                    text: config.value || '',
                    xtype: 'panel',
                    tpl: config.valueTpl || '{.}'
                }
            ]
        });
        this.callParent(arguments);
    },
    setValue: function(t) {
        this.valueItem.update(t);
    },
    setLabel: function(t) {
        this.labelItem.setText(t + ':');
    }
});


Ext.define('Zenoss.inspector.BaseInspector', {
    extend: 'Ext.Panel',
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            defaultType: 'devdetailitem',
            layout: 'anchor',
            bodyBorder: false,
            items: [],
            titleTpl: '<div class="name">{name}</div>'
        });

        config.items = [
            {
                layout: 'hbox',
                cls: 'inspector-header',
                ref: 'headerItem',
                items: [
                    {
                        cls: 'header-icon',
                        ref: 'iconItem',
                        xtype: 'panel'
                    },
                    {
                        cls: 'header-title',
                        ref: 'titleItem',
                        xtype: 'label',
                        tpl: config.titleTpl
                    }
                ]
            },
            {
                xtype: 'container',
                cls: 'inspector-body',
                autoEl: 'div',
                layout: 'anchor',
                ref: 'bodyItem',
                defaultType: 'inspectorprop',
                items: config.items
            }
        ];

        this.callParent(arguments);
    },
    setIcon: function(url) {
        if (this.headerItem.iconItem.getEl()) {
            this.headerItem.iconItem.getEl().setStyle(
                'background-image', 'url(' + url + ')'
            );
        } else {
            this.headerItem.iconItem.update(Ext.String.format('<img height="43px" src="{0}" />', url));
        }
    },
    /**
     * Overwrite to add any properties dynamically from the data. Must
     * return true if added any.
     */
    addNewDataItems: function() {
        return false;
    },
    update: function(data) {

        if (this.addNewDataItems(data)) {
            this.doLayout();
        }

        // update all the children that have templates
        var self = this;
        this.cascade(function(item) {
            if (item !== self && item.tpl) {
                item.data = data;
                item.update(data);
            }

            return true;
        });
        if (data.icon) {
            this.setIcon(data.icon);
        }

        if (this.ownerCt) {
            this.ownerCt.doLayout();
        }
        else {
            this.doLayout();
        }

    },
    /**
     * Add a property to the inspector panel for display.
     *@param {string} label
     *@param {string} id The key from the data to display.
     */
    addProperty: function(label, id) {
        this.addPropertyTpl(label, '{[values.' + id + ' || ""]}');
    },
    /**
     * Add a property to the inspector panel using a template to display.
     * @param {string} label label of template.
     * @param {string} tpl A string in XTemplate format to display
     * this property. Data values are in `values`.
     */
    addPropertyTpl: function(label, tpl) {
        this.bodyItem.add({
            xtype: 'inspectorprop',
            label: label,
            valueTpl: tpl
        });
    }
});

/**
 * An inspector that gets its data via a directFn remote call.
 */
Zenoss.inspector.DirectInspector = Ext.extend(Zenoss.inspector.BaseInspector, {
    _contextUid: null,
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            directFn: Ext.emptyFn
        });

        Zenoss.inspector.DirectInspector.superclass.constructor.call(this, config);
    },
    initComponent: function() {
        Zenoss.inspector.DirectInspector.superclass.initComponent.call(this);
        this.addEvents('contextchange');
        this.on('contextchange', this._onContextChange, this);
    },
    refresh: function() {
        this.load();
    },
    load: function() {
        if (this._contextUid) {
            this.directFn(
                { uid: this._contextUid, keys: this.keys },
                function(result) {
                    if (result.success) {
                        this.fireEvent('contextchange', result.data, this);
                    }
                },
                this
            );
        }
    },
    setContext: function(uid, load) {
        this._contextUid = uid;
        load = Ext.isDefined(load) ? load : true;

        if (load) {
            this.load();
        }
    },
    _onContextChange: function(data) {
        this.onData(data);
    },
    onData: function(data) {
        this.update(data);
    }
});

Ext.define('Zenoss.inspector.DeviceInspector', {
    alias: ['widget.deviceinspector'],
    extend: 'Zenoss.inspector.DirectInspector',
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            directFn: Zenoss.remote.DeviceRouter.getInfo,
            keys: ['ipAddress', 'device', 'deviceClass'],
            cls: 'inspector',
            titleTpl: '<div class="name"><a href="{uid}" target="_top">{name}</a></div><div class="info">{[Zenoss.render.DeviceClass(values.deviceClass.uid)]}</div><div class="info">{[Zenoss.render.ipAddress(values.ipAddress)]}</div>'
        });
        this.callParent(arguments);

        var url = "{uid}/devicedetail?filter=default#deviceDetailNav:device_events";
        this.addPropertyTpl(_t('Events'), '<div onclick="location.href=\''+url+'\';">{[Zenoss.render.events(values.events, 4)]}</div>');
        this.addPropertyTpl(_t('Device Status'), '{[Zenoss.render.pingStatus(values.status)]}');
        this.addProperty(_t('Production State'), 'productionState');
        this.addPropertyTpl(_t('Location'), '{[(values.location && values.location.name) || ""]}');
    }
});


Ext.define('Zenoss.inspector.ComponentInspector', {
    alias: ['widget.componentinspector'],
    extend: 'Zenoss.inspector.DirectInspector',
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            directFn: Zenoss.remote.DeviceRouter.getInfo,
            keys: ['ipAddress', 'device'],
            cls: 'inspector',
            titleTpl: '<div class="name"><a href="{uid}" target="_top">{name}</a></div><div class="info"><a href="{[values.device.uid]}" target="_top">{[values.device.name]}</a></div><div class="info">{[Zenoss.render.ipAddress(values.ipAddress)]}</div>'
        });

        config.items = [(
            {
                label: _t('Events'),
                valueTpl: '{[Zenoss.render.events(values.events, 4)]}'
            }
        )];

        this.callParent(arguments);
    },
    update: function(data) {
        // our template relies on a device being present
        if (!data.device) {
            data.device = { };
        }
        this.callParent(arguments);
    }
});

var windowManager = new Zenoss.inspector.SingleInstanceManager();
Zenoss.inspector.createWindow = function(uid, xtype, x, y) {

    var win = new Ext.Window({
        x: (x || 0),
        y: (y || 0),
        cls: 'inspector-window',
        plain: true,
        frame: false,
        constrain: true,
        model: false,
        layout: 'fit',
        width: 300,
        items: [{
            xtype: xtype,
            ref: 'panelItem'
        }]
    });

    windowManager.register(uid, win);
    return win;
};

Zenoss.inspector.registeredInspectors = {
    device: 'deviceinspector'
};

Zenoss.inspector.registerInspector = function(inspector_type, inspector_xtype) {
    Zenoss.inspector.registeredInspectors[inspector_type.toLowerCase()] = inspector_xtype;
};

Zenoss.inspector.show = function(uid, x, y) {
    Zenoss.remote.DeviceRouter.getInfo({ uid: uid }, function(result) {
        if (result.success) {
            // Grasping at straws but assume it's a component unless otherwise stated
            var xtype = 'componentinspector';

            var itype = result.data.inspector_type || result.data.meta_type;
            if (itype) {
                itype = itype.toLowerCase();
                if (Zenoss.inspector.registeredInspectors[itype]) {
                    xtype = Zenoss.inspector.registeredInspectors[itype];
                }
            }
            var win = Zenoss.inspector.createWindow(uid, xtype, x, y);
            win.panelItem.setContext(uid, false);
            win.panelItem.update(result.data);
            win.show();
            win.toFront();
        }
    });
};

})();
/* jshint boss:true */
(function() {

    var directStoreWorkaroundListeners = {
        beforedestroy: function() {
            // This is to work around a bug in Sencha 4.x:
            // http://www.sencha.com/forum/archive/index.php/t-136583.html?s=0737c51cf4da51fa8cb875c351c5b2b4
            this.bindStore(null);
        },
        afterrender: function(comp, eOpts) {
            // TODO: why does this get called twice?
            this.getStore().load();
        }
    };


    var ZF = Ext.ns('Zenoss.form.rule'),
         unparen=/^\((.*)\)$/,
         nested=/\)( and | or )\(/,
         conjunctions_inverse = {};

    ZF.CONJUNCTIONS = {
        any: {
            text: _t('any'),
            tpl: ' or '
        },
        all: {
            text: _t('all'),
            tpl: ' and '
        }
    };

    var buttons = function(scope) {
        return [{
            xtype: 'button',
            ref: 'buttonAdd',
            iconCls: 'add',
            handler: function() {
                var realowner = scope.ownerCt,
                idx = realowner.items.indexOf(scope);
                var clause = realowner.insert(idx + 1, {
                    xtype: 'ruleclause',
                    nestedRule: scope,
                    builder: scope.getBuilder()
                });
                realowner.doComponentLayout();
                clause.subject.focus();
            },
            scope: scope
        },{
            xtype: 'button',
            ref: 'buttonDelete',
            iconCls: 'delete',
            handler: function() {
                scope.ownerCt.remove(scope, true /* autoDestroy */);
                scope.getBuilder().fireEvent('rulechange', scope);
            },
            scope: scope
        },{
            xtype: 'button',
            ref: 'buttonSubclause',
            iconCls: 'classify',
            handler: function() {
                var realowner = scope.ownerCt,
                idx = realowner.items.indexOf(scope);
                realowner.insert(idx + 1, {
                    xtype: 'nestedrule',
                    ruleBuilder: scope.getBuilder()
                });
                realowner.doComponentLayout();
            },
            scope: scope
        }];
    };

    ZF.CONJUNCTION_STORE = [];
    for (var conjunction in ZF.CONJUNCTIONS) {
        if (ZF.CONJUNCTIONS.hasOwnProperty(conjunction)) {
            var conj = ZF.CONJUNCTIONS[conjunction];
            ZF.CONJUNCTION_STORE.push([conjunction, conj.text]);
            conjunctions_inverse[Ext.String.trim(conj.tpl)] = conjunction;
        }
    }


    /*
     *  The order of the comparisons in the following object matters. The
     *  templates are used to create regular expressions where the {#} substitutions
     *  get replaced with '(.*)'. When loading a clause, the following object
     *  is iterated over and the first matching regular expression gets used to
     *  parse the clause.
     */
    ZF.COMPARISONS = {
        doesnotcontain: {
            text: _t('does not contain'),
            tpl: '{1} not in {0}'
        },
        doesnotstartwith: {
            text: _t('does not start with'),
            tpl: 'not {0}.startswith({1})'
        },
        doesnotendwith: {
            text: _t('does not end with'),
            tpl: 'not {0}.endswith({1})'
        },
        contains: {
            text: _t('contains'),
            tpl: '{1} in {0}'
        },
        startswith: {
            text: _t('starts with'),
            tpl: '{0}.startswith({1})'
        },
        endswith: {
            text: _t('ends with'),
            tpl: '{0}.endswith({1})'
        },
        equals: {
            text: _t('equals'),
            tpl: '{0} == {1}'
        },
        doesnotequal: {
            text: _t('does not equal'),
            tpl: '{0} != {1}'
        },
        lessthan: {
            text: _t('is less than'),
            tpl: '{0} < {1}',
            field: {xtype: 'numberfield'}
        },
        greaterthan: {
            text: _t('is greater than'),
            tpl: '{0} > {1}',
            field: {xtype: 'numberfield'}
        },
        lessthanorequalto: {
            text: _t('is less than or equal to'),
            tpl: '{0} <= {1}',
            field: {xtype: 'numberfield'}
        },
        greaterthanorequalto: {
            text: _t('is greater than or equal to'),
            tpl: '{0} >= {1}',
            field: {xtype: 'numberfield'}
        }
    };
    ZF.COMPARISON_STORE = [];
    var comparison_patterns = {};
    for (var comparison in ZF.COMPARISONS) {
        if (ZF.COMPARISONS.hasOwnProperty(comparison)) {
            var cmp = ZF.COMPARISONS[comparison];
            ZF.COMPARISON_STORE.push([comparison, cmp.text]);
            comparison_patterns[comparison] = new RegExp(
                cmp.tpl.replace('(', '\\(')
                       .replace(')', '\\)')
                       .xsplit(/\{\d+\}/)
                       .join('(.*)'));
        }
    }

    Ext.define("Zenoss.form.rule.RuleClause", {
        alias:['widget.ruleclause'],
        extend:"Ext.Toolbar",
        constructor: function(config) {
            config = Ext.applyIf(config||{}, {
                cls: 'rule-clause',
                items: [{
                    ref: 'subject',
                    xtype: 'combo',
                    autoSelect: true,
                    allowBlank: false,
                    editable: false,
                    forceSelection: true,
                    triggerAction: 'all',
                    store: [[null,null]],
                    listConfig: {
                        resizable: true,
                        maxWidth:200,
                        maxHeight: 250
                    },
                    getSubject: Ext.bind(function() {
                        return this.getBuilder().subject_map[this.subject.getValue()];
                    }, this),
                    listeners: {
                        change: function() {
                            // when opening a second window this
                            // change fires before the component is rendered
                            if (!this.subject) {
                                return;
                            }
                            // Get the associated subject
                            var subject = this.subject.getSubject(),
                                comparisons = [];
                            this.comparison.reset();
                            // Update comparisons
                            if (subject.comparisons) {
                                Ext.each(subject.comparisons, function(cmp) {
                                    var c = ZF.COMPARISONS[cmp];
                                    if (!c) {
                                        return;
                                    }
                                    comparisons.push([cmp, c.text]);
                                });
                            } else {
                                comparisons = ZF.COMPARISON_STORE;
                            }

                            this.comparison.store.loadData(comparisons);
                            this.comparison.setValue(comparisons[0][0]);
                            this.getBuilder().fireEvent(
                                'rulechange',
                                this
                            );
                        },
                        scope: this
                    }
                },{
                    ref: 'comparison',
                    xtype: 'combo',
                    autoSelect: true,
                    editable: false,
                    allowBlank: false,
                    store: ZF.COMPARISON_STORE,
                    value: ZF.COMPARISON_STORE[0][0],
                    forceSelection: true,
                    triggerAction: 'all',
                    listConfig: {
                        maxWidth:200
                    },
                    listeners: {
                        change: function() {
                            var cmp = ZF.COMPARISONS[this.comparison.getValue()],
                                field = this.subject.getSubject().field || (cmp && cmp.field) || {xtype:'textfield'},
                                idx = Ext.Array.indexOf(this.items.items, this.predicate),
                                oldvalue = this.predicate.getValue(),
                                oldxtype = this.predicate.xtype;
                            this.remove(this.predicate);
                            this.insert(idx, Ext.apply({
                                ref: 'predicate',
                                allowBlank: false,
                                width: 150,
                                listeners: {
                                    change: function() {
                                        this.getBuilder().fireEvent(
                                            'rulechange',
                                            this
                                        );
                                    },
                                    scope: this
                                }
                            }, field));
                            if (typeof oldvalue !== 'undefined' && this.predicate.xtype === oldxtype) {
                                this.predicate.setValue(oldvalue);
                            }
                            this.doComponentLayout();
                            this.getBuilder().fireEvent(
                                'rulechange',
                                this
                            );
                            this.predicate.focus();
                        },
                        scope: this
                    }
                },{
                    ref: 'predicate',
                    xtype: 'textfield',
                    width: 150,
                    listeners: {
                        change: function() {
                            this.getBuilder().fireEvent(
                                'rulechange',
                                this
                            );
                        },
                        scope: this
                    }
                },'->']
            });
            Ext.each(buttons(this), function(btn) {
                config.items.push(btn);
            });
            this.callParent([config]);
            var subjects = this.getBuilder().subject_store;
            this.subject.store.loadData(subjects);
            this.subject.setValue(subjects[0][0]);
            this.on('added', function(){
                this.getBuilder().fireEvent('rulechange', this);
            }, this);
        },
        getValue: function() {
            var comparison = this.comparison.getValue(),
                sub = this.subject.getValue(),
                pred = this.predicate.getValue();
            if (!comparison || !sub || Ext.isEmpty(pred)) { return; }
            var cmp = ZF.COMPARISONS[comparison];
            var clause = Ext.String.format(cmp.tpl, this.getBuilder().prefix + sub, Ext.encode(pred));

            var clause_code = Ext.String.format("({0})", clause);
            if (sub.startsWith(Zenoss.env.ZP_DETAILS_TRIGGER_PREFIX)) {
                // for custom details we add a check to ensure the detail is present
                var replace_text = Zenoss.env.ZP_DETAILS_TRIGGER_PREFIX + '.';
                var detail = sub.replace(replace_text, '');
                var exists_code = Ext.String.format("hasattr({0}, \"{1}\")", Zenoss.env.ZP_DETAILS_TRIGGER_PREFIX, detail);
                clause_code = Ext.String.format("({0} and {1})", exists_code, clause);
            }
            return clause_code;
        },
        setValue: function(expression) {
            var matchOperator = false;

            // If the clause is for a custom zenpack detail, we need to remove
            // the hasattr condition
            if (expression.startsWith("hasattr")) {
                expression = expression.replace(/hasattr.* and /, '');
            }
            for (var cmp in comparison_patterns) {
                if (comparison_patterns.hasOwnProperty(cmp)) {
                    var pat = comparison_patterns[cmp];
                    if (pat.test(expression)) {
                        var vals = pat.exec(expression).slice(1),
                            spots = pat.exec(ZF.COMPARISONS[cmp].tpl).slice(1),
                            sorted = Ext.zip.apply(this, Ext.zip(spots, vals).sort())[1],
                            subject = sorted[0],
                            value = sorted[1],
                            cleansub = subject.replace(
                                new RegExp("^"+this.getBuilder().prefix), '');

                        try {
                            this.subject.setValue(cleansub);
                        } catch(e) {
                            Zenoss.message.warning("Invalid rule '"+ expression +"'. could not find subject '"+ cleansub +"'");
                            return;
                        }

                        this.comparison.setValue(cmp);
                        this.predicate.setValue(Ext.decode(value));
                        matchOperator = true;

                        break;
                    }
                }
            }
            // loop's over but we didn't find a comparison operator
            // that matched :(
            if(!matchOperator){
                Zenoss.message.warning("Invalid rule '"+ expression +"'. Could not match comparison operator");
            }

        },
        getBuilder: function() {
            if (!this.builder) {
                this.builder = this.nestedRule.ruleBuilder || this.findParentByType('rulebuilder', true);
            }
            return this.builder;
        }
    });



    ZF.changeListener = function(me, cmp) {
        var items = this.items.items;

        // If we're somewhere in the middle of initialization don't do anything
        if (items.length===0) {
            return;
        }

        var item = items[0],
            delbtn = item instanceof ZF.NestedRule ?
                     item.items.items[0].buttonDelete :
                     item.buttonDelete;

        // Disable the delete button if only one at this level
        if (items.length===1) {
            delbtn.disable();
        } else if (items.length > 1){
            delbtn.enable();
        }

        // Disable the nested button if it would mean more than 4 deep
        var i = 0, btn;

        while (item = item.findParentByType('nestedrule')) {
            i++;
            if (i>=4) {
                if (btn = cmp.buttonSubclause) {
                    btn.disable();
                }
                break;
            }
        }

    };

    Ext.define("Zenoss.form.rule.NestedRule", {
        alias:['widget.nestedrule'],
        extend:"Ext.Container",
        constructor: function(config) {
            config = Ext.applyIf(config||{}, {
                showButtons: true,
                cls: 'nested-rule',
                items: [{
                    xtype: 'toolbar',
                    cls: 'rule-clause nested-rule-header',

                    items: [{
                        ref: '../conjunction',
                        xtype: 'combo',
                        width: 60,
                        store: ZF.CONJUNCTION_STORE,
                        editable: false,
                        allowBlank: false,
                        forceSelection: true,
                        triggerAction: 'all',
                        value: 'all',
                        listConfig: {
                            maxWidth:60
                        },
                        listeners: {
                            change: function() {
                                this.getBuilder().fireEvent(
                                    'rulechange',
                                    this
                                );
                            },
                            scope: this
                        }
                    },{
                        xtype: 'label',
                        html: _t('of the following rules:'),
                        style: 'margin-left: 7px; font-size: 12px; color: #444'
                    }]
                },{
                    ref: 'clauses',
                    xtype: 'container',
                    cls: 'rule-clause-container',
                    items: {
                        xtype: 'ruleclause',
                        nestedRule: this
                    },
                    listeners: {
                        add: ZF.changeListener,
                        remove: ZF.changeListener
                    }
                }]
            });
            if (config.showButtons) {
                var items = config.items[0].items;
                items.push('->');
                Ext.each(buttons(this), function(btn) {
                    items.push(btn);
                });
            }
            this.callParent([config]);
        },
        getBuilder: function() {
            if (!this.builder) {
                this.builder = this.findParentByType('rulebuilder', true);
            }
            return this.builder;
        },
        getValue: function() {
            var values = [],
                joiner = ZF.CONJUNCTIONS[this.conjunction.getValue()].tpl,
                result;
            Ext.each(this.clauses.items.items, function(clause) {
                var value = clause.getValue();
                if (value) {
                    values.push(value);
                }
            }, this);
            result = values.join(joiner);
            if (values.length > 1) {
                result = Ext.String.format('({0})', result);
            }
            return result;
        },
        setValue: function(expression) {
            var c, q, i=0, p=0, tokens=[], token=[],
                funcflag=false;
            c = expression.charAt(i);
            var savetoken = function() {
                var v = Ext.String.trim(token.join(''));
                if (v) {
                    tokens.push(v);
                }
                token = [];
            };
            while (c) {
                //token.push(c);
                // Don't deal with contents of string literals
                if (c==='"'||c==='\'') {
                    q = c;
                    token.push(c);
                    for (;;) {
                        i++;
                        c = expression.charAt(i);
                        token.push(c);
                        if (c===q) {
                            // Closing quote
                            break;
                        }
                        if (c === '\\') {
                            // Skip escaped chars
                            i++;
                            token.push(expression.charAt(i));
                        }
                    }
                } else if (c==="("||c===")"){
                    if (p===0) {
                        savetoken();
                    }
                    token.push(c);
                    if (c==='(') {
                        var prev = expression.charAt(i-1);
                        if (i>0 && prev!==' ' && prev!=='(') {
                            funcflag = true;
                        } else {
                            p++;
                        }
                    } else if (c===')') {
                        if (funcflag) {
                            funcflag = false;
                        } else {
                            p--;
                        }
                    }
                    if (p===0) {
                        savetoken();
                    }
                } else {
                    token.push(c);
                }
                i++;
                c = expression.charAt(i);
            }
            savetoken();

            if (tokens) {
                this.clauses.removeAll();
                var conjunction, rule;

                Ext.each(tokens, function(t) {
                    if (t) {
                        if (conjunction = conjunctions_inverse[t]){
                             this.conjunction.setValue(conjunction);
                             return;
                        } else if (nested.test(t)) {
                            // Nested rule
                            rule = this.clauses.add({xtype:'nestedrule', ruleBuilder: this.ruleBuilder});
                        } else {
                            // Clause
                            rule = this.clauses.add({xtype: 'ruleclause', nestedRule: this});
                        }
                        var clause = t;
                        try {
                            clause = unparen.exec(t)[1];
                        } catch(ignored) {}
                        rule.setValue(clause);
                    }
                }, this);
            }
        }
    });



    Ext.define("Zenoss.form.rule.RuleBuilder", {
        alias:['widget.rulebuilder'],
        extend:"Ext.form.FieldContainer",
        constructor: function(config) {
            config = Ext.applyIf(config||{}, {
                cls: 'rule-builder',
                prefix: '',
                items: [{
                    ref: 'rootrule',
                    xtype: 'nestedrule',
                    showButtons: false,
                    ruleBuilder: this
                }]
            });
            this.subject_store = [];
            this.subject_map = {};
            Ext.each(config.subjects, function(subject) {
                if (!Ext.isObject(subject)) {
                    subject = {value: subject};
                }
                this.subject_store.push([subject.value, subject.text || subject.value]);
                this.subject_map[subject.value] = subject;
            }, this);
            this.callParent([config]);
            this.addEvents('rulechange');
        },
        getValue: function() {
            var result = this.rootrule.getValue();
            if (result && nested.test(result)) {
                // There will be one extra paren set wrapping the clause as a
                // whole that is hard to prevent earlier than this; don't save
                // it, as it will be treated as an unnecessary nested rule
                result = unparen.exec(result)[1];
            }
            return result;
        },
        setValue: function(expression) {
            if (!expression) {
                this.reset();
            } else {
                this.rootrule.setValue(expression);
            }
            this.doComponentLayout();
        },
        reset: function() {
            this.rootrule.clauses.removeAll();
            this.rootrule.clauses.add({
                xtype: 'ruleclause',
                nestedRule: this.rootrule,
                builder: this
            });
        }
    });

    Ext.define("Zenoss.form.rule.DeviceCombo",{
            extend: "Zenoss.form.SmartCombo",
            alias: ["widget.rule.devicecombo"],
            constructor: function(config){
                config = Ext.applyIf(config || {}, {
                    queryMode: 'remote',
                    directFn: Zenoss.remote.DeviceRouter.getDeviceUuidsByName,
                    root: 'data',
                    model: 'Zenoss.model.BasicUUID',
                    remoteFilter: true,
                    minChars: 3,
                    typeAhead: true,
                    valueField: 'uuid',
                    displayField: 'name',
                    forceSelection: true,
                    triggerAction: 'all',
                    editable: true,
                    autoLoad: false
                });
                this.callParent([config]);
            },

            setValue: function() {
                var uuid = arguments[0];
                // uuid can be undefined, an empty string, a string containing the uuid,
                // or a BasicUUID object.  Force a reload if it is a uuid-containing string
                // and the corresponding device is not already in the combobox dropdown.
                if (uuid && Ext.isString(uuid)) {
                    this.getStore().setBaseParam("uuid", uuid);
                    if (this.getStore().getById(uuid) === null) {
                        this.getStore().load();
                    }
                }
                return this.callParent(arguments);
            }
        }
    );


    ZF.STRINGCOMPARISONS = [
        'contains',
        'doesnotcontain',
        'startswith',
        'doesnotstartwith',
        'endswith',
        'doesnotendwith',
        'equals',
        'doesnotequal'
    ];

    ZF.NUMBERCOMPARISONS = [
        'equals',
        'doesnotequal',
        'lessthan',
        'greaterthan',
        'lessthanorequalto',
        'greaterthanorequalto'
    ];

    ZF.IDENTITYCOMPARISONS = [
        'equals',
        'doesnotequal'
    ];

    ZF.LISTCOMPARISONS = [
        'contains',
        'doesnotcontain'
    ];


    Ext.apply(ZF, {
        EVENTSEVERITY: {
            text: _t('Severity'),
            value: 'severity',
            comparisons: ZF.NUMBERCOMPARISONS,
            field: {
                xtype: 'combo',
                queryMode: 'local',
                valueField: 'value',
                displayField: 'name',
                typeAhead: false,
                forceSelection: true,
                triggerAction: 'all',
                listConfig: {
                    maxWidth:200
                },
                store: new Ext.data.ArrayStore({
                    model: 'Zenoss.model.NameValue',
                    data: [[
                        _t('Critical'), 5
                    ],[
                        _t('Error'), 4
                    ],[
                        _t('Warning'), 3
                    ],[
                        _t('Info'), 2
                    ],[
                        _t('Debug'), 1
                    ],[
                        _t('Clear'), 0
                    ]]
                })
            }
        },
        PRODUCTIONSTATE: {
            text: _t('Production state'),
            value: 'productionState',
            field: {
                xtype: 'ProductionStateCombo',
                listConfig: {
                    maxWidth:200
                }
            },
            comparisons: ZF.NUMBERCOMPARISONS
        },
        DEVICEPRIORITY: {
            text: _t('Device priority'),
            value: 'priority',
            field: {
                xtype: 'PriorityCombo',
                listConfig: {
                    maxWidth:200
                }
            },
            comparisons: ZF.NUMBERCOMPARISONS
        },
        DEVICE: {
            text: _t('Device'),
            value: 'device_uuid',
            comparisons: ZF.IDENTITYCOMPARISONS,
            field: {
                xtype: 'rule.devicecombo',
                listConfig: {
                    maxWidth:200
                }
            }
        },
        DEVICECLASS: {
            text: _t('Device Class'),
            value: 'dev.device_class',
            comparisons: ZF.STRINGCOMPARISONS,
            field: {
                xtype: 'smartcombo',
                defaultListConfig: {
                    maxWidth:200
                },
                fields: ['name'],
                directFn: Zenoss.remote.DeviceRouter.getDeviceClasses,
                root: 'deviceClasses',
                listeners: directStoreWorkaroundListeners,
                typeAhead: true,
                editable: true,
                valueField: 'name',
                displayField: 'name',
                forceSelection: true
            }
        },
        SYSTEMS: {
            text: _t('Systems'),
            value: 'dev.systems',
            comparisons: ZF.LISTCOMPARISONS,
            field: {
                xtype: 'smartcombo',
                defaultListConfig: {
                    maxWidth:200
                },
                directFn: Zenoss.remote.DeviceRouter.getSystems,
                root: 'systems',
                fields: ['name'],
                listeners: directStoreWorkaroundListeners,
                typeAhead: true,
                editable: true,
                valueField: 'name',
                displayField: 'name',
                forceSelection: false
            }
        },
        DEVICEGROUPS: {
            text: _t('Device Groups'),
            value: 'dev.groups',
            comparisons: ZF.LISTCOMPARISONS,
            field: {
                xtype: 'smartcombo',
                defaultListConfig: {
                    maxWidth:200
                },
                directFn: Zenoss.remote.DeviceRouter.getGroups,
                root: 'groups',
                fields: ['name'],
                listeners: directStoreWorkaroundListeners,
                typeAhead: true,
                editable: true,
                valueField: 'name',
                displayField: 'name',
                forceSelection: false
            }
        }
    });
})();
Ext.onReady(function() {
    Ext.ns('Zenoss.eventdetail');

    /**
     * This property is used by ZenPacks and other things to specify custom
     * renderers for fields in the detail panel before the panel has
     * instantiated. Once instantiated, the panel will apply any renderers
     * found here.
     */
    if (!Zenoss.hasOwnProperty('event_detail_custom_renderers')) {
        Zenoss.event_detail_custom_renderers = {};
    }

    /**
     * This property is by ZenPacks and other things to specify custom sections
     * for the detail panel before the panel has been instantiated. The section
     * config must specify a `section_class` property which is the string name
     * of a section class ('Section', 'RepeatedSection', etc.).
     *
     * A section config may also specify `renderers` which will specify a custom
     * renderer for a field. This will override the renderer for that field for
     * the entire detail panel.
     *
     * If a section config specifies a `title` property, the title will be used
     * to toggle display of the section.
     */
    if (!Zenoss.hasOwnProperty('event_detail_custom_sections')) {
        Zenoss.event_detail_custom_sections = {};
    }


    /**
     * The header used for the top of the event detail pane.
     * WAS Zenoss.eventdetail.detail_table_template
     */
    Zenoss.eventdetail.detail_header_template = ['<table width="99%" id="evdetail_props_table">',
        '<tr><td class="dt">',_t('Resource:'),'</td>',
            '<td>',
                '<tpl if="device">',
                    '{device}',
                '</tpl>',
            '</td>',
        '</tr>',
        '<tr><td class="dt">',_t('Component:'),'</td>',
            '<td>',
                '<tpl if="component">',
                    '{component}',
                '</tpl>',
            '</td>',
        '</tr>',
        '<tr><td class="dt">',_t('Event Class:'),'</td>',
            '<td>',
                '<tpl if="eventClass">',
                    '{eventClass}',
                '</tpl>',
            '</td>',
        '</tr>',
        '<tr><td class="dt">',_t('Status:'),'</td> <td>{eventState}</td></tr>',
        '<tr><td class="dt" valign="top">',_t('Message:'),'</td> <td id="event_details_event_message"><div class="event_message">{message}</div></td></tr>',
    '</table><div style="clear:both;"></div>'];

    /**
     * The template used for regular event properties.
     * WAS: Zenoss.eventdetail.fullprop_table_template
     */
    Zenoss.eventdetail.detail_data_template = ['<table class="proptable" width="100%">',
        '<tpl for="properties">',
        '<tr class=\'{[xindex % 2 === 0 ? "even" : "odd"]}\'>',
        '<td class="proptable_key">{key}</td>',
        '<td class="proptable_value"><div>{value}</div></td></tr>',
        '</tpl>',
        '</table>'];

    /**
     * Template for log messages.
     * WAS: Zenoss.eventdetail.log_table_template
     */
    Zenoss.eventdetail.detail_log_template = ['<table>',
        '<tpl for="log">',
        '<tr><td class="time">{0} {1}: </td>',
            '<td class="message">{2}</td></tr>',
        '</tpl>',
        '</table>'];


    /**
     * This class will generate HTML based on a template that simply uses named
     * properties:
     *
     *      "<p>{my_key}</p>"
     *
     */
    Zenoss.eventdetail.Section = Ext.extend(Object, {
        constructor: function(config){
            Ext.applyIf(config || {}, {
                template: Zenoss.eventdetail.detail_data_template
            });
            Ext.apply(this, config);

            Zenoss.eventdetail.Section.superclass.constructor.apply(this, arguments);
        },

        /**
         * A section is asked to generate its own HTML using this method.
         *
         * @param renderedData This is the event data after each field has been
         *                     rendered according to any renderers specified.
         *                     This contains all rendered data, not just the data
         *                     for the keys specified in this section.
         * @param eventData This is the raw event data. This is made available to
         *                  the section so that it may use it for whatever it wants.
         */
        generateHtml: function(renderedData, eventData) {
            var template = new Ext.XTemplate(this.template),
                props = {};
            Ext.each(this.keys, function(key) {
                props[key] = renderedData[key];
            });
            return template.apply(props);
        }
    });


    /**
     * This class will generate HTML based on a template that utilizes 'for'
     * and iterates over data on the 'properties' property:
     *
     *  "<tpl for="properties">
     *      <p>{key}: {value}</p>
     *   </tpl>"
     *
     */
    Zenoss.eventdetail.RepeatedSection = Ext.extend(Zenoss.eventdetail.Section, {
        constructor: function(config) {
            Ext.applyIf(config || {} , {
                generateHtml: function(renderedData, eventData) {
                    var template = new Ext.XTemplate(this.template),
                        props = {
                            properties: []
                        };
                    Ext.each(this.keys, function(key) {
                        props.properties.push({
                            key: key,
                            value: renderedData[key]
                        });
                    });
                    return template.apply(props);
                }
            });
            Zenoss.eventdetail.RepeatedSection.superclass.constructor.apply(this, arguments);
        }
    });


    /**
     * This special details section knows how to iterate over event details. Any
     * keys specified will be looked for in an event's details data.
     */
    Zenoss.eventdetail.DetailsSection = Ext.extend(Zenoss.eventdetail.RepeatedSection, {
        constructor: function(config) {
            Ext.applyIf(config || {} , {
                generateHtml: function(renderedData, eventData) {
                    var template = new Ext.XTemplate(this.template),
                        props = {
                            properties: renderedData.details
                        },
                        details = [];

                    if (this.hasOwnProperty('keys')) {
                        Ext.each(this.keys, function(key) {
                            Ext.each(renderedData.details, function(detail) {
                                if (detail.key === key) {
                                    details.push(detail);
                                }
                            }, this);
                        }, this);
                        props.properties = details;
                    }

                    return template.apply(props);
                }
            });
            Zenoss.eventdetail.DetailsSection.superclass.constructor.apply(this, arguments);
        }
    });


    /**
     * This panel represents the event detail panel. An initial "zenoss" config
     * is automatically loaded during instantiation in the `init` method.
     */
    Ext.define("Zenoss.DetailPanel", {
        extend:"Ext.Panel",
        alias: "widget.detailpanel",
        isHistory: false,
        layout: 'border',
        constructor: function(config){
            this.sections = [];
            this.renderers = {};
            config.onDetailHide = config.onDetailHide || Ext.emptyFn;
            config.items = [
                // Details Toolbar
                {
                    id: 'evdetail_hd',
                    region: 'north',
                    layout: 'border',
                    height: 50,
                    cls: 'evdetail_hd',
                    items: [{
                        region: 'west',
                        width: 77,
                        height:47,
                        defaults:{height:47},
                        layout: 'hbox',
                        items: [{
                            id: 'severity-icon',
                            cls: 'severity-icon'
                        },{
                            id: 'evdetail-sep',
                            cls: 'evdetail-sep'
                        }]
                    }, {
                        region: 'center',
                        id: 'evdetail-summary',
                        html: ''
                    },{
                        region: 'east',
                        id: 'evdetail-tools',
                        layout: 'hbox',
                        width: 57,
                        items: [{
                            id: 'evdetail-popout',
                            cls: 'evdetail-popout'
                        },{
                            id: 'evdetail_tool_close',
                            cls: 'evdetail_close'
                        }]
                    }]
                },

                // Details Body
                {
                    id: 'evdetail_bd',
                    region: 'center',
                    width: "90%",
                    autoScroll: true,
                    cls: 'evdetail_bd',
                    items: [   {
                        xtype:'toolbar',
                        width: "105%",
                        style: {
                            position: "relative",
                            top: -1
                        },
                        hidden: !config.showActions,
                        id: 'actiontoolbar',
                        items:[ {
                                xtype: 'tbtext',
                                text: _t('Event Actions:')
                            },
                            Zenoss.events.EventPanelToolbarActions.acknowledge,
                            Zenoss.events.EventPanelToolbarActions.close,
                            Zenoss.events.EventPanelToolbarActions.reopen,
                            Zenoss.events.EventPanelToolbarActions.unclose

                        ]
                    },
                    {
                        id: 'event_detail_properties',
                        frame: false,
                        defaults: {
                            frame: false
                        },
                        layout: {
                            type: 'table',
                            columns: 1,
                            tableAttrs: {
                                style: {
                                    width: '90%'
                                }
                            }
                        }
                    },

                    // Event Log Header
                    {
                        id: 'evdetail-log-header',
                        cls: 'evdetail-log-header',
                        hidden: false,
                        html: '<'+'hr/><'+'h2>LOG<'+'/h2>'
                    },

                    // Event Audit Form
                    {
                        xtype: 'form',
                        id: 'log-container',
                        width: '90%',
                        layout: {
                            type: 'table',
                            columns: 1
                        },
                        style: {'margin-left':'3em'},
                        hidden: false,
                        fieldDefaults: {
                            labelWidth: 1
                        },
                        items: [{
                            id: 'detail-logform-evid',
                            xtype: 'hidden',
                            name: 'evid'
                        },{
                            style: 'margin:0.75em',
                            width: 300,
                            xtype: 'textfield',
                            name: 'message',
                            listeners: {
                                specialkey: function(formEl, e){
                                    if (e.getCharCode() == Ext.EventObject.ENTER) {
                                        this.up('form').down('button').handler()
                                    }
                                }
                            },
                            hidden: Zenoss.Security.doesNotHavePermission('Manage Events'),
                            id: 'detail-logform-message'
                        },{
                            xtype: 'button',
                            type: 'submit',
                            name: 'add',
                            hidden: Zenoss.Security.doesNotHavePermission('Manage Events'),
                            text: 'Add',
                            handler: function(btn, e) {
                                var form = Ext.getCmp('log-container'),
                                vals = form.getForm().getValues(),
                                params = {
                                    evid: Ext.getCmp('')
                                };
                                Ext.apply(params, vals);
                                Zenoss.remote.EventsRouter.write_log(
                                    params,
                                    function(provider, response){
                                        Ext.getCmp(
                                            'detail-logform-message').setRawValue('');
                                        Ext.getCmp(config.id).refresh();
                                    });
                            }
                        }]
                    },

                    // Event Log Content
                    {
                        id: 'evdetail_log',
                        cls: 'log-content',
                        hidden: false,
                        autoScroll: true,
                        height: 200
                    }
                    ]
                }

            ];

            this.callParent([config]);
            this.init();
        },
        init: function() {
            var default_renderers = {
                device: function(value, sourceData) {
                    var val = sourceData.device_title;
                    if (sourceData.device_url) {
                        val = Zenoss.render.default_uid_renderer(
                            sourceData.device_url,
                            sourceData.device_title);
                    }
                    return val;
                },
                component: function(value, sourceData) {
                    var val = Ext.htmlEncode(sourceData.component_title);
                    if (sourceData.component_url) {
                        val = Zenoss.render.default_uid_renderer(
                            sourceData.component_url,
                            sourceData.component_title);
                    }
                    return val;
                },
                eventClass: function(value, sourceData) {
                    return  Zenoss.render.EventClass(
                        sourceData.eventClass_url,
                        sourceData.eventClass
                    );
                },
                eventClassMapping: function(value, sourceData) {
                    if (sourceData.eventClassMapping_url) {
                        return Zenoss.render.link(null,
                            sourceData.eventClassMapping_url,
                            sourceData.eventClassMapping
                        );
                    }
                },
                Systems: function(value, sourceData) {
                    return Zenoss.render.LinkFromGridUidGroup(value);
                },


                /* We don't totally control what the source looks like, so escape any HTML*/
                eventState: function(value, sourceData) {
                    return Ext.htmlEncode(value);
                },
                summary: function(value, sourceData) {
                    return Zenoss.render.conditionalEscaping(value);
                },
                dedupid: function(value, sourceData) {
                    return Ext.htmlEncode(value);
                },
                message: function(value, sourceData) {
                    return Zenoss.render.conditionalEscaping(value);
                },
                eventClassKey: function(value, sourceData) {
                    return Ext.htmlEncode(value);
                },
                DeviceGroups: function(value, sourceData) {
                    return Zenoss.render.LinkFromGridUidGroup(value);
                },
                DeviceClass: function(value, sourceData) {
                    return Zenoss.render.LinkFromGridUidGroup(value);
                },
                Location: function(value, sourceData) {
                    return Zenoss.render.LinkFromGridUidGroup(value);
                }

            };
            Ext.apply(this.renderers, default_renderers);

            var eventInfoSection = new Zenoss.eventdetail.Section({
                id: "evdetail_props",
                cls: 'evdetail_props',
                template: Zenoss.eventdetail.detail_header_template,
                keys: ['device', 'component', 'eventClass', 'eventState', 'message']
            });
            this.addSection(eventInfoSection);

            var eventManagementSection = new Zenoss.eventdetail.RepeatedSection({
                id: "event_detail_management_section",
                title: _t("Event Management"),
                template: Zenoss.eventdetail.detail_data_template,
                keys: [ 'agent', 'component', 'dedupid', 'eventClass', 'eventClassKey',
                        'eventClassMapping', 'eventGroup', 'eventKey', 'eventState', 'evid',
                        'facility', 'message', 'ntevid', 'priority', 'severity', 'summary'
                ]
            });
            this.addSection(eventManagementSection);

            var deviceStateSection = new Zenoss.eventdetail.RepeatedSection({
                id: 'event_detail_device_state_section',
                title: _t('Device State'),
                template: Zenoss.eventdetail.detail_data_template,
                keys: [
                    'DeviceClass', 'DeviceGroups', 'DevicePriority',
                    'Location', 'Systems', 'device', 'ipAddress',
                    'monitor', 'prodState'
                ]
            });
            this.addSection(deviceStateSection);

            var eventMetaSection = new Zenoss.eventdetail.RepeatedSection({
                id: 'event_detail_meta_section',
                title: _t('Event Data'),
                template: Zenoss.eventdetail.detail_data_template,
                keys: [
                    'clearid', 'count', 'firstTime', 'lastTime',
                    'ownerid', 'stateChange'
                ]
            });
            this.addSection(eventMetaSection);

            var eventDetailsSection = new Zenoss.eventdetail.DetailsSection({
                id: 'event_detail_details_section',
                title: _t('Event Details'),
                template: Zenoss.eventdetail.detail_data_template
            });
            this.addSection(eventDetailsSection);

            this.checkCustomizations();
        },

        checkCustomizations: function() {
            // Apply any custom renderers that were registered before we loaded
            // the 'stock' sections and renderers.
            Ext.apply(this.renderers, Zenoss.event_detail_custom_renderers);

            // Add any sections that were registered before we loaded completely.
            Ext.each(Zenoss.event_detail_custom_sections, function(section) {
                if (section.hasOwnProperty('section_class')) {
                    this.addSection(section);
                }
            }, this);

            this.doLayout();
        },

        getBody: function() {
            return Ext.getCmp('event_detail_properties');
        },

        addSection: function(section) {
            if (section.hasOwnProperty('renderers')) {
                Ext.apply(this.renderers, section.renderers);
            }

            this.sections.push(section);

            if (section.hasOwnProperty('title')) {
                var section_title_config = {
                    id: section.id + '_title',
                    html: "<a href='#'>" + section.title + '...' + "</a>",
                    cls: 'show_details',
                    height: 30,
                    toggleFn: Ext.bind(this.toggleSection, this, [section.id])
                };
                this.getBody().add(section_title_config);
            }



            var content_cls = 'full_event_props';
            if (section.hasOwnProperty('cls')) {
                content_cls = section.cls;
            }
            var section_content_config = {
                layout: 'fit',
                id: section.id,
                hidden: false,
                cls: content_cls,
                // dummy html
                html: ''
            };
            this.getBody().add(section_content_config);
        },

        removeSection: function(section_id) {
            var remove_idx;
            Ext.each(this.sections, function(item, idx, sections) {
                if (item.id === section_id) {
                    remove_idx = idx;
                }
            });
            this.sections.splice(remove_idx, 1);

            Ext.getCmp(section_id).destroy();
            Ext.getCmp(section_id+'_title').destroy();
        },

        hideSection: function(section_id) {
            Ext.getCmp(section_id).hide();
        },

        showSection: function(section_id) {
            Ext.getCmp(section_id).show();
        },

        toggleSection: function(section_id) {
            var state = Ext.state.Manager.get("evDetailSectionsState", {});
            var cmp = Ext.getCmp(section_id), el = cmp.getEl();
            // workaround IE not hiding/showing sections by explicitly setting the style on the dom nodes
            if (el.dom.style.display === "none") {
                el.dom.style.display = "block";
                state[section_id] = true;
            }
            else {
                el.dom.style.display = "none";
                state[section_id] = false;
            }
            Ext.state.Manager.set("evDetailSectionsState", state);
            this.getBody().doLayout();
        },

        findSection: function(section_id) {
            var section;
            Ext.each(this.sections, function(item) {
                if (item.id === section_id) {
                    section = item;
                }
            });
            return section;
        },

        /**
         * This method will iterate over every property of the raw event data
         * and call extractData for that key.
         *
         * @param eventData The raw event data.
         */
        renderData: function(eventData) {
            var renderedData = {};
            Ext.iterate(eventData, function(key) {
                if (key === 'details') {
                    var detailsData = [];
                    Ext.each(eventData[key], function(item) {
                        var val = this.extractData(item.key, item.value, eventData);
                        detailsData.push({
                            key: item.key,
                            value: val
                        });
                    }, this);
                    renderedData[key] = detailsData;
                }
                else {
                    renderedData[key] = this.extractData(key, eventData[key], eventData);
                }
            }, this);

            return renderedData;
        },

        /**
         * Extract rendered data from raw event data. Handles the case where a
         * key does not have a specified renderer as well as the case where
         * the value is null or undefined.
         *
         * @param key The key to use for looking up a renderer.
         * @param value The value to be rendered.
         * @param sourceData All event data.
         */
        extractData: function(key, value, sourceData) {
            var data;
            if (this.renderers.hasOwnProperty(key)) {
                data = this.renderers[key](value, sourceData);
            }
            else if (value || typeof(value) == 'number') {
                data = value;
            }
            else {
                data = '';
            }
            return data;
        },

        setSummary: function(summary){
            var panel = Ext.getCmp('evdetail-summary');
            if (panel && panel.el){
                panel.update(summary);
            }
        },

        setSeverityIcon: function(severity){
            var panel = Ext.getCmp('severity-icon');
            this.clearSeverityIcon();
            panel.addCls(severity);
        },

        clearSeverityIcon: function() {
            var panel = Ext.getCmp('severity-icon');
            Ext.each(Zenoss.env.SEVERITIES, function(sev) {
                sev = sev[1];
                panel.removeCls(sev.toLowerCase());
            });
        },

        update: function(eventData) {
            // render event data
            eventData.firstTime = Zenoss.date.renderWithTimeZone(eventData.firstTime);
            eventData.lastTime = Zenoss.date.renderWithTimeZone(eventData.lastTime);
            eventData.stateChange = Zenoss.date.renderWithTimeZone(eventData.stateChange);
            // Convert log timestamps to localized dates
            eventData.log = eventData.log.map(
                function(item) {
                    item[1] = Zenoss.date.renderWithTimeZone(item[1]/1000);
                    return item;
                }
            )

            var renderedData = this.renderData(eventData);

            this.setSummary(renderedData.summary);
            this.setSeverityIcon(Zenoss.util.convertSeverity(eventData.severity));

            // Save the evid for popping out. This is also used when submitting
            // the log form.
            Ext.getCmp('detail-logform-evid').setValue(eventData.evid);

            var state = Ext.state.Manager.get("evDetailSectionsState", {});
            // Update the data sections
            Ext.each(this.sections, function(section) {
                var cmp = Ext.getCmp(section.id),
                    html, el = cmp.getEl();
                html = section.generateHtml(renderedData, eventData);
                cmp.update(html);
                var section_state = (section.id in state) ? state[section.id] : false;
                if (el) {
                    if (section_state) {
                        el.dom.style.display = "block";
                    } else {
                        el.dom.style.display = "none";
                    }
                }
            }, this);

            // Update Logs
            var logTemplate = new Ext.XTemplate(Zenoss.eventdetail.detail_log_template),
                logHtml;
            logHtml = logTemplate.apply(eventData);
            Ext.getCmp('evdetail_log').update(logHtml);
            this.doLayout();
            if (this.showActions) {
                this.updateEventActions(eventData);
            }
        },
        updateEventActions: function(eventData) {
            Zenoss.EventActionManager.configure({
                onFinishAction: Ext.bind(this.refresh, this),

                findParams: function() {
                    var params = {
                            evids: [eventData.evid]
                        };
                    return params;
                }
            });
            var actiontoolbar = Ext.getCmp('actiontoolbar'),
                ackButton = actiontoolbar.query("button[iconCls='acknowledge']")[0],
                closeButton = actiontoolbar.query("button[iconCls='close']")[0],
                unAckButton = actiontoolbar.query("button[iconCls='unacknowledge']")[0],
                reopenButton = actiontoolbar.query("button[iconCls='reopen']")[0],
                state = eventData.eventState.toLowerCase();

            // disable all buttons
            ackButton.disable();
            closeButton.disable();
            unAckButton.disable();
            reopenButton.disable();

            // enable based on state (i.e. Which state can I go to from my current?)
            if (Zenoss.Security.hasPermission('Manage Events')) {
                if (state === "new") {
                    ackButton.enable();
                    closeButton.enable();
                } else if (state === "acknowledged") {
                    unAckButton.enable();
                    closeButton.enable();
                } else if ((state === "closed") || (state === "cleared"))  {
                    reopenButton.enable();
                }
            }

        },
        bind: function() {
            var close_btn = Ext.getCmp('evdetail_tool_close').getEl(),
                pop = Ext.getCmp('evdetail-popout').getEl();

            Ext.each(this.sections, function(section) {
                var cmp = Ext.getCmp(section.id+'_title');

                // A section may opt to not have a title, in which case
                // we can't provide auto-collapsing.
                if (cmp) {
                    // We remove this first because we don't want to keep
                    // adding listeners for the same event each time a new
                    // event is loaded.
                    cmp.getEl().un('click', cmp.toggleFn);
                    cmp.getEl().on('click', cmp.toggleFn);
                }

            }, this);

            if (close_btn) {
                // The 'onDetailHide' property is set by the config
                // during instantiation.
                close_btn.un('click', this.onDetailHide);
                close_btn.on('click', this.onDetailHide);
            }

            if (pop) {
                pop.un('click', this.popout, this);
                pop.on('click', this.popout, this);
            }
        },

        popout: function(){
            var evid = Ext.getCmp('detail-logform-evid').getValue(),
                url = this.isHistory ? 'viewHistoryDetail' : 'viewDetail';
            url = url +'?evid='+evid;
            window.open(url, evid.replace(/-/g,'_'),
                        "status=1,width=600,height=500,resizable=1,name=EventDetails");
        },
        wipe: function() {
            // hook to perform clean up actions when the panel is closed
        },
        load: function(event_id) {
            if (event_id !== this.event_id) {
                this.event_id = event_id;
                this.refresh();
            }
        },
        refresh: function() {
            Zenoss.remote.EventsRouter.detail({
                evid: this.event_id
            }, function(result) {
                var event = result.event[0];
                this.update(event);
                this.bind();
                this.show();
            }, this);
        }
    });
});
/*
 * triggersComponents.js
 */
(function(){

var router = Zenoss.remote.TriggersRouter;

    /**
     * @class Zenoss.triggers.TriggersModel
     * @extends Ext.data.Model
     * Field definitions for the triggers
     **/
    Ext.define('Zenoss.triggers.TriggersModel',  {
        extend: 'Ext.data.Model',
        idProperty: 'uuid',
        fields: [
            { name:'uuid'},
            { name:'enabled'},
            { name:'name'},
            { name:'rule'},
            { name:'users'},
            { name:'globalRead'},
            { name:'globalWrite'},
            { name:'globalManage'},
            { name:'userRead'},
            { name:'userWrite'},
            { name:'userManage'}
        ]
    });



Ext.define("Zenoss.trigger.TriggerSubscriptions", {
    alias:['widget.triggersSubscriptions'],
    extend:"Ext.grid.Panel",
    constructor: function(config) {
        var me = this;
        Ext.applyIf(config, {
            ref: 'grid_panel',
            border: false,
            viewConfig: {forceFit: true},
            title: _t('Triggers'),
            autoExpandColumn: 'value',
            loadMask: {msg:_t('Loading...')},
            autoHeight: true,
            deferRowRender: true,
            keys: [{
                key: [Ext.EventObject.ENTER],
                handler: function() {
                    me.addValueFromCombo();
                }
            }],
            tbar: {
                items: [{
                    xtype: 'combo',
                    ref: 'data_combo',
                    typeAhead: true,
                    triggerAction: 'all',
                    lazyRender:true,
                    width: 505,
                    queryMode: 'local',
                    store: Ext.create('Zenoss.NonPaginatedStore', {
                        root: 'data',
                        autoLoad: true,
                        idProperty: 'uuid',
                        fields:['uuid','name'],
                        directFn: router.getTriggerList,
                        autoDestroy: false,
                        listeners:  {
                            load: function() {
                                // The renderer for the trigger column depends
                                // on this store being loaded in order for it to
                                // render the name correctly.
                                me.getView().refresh();
                            }
                        }
                    }),

                    valueField: 'uuid',
                    displayField: 'name'
                },{
                        xtype: 'button',
                        text: 'Add',
                        ref: 'add_button',
                        handler: function() {
                            me.addValueFromCombo();
                        }
                    },{
                        xtype: 'button',
                        ref: 'delete_button',
                        iconCls: 'delete',
                        handler: function() {
                            var row = me.getSelectionModel().getSelected();
                            me.getStore().remove(row);
                            me.getView().refresh();
                        }
                    }
                ]
            },
            store: new Ext.data.JsonStore({
                model: 'Zenoss.triggers.TriggersModel',
                storeId: 'triggers_combo_store',
                autoLoad: false,
                autoDestroy: false,
                data: []
            }),

            columns: [{
                header: _t('Trigger'),
                dataIndex: 'uuid',
                flex: 1,
                renderer: function(value) {
                    var toolbar = me.getDockedItems('toolbar')[0];
                    var comboStore = toolbar.data_combo.store;
                    var idx = comboStore.find('uuid', value);
                    if (idx > -1) {
                        return comboStore.getAt(idx).data.name;
                    }
                    else {
                        // instead of displaying a uuid to the user, just display
                        // something that lets them know the value is hidden.
                        return _t('(Hidden)');
                    }
                }
            }],

            selModel: Ext.create('Zenoss.SingleRowSelectionModel', {})
        });
        this.callParent(arguments);
    },
    addValueFromCombo: function() {
        var combo = this.getDockedItems('toolbar')[0].data_combo,
            val = combo.getValue(),
            existingIndex = this.getStore().findExact('uuid', val);

        if (!Ext.isEmpty(val) && existingIndex === -1) {

            var record =  Ext.create('Zenoss.triggers.TriggersModel', {uuid:val});
            this.getStore().add(record);
            combo.setValue('');
        }
        else if (existingIndex !== -1) {
            Zenoss.message.error(_t('Duplicate items not permitted here.'));
        }
    },
    loadData: function(data) {
        this.getStore().loadData(data);
    }
});


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {
    Ext.ns('Zenoss.form');
    Ext.define("Zenoss.form.SettingsGrid", {
        alias:['widget.settingsgrid'],
        extend:"Ext.form.FormPanel",
        constructor: function(config, itemsConfig) {
            config = config || {};
            var i,
                me = this,
                prop;

            // build the properties and editors
            for (i=0; i < itemsConfig.length; i++){
                prop = itemsConfig[i];
                prop.fieldLabel = prop.name;
                if (!Ext.isDefined(prop.value)) {
                    prop.value = prop.defaultValue;
                }
                if (prop.xtype === "checkbox") {
                    prop.checked = prop.value;
                }
                prop.ref = prop.id;
                prop.name = prop.id;
            }
            this.lastValues = itemsConfig;

            Ext.applyIf(config, {
                autoScroll: 'y',
                layout: 'anchor',
                fieldDefaults: {
                    labelAlign: 'top'
                },
                paramsAsHash: true,
                frame: false,
                buttonAlign: 'left',
                defaults: {
                    anchor: '95%',
                    labelStyle: 'font-size: 13px; color: #5a5a5a'
                },
                bodyStyle: 'padding: 10px',
                listeners: {
                    validitychange: function(form, isValid) {
                        me.query('button')[0].setDisabled(!isValid);
                    },
                    afterrender: function(form){
                        this.getForm().checkValidity();
                    },
                    scope: this
                },
                isDirty: function(){
                    return true;
                },
                buttons: [{
                    text: _t('Save'),
                    ref: '../savebtn',
                    formBind: true,
                    handler: function(btn){
                        var values = {};
                        Ext.each(itemsConfig, function(prop){
                            values[prop.id] = me[prop.id].getValue();
                        });
                        config.saveFn({values: values}, function(response){
                            if (response.success){
                                var message = _t("Configuration updated");
                                Zenoss.message.info(message);
                            }
                        });
                    }
                },{
                    text: _t('Cancel'),
                    ref: '../cancelbtn',
                    handler: function(btn) {
                        var form = btn.refOwner;
                        form.setInfo(form.lastValues);
                    }
                }],
                items: itemsConfig,
                setInfo: function(data) {
                    var me = this;
                    Ext.each(data, function(prop){
                        me[prop.id].setValue(prop.value);
                    });
                },

                autoHeight: true,
                viewConfig : {
                    forceFit: true
                }
            });
            this.callParent(arguments);
        }
    });


}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


/* package level */
(function() {
    Ext.ns('Zenoss.form');
    var router = Zenoss.remote.DeviceRouter;

    Ext.define("Zenoss.form.OrganizerDropDown", {
        extend:"Ext.form.ComboBox",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                xtype: 'combo',
                name: 'group',
                store: new Ext.data.DirectStore({
                    directFn: config.getGroupFn,
                    root: config.getGroupRoot,
                    fields: ['name']
                }),
                valueField: 'name',
                emptyText: _t('All...'),
                displayField: 'name',
                allowBlank: true,
                forceSelection: false,
                editable: true,
                autoSelect: true,
                triggerAction: 'all',
                listConfig: {
                    resizable: true
                }
            });
            Zenoss.form.OrganizerDropDown.superclass.constructor.apply(this, arguments);
        }
    });

    Ext.define("Zenoss.form.SystemDropDown", {
        alias:['widget.systemdropdown'],
        extend:"Zenoss.form.OrganizerDropDown",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                getGroupFn: router.getSystems,
                getGroupRoot: 'systems'
            });
            Zenoss.form.SystemDropDown.superclass.constructor.apply(this, arguments);
        }
    });


    Ext.define("Zenoss.form.GroupDropDown", {
        alias:['widget.groupdropdown'],
        extend:"Zenoss.form.OrganizerDropDown",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                getGroupFn: router.getGroups,
                getGroupRoot: 'groups'
            });
            Zenoss.form.GroupDropDown.superclass.constructor.apply(this, arguments);
        }
    });


    Ext.define("Zenoss.form.LocationDropDown", {
        alias:['widget.locationdropdown'],
        extend:"Zenoss.form.OrganizerDropDown",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                getGroupFn: router.getLocations,
                getGroupRoot: 'locations'
            });
            Zenoss.form.LocationDropDown.superclass.constructor.apply(this, arguments);
        }
    });


    Ext.define("Zenoss.form.DeviceClassDropDown", {
        alias:['widget.deviceclassdropdown'],
        extend:"Zenoss.form.OrganizerDropDown",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                getGroupFn: router.getDeviceClasses,
                getGroupRoot: 'deviceClasses'
            });
            Zenoss.form.DeviceClassDropDown.superclass.constructor.apply(this, arguments);
        }
    });



}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){
    var router = Zenoss.remote.PropertiesRouter,
        ZPROP_NAME = 'zCollectorPlugins';
    Ext.define("Zenoss.form.ModelerPluginForm", {
        extend:"Ext.form.FormPanel",
        alias: "widget:modelerpluginform",
        constructor: function(config) {
            var me = this;
            config = config || {};
            Ext.apply(config, {
                fieldDefaults: {
                    labelAlign: 'top'
                },
                paramsAsHash: true,
                frame: false,
                autoScroll: 'y',
                defaults: {
                    labelStyle: 'font-size: 13px; color: #5a5a5a',
                    anchor: '100%'
                },
                items: [{
                    xtype: 'displayfield',
                    name: 'path',
                    id: 'modeler-plugin-path',
                    fieldLabel: _t('Path')
                },{
                    xtype: 'displayfield',
                    id: 'modeler-plugin-doc',
                    name: 'doc',
                    height: 65,
                    autoScroll: true,
                    ref: 'doc',
                    tooltip: 'Select a single plugin to see the docs',
                    fieldLabel: _t('Plugin Documentation')
                }],
                buttonAlign: 'left',
                buttons: [{
                    text: _t('Save'),
                    ref: 'savebtn',
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    handler: function(btn){
                        var values = me.modelerPlugins.getValue(),
                            panel = me,
                            uid = me.uid;
                        router.setZenProperty({uid:uid,
                                               zProperty: ZPROP_NAME,
                                               value:values},
                                              function(response){
                                                  if (response.success){
                                                      Zenoss.message.info('Updated Modeler Plugins');
                                                      Ext.getCmp('modeler-plugin-path').setValue(response.data.path);
                                                      panel.toggleDeleteButton(response.data.path);
                                                  }

                                              });
                    }
                },{
                    text: _t('Cancel'),
                    ref: 'cancelbtn',
                    handler: function() {
                        if (me.uid) {
                            me.setContext(me.uid);
                        }
                    }
                },{
                    text: _t('Delete Local Copy'),
                    ref: 'deleteBtn',
                    hidden: true,
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    handler: function(btn) {
                        var panel = me;
                        // show a confirmation
                        new Zenoss.dialog.SimpleMessageDialog({
                            title: _t('Delete zProperty'),
                            msg: _t("Are you sure you want to delete the local copy of zCollectorPlugin?"),
                            buttons: [{
                                xtype: 'DialogButton',
                                text: _t('OK'),
                                handler: function() {
                                    if (panel.uid) {
                                        router.deleteZenProperty({
                                            uid: panel.uid,
                                            zProperty: ZPROP_NAME
                                        }, function(response){
                                            panel.setContext(panel.uid);
                                        });
                                    }
                                }
                            }, {
                                xtype: 'DialogButton',
                                text: _t('Cancel')
                            }]
                        }).show();
                    }
                }],
                cls: 'device-overview-form-wrapper',
                bodyCls: 'device-overview-form'
            });
            this.callParent(arguments);
        },
        setContext: function(uid) {
            if (this.modelerPlugins) {
                this.modelerPlugins.destroy();
            }
            if(this.pluginsHeader){
                this.pluginsHeader.destroy();
            }
            if(this.panelHeaders){
                this.panelHeaders.destroy();
            }
            Ext.getCmp('modeler-plugin-doc').setValue('');
            this.uid = uid;
            // get the modeler plugins
            router.getZenProperty({
                uid: uid,
                zProperty: ZPROP_NAME
            }, Ext.bind(this.loadData, this));

            Zenoss.remote.DeviceRouter.getModelerPluginDocStrings({
                uid: uid
            }, Ext.bind(this.loadDocs, this));

        },
        toggleDeleteButton: function(path){
            // show the delete button if locally defined
            var localPath = this.uid.replace('/zport/dmd/Devices', ''),
                deleteBtn = this.getButton("deleteBtn");

            // can't delete the root
            if (path === '/') {
                deleteBtn.hide();
                return;
            }
            if (localPath === path) {
                deleteBtn.show();
            }else{
                deleteBtn.hide();
            }
        },
        getButton: function(ref) {
            return this.query(Ext.String.format("button[ref='{0}']", ref))[0];
        },
        loadData: function(response) {
            if (response.success) {
                var data = response.data,
                    clickHandler,
                    panel = this;

                Ext.getCmp('modeler-plugin-path').setValue(data.path);
                this.toggleDeleteButton(data.path);
                clickHandler = function(select, record) {
                    // display the docs for the record clicked
                    var value = record.data.name;
                    panel.showDocFor(value);
                };
                var store = [];
                Ext.each(data.options, function(item) {
                    store.push([item]);
                });
                // add the multi select
                this.add({
                        xtype:'label',
                        text: 'Modeler Plugins:',
                        ref: 'pluginsHeader',
                        style: {'color':'#5A5A5A', 'fontWeight':'bold', 'display':'block', 'padding':'0 0 8px 0'}
                });
                this.add({
                    xtype: 'panel',
                    width: 800,
                    layout:'column',
                    ref: 'panelHeaders',
                    items: [
                    {
                        xtype:'label',
                        columnWidth: 0.515,
                        text: 'Available',
                        style: {'color':'#5A5A5A'}
                    },{
                        xtype: 'label',
                        columnWidth: 0.485,
                        text: 'Selected',
                        style: {'color':'#5A5A5A'}
                    }]
                });
                this.add({
                    name: 'modelerPlugins',
                    ref: 'modelerPlugins',
                    xtype: 'itemselector',
                    imagePath: "/++resource++zenui/img/xtheme-zenoss/icon",
                    height: 250,
                    drawUpIcon: true,
                    drawDownIcon: true,
                    drawTopIcon: true,
                    drawBotIcon: true,
                    displayField: 'name',
                    valueField: 'name',
                    store:  Ext.create('Ext.data.ArrayStore', {
                        model: 'Zenoss.model.Name',
                        data: store
                    }),
                    value: data.value,
                    listeners: {
                        afterrender: function() {
                            // HACK: have to go into the internals of MultiSelect to properly register a click
                            // handler
                            this.modelerPlugins.fromField.boundList.on('itemclick', clickHandler);
                            this.modelerPlugins.toField.boundList.on('itemclick', clickHandler);
                        },
                        scope: this
                    }
                });
                this.doLayout();

            }
        },
        showDocFor: function(plugin) {
            if (plugin && this.docs && this.docs[plugin]) {
                this.doc.setValue(this.docs[plugin]);
            }else{
                this.doc.setValue(Ext.String.format(_t('No documentation found for {0}'), plugin));
            }
        },
        loadDocs: function(response) {
            if (response.success) {
                this.docs = response.data;
            }
        }
    });


    /**
     * Place the form inside a panel for sizing
     **/
    Ext.define("Zenoss.form.ModelerPluginPanel", {
        alias:['widget.modelerpluginpanel'],
        extend:"Ext.Panel",
        constructor: function(config) {
            config = config || {};
            var form = Ext.create("Zenoss.form.ModelerPluginForm", {});
            Ext.applyIf(config, {
                layout: 'fit',
                width: 800,
                autoScroll: 'auto',
                items: [form]
            });
            this.callParent(arguments);
            this.form = form;
        },
        setContext: function(uid) {
            this.form.setContext(uid);
        }
    });




})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){
    var router = Zenoss.remote.PropertiesRouter,
        zpropertyConfigs = {};

    Ext.ns('Zenoss.zproperties');
    Zenoss.zproperties.inferZPropertyLabel= function(id) {
        var fieldLabel = id;
        // special case for vsphere end point.
        if (id.toLowerCase().indexOf('endpointhost')!== -1) {
            fieldLabel = _t("vSphere Endpoint Host");
        }else if (id.toLowerCase().indexOf('winkdc')!== -1) {
            fieldLabel = _t('AD Domain Controller');
        }else if (id.toLowerCase().indexOf('winscheme')!== -1) {
            fieldLabel = _t('Protocol (http/https)');
        }else if (id.toLowerCase().indexOf('user')!== -1) {
            fieldLabel = _t('Username');
        } else if (id.toLowerCase().indexOf('password')!== -1) {
            fieldLabel = _t('Password');
        } else if (id.toLowerCase().indexOf('snmpcomm')!== -1) {
            fieldLabel = _t('SNMP Community String');
        } else if (id.toLowerCase().indexOf('port')!== -1) {
            fieldLabel = _t('Port');
        } else if (id.toLowerCase().indexOf('ssl')!== -1) {
            fieldLabel = _t('Use SSL?');
        } else {
            fieldLabel = id;
        }
        return fieldLabel;
    };

    Ext.apply(zpropertyConfigs, {
        'int': {
            xtype: 'numberfield',
            allowDecimals: false,
            width: 100
        },
        'float': {
            xtype: 'numberfield',
            width: 100
        },
        'string': {
            xtype: 'textfield'
        },
        'lines': {
            xtype: 'textarea',
            resizable: true,
            width: 300
        },
        'severity': {
            xtype: 'severity'
        },
        'boolean': {
            xtype: 'checkbox'
        },
        'password': {
            xtype: 'password'
        },
        'options': {
            xtype: 'combo',
            editable: false,
            forceSelection: true,
            autoSelect: true,
            triggerAction: 'all',
            queryMode: 'local'
        },
        'zSnmpCommunity': {
            xtype: Zenoss.Security.doesNotHavePermission('zProperties Edit') ? 'password' : 'textfield'
        },
        'zEventAction': {
            xtype: 'eventaction'
        },
        'zEventSeverity': {
            xtype: 'defaultseverity'
        },
        'zFailSeverity': {
            xtype: 'severity'
        },
        'zFlappingSeverity': {
            xtype: 'severity'
        },
        'zFlappingThreshold': {
            xtype: 'numberfield',
            allowDecimals: false,
            width: 100
        },
        'zWinEventlogMinSeverity': {
            xtype: 'reverseseverity'
        },
        'zFlappingIntervalSeconds': {
            xtype: 'numberfield',
            allowDecimals: false,
            width: 100,
            minValue: 600
        }
    });

    /**
     * Allow zenpack authors to register custom zproperty
     * editors.
     **/
    Zenoss.zproperties.registerZPropertyType = function(id, config){
        zpropertyConfigs[id] = config;
    };

    Zenoss.zproperties.createZPropertyField = function(data) {
        var editConfig = {},
            type = data.type;
        // in case of drop down lists
        if (Ext.isArray(data.options) && data.options.length > 0 && type === 'string') {
            // make it a combo and the options is the store
            editConfig.store = data.options.map(function(value){
                var valueField = value,
                    displayField = value;

                // if the value is an empty string, indicate
                // with a special string
                if(value === ""){
                    displayField = "- None -";
                }
                return [valueField, displayField];
            });
            Ext.apply( editConfig, zpropertyConfigs['options']);
        } else {
            // Try the specific property id, next the type and finally default to string
            Ext.apply( editConfig, zpropertyConfigs[data.id] || zpropertyConfigs[type] || zpropertyConfigs['string']);
        }

        // set the default values common to all configs
        Ext.applyIf(editConfig, {
            fieldLabel: data.label || Zenoss.zproperties.inferZPropertyLabel(data.id),
            value: data.value,
            ref: 'editConfig',
            checked: data.value || data.valueAsString,
            name: data.id,
            width: 250
        });

        // lines come in as comma separated and should be saved as such
        if (type === 'lines' && Ext.isArray(editConfig.value)){
            editConfig.value = editConfig.value.join('\n');
        }
        return editConfig;
    };

    function showEditConfigPropertyDialog(data, grid) {
        var handler, config, editConfig, dialog, type;

        editConfig = Zenoss.zproperties.createZPropertyField(data);


        handler = function() {
            // save the junk and reload
            var values = dialog.getForm().getForm().getValues(),
                value = values[data.id];
            if (type === 'lines') {
                if (value) {
                    // send back as an array separated by a new line
                    value = Ext.Array.map(value.split('\n'), function(s) {return Ext.String.trim(s);});
                } else {
                    // send back an empty list if nothing is set
                    value = [];
                }
            }
            var params = {
                uid: grid.uid,
                zProperty: data.id,
                value: value
            };
            Zenoss.remote.PropertiesRouter.setZenProperty(params, function(response){
                if (response.success) {
                    grid.refresh();
                }
            });

        };

        // form config
        config = {
            submitHandler: handler,
            minHeight: 300,
            autoHeight: true,
            width: 500,
            title: _t('Edit Config Property'),
            listeners: {
                show: function() {
                    dialog.getForm().query("field[ref='editConfig']")[0].focus(true, 500);
                }
            },
            items: [{
                    xtype: 'displayfield',
                    name: 'name',
                    fieldLabel: _t('Name'),
                    value: data.id
                },{
                    xtype: 'displayfield',
                    name: 'path',
                    ref: 'path',
                    fieldLabel: _t('Path'),
                    value: data.path
                },{
                    xtype: 'displayfield',
                    name: 'description',
                    ref: 'description',
                    fieldLabel: _t('Description'),
                    value: data.description
                },{
                    xtype: 'displayfield',
                    name: 'type',
                    ref: 'type',
                    fieldLabel: _t('Type'),
                    value: data.type
                }, {
                    // spacer for metadata/value
                    height: 25,
                    xtype: 'container'
                },{
                    //Add hidden input fields to prevent password autocomplete
                    xtype: 'textfield',
                    hidden: true
                },{
                    xtype: 'password',
                    hidden: true
                }, editConfig
            ],
            // explicitly do not allow enter to submit the dialog
            keys: {

            }
        };
        dialog = new Zenoss.SmartFormDialog(config);

        if (Zenoss.Security.hasPermission('zProperties Edit')) {
            dialog.show();
        }
    }

    /**
     * @class Zenoss.ConfigProperty.Model
     * @extends Ext.data.Model
     * Field definitions for the Config Properties
     **/
    Ext.define('Zenoss.ConfigProperty.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: [
            {name: 'id'},
            {name: 'islocal'},
            {name: 'value'},
            {name: 'category'},
            {name: 'valueAsString'},
            {name: 'type'},
            {name: 'path'},
            {name: 'options'},
            {name: 'label'},
            {name: 'description'}
        ]
    });

    /**
     * @class Zenoss.ConfigProperty.Store
     * @extends Zenoss.DirectStore
     * Store for our configuration properties grid
     **/
    Ext.define("Zenoss.ConfigProperty.Store", {
        extend: "Zenoss.DirectStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.ConfigProperty.Model',
                initialSortColumn: 'id',
                pageSize: 300,
                directFn: Zenoss.remote.PropertiesRouter.getZenProperties
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.ConfigProperty.Grid", {
        alias: ['widget.configpropertygrid'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};

            Zenoss.Security.onPermissionsChange(function() {
                this.disableButtons(Zenoss.Security.doesNotHavePermission('zProperties Edit'));
            }, this);

            Ext.applyIf(config, {
                stateId: config.id || 'config_property_grid',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                tbar:[

                    {
                    xtype: 'button',
                    iconCls: 'customize',
                    tooltip: _t('Customize'),
                    disabled: Zenoss.Security.doesNotHavePermission('zProperties Edit'),
                    ref: 'customizeButton',
                    handler: function(button) {
                        var grid = button.up("configpropertygrid"),
                            data,
                            selected = grid.getSelectionModel().getSelection();
                        if (Ext.isEmpty(selected)) {
                            return;
                        }
                        // single selection
                        data = selected[0].data;
                        showEditConfigPropertyDialog(data, grid);
                    }
                    }, {
                    xtype: 'button',
                    iconCls: 'refresh',
                    tooltip: _t('Refresh'),
                    ref: '../refreshButton',
                    disabled: Zenoss.Security.doesNotHavePermission('zProperties Edit'),
                    handler: function(button) {
                        var grid = button.up("configpropertygrid");
                        grid.refresh();
                    }
                    },{
                        xtype: 'button',
                        ref: '../deleteButton',
                        tooltip: _t('Delete'),
                        text: _t('Delete Local Copy'),
                        handler: function(button) {
                            var grid = button.up("configpropertygrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();
                            if (Ext.isEmpty(selected)) {
                                return;
                            }

                            data = selected[0].data;
                            if (data.islocal && data.path === '/') {
                                Zenoss.message.info(_t('{0} can not be deleted from the root definition.'), data.id);
                                return;
                            }
                            if (!data.islocal){
                                Zenoss.message.info(_t('{0} is not defined locally'), data.id);
                                return;
                            }
                            new Zenoss.dialog.SimpleMessageDialog({
                                title: _t('Delete Local Property'),
                                message: Ext.String.format(_t("Are you sure you want to delete the local copy of {0}?"), data.id),
                                buttons: [{
                                    xtype: 'DialogButton',
                                    text: _t('OK'),
                                    handler: function() {
                                        if (grid.uid) {
                                            router.deleteZenProperty({
                                                uid: grid.uid,
                                                zProperty: data.id
                                            }, function(response){
                                                grid.refresh();
                                            });
                                        }
                                    }
                                }, {
                                    xtype: 'DialogButton',
                                    text: _t('Cancel')
                                }]
                            }).show();
                        }
                    }
                ],
                store: Ext.create('Zenoss.ConfigProperty.Store', {
                }),
                columns: [{
                        header: _t("Is Local"),
                        dataIndex: 'islocal',
                        width: 60,
                        sortable: true,
                        filter: false,
                        renderer: function(value){
                            if (value) {
                                return 'Yes';
                            }
                            return '';
                        }
                    },{
                        dataIndex: 'category',
                        header: _t('Category'),
                        sortable: true,
                        renderer: function(value) {
                            return Ext.htmlEncode(value);
                        }
                    },{
                        dataIndex: 'id',
                        header: _t('Name'),
                        width: 200,
                        sortable: true,
                        renderer: function(value) {
                            return Ext.htmlEncode(value);
                        }
                    },{
                        dataIndex: 'valueAsString',
                        header: _t('Value'),
                        width: 150,
                        renderer: function(v, row, record) {
                            if (Zenoss.Security.doesNotHavePermission("zProperties Edit") &&
                                record.data.id === 'zSnmpCommunity') {
                                return "*******";
                            }
                            return Zenoss.render.zProperty(v, record);
                        },
                        sortable: false
                    },{
                        dataIndex: 'label',
                        header: _t('Label'),
                        width: 200,
                        sortable: true
                    },{
                        dataIndex: 'description',
                        header: _t('Description'),
                        flex: 1
                    },{
                        //id: 'path',
                        dataIndex: 'path',
                        header: _t('Path'),
                        width: 380,
                        sortable: true,
                        renderer: function(value) {
                            return Ext.htmlEncode(value);
                        }
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
        },
        setContext: function(uid) {
            if (uid === '/zport/dmd/Devices'){
                this.deleteButton.setDisabled(true);
            } else {
                this.deleteButton.setDisabled(Zenoss.Security.doesNotHavePermission('zProperties Edit'));
            }

            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        },
        onRowDblClick: function(grid, rowIndex, e) {
            var data,
                selected = this.getSelectionModel().getSelection();
            if (!selected) {
                return;
            }
            data = selected[0].data;
            showEditConfigPropertyDialog(data, this);
        },
        disableButtons: function(bool) {
            var btns = this.query("button");
            Ext.each(btns, function(btn){
                btn.setDisabled(bool);
            });
        }
    });

    Ext.define("Zenoss.form.ConfigPropertyPanel", {
        alias:['widget.configpropertypanel'],
        extend:"Ext.Panel",
        constructor: function(config) {
            config = config || {};
            this.gridId = Ext.id();
            Ext.applyIf(config, {
                layout: 'fit',
                autoScroll: 'y',
                height: 800,
                items: [{
                    id: this.gridId,
                    xtype: "configpropertygrid",
                    ref: 'configGrid',
                    displayFilters: config.displayFilters
                }]
            });
            this.callParent(arguments);
        },
        setContext: function(uid) {
            Ext.getCmp(this.gridId).setContext(uid);
        }
    });



    var checkText = function(e){
        var uid = "/zport/dmd/Devices"; // to force it to apply to the root on every newly created prop

        var params = {
            uid: uid,
            zProperty: e
        };
        Zenoss.remote.PropertiesRouter.getZenProperty(params, function(response){
            if (response.success) {
                if (response.data.type === "lines"){
                    var lines = response.data.value, context = Zenoss.zproperties.configPropertyConfigs.state.context, formID;
                    if (context === "add") {
                        formID = "addCustomDialog";
                    } else {
                        formID = "editCustomDialog";
                    }
                    var c = Ext.getCmp(formID).getForm().getForm().getValues();

                    // we got a 'lines' item back. Lets save this so we can complete the users goal

                    var params = {
                        uid: uid,
                        id: c.name,
                        type: c.type,
                        label: c.description,
                        value: c.value
                    };
                    var addToCombo = function(){
                        // dump the array response.data.value into the combo box by breaking it up into an array of arrays
                        // for loadData to work
                        var combo = Ext.getCmp('linesCombo');
                        var data = [];
                        for (var i = 0; i < lines.length; i++){
                            data.push([lines[i]]);
                        }
                        combo.store.loadData(data);
                        // fire off a selection of the first item so it's obvious something happened
                        combo.setValue(combo.store.getAt('0'));
                    };
                    if (context === "add"){
                        Zenoss.remote.PropertiesRouter.addCustomProperty(params, function(response){
                            if (response.success) {
                                addToCombo();
                            }
                        });
                    }else{
                        addToCombo();
                    }

                }
            }
        });
    };


    Zenoss.zproperties.configPropertyConfigs = {};
    var propValText = _t("Value for Property Type:");
    Ext.apply(Zenoss.zproperties.configPropertyConfigs, {
        'state': {
            context: 'add'
        },
        'int': {
            xtype: 'numberfield',
            allowDecimals: false,
            fieldLabel: _t(propValText+' INT'),
            name: 'value'
        },
        'float': {
            xtype: 'numberfield',
            fieldLabel: _t(propValText+' FLOAT'),
            name: 'value'
        },
        'long': {
            xtype: 'numberfield',
            fieldLabel: _t(propValText+' LONG'),
            name: 'value'
        },
        'date': {
            xtype: 'datefield',
            name: 'value',
            fieldLabel: _t(propValText+' DATE')
        },
        'string': {
            xtype: 'textfield',
            name: 'value',
            fieldLabel: _t(propValText+' STRING'),
            width:220
        },
        'lines': {
            xtype: 'textarea',
            name: 'value',
            width: 220,
            height: 70,
            fieldLabel: _t(propValText+' LINES')
        },
        'boolean': {
            xtype: 'checkbox',
            name: 'value',
            fieldLabel: _t(propValText+' BOOLEAN')
        },
        'password': {
            xtype: 'password',
            name: 'value',
            width:220,
            fieldLabel: _t(propValText+' PASSWORD')
        },
        'selection': {
            xtype: 'panel',
            items: [
                {
                    xtype: 'panel',
                    width: 260,
                    layout: 'hbox',
                    items: [
                        {
                            xtype: 'textfield',
                            name: 'value',
                            id: 'linesSearchBox',
                            width:220,
                            margin: '0 0 5px 0',
                            fieldLabel: _t(propValText+' SELECTION')
                        },{
                            xtype: 'button',
                            iconCls: 'acknowledge',
                            listeners: {
                                afterrender: function(button){
                                    button.setTooltip(_t("Retrieve lines from property for selection"));
                                    if (Zenoss.zproperties.configPropertyConfigs.state.context === "add"){
                                        button.setTooltip(_t("Save new property and retrieve lines for selection"));
                                    }
                                }
                            },
                            margin: '17px 0 0 7px',
                            handler: function(button) {
                                var txt = Ext.getCmp('linesSearchBox').getValue(), uid;
                                if(Zenoss.zproperties.configPropertyConfigs.state.context === "add"){
                                    uid = Ext.getCmp('addCustomDialog').uid;
                                }else{
                                    uid = Ext.getCmp('editCustomDialog').uid;
                                }
                                if(txt!== ""){
                                    checkText(txt, uid);
                                }else{
                                    Zenoss.message.info(_t('The "Lines" property type cannot be empty. Please enter a "Lines" custom property.'));
                                }
                            }
                        }
                        ]
                },{
                    xtype: 'combo',
                    name: 'selection',
                    queryMode:'local',
                    id: 'linesCombo',
                    store: [ 'none' ],
                    width:220
                }
            ]
        }
    });

    Zenoss.zproperties.showEditPropertyDialog = function(pkg) {
        var handler, config, editConfig, dialog;

        Zenoss.zproperties.configPropertyConfigs.state.context = "edit";
        // Try the specific property id, next the type and finall default to string
        editConfig = Zenoss.zproperties.configPropertyConfigs[pkg.name] || Zenoss.zproperties.configPropertyConfigs[pkg.type] || Zenoss.zproperties.configPropertyConfigs['string'];

        // in case of drop down lists
        if (Ext.isArray(pkg.options) && pkg.options.length > 0 && pkg.type === 'string') {
            // make it a combo and the options is the store
            editConfig = Zenoss.zproperties.configPropertyConfigs['options'];
            editConfig.store = pkg.options;
        }
        // set the default values common to all configs
        Ext.apply(editConfig, {
            fieldLabel: _t('Value'),
            value: pkg.value,
            ref: 'editConfig',
            checked: pkg.value,
            name: pkg.name
        });

        // lines come in as comma separated and should be saved as such
        if (pkg.type === 'lines' && Ext.isArray(editConfig.value)){
            editConfig.value = editConfig.value.join('\n');
        }

        handler = function() {
            // save the junk and reload
            var values = dialog.getForm().getForm().getValues(),
                value = values[pkg.name];
            if ((typeof values.applylocal) === "undefined"){
                values.applylocal = false;
            }
            if (pkg.type === 'lines') {
                if (value) {
                    // send back as an array separated by a new line
                    value = Ext.Array.map(value.split('\n'), function(s) {return Ext.String.trim(s);});
                } else {
                    // send back an empty list if nothing is set
                    value = [];
                }
            }
            // if this is a selection type, then the value is form.selection and not form.value
            if (pkg.type === 'selection'){
                value = values.selection;
            }
            var params = {
                uid: "/zport/dmd"+pkg.uid,
                zProperty: pkg.name,
                value: value
            };
            Zenoss.remote.PropertiesRouter.setZenProperty(params, function(response){
                if (response.success) {
                    pkg.grid.refresh();
                }
            });

        };
        var items = pkg.items;
        Ext.Array.splice( items, -1, 0, editConfig );
        var config = {
            submitHandler: handler,
            minHeight: pkg.minHeight,
            autoHeight: true,
            width: pkg.width,
            uid: pkg.uid,
            id: "editCustomDialog",
            title: _t('Edit Configuration Property'),
            items: items,
            keys: {}
        };

        dialog = new Zenoss.SmartFormDialog(config);

        if (Zenoss.Security.hasPermission('zProperties Edit')) {
            dialog.show();
        }
    };


})();
/* global zenoss:true, moment: true */
/* jshint freeze: false */
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2010, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/
(function(){
    Ext.ns('Zenoss');
    /**********************************************************************
     *
     * Graph Panel
     *
     */
    function syntaxHighlight(json) {
        json = JSON.stringify(json, undefined, 4);
        json = json.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
        json = json.replace(/ /g, '&nbsp;');
        return json.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function (match) {
            var cls = 'syntax-number';
            if (/^"/.test(match)) {
                if (/:$/.test(match)) {
                    cls = 'syntax-string';
                } else {
                    cls = 'syntax-text';
                }
            } else if (/true|false/.test(match)) {
                cls = 'syntax-boolean';
            } else if (/null/.test(match)) {
                cls = 'syntax-null';
            }
            return '<span class="' + cls + '">' + match + '</span>';
        });
    }


    function isSingleComponentChart(pts){

        var allTheSame = true;
        var first = true;
        var componentName = "";
        pts.forEach(function(pt){
            var cname = pt.legend.substr(0,pt.legend.indexOf(" "));
            allTheSame = allTheSame && ( componentName === cname || first );
            componentName = cname;
            first = false;
        });
        return allTheSame ? componentName : '';
    }

    function removeRepeatedComponent(pts, cname) {
        // Remove repeated component name from beginning of all strings
        pts.forEach(function (pt) {
            if (pt.legend.substr(0, cname.length) === cname) {
                pt.legend = pt.legend.substr(cname.length).trim();
            }
            // also trim off any leading "- " strings
            if (pt.legend.substr(0, 2) === "- ") {
                pt.legend = pt.legend.substr(2);
            }
        });
    }

    function removeRepeatedContext(pts) {
        pts.forEach(function (pt) {
            var usedused = pt.legend.indexOf('usedBlocks Used');
            if ( usedused > -1) {
                pt.legend = pt.legend.substr(0, usedused) + "Used";
            }
        });
    }

    var router = Zenoss.remote.DeviceRouter,

        CURRENT_TIME = "0s-ago",
        DATE_RANGES = [
            ["1h-ago", _t('Last Hour')],
            ["1d-ago", _t('Last 24 Hours')],
            ["7d-ago", _t('Last Week')],
            ["30d-ago", _t('Last 30 days')],
            ["1y-ago", _t('Last Year')]
        ],
        RANGE_TO_MILLISECONDS = {
            '1h-ago': 3600000,
            '1d-ago': 86400000,
            '7d-ago': 604800000,
            '30d-ago': 2419200000,
            '1y-ago': 31536000000
        },

        /*
         * If a given request is over GRAPHPAGESIZE then
         * the results will be paginated.
         * Lower the number of graphs that are displayed for IE
         * since it dramatically speeds up the rendering speed.
         **/
        GRAPHPAGESIZE = Ext.isIE ? 25 : 50;

    Number.prototype.pad = function(count) {
        var zero = count - this.toString().length + 1;
        return new Array(+(zero > 0 && zero)).join("0") + this;
    };

    function now() {
        return new Date().getTime();
    }

    function rangeToMilliseconds(range){
        if (RANGE_TO_MILLISECONDS[range]) {
            return RANGE_TO_MILLISECONDS[range];
        }
        return range;
    }

    Date.prototype.minus = function(secs) {
        return new Date(this.valueOf()-(secs*1000));
    };


    Ext.define("Zenoss.EuropaGraph", {
        alias:['widget.europagraph'],
        extend: "Ext.Panel",

        zoom_factor: 1.25,
        pan_factor: 4,

        /**
         * @cfg {int} start
         * The start time of the graph in unix seconds.
         * Defaults to <code>new DateTime().getTime()</code>.
         */
        start: DATE_RANGES[0][0],

        /**
         * @cfg {int} end
         * The end time of the graph in unix seconds.
         * Defaults to <code>new DateTime().getTime() - 3600 (an hour ago)</code>.
         */
        end: CURRENT_TIME,

        /**
         * @cfg {String}  graphId
         * The id of the div that is used to contain the graph
         *
         */
        graphId: "",

        /**
         * @cfg {String}  graphTitle
         * The upper left hand title of the graph
         * Defaults to an empty string.
         */
        graphTitle: "",

        /**
         * @cfg {Object}  tags
         * The tags we are sending to the metric service. This is a key value pair of tagname, value.
         * For Example: <code>{'ip_address': '191.168.4.10'}
         * Defaults to an empty object
         */
        tags: {},

        /**
         * @cfg {String} type
         * The type of chart we are asking the metric service to render. See the central-query documentation for a list of valid types.
         * Defaults to an <code>line</code>
         */
        type: 'line',

        /**
         * @cfg {boolean} hasMenu
         * Toggles building the gear menu.
         */
        hasMenu: true,

        /**
         * @cfg {Array} datapoints
         * Datapoints we are sending to the metric service. This is an
         * array of objects. A datapoint object has the following
         * available properties:
         * <ul>
         *   <li> aggregator - example: "avg". The available aggregator functions  are: ("avg", "min", "max", "sum")</li>
         *   <li> color - example: "#0000ff99"</li>
         *   <li> format - example: "%6.2lf"</li>
         *   <li> id - example: "laLoadInt15"</li>
         *   <li> legend - example: "15 Minute"</li>
         *   <li> metric - example: "laLoadInt15"</li>
         *   <li> rpn - example: "100,/"</li>
         *   <li> uuid - example: "480fc36d-1ffa-4bbd-a41d-2ad0e459fb85""</li>
         * </ul>
         **/
        datapoints: [],
        graphTemplate: new  Ext.Template('<div id="{graphId}" class="europagraph" style="{graphPadding}height:{graphHeight}px;"> ' +
                                         '     <div class="graph_title">{graphTitle}' +
                                         '        <div class="graph_description">{description}</div>'+
                                         '     </div> ' +
                                         '    <img id="{buttonId}" class="europaGraphGear" src="/++resource++zenui/img/gear.png"  />' +
                                         '</div>'),
        constructor: function(config) {
            var padding = "padding:5px 5px 5px 0px;";
            // backcompat from graph dimensions from rrd
            // the properties were saved on each graph definition and we want to
            // preserve backward compabability
            if (config.height === 100 && config.width === 500) {
                config.height = 500;
            } else if (config.height == undefined) {
                config.height = 500;
            }
            // width is not customizable - always fills column
            delete config.width;

            // dynamically adjust the height;
            config.graphPadding = padding;
            config.graphHeight = config.height - 50;
            config.buttonId = Ext.id();
            config = Ext.applyIf(config||{}, {
                html: this.graphTemplate.apply(config),
                maxWidth: 800,
                cls: 'graph-panel',
                bodyStyle: {
                    padding: "5px"
                },
                graph_params: {
                    drange: DATE_RANGES[0][0],
                    end: config.end || CURRENT_TIME,
                    start: config.start || DATE_RANGES[0][0]
                },
                dockedItems: [{
                    xtype: 'toolbar',
                    dock: 'top',
                    items: ['->',{
                        text: _t('Open in New Tab'),
                        ref: "../newtab",
                        handler: Ext.bind(function(btn, e) {
                                this.newTab(this);
                        }, this)
                    },{
                        text: '&lt;',
                        width: 40,
                        handler: Ext.bind(function(btn, e) {
                                this.onPanLeft(this);
                        }, this)
                    },{
                        text: _t('Zoom In'),
                        ref: '../zoomin',
                        handler: Ext.bind(function(btn, e) {
                            this.doZoom.call(this, 0, 1/this.zoom_factor);
                        }, this)
                    },{
                        text: _t('Zoom Out'),
                        ref: '../zoomout',
                        handler: Ext.bind(function(btn, e) {
                            this.doZoom.call(this, 0, this.zoom_factor);
                        }, this)
                    },{
                        text: '&gt;',
                        width: 40,
                        handler: Ext.bind(function(btn, e) {
                            this.onPanRight(this);
                        }, this)
                    }]
                }]
            });

            Zenoss.EuropaGraph.superclass.constructor.call(this, config);
        },
        initComponent: function() {
            // the visualization library depends on our div rendering,
            // let's make sure that has happened
            this.on('afterrender', this.initChart, this);
            this.on('afterrender', this.buildMenu, this, {single: true});
            this.callParent(arguments);
        },
        beforeDestroy: function() {
            if (this.menu) {
                this.menu.destroy();
            }
        },
        buildMenu: function() {
            if (! this.hasMenu) {
                return;
            }
            var item = Ext.get(this.buttonId);
            this.menu = Ext.create('Ext.menu.Menu', {
                items: [{
                    text: _t('Definition'),
                    handler: Ext.bind(this.displayDefinition, this)
                }, {
                    text: _t('Export to CSV'),
                    handler: Ext.bind(this.exportData, this)
                }, {
                    text: _t('Link to this Graph'),
                    handler: Ext.bind(this.displayLink, this)
                }, {
                    text: _t('Expand Graph'),
                    handler: Ext.bind(this.expandGraph, this)
                }]
            });
            item.on('click', function(event, t) {
                event.preventDefault();
                var rect = event.target.getBoundingClientRect(),
                    x = rect.left,
                    y = rect.top + rect.height;
                this.menu.showAt(x, y);
            }, this);

        },
        initChart: function() {
            var cname = isSingleComponentChart(this.datapoints);
            // returns repeated legend name if all legends share same component
            if(cname.length > 0) {
                [this.datapoints, this.thresholds].forEach(function (datapts) {
                    removeRepeatedComponent(datapts, cname);
                }, this);
            }

            removeRepeatedContext(this.datapoints);

            // these assume that the graph panel has already been rendered
            var height = this.getEl().getHeight();
            var visconfig = {
                returnset: "EXACT",
                range : {
                    start : this.graph_params.start,
                    end : this.graph_params.end
                },
                base: this.base,
                tags: this.tags,
                datapoints: this.datapoints,
                overlays: this.thresholds,
                projections: this.projections,
                printOptimized: this.printOptimized,
                type: this.type,
                // lose the footer and yaxis label as the image gets smaller
                footer: (height >= 350) ? true : false,
                yAxisLabel: this.units,
                supressLegend: true,
                miny: (this.miny !== -1) ? this.miny : null,
                maxy: (this.maxy !== -1) ? this.maxy : null,
                // the visualization library currently only supports
                // one format for chart, not per metric
                format: (this.datapoints.length > 0) ? this.datapoints[0].format: "",
                timezone: Zenoss.USER_TIMEZONE
            };

            // determine scaling
            if (this.autoscale) {
                visconfig.autoscale = {
                    factor: this.base,
                    ceiling: this.ceiling
                };
            }
            this.chartdefinition = visconfig;

            var self = this;
            var p = zenoss.visualization.chart.create(this.graphId, visconfig);
            p.then(function(chart){
                chart.afterRender = function(){
                    var legenddiv = chart.$div.find(".nv-legend").length;
                    // 40 will trigger resize below.
                    var legendHeight = legenddiv ? chart.$div.find(".nv-legend")[0].getBBox().height : 0;

                    // adjust height based on graph content
                    var footerHeight = chart.$div.find(".zenfooter").outerHeight() || 0,
                        graphHeight = self.height,
                        adjustedHeight = footerHeight + graphHeight + legendHeight;

                    // if tall footer is squishing the chart, recalculate panel height
                    if(footerHeight > 150){
                        chart.$div.height(adjustedHeight);
                        self.setHeight(adjustedHeight + 60);
                        chart.resize();
                    }
                };
                // Here we set an onUpdate function for the chart, which takes a promise as an argument (the update
                // ajax request) and disables the controls until the promise is either fulfilled or it fails.
                chart.onUpdate = function(p1){
                    // start gear spinning
                    var delement = Ext.query(".europaGraphGear", self.getEl().dom)[0];
                    delement.classList.add("spinnerino");

                    // set the graph panel to 'updating'. This will disable controls for combined graphs
                    var graphPanel = self.up("graphpanel");
                    if(graphPanel){
                        graphPanel.updateStart();
                    }

                    // disable the controls for europa graphs which are not combined, such as component graphs
                    var items = self.dockedItems.items;
                    var disableTimer = setTimeout(function(){
                        items.forEach(function(item){
                            if(item.disable){
                                item.disable();
                            }
                        });
                    }, 1000);

                    // regardless of whether the promise is fulfilled or it fails, we will re-enable controls for both types of graphs
                    p1.always(function(){
                        if (graphPanel){
                            graphPanel.updateEnd();
                        }
                        clearTimeout(disableTimer);
                        delement.classList.remove("spinnerino");
                        items.forEach(function(item){
                            item.enable();
                        });
                    });
                };

                chart.zoomTo = function (zoomTime) {
                    if (Ext.isNumeric(zoomTime) && zoomTime > 0) {

                        var zoom_factor = 1.25;
                        var chart_min_range = 1000 * 60 * 20;
                        var curRange = rangeToMilliseconds(self.graph_params.drange);

                        var zoomedRange = Math.floor(curRange / zoom_factor);
                        zoomedRange = Math.max(zoomedRange, chart_min_range);
                        var zoomStart = zoomTime - Math.floor(zoomedRange / 2);
                        var zoomEnd = zoomStart + zoomedRange;

                        var gParams = {
                            'drange': zoomedRange,
                            'start': zoomStart,
                            'end': zoomEnd
                        };
                        if (self.dockedItems.items.length) {
                            // handle own chart changes
                            self.updateGraph(gParams);
                        } else {
                            // handle update at graphPanel level
                            self.fireEvent("zoomPanel", gParams);
                        }
                    }
                }

                chart.normalizeTimeToMs = function (val) {
                    var TIME_UNITS = {
                        s: 1000,
                        m: 1000 * 60,
                        h: 1000 * 60 * 60,
                        d: 1000 * 60 * 60 * 24
                    };
                    var timeUnitsRegExp = /[smhd]/;
                    var timeNow = new Date().getTime();
                    var agoMatch, unitMatch, count, unit, msTime;

                    agoMatch = /ago/.exec(val);
                    if (agoMatch === null) {
                        msTime = val;
                    } else {
                        unitMatch = timeUnitsRegExp.exec(val);
                        count = +val.slice(0, unitMatch.index);
                        unit = val.slice(unitMatch.index, agoMatch.index - 1);
                        msTime = timeNow - count * TIME_UNITS[unit];
                    }
                    return msTime;
                }

            });

        },
        displayLink: function(){
            var config = {},
                encodedConfig, link,
                drange="null",
                // keys to exclude when cloning config object
                exclusions = ["dockedItems"];

            // shallow clone initialConfig object as long as the
            // key being copied is no in the exclusions list.
            // This is useful because the final JSON string needs
            // to be as small as possible!
            for(var i in this.initialConfig){
                if(exclusions.indexOf(i) === -1){
                    config[i] = this.initialConfig[i];
                }
            }
            // see if we can find the date range selected
            var graphPanel = this.up('graphpanel');
            if (graphPanel && Ext.isNumber(graphPanel.drange)) {
                drange = graphPanel.drange;
            }
            // get the zipped + encoded config from the server
            router.gzip_b64({string: Ext.JSON.encode(config)}, function(resp) {
                if (resp.success && resp.data && resp.data.data !== undefined) {
                    link = Ext.String.format("/zport/dmd/viewGraph?drange={0}&data={1}", drange, resp.data.data);
                    if (link.length > 2000) {
                        Zenoss.message.error('Unable to generate link, length is too great');
                    } else {
                        new Zenoss.dialog.ErrorDialog({
                            message: Ext.String.format(_t('<div>' + Ext.String.format(_t('Drag this link to your bookmark bar to link directly to this graph. {0}'),
                                '<br/><br/><a href="' + link + '">Graph: ' + this.graphTitle +  ' </a>') + '</div>')),
                            title: _t('Link to this Graph')
                        });
                    }
                }
            },
            this);
        },
        expandGraph: function(){
            var config = {},
                drange = "null",
                exclusions = ["dockedItems"];

            for(var i in this.initialConfig){
                if(exclusions.indexOf(i) === -1){
                    config[i] = this.initialConfig[i];
                }
            }
            // see if we can find the date range selected
            var graphPanel = this.up('graphpanel');
            if (graphPanel && Ext.isNumber(graphPanel.drange)) {
                drange = graphPanel.drange;
            }

            config.graphId = Ext.id();
            config.autoscale = true;
            config.hasMenu = false;
            config.xtype = 'europagraph';
            config.height = window.outerHeight * 0.75;
            config.width = Math.min(window.outerWidth * 0.80, config.height * 1.6180339887);
            config.maxWidth = 2000;
            config.autoScroll = true;
            delete config.html;

            var win = Ext.create('Zenoss.dialog.BaseWindow', {
                cls: 'white-background-panel',
                layout: 'fit',
                width: config.width * 1.15,
                height: config.height * 1.05,
                resizable: false,
                items: [config]
            });
            win.show();
        },
        displayDefinition: function(){
            Ext.create('Zenoss.dialog.BaseWindow', {
                closeAction: 'destroy',
                title: _t('Graph JSON Definition'),
                autoScroll: true,
                minWidth: 700,
                height: 500,
                items: [{
                    xtype: 'panel',
                    autoScroll: true,
                    html: Ext.String.format('<pre>{0}</pre>', syntaxHighlight(this.chartdefinition))
                }]
            }).show();
        },
        exportData: function() {
            var chart = zenoss.visualization.chart.getChart(this.graphId),
                plots = Ext.JSON.encode(chart.plots),
                form, start, end,
                startDate = chart.getStartDate(),
                endDate = chart.getEndDate(),
                uid = this.uid,
                units = chart.yAxisLabel;

            if(!startDate || !endDate){
                Zenoss.message.error('Cannot export data: graph missing start or end date');
            }

            start = startDate.unix();
            end = endDate.unix();

            form = Ext.DomHelper.append(document.body, {
                tag: 'form',
                method: 'POST',
                action: '/zport/dmd/exportGraph',
                children: [{
                    tag: 'textarea',
                    style: {
                        display: 'none'
                    },
                    name: 'plots',
                    html: plots
                },{
                    tag: 'input',
                    type: 'hidden',
                    name: 'start',
                    value: Zenoss.date.renderWithTimeZone(start, "YYMMDD_hhmmss")
                },{
                    tag: 'input',
                    type: 'hidden',
                    name: 'end',
                    value: Zenoss.date.renderWithTimeZone(end, "YYMMDD_hhmmss")
                },{
                    tag: 'input',
                    type: 'hidden',
                    name: 'uid',
                    value: uid
                },{
                    tag: 'input',
                    type: 'hidden',
                    name: 'title',
                    value: this.graphTitle
                },{
                    tag: 'input',
                    type: 'hidden',
                    name: 'units',
                    value: units
                }]
            });
            form.submit();
        },
        /**
         * In response to ZEN-14295 if we have a lot of series we are trying to plot
         * ignore the set height and make the graph larger so the graph shows up.
         **/
        adjustHeightBasedOnMetrics: function(height, datapoints) {
            height = height || 500;
            // not necessary until we get above 5 datapoints
            if (datapoints.length <= 5) {
                return height;
            }

            var adjustment = height * 0.05;
            return height + (adjustment * datapoints.length);
        },
        initEvents: function() {
            Zenoss.EuropaGraph.superclass.initEvents.call(this);
            this.addEvents(
                /**
                 * @event updateimage
                 * Fire this event to force the chart to redraw itself.
                 * @param {object} params The parameters we are sending to the object.
                 **/
                'updateimage'
            );
            this.on('updateimage', this.updateGraph, this);
            this.graphEl = Ext.get(this.graphId);
        },
        updateGraph: function(params) {
            var gp = Ext.apply({}, params, this.graph_params);
            gp.start = params.start || gp.start;
            if (gp.start < 0) {
                gp.start = 0;
            }

            // see if end is explicitly defined on the params
            if (Ext.isDefined(params.end) && (params.end > params.start)){
                gp.end = params.end;
            }

            var changes = {
                range : {
                    start: gp.start,
                    end: gp.end
                }
            };
            zenoss.visualization.chart.update(this.graphId, changes);

            this.graph_params = gp;
        },
        convertStartToAbsoluteTime: function(start) {
            if (Ext.isNumber(start)) {
                return start;
            }
            return parseInt(new Date() - rangeToMilliseconds(start));
        },
        convertEndToAbsolute: function(end) {
            if (end === CURRENT_TIME) {
                return now();
            }
            return end;
        },
        newTab: function(graph) {
            var config = {},
                link,
                drange="null",
                exclusions = ["dockedItems"];

            for(var i in this.initialConfig){
                if(exclusions.indexOf(i) === -1){
                    config[i] = this.initialConfig[i];
                }
            }
            // see if we can find the date range selected
            var graphPanel = this.up('graphpanel');
            if (graphPanel && Ext.isNumber(graphPanel.drange)) {
                drange = graphPanel.drange;
            }
            else if(graph.graph_params && Ext.isNumber(graph.graph_params.drange)) {
                drange = graph.graph_params.drange;
            }

            // create a new window that will later be
            // redirected to the graph url
            // NOTE: this circumvents popup blocking
            var redirect = window.open("", "_blank");

            // get the zipped + encoded config from the server
            router.gzip_b64({string: Ext.JSON.encode(config)}, function(resp) {
                if (resp.success && resp.data && resp.data.data !== undefined) {
                    link = Ext.String.format("/zport/dmd/viewGraph?drange={0}&data={1}", drange, resp.data.data);
                    if (link.length > 2000) {
                        Zenoss.message.error('Unable to generate link, length is too great');
                    } else {
                        redirect.location = link;
                    }
                }
            });

        },
        onPanLeft: function(graph) {
            var gp = this.graph_params;
            gp.start = this.convertStartToAbsoluteTime(gp.start);
            var delta = Math.round(rangeToMilliseconds(gp.drange)/this.pan_factor);
            var newstart = (gp.start) - delta > 0 ? gp.start - delta : 0;
            var newend = parseInt(newstart + rangeToMilliseconds(gp.drange));

            this.fireEvent("updatelimits", {start:newstart, end:newend});
            this.fireEvent("updateimage", {start:newstart, end:newend});
        },
        onPanRight: function(graph) {
            var gp = this.graph_params;
            gp.start = this.convertStartToAbsoluteTime(gp.start);
            var delta = Math.round(rangeToMilliseconds(gp.drange)/this.pan_factor);
            var newstart = gp.start + delta > 0 ? gp.start + delta : 0;
            var newend = parseInt(newstart + rangeToMilliseconds(gp.drange));

            this.fireEvent("updatelimits", {start:newstart, end:newend});
            this.fireEvent("updateimage", {start:newstart, end:newend});
        },
        doZoom: function(xpos, factor) {
            var gp = this.graph_params;

            gp.end = this.convertEndToAbsolute(gp.end);
            gp.drange = rangeToMilliseconds(gp.drange) * factor;

            var end = gp.end,
                start = parseInt(gp.end - gp.drange);

            this.fireEvent("updatelimits", { start: start, end: end });
            this.fireEvent("updateimage", { start: start, end: end });
        }
    });


    Ext.define("Zenoss.form.GraphRefreshButton", {
        alias:['widget.graphrefreshbutton'],
        extend:"Zenoss.RefreshMenuButton",
        constructor: function(config) {
            config = config || {};
            var menu = {
                xtype: 'statefulrefreshmenu',
                id: config.stateId || Ext.id(),
                trigger: this,
                items: [{
                    cls: 'refreshevery',
                    text: _t('Refresh every')
                },{
                    xtype: 'menucheckitem',
                    text: _t('1 minute'),
                    value: 60,
                    group: 'refreshgroup'
                },{
                    xtype: 'menucheckitem',
                    text: _t('5 minutes'),
                    value: 300,
                    group: 'refreshgroup'
                },{
                    xtype: 'menucheckitem',
                    text: _t('10 Minutes'),
                    value: 600,
                    group: 'refreshgroup'
                },{
                    xtype: 'menucheckitem',
                    text: _t('30 Minutes'),
                    checked: true,
                    value: 1800,
                    group: 'refreshgroup'
                },{
                    xtype: 'menucheckitem',
                    text: _t('1 Hour'),
                    value: 3600,
                    group: 'refreshgroup'
                },{
                    xtype: 'menucheckitem',
                    text: _t('Manually'),
                    value: -1,
                    group: 'refreshgroup'
                }]
            };
            Ext.apply(config, {
                menu: menu
            });
            this.callParent(arguments);
        }
    });


    Ext.define("Zenoss.form.DRangeSelector", {
        alias:['widget.drangeselector'],
        extend:"Ext.form.ComboBox",
        constructor: function(config) {
            config = config || {};
            Ext.apply(config, {
                fieldLabel: _t('Range'),
                name: 'ranges',
                editable: false,
                forceSelection: true,
                autoSelect: true,
                triggerAction: 'all',
                value: '1h-ago',
                queryMode: 'local',
                store: new Ext.data.ArrayStore({
                    id: 0,
                    model: 'Zenoss.model.IdName',
                    data: DATE_RANGES.concat([[0,"<hr>"],["custom", "["+ _t("Custom") +"]"]])
                }),
                valueField: 'id',
                displayField: 'name'
            });
            this.callParent(arguments);
        }
    });

    // datefield that uses number of ms since epoch to
    // get and set date. Also accepts a `displayTZ` property
    // which is used to offset the date that is displayed to
    // the end user.
    // NOTE: when you set the date, it should be UTC. When you
    // get the date, it will be UTC. The `displayTZ` property is
    // only to adjust what the user sees, not to adjust the
    // actual date value.
    Ext.define("Zenoss.form.UTCDateField", {
        alias:['widget.utcdatefield'],
        extend:"Ext.form.DateField",
        constructor: function(config) {
            config = config || {};

            // "Africa/Abidjan" is moment.tz default UTC zone
            this.setDisplayTimezone(config.displayTZ || "Africa/Abidjan");

            // get the browser's local timezone so we can offset that as well :/
            // NOTE: zone returns minutes, so convert to milliseconds
            this.TZLocalMS = moment().utcOffset() * 60 * 1000;

            this.callParent(arguments);
        },

        setDisplayTimezone: function(tz){
            // store provided timezone
            this.displayTZ = tz;

            // get timezone offset *in hours* and convert hours to ms
            this.TZOffsetMS = (+moment.utc().tz(this.displayTZ).format("ZZ") * 0.01) * 1000 * 60 * 60;
        },

        setValue: function(ms, isAdjusted){
            if(!ms){
                return;
            }

            if (isAdjusted === undefined) {
                isAdjusted = true;
            }

            var adjustedTime;

            // if someone is using a date object, get
            // the UTC time and use that
            if(ms instanceof Date){
                ms = ms.getTime();
            }

            // if incoming time is already adjusted, do not attempt
            // to offset it (this is the case when the value is coming
            // from inside the datepicker instead of outside)
            if(!isAdjusted){
                // take provided time, offset with local timezone, then
                // offset with displayTZ
                adjustedTime = ms + this.TZOffsetMS - this.TZLocalMS;
            } else {
                adjustedTime = ms;
            }

            this.callParent([new Date(adjustedTime)]);

            return this;
        },

        // returns ms since epoch (UTC)
        getValue: function(){
            // clear any previous offsets and return just the UTC
            // time since epoch
            return +moment.utc(this.value - this.TZOffsetMS + this.TZLocalMS).toDate();
        },

        beforeBlur : function(){
            var me = this,
                v = me.parseDate(me.getRawValue()),
                focusTask = me.focusTask;

            if (focusTask) {
                focusTask.cancel();
            }

            if (v) {
                // setValue but do not adjust for timezones because
                // it is already adjusted
                me.setValue(v, true);
            }
        },

        onSelect: function(m, d) {
            var me = this;

            // setValue but do not adjust for timezones because
            // it is already adjusted
            me.setValue(d, true);

            me.fireEvent('select', me, d);
            me.collapse();
        }
    });

    var tbarConfig = [
        '->',

        {
            xtype: 'drangeselector',
            cls: 'drange_select',
            labelWidth: 40,
            labelAlign: "right",
            listeners: {
                select: function(self, records, index){
                    var value = records[0].data.id,
                        panel = self.up("graphpanel");

                    // if value is "custom", then reveal the date
                    // picker container
                    if(value === "custom"){
                        panel.showDatePicker();

                    // if user selected the separator, select custom
                    } else if(value === 0){
                        self.setValue("custom");
                        panel.showDatePicker();

                    // otherwise, update graphs
                    } else {
                        // all ranges are relative to now, so set
                        // end to current time
                        panel.setEndToNow();
                        panel.hideDatePicker();
                        // update drange and start values based
                        // on the new end value
                        panel.setDrange(value);
                    }
                }
            }
        },

        "-",

        {
            xtype: "container",
            layout: "hbox",
            cls: "date_picker_container",
            padding: "0 10 0 0",
            items: [
                {
                    xtype: 'utcdatefield',
                    cls: 'start_date',
                    width: 175,
                    fieldLabel: _t('Start'),
                    labelWidth: 40,
                    labelAlign: "right",
                    format:'Y-m-d H:i:s',
                    displayTZ: Zenoss.USER_TIMEZONE,
                    listeners: {
                        change: function(self, val){
                            var panel = self.up("graphpanel");
                            //update graphpanel.start with *UTC time*
                            //NOTE: panel.start should *always* be UTC!
                            panel.start = moment.utc(self.getValue());
                        }
                    }
                },{
                    xtype: 'utcdatefield',
                    cls: 'end_date',
                    width: 175,
                    fieldLabel: _t('End'),
                    labelWidth: 40,
                    labelAlign: "right",
                    disabled: true,
                    format:'Y-m-d H:i:s',
                    displayTZ: Zenoss.USER_TIMEZONE,
                    listeners: {
                        change: function(self, val){
                            var panel = self.up("graphpanel");
                            //update graphpanel.end with *UTC time*
                            //NOTE: panel.end should *always* be UTC!
                            panel.end = moment.utc(self.getValue());
                        }
                    }
                },{
                    xtype: 'checkbox',
                    cls: 'checkbox_now',
                    fieldLabel: _t('Now'),
                    labelWidth: 40,
                    labelAlign: "right",
                    checked: true,
                    listeners: {
                        change: function(self, val) {
                            var panel = self.up("graphpanel");
                            panel.query("datefield[cls='end_date']")[0].setDisabled(val);

                            // if it should be now, update it
                            if(val){
                                panel.setEndToNow();
                            }
                        }
                    }
                }
            ]
        },

        {
            xtype: 'graphrefreshbutton',
            ref: '../refreshmenu',
            iconCls: 'refresh',
            text: _t('Refresh'),
            handler: function(btn) {
                if (btn) {
                    var panel = btn.up("graphpanel");
                    if(panel && panel.isVisible()){
                        panel.refresh();
                    }
                }
            }
        },
        '-',
        {
            xtype: 'button',
            ref: '../newwindow',
            iconCls: 'newwindow',
            hidden: true,
            handler: function(btn) {
                var panel = btn.refOwner;
                var config = panel.initialConfig,
                win = Ext.create('Zenoss.dialog.BaseWindow',  {
                    cls: 'white-background-panel',
                    layout: 'fit',
                    items: [Ext.apply(config,{
                        id: 'device_graphs_window',
                        xtype: 'graphpanel',
                        ref: 'graphPanel',
                        uid: panel.uid,
                        newWindowButton: false
                    })],
                    maximized: true
                });
                win.show();
                win.graphPanel.setContext(panel.uid);
            }
        }];


    Ext.define("Zenoss.form.GraphPanel", {
        alias:['widget.graphpanel'],
        extend:"Ext.Panel",
        tbar: tbarConfig,
        cls: "graphpanel",
        constructor: function(config) {
            config = config || {};

            Ext.applyIf(config, {
                drange: DATE_RANGES[0][0],
                newWindowButton: true,
                columns: 1,
                // images show up after Ext has calculated the
                // size of the div
                bodyStyle: {
                    overflow: 'auto',
                    paddingTop: "15px"
                },
                directFn: router.getGraphDefs
            });

            Zenoss.form.GraphPanel.superclass.constructor.apply(this, arguments);

            // assumes just one docked item
            this.toolbar = this.getDockedItems()[0];

            this.startDatePicker = this.toolbar.query("utcdatefield[cls='start_date']")[0];
            this.endDatePicker = this.toolbar.query("utcdatefield[cls='end_date']")[0];
            this.nowCheck = this.toolbar.query("checkbox[cls='checkbox_now']")[0];

            this.startDatePicker.setDisplayTimezone(Zenoss.USER_TIMEZONE);
            this.endDatePicker.setDisplayTimezone(Zenoss.USER_TIMEZONE);

            // this variable stores the number of current graphs which are actively loading or refreshing
            this.graphBusy = 0;
            // add title to toolbar
            this.toolbar.insert(0, [{
                xtype: 'tbtext',
                text: config.tbarTitle || _t('Performance Graphs')
            }, '-', {
                text: '&lt;',
                width: 40,
                handler: Ext.bind(function(btn, e) {
                    Ext.Array.each(this.getGraphs(), function(graph) {
                        graph.onPanLeft(graph);
                    });
                }, this)
            },{
                text: _t('Zoom In'),
                ref: '../zoomin',
                handler: Ext.bind(function(btn, e) {
                    Ext.Array.each(this.getGraphs(), function(graph) {
                        graph.doZoom.call(this, 0, 1/graph.zoom_factor);
                    });
                }, this)

            },{
                text: _t('Zoom Out'),
                ref: '../zoomout',
                handler: Ext.bind(function(btn, e) {
                    Ext.Array.each(this.getGraphs(), function(graph) {
                        graph.doZoom.call(this, 0, graph.zoom_factor);
                    });
                }, this)
            },{
                text: '&gt;',
                width: 40,
                handler: Ext.bind(function(btn, e) {
                    Ext.Array.each(this.getGraphs(), function(graph) {
                        graph.onPanRight(graph);
                    });
                }, this)
            }]);

            // default range value of 1 hour
            // NOTE: this should be a real number, not a relative
            // measurement like "1h-ago"
	    this.toolbar.query("drangeselector[cls='drange_select']")[0].setValue(this.drange);
            this.drange = rangeToMilliseconds(config.drange);

            // default start and end values in UTC time
            // NOTE: do not apply timezone adjustments to these values!
            this.start = moment.utc().subtract(this.drange, "ms");
            this.setEndToNow();

            // set start and end dates
            this.updateStartDatePicker();
            this.updateEndDatePicker();

            this.hideDatePicker();

            if (config.hideToolbar){
                this.toolbar.hide();
            }
        },
        setContext: function(uid) {
            if (this.newwindow) {
                if (this.newWindowButton) {
                    this.newwindow.show();
                } else {
                    this.newwindow.hide();
                }
            }

            // remove all the graphs
            this.removeAll();
            this.lastShown = 0;
            var params = {
                uid: uid,
                drange: this.drange
            };
            this.uid = uid;
            this.directFn(params, Ext.bind(this.loadGraphs, this));
        },
        loadGraphs: function(result){
            if (!result.success){
                return;
            }
            var data = result.data,
                el = this.getEl();

            if (el && el.isMasked()) {
                el.unmask();
            }
            // this is defined by the visualization library, if it is missing then we can not
            // render any charts
            if (!Ext.isDefined(window.zenoss)) {
                el.mask(_t('Unable to load the visualization library.') , 'x-mask-msg-noicon');

            } else if (data.length > 0){
                this.addGraphs(data);

            }else{

                // no graphs were returned
                if(el){
                   el.mask(_t('No Graph Data') , 'x-mask-msg-noicon');
                }
                // disable the refresh since there are no graphs
                var buttons = this.toolbar.query("graphrefreshbutton");
                if (buttons.length > 0){
                    buttons[0].setInterval(-1);
                }
            }
        },
        addGraphs: function(data) {
            var graphs = [],
                graph,
                graphId,
                me = this,
                start = this.lastShown,
                end = this.lastShown + GRAPHPAGESIZE,
                graphTitle,
                i;

            // load graphs until we have either completed the page or
            // we ran out of graphs
            for (i=start; i < Math.min(end, data.length); i++) {
                graph = data[i];
                graphId = Ext.id();

                graphTitle = graph.contextTitle || graph.title;

                delete graph.title;
                graphs.push(new Zenoss.EuropaGraph(Ext.applyIf(graph, {
                    uid: this.uid,
                    graphId: graphId,
                    graphTitle: graphTitle,
                    ref: graphId,
                    printOptimized: this.printOptimized,
                    // when a europa graph appears in a graph panel then don't show controls
                    dockedItems: [],
                    // set the date range incase we are refreshing after a resize or
                    // tab change
                    graph_params: {
                        drange: this.drange,
                        start: this.start.valueOf(),
                        end: this.end.valueOf()
                    }
                })));
                graphs[graphs.length-1].on("updatelimits", function(limits){
                    this.setLimits(limits.start, limits.end);
                }, this);
                graphs[graphs.length - 1].on("zoomPanel", function (gParams) {
                    this.setGraph(gParams);
                }, this);
            }

            // set up for the next page
            this.lastShown = end;

            // if we have more to show, add a button
            if (data.length > end) {
                graphs.push({
                    xtype: 'button',
                    text: _t('Show more results...'),
                    margin: '0 0 7 7',
                    handler: function(t) {
                        t.hide();
                        // will show the next page by looking at this.lastShown
                        me.addGraphs(data);
                    }
                });
            }
            // render the graphs
            this.organizeGraphsIntoColumns(graphs, this.columns);
        },
        organizeGraphsIntoColumns: function(graphs, numCols) {
            var columns = [], i, col=0;
            // create a column container for each column specified
            for (i=0; i < numCols; i ++) {
                columns.push({
                    xtype: 'container',
                    items: [],
                    // make them equal space
                    columnWidth: 1 / numCols
                });
            }

            // divide the graphs into buckets based on the order in which they were defined.
            for (i=0; i < graphs.length; i ++) {
                columns[col].items.push(graphs[i]);
                col++;
                if (col>=numCols) {
                    col = 0;
                }
            }

            this.add({
                layout: 'column',
                items: columns
            });
        },
        refresh: function() {
            // if end should be set to `now`, set it
            if(this.nowCheck.getValue()){
                this.setEndToNow();
            }

            var graphConfig = {
                drange: this.drange,
                // start and end are moments so they need to be
                // converted to millisecond values
                start: this.start.valueOf(),
                end: this.end.valueOf()
            };

            // if we are rendered but not visible do not refresh
            if(this.isVisible()){
                Ext.each(this.getGraphs(), function(g) {
                    g.fireEvent("updateimage", graphConfig, this);
                });
            }
        },
        getGraphs: function() {
            return this.query('europagraph');
        },

        ///////////////////////////
        // graph time and range type stuff
        //

        setDrange: function(drange) {
            this.drange = drange || this.drange;

            // if drange is relative measurement, convert to ms
            if(!Ext.isNumeric(this.drange)){
                this.drange = rangeToMilliseconds(this.drange);
            }

            // check `now` checkbox since drange is always set from now
            this.nowCheck.setValue(true);

            // update start to reflect new range
            this.start = this.end.clone().subtract(this.drange, "ms");

            //  set the start and end dates to the selected range.
            this.updateStartDatePicker();
            this.updateEndDatePicker();

            this.refresh();
        },

        setGraph: function (gParams) {
            this.setLimits(gParams.start, gParams.end);
            this.drange = gParams.drange;
            this.refresh();
        },

        setLimits: function(start, end){
            // TODO - validate end is greater than start
            this.start = moment.utc(start);
            this.end = moment.utc(end);

            // these limits require a custom date range
            this.drange = end - start;

            // set the range combo to custom
            this.toolbar.query("drangeselector[cls='drange_select']")[0].setValue("custom");

            // uncheck `now` checkbox since we're using a custom range
            this.nowCheck.setValue(false);

            this.showDatePicker();

            //  set the start and end dates to the selected range.
            this.updateStartDatePicker();
            this.updateEndDatePicker();
        },

        setEndToNow: function(){
            this.end = moment.utc();
            this.updateEndDatePicker();

            // if the "now" checkbox is set and range isn't custom, start time should be updated as well
            if(this.nowCheck.getValue() && this.toolbar.query("drangeselector")[0].getValue() !== "custom"){
                this.start = this.end.clone().subtract(this.drange, "ms");
                this.updateStartDatePicker();
            }
        },

        // updates date picker with stored date value, offset for timezone,
        // but forced to be treated as UTC to prevent additional timezone offset
        updateStartDatePicker: function(){
            this.startDatePicker.suspendEvents();
            this.startDatePicker.setValue(this.start.valueOf(), false);
            this.startDatePicker.resumeEvents(false);
        },
        updateEndDatePicker: function(){
            this.endDatePicker.suspendEvents();
            this.endDatePicker.setValue(this.end.valueOf(), false);
            this.endDatePicker.resumeEvents(false);
        },

        showDatePicker: function(){
            // show date picker stuff
            this.toolbar.query("container[cls='date_picker_container']")[0].show();
        },
        hideDatePicker: function(){
            // hide date picker stuff
            this.toolbar.query("container[cls='date_picker_container']")[0].hide();
        },
        updateStart: function(){
            // indicate to the panel that a graph is loading
            this.graphBusy++;
            this.disableControls();
        },
        updateEnd: function(){
            // indicate to the panel that a graph has completed loading / refreshing. Note that the controls should
            // only be re-enabled when all graphs are ready
            this.graphBusy--;
            if(this.graphBusy <= 0){
                this.graphBusy = 0;
            }
            if(this.graphBusy === 0){
                this.enableControls();
            }
        },
        disableControls: function() {
            // this function disables controls for combined graphs with a single panel
            var items = this.getDockedItems()[0].items.items;
            if(this.controlsDisableTimer){
                // if a timer is already set, clear it and start a new one
                clearTimeout(this.controlsDisableTimer);
            }
            var disableTimer = setTimeout(function(){
                items.forEach(function(item){
                    if(item.disable){
                        item.disable();
                    }
                });
            }, 1000);
            this.controlsDisableTimer = disableTimer;
        },
        enableControls: function(){
            // re-enable controls for combined graphs with a single panel
            clearTimeout(this.controlsDisableTimer);
            var items = this.getDockedItems()[0].items.items;
            items.forEach(function(item){
                if(item.enable){
                    item.enable();
                }
            });
        }
    });
}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2014, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/
(function() {
    Ext.define("Zenoss.form.DateDisplayField", {
        alias:['widget.datedisplayfield'],
        extend:"Ext.form.field.Display",
        dateFormat: "YYYY-MM-DD HH:mm:ss a",
        constructor: function(config) {
            config = config || {};
            config.value = this.formatDate(config.value);
            this.callParent([config]);
        },
        formatDate: function(date) {
            var value = date;
            if (value && Ext.isNumeric(value)){
                // assume it is a timestamp and format it using the timezone
                value = Zenoss.date.renderWithTimeZone(value, this.format);
            }
            return value;
        },
        setValue: function(value) {
            this.callParent([this.formatDate(value)]);
        }
    });
}());/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2013, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/
(function(){
    Ext.ns('Zenoss');
    var ZC = Ext.ns('Zenoss.component');

    ZC.renderMap = {};

    /**
     * Register a custom handler for displaying graphs.
     **/
    ZC.registerComponentGraphRenderer = function(meta_type, fn, scope) {
        if (scope) {
            ZC.renderMap[meta_type] = Ext.bind(fn, scope);
        } else {
            ZC.renderMap[meta_type] = fn;
        }
    };

    /**
     * Grabs the registered graph renderer based on the meta_type of the
     * component. If a custom renderer is not found the 'default' one is
     * returned.
     **/
    ZC.getComponentGraphRenderer = function(meta_type) {
        return ZC.renderMap[meta_type] || ZC.renderMap['default'];
    };

    /**
     * The default graph renderer. This simply displays
     * a europa graph for each result returned.
     **/
    ZC.registerComponentGraphRenderer('default',
        function(meta_type, uid, graphId, allOnSame, data) {
            var id, graph, graphTitle, i, graphs = [];
            for(i=0; i< data.length; i++) {
                graph = data[i];
                graphTitle = graph.contextTitle || graph.title;
                delete graph.title;
                id = Ext.id();
                graphs.push(new Zenoss.EuropaGraph(Ext.applyIf(graph, {
                    uid: uid,
                    graphId: id,
                    allOnSame: allOnSame,
                    graphTitle: graphTitle,
                    ref: id,
                    height: 500
                })));
            }
            return graphs;
    });


    /**
     * An example of using a custom renderer to show stacked graphs
     * for processes if you are viewing the memory or cpu and
     * viewing all the graphs on the same page.
     * Notice that it uses the default renderer to actually
     * construct the graphs.
     **/
    ZC.registerComponentGraphRenderer('OSProcess',
        function(meta_type, uid, graphId, allOnSame, data) {
            var fn = ZC.getComponentGraphRenderer('default'), graphs, i;
            for (i=0;i<data.length;i++) {
                if (allOnSame && (graphId === 'Memory' || graphId === 'CPU Utilization')) {
                    data[i].type = 'area';
                }
            }

            graphs = fn(meta_type, uid, graphId, allOnSame, data);
            return graphs;
    });

    Ext.define("Zenoss.form.ComponentGraphPanel", {
        alias:['widget.componentgraphpanel'],
        extend:"Ext.Panel",
        compType: "",
        constructor: function(config) {
            config = config || {};


            Ext.applyIf(config, {
                style: {
                    paddingTop: "10px",
                    paddingLeft: "10px"
                },
                bodyStyle: {
                    overflow: 'auto'
                },
                dockedItems: [{
                    layout: 'hbox',
                    dock: 'top',
                    style: {
                        paddingBottom: "10px"
                    },
                    items: [{
                        xtype: 'combo',
                        queryMode: 'local',
                        displayField: 'name',
                        valueField:'value',
                        ref: '../component',
                        fieldLabel: _t('Component Type'),
                        listeners: {
                            scope: this,
                            select: this.onSelectComponentType
                        }
                    }, {
                        xtype: 'container',
                        width: 10
                    },{
                        xtype: 'combo',
                        disabled: true,
                        queryMode: 'local',
                        labelWidth: 50,
                        ref: '../graphTypes',
                        fieldLabel: _t('Graph'),
                        displayField: 'name',
                        valueField: 'name',
                        listeners: {
                            scope: this,
                            select: this.onSelectGraph
                        }
                    }, {
                        xtype: 'container',
                        width: 10
                    }, {
                        xtype: 'checkbox',
                        baseCls: 'zencheckbox_allonsame',
                        fieldLabel: _t('All on same graph'),
                        labelAlign: 'right',
                        labelSeparator: '',
                        labelWidth: 272,
                        ref: '../allOnSame',
                        listeners: {
                            change: this.updateGraphs,
                            scope: this
                        }
                    }]
                }]
            });
            Zenoss.form.ComponentGraphPanel.superclass.constructor.apply(this, arguments);
        },
        setContext: function(uid) {
            this.uid = uid;
            Zenoss.remote.DeviceRouter.getGraphDefintionsForComponents({
                uid: this.uid
            }, this.updateComboStores, this);
        },
        updateComboStores: function(response){
            if (response.success) {
                this.componentGraphs = response.data;

                // create the component drop down store
                var data = [], componentType;
                for (componentType in this.componentGraphs) {
                    if (this.componentGraphs.hasOwnProperty(componentType) && this.componentGraphs[componentType].length) {
                        data.push([Zenoss.component.displayName(componentType)[0],
                               componentType]);
                    }

                }
                var store = Ext.create('Ext.data.Store', {
                    model: 'Zenoss.model.NameValue',
                    data: data
                });
                this.component.bindStore(store, true);
                if (data.length) {
                    this.component.select(data[0][0]);
                    this.onSelectComponentType(this.component,[
                        this.component.store.getAt(0)
                    ]);
                }
            }
        },
        onSelectComponentType: function(combo, selected) {
            this.compType = selected[0].get('value');
            var store, i, graphIds = this.componentGraphs[this.compType], data=[];
            for (i=0;i<graphIds.length;i++) {
                data.push([
                    graphIds[i]
                ]);
            }
            store = Ext.create('Ext.data.Store', {
                model: 'Zenoss.model.Name',
                data: data
            });
            this.graphTypes.bindStore(store, true);
            if (!data.length) {
                this.graphTypes.disable();
            } else {
                this.graphTypes.enable();
                this.graphTypes.select(data[0][0]);
                // go ahead and show the graphs for the first
                // selected option
                this.onSelectGraph(this.graphTypes,
                                   [this.graphTypes.store.getAt(0)]
                                  );
            }
        },
        onSelectGraph: function(combo, selected) {
            // go to the server and return a list of graph configs
            // from which we can create EuropaGraphs from
            var graphId = selected[0].get('name');
            this.graphId = graphId;
            this.updateGraphs();

        },
        updateGraphs: function() {
            var meta_type = this.compType, uid = this.uid,
                graphId = this.graphId, allOnSame = this.allOnSame.checked;
            Zenoss.remote.DeviceRouter.getComponentGraphs({
                uid: uid,
                meta_type: meta_type,
                graphId: graphId,
                allOnSame: allOnSame
            }, function(response){
                if (response.success) {
                    var graphs=[], fn;
                    this.removeAll();
                    fn = ZC.getComponentGraphRenderer(meta_type);
                    graphs = fn(meta_type, uid, graphId, allOnSame, response.data);
                    this.add(graphs);
                }
            }, this);

        }
    });



}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2011, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

    Ext.define("Zenoss.ComponentTemplatePanel", {
        alias:['widget.componenttemplatepanel'],
        extend:"Ext.Panel",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                layout: 'border',
                defaults: {
                    split: true
                },
                items:[{
                    region: 'center',
                    width: '65%',
                    xtype: 'DataSourceTreeGrid',
                    ref: 'dataSourceTreeGrid',
                    title: null,
                    tbarItems:[{
                        xtype: 'tbtext',
                        text: _t('Data Sources')
                    }, '-'],
                    dsAddHandler: function() {
                        var templateUid = this.ownerCt.ownerCt.ownerCt.getContext();
                        if (!templateUid) {
                            new Zenoss.dialog.ErrorDialog({message: _t('There is no template to which to add a datasource.')});
                            return;
                        }
                        Ext.create('Zenoss.templates.AddDataSourceDialog', {templateUid: templateUid}).show();
                    },
                    dpAddHandler: function() {
                        var selectedNode = this.ownerCt.ownerButton.ownerCt.ownerCt.getSelectionModel().getSelectedNode();
                        if (!selectedNode) {
                            new Zenoss.dialog.ErrorDialog({message: _t('You must select a data source.')});
                            return;
                        }
                        Ext.create('Zenoss.templates.AddDataPointDialog', {
                            dataSourceUid: selectedNode.data.uid,
                            dataSourceId: selectedNode.data.id
                        }).show();
                    },
                    root: {
                        uid: config.contextUid
                    }
                },{
                    xtype: 'panel',
                    layout: 'border',
                    region: 'east',
                    width: '35%',
                    defaults: {
                        split: true
                    },
                    items:[{
                        id: 'component_template_threshold',
                        region: 'north',
                        height: '50%',
                        title: null,
                        stateId: 'component_template_thresholds',
                        xtype: 'thresholddatagrid',
                        ref: '../thresholds',
                        getTemplateUid: function() {
                            return this.getContext();
                        },
                        tbarItems:[{
                            xtype: 'tbtext',
                            text: _t('Thresholds')
                        }, '-']
                    }, {
                        xtype: 'graphgrid',
                        ref: '../graphGrid',
                        height: '50%',
                        region: 'center',
                        title: null,
                        tbarItems:[{
                            xtype: 'tbtext',
                            text: _t('Graph Definitions')
                        }, '-'],
                        getSelectedTemplateUid: function() {
                            return this.uid;
                        }
                    }]
                }]
            });
            this.callParent(arguments);
        },
        setContext: function(uid) {
            this.uid = uid;
            this.dataSourceTreeGrid.setContext(uid);
            this.thresholds.setContext(uid);
            this.graphGrid.setContext(uid);
        },
        getContext: function() {
            return this.uid;
        }
    });

})();
(function(){

    /**
     * @class Zenoss.lockpanel
     * @extends Ext.Panel
     * Panel for locking an object
     **/
    Ext.define('Zenoss.LockPanel', {
        extend: 'Ext.panel.Panel',
        alias: ['widget.lockpanel'],
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                listeners:  {
                    show: Ext.bind(this.setCheckboxes, this)
                },
                items: [{
                    xtype: 'panel',
                    frame: false,
                    border: false,
                    layout: 'vbox',
                    height: 110,
                    defaults: {
                        xtype: 'checkbox',
                        flex: 1,
                        align: 'stretch'
                    },
                    items: [{
                        name: 'updates',
                        ref: '../updates',
                        boxLabel: _t('Lock from updates'),
                        listeners: {
                            change: Ext.bind(this.setCheckboxes, this)
                        },
                        checked: config.updatesChecked
                    },{
                        name: 'deletion',
                        ref: '../deletion',
                        boxLabel: _t('Lock from deletion'),
                        listeners: {
                            change: Ext.bind(this.setCheckboxes, this)
                        },
                        checked: config.deletionChecked
                    },{
                        name: 'sendEvent',
                        ref: '../sendEventWhenBlocked',
                        boxLabel: _t('Send an event when an action is blocked'),
                        checked: config.sendEventChecked
                    }]
                }]

            });

            this.callParent([config]);
        },
        /**
          * This dialog consists of three checkboxes.
          * The rules for enabling or disabling the checkboxes:
          *
          * 1. If either updates or deleted is enabled then
          * sendEvents is enabled.
          *
          * 2. If updates is checked then deletion is disabled.
          **/
        setCheckboxes: function() {
            var updatesChecked = this.updates.getValue(),
                    deletionChecked = this.deletion.getValue();
            // rule 1. sendEvents
            if (updatesChecked || deletionChecked) {
                this.sendEventWhenBlocked.enable();
            } else {
                this.sendEventWhenBlocked.setValue(false);
                this.sendEventWhenBlocked.disable();
            }

            // rule 2. updates and deletion
            if (updatesChecked) {
                this.deletion.setValue(true);
                this.deletion.disable();
            } else {
                this.deletion.enable();
            }

        }

    });


    /**
     * @class Zenoss.dialog.LockForm
     * @extends Zenoss.SmartFormDialog
     * Dialog for locking an object.
     **/
    Ext.define('Zenoss.dialog.LockForm',{
        extend: 'Zenoss.SmartFormDialog',
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                submitFn: Zenoss.remote.DeviceRouter.lockDevices,
                applyOptions: Ext.emptyFn,
                height: 220,
                title: _t('Locking'),
                submitHandler: Ext.bind(this.lockObject, this),
                items:{
                    xtype: 'lockpanel',
                    updatesChecked: config.updatesChecked,
                    deletionChecked: config.deletionChecked,
                    sendEventChecked: config.sendEventChecked
                }
            });
            this.callParent([config]);
        },

        lockObject: function(values) {
            this.applyOptions(values);
            this.submitFn(values);
        }
    });
}());
/*****************************************************************************
 * 
 * Copyright (C) Zenoss, Inc. 2012, all rights reserved.
 * 
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 * 
 ****************************************************************************/


(function() {
    /**
     * A regular text field that allows a person to provide an example of
     * valid input. This will appear in italics next to the input
     *
     *
     *@class Zenoss.form.ExampleTextField
     *@extends Ext.form.field.Text
     */
    Ext.define("Zenoss.form.ExampleTextField", {
        alias:['widget.exampletextfield'],
        extend:"Ext.form.field.Text",
        /**
         * @cfg {String} example A custom example message to display to the bottom right of the field.
         */
        config: {
            example: null
        },
        fieldSubTpl: [ // note: {id} here is really {inputId}, but {cmpId} is available
            '<input id="{id}" type="{type}" {inputAttrTpl}',
            ' size="1"', // allows inputs to fully respect CSS widths across all browsers
            '<tpl if="name"> name="{name}"</tpl>',
            '<tpl if="value"> value="{[Ext.util.Format.htmlEncode(values.value)]}"</tpl>',
            '<tpl if="placeholder"> placeholder="{placeholder}"</tpl>',
            '<tpl if="maxLength !== undefined"> maxlength="{maxLength}"</tpl>',
            '<tpl if="readOnly"> readonly="readonly"</tpl>',
            '<tpl if="disabled"> disabled="disabled"</tpl>',
            '<tpl if="tabIdx"> tabIndex="{tabIdx}"</tpl>',
            '<tpl if="fieldStyle"> style="{fieldStyle}"</tpl>',
            ' class="{fieldCls} {typeCls} {editableCls}" autocomplete="off"/><span class="example">Example: {example}</span>',
            {
                disableFormats: true
            }
        ],

        constructor: function(config){
            config = config || {};
            config.style = config.style || {};
            // make sure there is enough room for the example
            Ext.applyIf(config.style, {
                paddingBottom: 20
            });
            this.callParent([config]);
        },
        getSubTplData: function() {
            var me = this;

            return Ext.apply(me.callParent(), {
                example   : me.example
            });
        }
    });


}());
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2012, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function() {

Ext.define('Zenoss.software.SoftwareGridPanel', {
    extend: 'Ext.grid.GridPanel',
    alias: ['widget.softwarepanel'],
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            store: Ext.create('Zenoss.software.SoftwareStore', {}),
            columns: [{
                dataIndex: 'manufacturer',
                width: 280,
                header: _t('Manufacturer'),
                renderer: function(v) {
                    return Ext.String.format('<a href="{0}">{1}</a>', v.uid, v.name);
                }
            }, {
                dataIndex: 'namelink',
                header: _t('Name'),
                width: 340,
                sortable: true,
                renderer: function(v, col, record) {
                    return Ext.String.format('<a href="{0}">{1}</a>', v, record.data.name);
                }

            }, {
                flex: 1,
                dataIndex: 'installdate',
                header: _t('Install Date')
            }]
        });
        this.callParent([config]);
    },
    refresh: function() {
        this.setContext(this.contextUid);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.getStore().load({
            params: {uid: uid, keys: ['uid', 'manufacturer', 'name', 'namelink', 'installdate']}
        });
    }
});

Ext.define('Zenoss.software.SoftwareModel',  {
    extend: 'Ext.data.Model',
    idProperty: 'uid',
    fields: [
            {name: 'manufacturer'},
            {name: 'name'},
            {name: 'namelink'},
            {name: 'installdate'}
    ]
});

Ext.define("Zenoss.software.SoftwareStore", {
    extend: "Zenoss.NonPaginatedStore",
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            model: 'Zenoss.software.SoftwareModel',
            initialSortColumn: "name",
            directFn: Zenoss.remote.DeviceRouter.getSoftware,
            root: 'data'
        });
        this.callParent(arguments);
    }
});


})();
/* jshint boss:true */
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2012, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){
    var router = Zenoss.remote.PropertiesRouter;

    Ext.ns('Zenoss.cproperties');


    /**
     * Allow zenpack authors to register custom cproperty
     * editors.
     **/
    Zenoss.cproperties.registerCPropertyType = function(id, config){
        Zenoss.zproperties.configPropertyConfigs[id] = config;
    };

    Zenoss.cproperties.hideApplyCheckbox = function(grid){
        if(grid.uid !== "/zport/dmd/Devices"){
            return false;
        }else{
            return true;
        }
    };

    function showAddNewPropertyDialog(grid) {
        var addhandler, uid, config, dialog, msg,
        clearPanel = function(panel){
            var f;
            while(f = panel.items.first()){
              panel.remove(f, true);// can do false to reuse the elements
            }
        };
        Zenoss.zproperties.configPropertyConfigs.state.context = "add";
        msg = '<span style="color:#aaa;">';
        msg += _t('Names must start with a lower case c')+' <br>';
        msg += _t('Example: ');
        msg += '<span style="color:yellow;">';
        msg += _t('cPropertyName');
        msg += '</span></span>';

        addhandler = function() {
            var c = dialog.getForm().getForm().getValues(), value;
            uid = "/zport/dmd/Devices"; // to force it to apply to the root on every newly created prop

            // if it's type 'lines' then it was already saved. the submit should be 'updating' it instead
            if (c.type === 'lines') {
                if (c.value) {
                    // send back as an array separated by a new line
                    value = Ext.Array.map(c.value.split('\n'), function(s) {return Ext.String.trim(s);});
                } else {
                    // send back an empty list if nothing is set
                    value = [];
                }
            }else{
                value = c.value;
            }

            if (c.type === 'selection'){
            // this is a submit on a selection, which was already added as new with the selection action on dialog
            // so here we're just updating the newly created item with the line item
                var params = {
                    uid: uid,
                    zProperty: c.name,
                    value: c.selection
                };
                Zenoss.remote.PropertiesRouter.setZenProperty(params, function(response){
                    if (response.success) {
                        grid.refresh();
                    }
                });
            }else{
                var params = {
                    uid: uid,
                    id: c.name,
                    type: c.type,
                    label: c.description,
                    value: value
                };

                Zenoss.remote.PropertiesRouter.addCustomProperty(params, function(response){
                    if (response.success) {
                        grid.refresh();
                    }
                });
            }
        };

        // form config
        config = {
            submitHandler: addhandler,
            minHeight: 315,
            autoHeight: true,
            width: 480,
            id: 'addCustomDialog',
            defaults:{
                applyLocalHidden: true
            },
            title: _t('Add Custom Config Property'),
            items: [{
                    xtype: 'panel',
                    layout: 'hbox',
                    margin: '0 0 1px 0',
                    items: [
                        {
                            xtype: 'textfield',
                            name: 'name',
                            fieldLabel: _t('Property Name'),
                            width:220,
                            regex: /^c[A-Z]/,
                            regexText: _t("Custom Properties must start with a lower case c"),
                            value: ""
                        },{
                            xtype: 'panel',
                            padding: '0 0 0 10px',
                            items: [
                                {
                                    xtype: 'label',
                                    width: 230,
                                    height: 50,
                                    html: msg
                                }
                            ]
                        }
                    ]
                },{
                    xtype: 'textareafield',
                    name: 'description',
                    margin: '3px 0 10px 0',
                    width:457,
                    height:55,
                    ref: 'desc',
                    fieldLabel: _t('Description')
                },{
                    xtype: 'panel',
                    layout: 'hbox',
                    padding: '8px',
                    style: {border:'1px solid #555'},
                    items: [
                        {
                            xtype: 'combo',
                            name: 'type',
                            ref: 'type',
                            valueField: 'name',
                            value:'string',
                            displayField: 'name',
                            typeAhead: false,
                            forceSelection: true,
                            triggerAction: 'all',
                            fieldLabel: _t('Type'),
                            listeners:{
                                change: function(e){
                                    var panel = Ext.getCmp('typeSelectionPanel');
                                    clearPanel(panel);
                                    panel.add(Zenoss.zproperties.configPropertyConfigs[e.value]);
                                }
                            },
                            listConfig: {
                                maxWidth:185
                            },
                            store: new Ext.data.ArrayStore({
                                model: 'Zenoss.model.Name',
                                data: [
                                ['boolean'],['date'],['float'],['int'],['lines'],
                                ['long'],['password'],['string' ],['selection']
                                ]
                            })
                        },{
                            xtype: 'panel',
                            padding: '0 0 0 10px',
                            id:'typeSelectionPanel',
                            items:[Zenoss.zproperties.configPropertyConfigs.string]

                        }
                ]
                }
            ],
            // explicitly do not allow enter to submit the dialog
            keys: {

            }
        };

        dialog = new Zenoss.SmartFormDialog(config);

        if (Zenoss.Security.hasPermission('zProperties Edit')) {
            dialog.show();
        }


    }

    function showEditCustPropertyDialog(data, grid){
        var path;
        if (grid.uid === "/zport/dmd/Devices"){
            path = "/";
        }else{
            path = grid.uid.split("Devices")[1];
        }
        var pathMsg = '<div style="padding-top:10px;">';
        pathMsg += _t(" This value will be changed to apply to the current location:");
        pathMsg += '<div style="color:#aaa;padding-top:3px;">';
        pathMsg += '<div style="color:#fff;background:#111;margin:2px 7px 2px 0;padding:5px;"> '+path+' </div>';
        pathMsg += '</div></div>';

        var lbltemplate = '<div style="margin:5px 0 15px 0;"><b>{0}</b> <span style="display:inline-block;padding-left:5px;color:#aaccaa">{1}</span></div>';

        var items = [
                {
                    xtype: 'label',
                    width: 500,
                    height: 50,
                    html: Ext.String.format(lbltemplate, _t("Name:"), data.id)
                },{
                    xtype: 'label',
                    width: 500,
                    height: 50,
                    html: Ext.String.format(lbltemplate, _t("Description:"),data.label)
                },{
                    xtype: 'label',
                    width: 500,
                    height: 50,
                    html: Ext.String.format(lbltemplate, _t("Path:"),data.path)
                },{
                    xtype: 'label',
                    width: 500,
                    height: 50,
                    html: Ext.String.format(lbltemplate, _t("Type:"),data.type)
                },{
                    xtype: 'label',
                    width: 300,
                    height: 90,
                    html: pathMsg
                }
        ];

        var pkg = {
            'items': items,
            'uid': grid.uid,
            'type': data.type,
            'value': data.valueAsString,
            'name': data.id,
            'grid': grid,
            'dialogId': 'editCustomDialog',
            'minHeight': 300,
            'width': 500,
            'options': data.options
        };
        Zenoss.zproperties.showEditPropertyDialog(pkg);

    }
    /**
     * @class Zenoss.CustomProperty.Model
     * @extends Ext.data.Model
     * Field definitions for the Config Properties
     **/
    Ext.define('Zenoss.CustomProperty.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: [
            {name: 'id'},
            {name: 'label'},
            {name: 'valueAsString'},
            {name: 'path'},
            {name: 'type'},
            {name: 'islocal'}
        ]
    });

    /**
     * @class Zenoss.CustomProperty.Store
     * @extends Zenoss.DirectStore
     * Store for our configuration properties grid
     **/
    Ext.define("Zenoss.CustomProperty.Store", {
        extend: "Zenoss.DirectStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.CustomProperty.Model',
                initialSortColumn: 'id',
                pageSize: 300,
                directFn: Zenoss.remote.PropertiesRouter.getCustomProperties
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.CustomProperty.Grid", {
        alias: ['widget.custompropertygrid'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};

            Zenoss.Security.onPermissionsChange(function() {
                this.disableButtons(Zenoss.Security.doesNotHavePermission('zProperties Edit'));
            }, this);

            Ext.applyIf(config, {
                stateId: config.id || 'custom_property_grid',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                stateful: true,
                tbar:[
                    {
                    xtype: 'button',
                    iconCls: 'add',
                    tooltip: _t('Add a new Custom Property'),
                    disabled: Zenoss.Security.doesNotHavePermission('zProperties Edit'),
                    ref: 'addButton',
                        handler: function(button) {
                            var grid = button.up("custompropertygrid");
                            showAddNewPropertyDialog(grid);
                        }
                    },
                    {
                    xtype: 'button',
                    iconCls: 'customize',
                    tooltip: _t('Edit selected Custom Property'),
                    disabled: Zenoss.Security.doesNotHavePermission('zProperties Edit'),
                    ref: 'customizeButton',
                        handler: function(button) {
                            var grid = button.up("custompropertygrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();

                            if (Ext.isEmpty(selected)) {
                                return;
                            }
                            // single selection
                            data = selected[0].data;
                            showEditCustPropertyDialog(data, grid);
                        }
                    }, {
                    xtype: 'button',
                    iconCls: 'refresh',
                    tooltip: _t('Refresh'),
                    ref: '../refreshButton',
                    disabled: Zenoss.Security.doesNotHavePermission('zProperties Edit'),
                    handler: function(button) {
                        var grid = button.up("custompropertygrid");
                        grid.refresh();
                    }
                    },{
                        xtype: 'button',
                        iconCls: 'delete',
                        tooltip: _t('Delete Custom Property'),
                        handler: function(button) {
                            var grid = button.up("custompropertygrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();
                            if (Ext.isEmpty(selected)) {
                                return;
                            }

                            data = selected[0].data;
                            new Zenoss.dialog.SimpleMessageDialog({
                                title: _t('Delete Local Property'),
                                message: Ext.String.format(_t("Are you sure you want to delete the Custom Property {0}?"), data.id),
                                buttons: [{
                                    xtype: 'DialogButton',
                                    text: _t('OK'),
                                    handler: function() {
                                        if (grid.uid) {
                                            router.deleteZenProperty({
                                                uid: grid.uid,
                                                zProperty: data.id
                                            }, function(response){
                                                grid.refresh();
                                            });
                                        }
                                    }
                                }, {
                                    xtype: 'DialogButton',
                                    text: _t('Cancel')
                                }]
                            }).show();
                        }
                    }
                ],
                store: Ext.create('Zenoss.CustomProperty.Store', {}),
                columns: [{
                        header: _t("Property Name"),
                        id: 'property',
                        dataIndex: 'id',
                        width: 150,
                        sortable: true
                    },{
                        id: 'label',
                        dataIndex: 'label',
                        header: _t('Description'),
                        width: 400,
                        sortable: true
                    },{
                        id: 'valueString',
                        dataIndex: 'valueAsString',
                        header: _t('Value'),
                        flex: 1,
                        sortable: true
                    },{
                        id: 'pathid',
                        dataIndex: 'path',
                        header: _t('Path'),
                        width: 90,
                        sortable: true
                    },{
                        id: 'type',
                        dataIndex: 'type',
                        header: _t('Type'),
                        width: 60,
                        sortable: true
                    },{
                        id: 'is_local',
                        dataIndex: 'islocal',
                        header: _t('Is Local'),
                        width: 50,
                        sortable: true,
                        filter: false,
                        renderer: function(value){
                            if (value) {
                                return 'Yes';
                            }
                            return '';
                        }
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
        },
        setContext: function(uid) {

            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        },
        onRowDblClick: function(grid, rowIndex, e) {
            var data,
                selected = this.getSelectionModel().getSelection();
            if (!selected) {
                return;
            }
            data = selected[0].data;
            showEditCustPropertyDialog(data, this);
        },
        disableButtons: function(bool) {
            var btns = this.query("button");
            Ext.each(btns, function(btn){
                btn.setDisabled(bool);
            });
        }
    });

    Ext.define("Zenoss.form.CustomPropertyPanel", {
        alias:['widget.custompropertypanel'],
        extend:"Ext.Panel",
        constructor: function(config) {
            config = config || {};
            this.gridId = "custompropgridId";
            Ext.applyIf(config, {
                layout: 'fit',
                autoScroll: 'y',
                height: 800,
                items: [{
                    id: this.gridId,
                    xtype: "custompropertygrid",
                    ref: 'customGrid',
                    displayFilters: config.displayFilters
                }]
            });
            this.callParent(arguments);
        },
        setContext: function(uid) {
            Ext.getCmp(this.gridId).setContext(uid);
        }
    });

})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2012, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function () {

Ext.ns('Zenoss');

var REMOTE = Zenoss.remote.JobsRouter;

function lengthOrZero(o) {
    if (o) {
        return o;
    }
    return 0;
}

function isObjectEmpty(o) {
    for (var i in o) {
        if (o.hasOwnProperty(i)) {
             return false;
        }
    }
    return true;
}

function jobLinkRenderer(value, metadata, record) {
    var job = record.data,
    description = job.description;
    description = description.length > 58 ? description.substring(0, 55) + '...' : description;
    // Not htmlEncod()ing the 'description' below, as it's already coming to use encoded
    return "<a href='/zport/dmd/joblist#jobs:" + job.uuid + "'>" + description + "</a>";
}

function readableRenderer(value, metadata, record) {
    var adjective,
        job = record.data,
        date = new Date(0);
    switch (job.status) {
        case "SUCCESS":
            adjective = _t("Finished");
            break;
        case "STARTED":
            adjective = _t("Started");
            break;
        case "PENDING":
            adjective = _t("Scheduled");
            break;
    }
    date.setUTCSeconds(value);
    return adjective + " " + date.readable(1);
}

Ext.define("Zenoss.model.Job", {
    extend: 'Ext.data.Model',
    idProperty: 'uuid',
    fields: [
        'uuid',
        'type',
        'description',
        'scheduled',
        'started',
        'finished',
        'status',
        'result',
        'user'
    ]
});

var runningJobsStore = Ext.create('Ext.data.Store', {
    model: 'Zenoss.model.Job',
    initialSortColumn: 'running'
});

var pendingJobsStore = Ext.create('Ext.data.Store', {
    model: 'Zenoss.model.Job',
    initialSortColumn: 'pending'
});

var finishedJobsStore = Ext.create('Ext.data.Store', {
    model: 'Zenoss.model.Job',
    initialSortColumn: 'finished'
});

Ext.define("Zenoss.JobsWidget", {
    alias: ['widget.jobswidget'],
    extend: "Ext.Button",
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            menuAlign: 'br-tr?',
            menu: {
                layout: 'fit',
                plain: true,
                bodyStyle: 'background:transparent;padding:0;border:0',
                style: 'background:transparent;padding:0;border:0',
                items: [{
                    xtype: 'container',
                    width: 425,
                    style: {
                        border: '1px solid #444',
                        '-webkit-border-radius': 5,
                        '-moz-border-radius': 5,
                        '-ms-border-radius': 5,
                        borderRadius: 5
                    },
                    items: [{
                        id: 'nojobspanel',
                        xtype: 'panel',
                        bodyStyle: 'padding:4px',
                        hidden: true,
                        html: _t('No jobs')
                    },{
                        id: 'jobs-grid-running',
                        xtype: 'grid',
                        hideHeaders: true,
                        forceFit: true,
                        stateful: false,
                        disableSelection: true,
                        title: _t('Running jobs'),
                        store: runningJobsStore,
                        columns: [{
                            header: 'description',
                            dataIndex: 'description',
                            renderer: jobLinkRenderer
                        }, {
                            header: 'started',
                            dataIndex: 'started',
                            renderer: readableRenderer,
                            width: 150
                        }]
                    },{
                        id: 'jobs-grid-pending',
                        xtype: 'grid',
                        hideHeaders: true,
                        forceFit: true,
                        stateful: false,
                        disableSelection: true,
                        title: _t('Pending jobs'),
                        store: pendingJobsStore,
                        columns: [{
                            header: 'description',
                            dataIndex: 'description',
                            renderer: jobLinkRenderer
                        }, {
                            header: 'scheduled',
                            dataIndex: 'scheduled',
                            renderer: readableRenderer,
                            width: 150
                        }]
                    },{
                        id: 'jobs-grid-finished',
                        xtype: 'grid',
                        hideHeaders: true,
                        forceFit: true,
                        stateful: false,
                        disableSelection: true,
                        title: _t('Finished jobs'),
                        store: finishedJobsStore,
                        columns: [{
                            header: 'description',
                            dataIndex: 'description',
                            renderer: jobLinkRenderer,
                            width:60,
                            flex: 1
                        }, {
                            header: 'finished',
                            dataIndex: 'finished',
                            renderer: readableRenderer,
                            width: 150
                        }]
                    },{
                        xtype: 'panel',
                        id: 'more-jobs-link',
                        bodyStyle: 'font-size:11px; padding:3px; padding-left: 5px',
                        html: '<a href="/zport/dmd/joblist">More...</a>'
                    }]
                }]
            }
        });
        this.lastchecked = 0;
        this.callParent([config]);
        this.on('beforerender', this.on_render, this, {single:true});
        this.pollTask = new Ext.util.DelayedTask(this.poll, this);
    },
    initComponent: function() {
        this.lastchecked = Ext.util.Cookies.get('jobswidget_update') || 0;
        this.callParent(arguments);
    },
    on_render: function() {
        this.menucontainer = this.menu.items.items[0];
        this.init_tip();
        // initially delay before rending the jobs widget so that
        // other, more important, router requests have time to complete.
        this.pollTask.delay(2000);
    },
    initEvents: function() {
        this.callParent(arguments);
        this.addEvents('update');
        // Listen to Direct requests for job results
        Ext.Direct.on('event', function(e) {
            if (Ext.isDefined(e.result) && e.result && Ext.isDefined(e.result.new_jobs)) {
                Ext.each(e.result.new_jobs, function(job) {
                    this.alert_new(job);
                }, this);
            }
        }, this);
    },
    init_tip: function() {
        var me = this,
            klass = Ext.ClassManager.getByAlias("widget.tooltip"),
            tip = this.tip = Ext.create('Ext.tip.ToolTip', {
                target: this,
                dismissDelay: 9000,
                show: function() {
                    klass.prototype.show.call(this);
                    this.triggerElement = me.el;
                    this.clearTimer('hide');
                    this.targetXY = this.el.getAlignToXY(me.el, 'br-tr', [-15, -18]);
                    klass.prototype.show.call(this);
                }
            });
        tip.mun(this, 'mouseover', tip.onTargetOver, tip);
        tip.mun(this, 'mouseout', tip.onTargetOut, tip);
        tip.mun(this, 'mousemove', tip.onMouseMove, tip);
    },
    alert: function(msg) {
        if (this.tip.isVisible()) {
            msg = this.tip.body.dom.innerHTML + '<br/><br/>' + msg;
        }
        this.tip.update(msg);
        this.tip.show();
    },
    alert_new: function(job) {
        this.alert("<b>New job</b>: " + Ext.htmlEncode(job.description));
    },
    alert_finished: function(job) {
        this.alert("<b>Finished job</b>: " + Ext.htmlEncode(job.description));
    },
    poll: function() {
        this.update();
        this.pollTask.delay(Zenoss.settings.zenjobsRefreshInterval * 1000);
    },
    pause: function() {
        this.pollTask.cancel();
    },
    check_for_recently_finished: function(jobs) {
        if (Ext.isDefined(jobs.SUCCESS)) {
            Ext.each(jobs.SUCCESS, function(job) {
                if (job.finished >= this.lastchecked) {
                    this.alert_finished(job);
                }
            }, this);
        }
        this.set_lastchecked();
    },
    update_button: function(totals) {
        var pending = lengthOrZero(totals.PENDING),
            running = lengthOrZero(totals.STARTED),
            text, cl, jobText;
        if (!pending && !running) {
            text = _t("0 Jobs");
            cl = 'circle_arrows_still';
        } else {
            if (running > 1 ) {
                jobText = " Jobs";
            }else {
                jobText = " Job";
            }
            text = running + jobText + " ("+pending+" Pending)";
            cl = 'circle_arrows_ani';
        }
        this.setIconCls(cl);
        this.setText(text);
    },
    update_menu: function(jobs) {
        if (!isObjectEmpty(jobs)) {
            Ext.getCmp('nojobspanel').hide();
            Ext.getCmp('jobs-grid-running').setVisible(jobs.STARTED);
            if (jobs.STARTED) {
                runningJobsStore.loadData(jobs.STARTED);
            }
            Ext.getCmp('jobs-grid-pending').setVisible(jobs.PENDING);
            if (jobs.PENDING) {
                pendingJobsStore.loadData(jobs.PENDING);
            }
            Ext.getCmp('jobs-grid-finished').setVisible(jobs.SUCCESS);
            if (jobs.SUCCESS) {
                finishedJobsStore.loadData(jobs.SUCCESS);
            }
            Ext.getCmp('more-jobs-link').show();
        } else {
            Ext.getCmp('nojobspanel').show();
            Ext.getCmp('jobs-grid-running').hide();
            Ext.getCmp('jobs-grid-pending').hide();
            Ext.getCmp('jobs-grid-finished').hide();
            Ext.getCmp('more-jobs-link').hide();
        }
    },
    set_lastchecked: function() {
        // expires two weeks from now
        var cookieExpires = Ext.Date.add(new Date(), Ext.Date.DAY, 14);
        this.lastchecked = (new Date().getTime()/1000);
        Ext.util.Cookies.set('jobswidget_update', this.lastchecked, cookieExpires);
    },
    update: function() {
        if (!Zenoss.Security.doesNotHavePermission('Manage DMD') && Zenoss.settings.zenjobsRefreshInterval) {
            REMOTE.userjobs({}, function(result){
                if (result && result.jobs) {
                    var jobs = result.jobs;
                    this.update_button(result.totals);
                    this.update_menu(jobs);
                    this.check_for_recently_finished(jobs);
                }

            }, this);
        }
    }
});

})(); // End local namespace
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2012, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/

(function(){

Ext.ns('Zenoss', 'Zenoss.devicemanagement');

    var timeZone = "";

    Zenoss.devicemanagement = {
        createTooltip: function(text, overtext){
            return '<span style="cursor:pointer" title="'+overtext+'" >'+text+'</span>';
        },
        getProdStateValue: function(prodStateName){
            for ( var key in Zenoss.env.PRODUCTION_STATES_MAP) {
                if (Zenoss.env.PRODUCTION_STATES_MAP[key] === prodStateName){
                     return parseFloat(key);
                }
            }
            return false;
        },
        getStartTime: function(startTime){
            // @startTime format: 2012/10/22 08:05:00.000
            // break it up for the form elements
            var t = startTime.split(" ");
            var _start = {};
            _start.date = t[0].split("/")[1]+"/"+t[0].split("/")[2]+"/"+t[0].split("/")[0];
            _start.hr = t[1].split(":")[0];
            _start.min = t[1].split(":")[1];
            return _start;
        },
        getDuration: function(duration){
            var patt1=/(?:(\d+) days )?(?:(\d\d):)?(\d\d):00/g;
            var match = patt1.exec(duration);
            var _duration = {};
            _duration.days = match[1] || 0;
            _duration.hrs  = match[2] || 0;
            _duration.mins = match[3] || 0;
            return _duration;
        },
        setUsersCombo: function(grid, combo, index){
            if(!Ext.isDefined(index)) {
                index = -1;
            }

            // create an object of users and return it - format like productionStates
            Zenoss.remote.DeviceManagementRouter.getUserList({uid:grid.uid}, function(response){
                if (response.success) {
                    Zenoss.devicemanagement.setComboFromData(response, combo, grid, 'user', index);
                }
            });
        },
        setRolesCombo: function(grid, uid, combo, index){
            if(!Ext.isDefined(index)) {
                index = -1;
            }
            Zenoss.remote.DeviceManagementRouter.getRolesList({uid:uid}, function(response){
                if (response.success) {
                    Zenoss.devicemanagement.setComboFromData(response, combo, grid, 'role', index);
                }
            });
        },
        setComboFromData: function(response, combo, grid, type, index){
            var data = [], gdata = grid.getView().getStore().data;
            for(var i=0;i < response.data.length; i++){
                if(type === "user"){
                    if( !this.alreadyInCombo(response.data[i], gdata, type)  ){
                        data.push([response.data[i]]);
                    }
                }else{
                    data.push([response.data[i]]);
                }
            }
            combo.store.loadData(data);
            if(index === -1) {
                return;
            }
            combo.setValue(combo.store.getAt(index));
        },
        alreadyInCombo: function(data, gdata){
            for(var i=0;i < gdata.items.length; i++){
                if(data === gdata.items[i].data.id) {
                    return true;
                }
            }
            return false;
        },
        setComboSleep: function(combo){
            combo.setDisabled(true);
            combo.setValue(combo.store.getAt(-1));
        }
    };

// ----------------------------------------------------------------- DIALOGS
    function maintDialog(grid, data) {
        if (!Ext.isDefined(data)) {
            data = "";
        }
        var addhandler, config, dialog, newEntry;
        var labelmargin = '5px 5px 0 0';
        newEntry = (data === "");

        addhandler = function() {
            var c = dialog.getForm().getForm().getValues();
            var padZero = function(num){
                if(num === "") {
                    return 0;
                }
                if(num.length === 1){
                    return "0"+num;
                }
                return num;
            };
            if(c.duration_days === 0 && c.duration_hrs === 0 && c.duration_mins === 0){ // if they didn't enter anything
                c.duration_mins = "1"; // have to have at least 1 minute
            }

            var params = {
                uid:                    grid.uid,
                id:                     c.id,
                name:                   c.name,
                startDate:              c.start_date,
                startHours:             padZero(c.start_hr),
                startMinutes:           padZero(c.start_min),
                durationDays:           c.duration_days,
                durationHours:          padZero(c.duration_hrs),
                durationMinutes:        padZero(c.duration_mins),
                repeat:                 c.repeat,
                startProductionState:   c.start_state,
                enabled:                c.enabled,
                days:                   c.days,
                occurrence:             c.occurrence
            };

          if(newEntry){
                Zenoss.remote.DeviceManagementRouter.addMaintWindow({params:params}, function(response){
                    if (response.success) {
                        params.id = c.name; // since not an edit, need to manually populate the id as the hidden field will be empty
                        Zenoss.remote.DeviceManagementRouter.editMaintWindow({params:params}, function(response){
                            if (response.success) {
                                grid.refresh();
                            }
                        });
                    }
                });
            }else{
                Zenoss.remote.DeviceManagementRouter.editMaintWindow({params:params}, function(response){
                    if (response.success) {
                        grid.refresh();
                    }
                });
            }
        };

        // form config
        config = {
            submitHandler: addhandler,
            minHeight: 340,
            width: 400,
            id: 'addDialog',
            title: _t("Add New Maintenance Window"),
            listeners: {
                'afterrender': function(e){
                    if(!newEntry){ // this window will be used to EDIT the values instead of create from scratch
                        e.setTitle(_t("Edit Maintenance Window"));
                        var fields = e.getForm().getForm().getFields();
                        var prodState = Zenoss.devicemanagement.getProdStateValue(data.startProdState);
                        var startTime = Zenoss.devicemanagement.getStartTime(data.startTime_data);
                        var duration = Zenoss.devicemanagement.getDuration(data.duration_data); // returns object with days,hrs,mins
                        fields.findBy(
                            function(record){
                                switch(record.getName()){
                                    case "start_state"    : record.setValue(prodState);  break;
                                    case "start_date"     : record.setValue(startTime.date);  break;
                                    case "start_hr"       : record.setValue(startTime.hr); break;
                                    case "start_min"      : record.setValue(startTime.min); break;
                                    case "name"           : record.setValue(data.name);  break;
                                    case "id"             : record.setValue(data.name);  break;
                                    case "duration_days"  : record.setValue(duration.days);  break;
                                    case "duration_hrs"   : record.setValue(duration.hrs);  break;
                                    case "duration_mins"  : record.setValue(duration.mins);  break;
                                    case "repeat"         : record.setValue(data.repeat);  break;
                                    case "enabled"        : record.setValue(data.enabled);  break;
                                    case "days"           : record.setValue(data.days);  break;
                                    case "occurrence"     : record.setValue(data.occurrence);  break;
                                }
                            }
                        );
                    }
                }
            },
            items: [
                {
                    xtype: 'panel',
                    layout: 'hbox',
                    margin: '0 0 30px 0',
                    items: [
                        {
                            xtype: 'textfield',
                            name: 'name',
                            disabled: !newEntry,
                            fieldLabel: _t('Name'),
                            margin: '0 10px 0 0',
                            width:220,
                            regex: Zenoss.env.textMasks.allowedNameText,
                            regexText: Zenoss.env.textMasks.allowedNameTextFeedback,
                            allowBlank: false
                        },{
                            xtype: 'hiddenfield',
                            name: 'id'
                        },{
                            xtype: 'panel',
                            layout: 'hbox',
                            margin: '18px 0 0 34px',
                            items:[
                                {
                                    xtype: 'label',
                                    text: _t('Enabled?'),
                                    margin: labelmargin
                                },{
                                    xtype: 'checkbox',
                                    name: 'enabled'
                                }
                            ]
                        }
                    ]

                },{
                    xtype: 'panel',
                    layout: 'hbox',
                    margin: '0 0 30px 0',
                    items: [
                        {
                            xtype: 'datefield',
                            allowBlank: false,
                            name: 'start_date',
                            fieldLabel: _t('Start Date and Time')
                        },{
                            xtype: 'panel',
                            layout: 'hbox',
                            margin: '17px 0 0 45px',
                            items: [
                                {
                                    xtype: 'panel',
                                    layout: 'hbox',
                                    items: [
                                        {
                                            xtype: 'label',
                                            text: Ext.String.format(_t('Time ({0})'), timeZone),
                                            margin: labelmargin
                                        },{
                                            xtype: 'numberfield',
                                            maxValue: 23,
                                            minValue: 0,
                                            value: 0,
                                            name: 'start_hr',
                                            width: 45,
                                            allowDecimals: false,
                                            margin: '0 0px 0 0',
                                            allowBlank: false
                                        }
                                    ]
                                },{
                                    xtype: 'panel',
                                    layout: 'hbox',
                                    items: [
                                        {
                                            xtype: 'label',
                                            text: _t(':'),
                                            margin: '5px 1px 0 1px'
                                        },{
                                            xtype: 'numberfield',
                                            name: 'start_min',
                                            maxValue: 55,
                                            minValue: 0,
                                            value: 0,
                                            step: 5,
                                            regexMask: /[0-9]*[05]/,
                                            regex: /^[0-9]*[05]$/,
                                            allowDecimals: false,
                                            listeners: {
                                                'validitychange': function(field, isValid){
                                                    if(!isValid){
                                                        field.setValue(5);
                                                    }
                                                }
                                            },
                                            width: 45,
                                            allowBlank: false
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                },{
                    xtype: 'panel',
                    layout: 'hbox',
                    margin: '30px 0 30px 0px',
                    items: [
                        {
                            xtype: 'panel',
                            layout: 'hbox',
                            items: [
                                {
                                    xtype: 'label',
                                    text: _t('Duration in Days:'),
                                    margin: labelmargin
                                },{
                                    xtype: 'numberfield',
                                    name: 'duration_days',
                                    width: 50,
                                    value: 0,
                                    minValue: 0,
                                    maxValue: 999,
                                    margin: '0 14px 0 0',
                                    allowDecimals: false,
                                    allowBlank: false
                                }
                            ]
                        },{
                            xtype: 'panel',
                            layout: 'hbox',
                            items: [
                                {
                                    xtype: 'label',
                                    text: _t('Hrs:'),
                                    margin: labelmargin
                                },{
                                    xtype: 'numberfield',
                                    name: 'duration_hrs',
                                    width: 50,
                                    value: 0,
                                    maxValue: 23,
                                    minValue: 0,
                                    margin: '0 13px 0 0',
                                    allowDecimals: false,
                                    allowBlank: false
                                }
                            ]
                        },{
                            xtype: 'panel',
                            layout: 'hbox',
                            items: [
                                {
                                    xtype: 'label',
                                    text: _t('Mins:'),
                                    margin: labelmargin
                                },{
                                    xtype: 'numberfield',
                                    name: 'duration_mins',
                                    value: 0,
                                    maxValue: 59,
                                    minValue: 0,
                                    allowDecimals: false,
                                    width: 50,
                                    allowBlank: false
                                }
                            ]
                        }
                      ]
                    },{
                        xtype: 'panel',
                        layout: 'hbox',
                        margin: '0 0 30px 0',
                        items: [
                            {
                                xtype: 'combo',
                                name: 'repeat',
                                id: 'repeat-combo',
                                ref: 'repeat',
                                margin: '0 20px 0 0',
                                valueField: 'name',
                                value:'Never',
                                width: 160,
                                displayField: 'name',
                                typeAhead: false,
                                forceSelection: true,
                                triggerAction: 'all',
                                fieldLabel: _t('Repeat'),
                                listeners: {
                                                'select': function(field, value){
                                                    var cmb_occ = Ext.getCmp('occurrence-combo');
                                                    var cmb_days = Ext.getCmp('days-combo');
                                                    if (value && value[0].index === 5) {
                                                        cmb_occ.enable();
                                                        cmb_days.enable();
                                                    } else {
                                                        cmb_occ.disable();
                                                        cmb_days.disable();
                                                    }
                                                }
                                            },
                                listConfig: {
                                    maxWidth:160
                                },
                                store: new Ext.data.ArrayStore({
                                    model: 'Zenoss.model.Name',
                                    data: [
                                        ['Never'],['Daily'],['Every Weekday'],['Weekly'],
                                        ['Monthly: day of month'],['Monthly: day of week']
                                    ]
                                })
                            },{
                                xtype: 'combo',
                                disabled: true,
                                name: 'occurrence',
                                id: 'occurrence-combo',
                                ref: 'occurrence',
                                margin: '0 20px 0 0',
                                valueField: 'name',
                                value:'1st',
                                width: 70,
                                displayField: 'name',
                                typeAhead: false,
                                forceSelection: true,
                                triggerAction: 'all',
                                fieldLabel: _t('Occurrence'),
                                listConfig: {
                                    maxWidth:70
                                },
                                store: new Ext.data.ArrayStore({
                                    model: 'Zenoss.model.Name',
                                    data: [
                                        ['1st'], ['2nd'], ['3rd'], ['4th'], ['5th'], ['Last']
                                    ]
                                })
                            },{
                                xtype: 'combo',
                                disabled: true,
                                name: 'days',
                                id: 'days-combo',
                                ref: 'days',
                                margin: '0 20px 0 0',
                                valueField: 'name',
                                value:'Sunday',
                                width: 95,
                                displayField: 'name',
                                typeAhead: false,
                                forceSelection: true,
                                triggerAction: 'all',
                                fieldLabel: _t('Days'),
                                listConfig: {
                                    maxWidth:95
                                },
                                store: new Ext.data.ArrayStore({
                                    model: 'Zenoss.model.Name',
                                    data: [
                                        ['Monday'], ['Tuesday'], ['Wednesday'],
                                        ['Thursday'], ['Friday'], ['Saturday'],
                                        ['Sunday']
                                    ]
                                })
                            }
                       ]
                },  {
                        xtype: 'panel',
                        layout: 'hbox',
                        margin: '0 0 30px 0',
                        items: [
                            {
                                xtype: 'combo',
                                name: 'start_state',
                                ref: 'start_state',
                                id: 'start_state',
                                valueField: 'value',
                                displayField: 'name',
                                queryMode: 'local',
                                typeAhead: false,
                                forceSelection: true,
                                triggerAction: 'all',
                                fieldLabel: _t('Window Production State'),
                                listConfig: {
                                    maxWidth:185
                                },
                                listeners: {
                                    'afterrender': function(combo){
                                        combo.setValue(combo.store.getAt(0));
                                    }
                                },
                                store: Ext.create('Ext.data.Store', {
                                    fields: ['value', 'name'],
                                    data : Zenoss.env.PRODUCTION_STATES
                                })
                            }

                       ]
                }
            ],
            // explicitly do not allow enter to submit the dialog
            keys: {}
        };


        if (Zenoss.Security.hasPermission('Manage Device')) {
            dialog = new Zenoss.SmartFormDialog(config);
            dialog.show();
        }else{ return false; }
    }


    function commandsDialog(grid, data) {
        if (!Ext.isDefined(data)) {
            data = "";
        }
        var addhandler, config, dialog, newEntry;
        var wintitle = _t("Add New User Command");
        newEntry = data === "";

        addhandler = function() {
            var c = dialog.getForm().getForm().getValues();

            var params = {
                uid:          grid.uid,
                id:           c.id,
                name:         c.name,
                description:  c.description,
                command:      c.command,
                password:     c.psword
            };
            if(newEntry){
                Zenoss.remote.DeviceManagementRouter.addUserCommand({params:params}, function(response){
                    if (response.success) {
                        grid.refresh();
                    }
                });
            }else{
                Zenoss.remote.DeviceManagementRouter.updateUserCommand({params:params}, function(response){
                    if (response.success) {
                        grid.refresh();
                    }
                });
            }
        };

        // form config
        config = {
            submitHandler: addhandler,
            minHeight: 315,
            autoHeight: true,
            width: 480,
            id: 'commandsDialog',
            listeners:{
                'afterrender': function(e){
                    if(!newEntry){ // this window will be used to EDIT the values instead of create from scratch
                        wintitle = _t("Edit User Command");
                        // set the dialog values after loading
                        e.setTitle(wintitle);
                        var fields = e.getForm().getForm().getFields();
                        fields.findBy(
                            function(record){
                                switch(record.getName()){
                                    case "name"         : record.setValue(data.id);  break;
                                    case "id"           : record.setValue(data.id);  break;
                                    case "description"  : record.setValue(data.description);  break;
                                    case "command"      : record.setValue(data.command); break;
                                }
                            }
                        );
                    }
                }
            },
            title: wintitle,
            items: [
                {
                    xtype: 'textfield',
                    name: 'name',
                    disabled: !newEntry,
                    fieldLabel: _t('Name'),
                    width:220,
                    regexText: Zenoss.env.textMasks.allowedNameTextFeedback,
                    regex: Zenoss.env.textMasks.allowedNameText,
                    allowBlank: false
                },{
                    xtype: 'hiddenfield',
                    name: 'id'
                },{
                    xtype: 'textareafield',
                    name: 'description',
                    margin: '3px 0 10px 0',
                    width:457,
                    height:55,
                    ref: 'desc',
                    regexText: Zenoss.env.textMasks.allowedDescTextFeedback,
                    regex: Zenoss.env.textMasks.allowedDescText,
                    fieldLabel: _t('Description')
                },{
                    xtype: 'textareafield',
                    name: 'command',
                    margin: '3px 0 10px 0',
                    width:457,
                    height:55,
                    ref: 'cmd',
                    fieldLabel: _t('Command')
                },{
                    xtype: 'password',
                    name: 'psword',
                    width:220,
                    fieldLabel: _t('Confirm Password')
                }
            ],
            // explicitly do not allow enter to submit the dialog
            keys: {}
        };

        if (Zenoss.Security.hasPermission('Manage Device')) {
            dialog = new Zenoss.SmartFormDialog(config);
            dialog.show();
        }else{ return false; }

    }    // end add new commands dialog


    // ================================================= ADMIN DIALOG



    function adminsDialog(grid) {
        var addhandler, config, dialog;

        addhandler = function() {
            var c = dialog.getForm().getForm().getValues();
            var params = {
                uid:    grid.uid,
                name:   c.admin_combo,
                role:   c.role_combo
            };

            Zenoss.remote.DeviceManagementRouter.addAdminRole({params:params}, function(response){
                if (response.success) {
                    Zenoss.remote.DeviceManagementRouter.updateAdminRole({params:params}, function(response){
                        if (response.success) {
                            grid.reset();
                        }
                    });
                }
            });

        };

        // form config
        config = {
            submitHandler: addhandler,
            minHeight: 200,
            autoHeight: true,
            width: 250,
            autoScroll: false,
            id: 'adminsDialog',
            defaults:{
                applyLocalHidden: true
            },
            title: _t('Add Administrator'),
            items: [
                {
                    xtype: 'combo',
                    name: 'admin_combo',
                    ref: 'admin_combo',
                    editable: false,
                    fieldLabel: _t('Administrators'),
                    queryMode:'local',
                    listConfig: {
                        maxWidth:185
                    },
                    listeners: {
                        afterrender: function(combo){
                            Zenoss.devicemanagement.setUsersCombo(grid, combo, 0);
                        }
                    },
                    store: ['none']
                },{
                    xtype: 'combo',
                    name: 'role_combo',
                    margin: '20px 0 0 0',
                    ref: 'role_combo',
                    editable: false,
                    fieldLabel: _t('Role'),
                    queryMode:'local',
                    listConfig: {
                        maxWidth:185
                    },
                    listeners: {
                        afterrender: function(combo){
                            Zenoss.devicemanagement.setRolesCombo(grid, grid.uid, combo, 0);
                        }
                    },
                    store: ['none']
                }
            ],
            // explicitly do not allow enter to submit the dialog
            keys: {}
        };

        dialog = new Zenoss.SmartFormDialog(config);

        if (Zenoss.Security.hasPermission('Manage Device')) {
            dialog.show();
        }
    }    // end add new admin dialog


// --------------------------------------------------------------- GRIDS & WINDOW
Ext.define("Zenoss.devicemanagement.Administration", {
    alias:['widget.devadmincontainer'],
    extend:"Ext.Panel",
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'border',
            defaults: {
                split: true
            },
            items: [{
                xtype: 'MaintWindowsGrid',
                id: 'maintWindowGrid',
                region: 'north',
                height: '60%',
                ref: 'maintWindow'
            },{
                xtype: 'panel',
                layout: 'border',
                region: 'center',
                defaults: {
                    split: true
                },
                items: [{
                    xtype: 'AdminCommandsGrid',
                    id: 'deviceCommandsGrid',
                    ref: '../devicecommands',
                    region: 'west',
                    width: '50%'
                }, {
                    xtype: 'AdministratorsGrid',
                    id: 'adminsGrid',
                    ref: '../admins',
                    region: 'center'
                }]
            }]
        });
        Zenoss.devicemanagement.Administration.superclass.constructor.call(this, config);
    },
    setContext: function(uid){
        Ext.getCmp('maintWindowGrid').setContext(uid);
        Ext.getCmp('deviceCommandsGrid').setContext(uid);
        Ext.getCmp('adminsGrid').setContext(uid);
    }
});




// ------------------------------------------------------- Maint Window:
    /**
     * @class Zenoss.maintwindow.Model
     * @extends Ext.data.Model
     **/
    Ext.define('Zenoss.maintwindow.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'uid',
        fields: [
            {name: 'name'},
            {name: 'enabled'},
            {name: 'startTime'},
            {name: 'startTime_data', mapping:'startTime'},
            {name: 'duration'},
            {name: 'duration_data', mapping:'duration'},
            {name: 'repeat'},
            {name: 'niceRepeat'},
            {name: 'startProdState'},
            {name: 'days'},
            {name: 'occurrence'}
        ]
    });

    /**
     * @class Zenoss.maintwindow.Store
     * @extends Zenoss.DirectStore
     * Store for our maintenance window grid
     **/
    Ext.define("Zenoss.maintwindow.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.maintwindow.Model',
                initialSortColumn: "name",
                directFn: Zenoss.remote.DeviceManagementRouter.getMaintWindows,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.maintwindow.Grid", {
        alias: ['widget.MaintWindowsGrid'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};

            Zenoss.Security.onPermissionsChange(function() {
                disableButtons(this);
            }, this);

            Ext.applyIf(config, {
                stateId: config.id || 'maintwindow_grid',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                stateful: true,
                title: "Maintenance Windows",
                tbar:[
                    {
                    xtype: 'button',
                    iconCls: 'add',
                    tooltip: _t('Set up a new Maintenance Window'),
                    requiredPermission: 'Maintenance Windows Edit', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Maintenance Windows Edit'),
                    ref: 'addButton',
                        handler: function() {
                            var grid = Ext.getCmp("maintWindowGrid");
                            maintDialog(grid);
                        }
                    },{
                        xtype: 'button',
                        iconCls: 'delete',
                        tooltip: _t('Delete selected Maintenance Window'),
                        requiredPermission: 'Maintenance Windows Edit', // change the line below as well
                        disabled: Zenoss.Security.doesNotHavePermission('Maintenance Windows Edit'),
                        handler: function() {
                            var grid = Ext.getCmp("maintWindowGrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();
                            if (Ext.isEmpty(selected)) {
                                return;
                            }

                            data = selected[0].data;
                            new Zenoss.dialog.SimpleMessageDialog({
                                title: _t('Delete Maintenance Window'),
                                message: Ext.String.format(_t("Are you sure you want to delete {0}?"), data.name),
                                buttons: [{
                                    xtype: 'DialogButton',
                                    text: _t('OK'),
                                    handler: function() {
                                        if (grid.uid) {
                                            Zenoss.remote.DeviceManagementRouter.deleteMaintWindow({uid:grid.uid, id:data.name}, function(response){
                                                if (response.success) {
                                                    grid.refresh();
                                                }
                                            });
                                        }
                                    }
                                }, {
                                    xtype: 'DialogButton',
                                    text: _t('Cancel')
                                }]
                            }).show();
                        }
                    },
                    {
                    xtype: 'button',
                    iconCls: 'customize',
                    tooltip: _t('Edit selected Maintenance Window'),
                    requiredPermission: 'Maintenance Windows Edit', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Maintenance Windows Edit'),
                    ref: 'customizeButton',
                        handler: function() {
                            var grid = Ext.getCmp("maintWindowGrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();

                            if (Ext.isEmpty(selected)) {
                                return;
                            }
                            // single selection
                            data = selected[0].data;
                            maintDialog(grid, data);
                        }
                    },
                    {
                    xtype: 'button',
                    iconCls: 'suppress',
                    tooltip: _t('Toggle Enable/Disable on selected Maintenance Window'),
                    requiredPermission: 'Maintenance Windows Edit', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Maintenance Windows Edit'),
                    ref: 'disableMaintButton',
                        handler: function() {
                            var grid = Ext.getCmp("maintWindowGrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();

                            if (Ext.isEmpty(selected)) {
                                return;
                            }
                            data = selected[0].data;
                            if (grid.uid) {
                                var prodState = Zenoss.devicemanagement.getProdStateValue(data.startProdState);
                                var startTime = Zenoss.devicemanagement.getStartTime(data.startTime_data);
                                var duration = Zenoss.devicemanagement.getDuration(data.duration_data);
                                var enabled = (!data.enabled);

                                var params = {
                                    uid:                    grid.uid,
                                    id:                     data.name,
                                    name:                   data.name,
                                    startDate:              startTime.date,
                                    startHours:             startTime.hr,
                                    startMinutes:           startTime.min,
                                    durationDays:           duration.days,
                                    durationHours:          duration.hrs,
                                    durationMinutes:        duration.mins,
                                    repeat:                 data.repeat,
                                    days:                   data.days,
                                    occurrence:             data.occurrence,
                                    startProductionState:   prodState,
                                    enabled:                enabled
                                };
                                Zenoss.remote.DeviceManagementRouter.editMaintWindow({params:params}, function(response){
                                    if (response.success) {
                                        grid.refresh();
                                    }
                                });
                            }
                            // do an update that only toggles the enabled
                        }
                    }
                ],
                store: Ext.create('Zenoss.maintwindow.Store', {}),
                columns: [
                    {
                        id: 'maint_enabled',
                        dataIndex: 'enabled',
                        header: _t('Enabled'),
                        width: 50,
                        sortable: true,
                        filter: false,
                        renderer: function(value){
                            if (value) {
                                return '<span style="color:green;">Yes</span>';
                            }
                            return '<span style="color:red;">No</span>';
                        }
                    },{
                        header: _t("Name"),
                        id: 'maint_name',
                        dataIndex: 'name',
                        flex: 1,
                        filter: false,
                        sortable: true
                    },{
                        id: 'maint_start_data',
                        dataIndex: 'startTime_data',
                        hidden:true
                    },{
                        id: 'maint_start',
                        dataIndex: 'startTime',
                        header: _t('Start'),
                        width: 140,
                        filter: false,
                        sortable: true,
                        renderer: function(val){
                            var chunk = val.split(".");
                            var tz = val.split(" ")[2]
                            return chunk[0]+" "+tz;
                        }
                    },{
                        id: 'maint_duration_data',
                        dataIndex: 'duration_data',
                        hidden:true
                    },{
                        id: 'maint_duration',
                        dataIndex: 'duration',
                        header: _t('Duration'),
                        width: 140,
                        filter: false,
                        sortable: true,
                        renderer: function(val){
                            if(val.length === 5){ // it's only minutes
                                return val+" mins";
                            }else if(val.length === 8){
                                return val+" hrs";
                            }

                            return val;
                        }
                    },{
                        id: 'maint_repeat',
                        dataIndex: 'niceRepeat',
                        header: _t('Repeat'),
                        width: 150,
                        filter: false,
                        sortable: true
                    },{
                        id: 'maint_startstate',
                        dataIndex: 'startProdState',
                        header: _t('State'),
                        width: 100,
                        filter: false,
                        sortable: true
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
        },
        setContext: function(uid) {
            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        },
        onRowDblClick: function() {
            var data,
                selected = this.getSelectionModel().getSelection();
            if (!selected) {
                return;
            }
            data = selected[0].data;
            maintDialog(this, data);
        },
        beforeRender: function() {
            Zenoss.remote.DeviceManagementRouter.getTimeZone({}, function(response){
                if(response.success){
                    timeZone = response.data;
                }
            });
        }
    });

    var contextIsDevice = function(uid){
        var split = uid.split('/devices/');
        if(split[1]){
            return true;
        }
        return false;
    };

    var disableButtons = function(cmp) {
        var btns = cmp.query("button");
        Ext.each(btns, function(btn){
            if(btn.requiredPermission) {
                btn.setDisabled(Zenoss.Security.doesNotHavePermission(btn.requiredPermission));
            }
        });
    }


// ------------------------------------------------------- Commands:

    Ext.define('Zenoss.admincommands.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: [
            {name: 'id'},
            {name: 'command'},
            {name: 'description'}
        ]
    });

    /**
     * @class Zenoss.admincommands.Store
     * @extends Zenoss.DirectStore
     * Store for our admincommands window grid
     **/
    Ext.define("Zenoss.admincommands.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.admincommands.Model',
                initialSortColumn: "id",
                directFn: Zenoss.remote.DeviceManagementRouter.getUserCommands,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.admincommands.Grid", {
        alias: ['widget.AdminCommandsGrid'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};

            Zenoss.Security.onPermissionsChange(function() {
                disableButtons(this);
            }, this);

            Ext.applyIf(config, {
                stateId: config.id || 'admincommands_grid',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                stateful: true,
                title: "User Commands",
                tbar:[
                    {
                    xtype: 'button',
                    iconCls: 'add',
                    tooltip: _t('Add a User Command'),
                    requiredPermission: 'Manage Device', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                    ref: 'addButton',
                        handler: function() {
                            var grid = Ext.getCmp("deviceCommandsGrid");
                            commandsDialog(grid);
                        }
                    },{
                        xtype: 'button',
                        iconCls: 'delete',
                        tooltip: _t('Delete selected User Command'),
                        requiredPermission: 'Maintenance Windows Edit', // change the line below as well
                        disabled: Zenoss.Security.doesNotHavePermission('Maintenance Windows Edit'),
                        handler: function() {
                            var grid = Ext.getCmp("deviceCommandsGrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();
                            if (Ext.isEmpty(selected)) {
                                return;
                            }

                            data = selected[0].data;
                            new Zenoss.dialog.SimpleMessageDialog({
                                title: _t('Delete User Command'),
                                message: Ext.String.format(_t("Are you sure you want to delete the User Command {0}?"), data.id),
                                buttons: [{
                                    xtype: 'DialogButton',
                                    text: _t('OK'),
                                    handler: function() {
                                        if (grid.uid) {
                                            Zenoss.remote.DeviceManagementRouter.deleteUserCommand({uid:grid.uid, id:data.id}, function(response){
                                                if (response.success) {
                                                    grid.refresh();
                                                }
                                            });
                                        }
                                    }
                                }, {
                                    xtype: 'DialogButton',
                                    text: _t('Cancel')
                                }]
                            }).show();
                        }
                    },
                    {
                    xtype: 'button',
                    iconCls: 'customize',
                    tooltip: _t('Edit selected User Command'),
                    requiredPermission: 'Manage Device', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                    ref: 'customizeButton',
                        handler: function() {
                            var grid = Ext.getCmp("deviceCommandsGrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();

                            if (Ext.isEmpty(selected)) {
                                return;
                            }
                            // single selection
                            data = selected[0].data;
                            commandsDialog(grid, data);
                        }
                    }, {
                    xtype: 'button',
                    iconCls: 'export',
                    tooltip: _t('Add selected User Command to ZenPack'),
                    ref: '../refreshButton',
                    requiredPermission: 'Manage Device', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                    handler: function() {
                        var grid = Ext.getCmp("deviceCommandsGrid");
                        var zp_dialog = Ext.create('Zenoss.AddToZenPackWindow', {});
                        zp_dialog.setTarget(grid.uid);
                        zp_dialog.show();
                    }
                    },{
                        xtype: 'button',
                        iconCls: 'acknowledge',
                        tooltip: _t('Run the selected command'),
                        requiredPermission: 'Manage Device', // change the line below as well
                        disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                        listeners: {
                            afterrender: function(b){
                                b.setVisible(contextIsDevice(Ext.getCmp("deviceCommandsGrid").uid));
                            }
                        },
                        handler: function(){
                                var grid = Ext.getCmp("deviceCommandsGrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();

                            if (Ext.isEmpty(selected)) {
                                return;
                            }
                            // single selection
                            data = selected[0].data;
                            var win = new Zenoss.CommandWindow({
                                uids: [grid.uid],
                                target: grid.uid + '/run_command',
                                command: data.id
                            });
                            win.show();
                        }
                    }
                ],
                store: Ext.create('Zenoss.admincommands.Store', {}),
                columns: [{
                        header: _t("Name"),
                        id: 'cmd_name',
                        dataIndex: 'id',
                        width: 150,
                        filter: false,
                        sortable: true,
                        renderer: function(val, o, fields){
                            return Zenoss.devicemanagement.createTooltip(val, fields.data.description);
                        }
                    },{
                        id: 'cmd_command',
                        dataIndex: 'command',
                        header: _t('Command'),
                        flex: 1,
                        filter: false,
                        sortable: true,
                        renderer: function(val, o, fields){
                            return Zenoss.devicemanagement.createTooltip(val, fields.data.description);
                        }
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
        },
        setContext: function(uid) {
            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        },
        onRowDblClick: function() {
            var data,
                selected = this.getSelectionModel().getSelection();
            if (!selected) {
                return;
            }
            data = selected[0].data;
            if(!data) {
                return;
            }
            commandsDialog(this, data);
        }
    });


// ------------------------------------------------------- Admins:
    /**
     * @class Zenoss.maintwindow.Model
     * @extends Ext.data.Model
     **/
    Ext.define('Zenoss.administrators.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: [
            {name: 'id'},
            {name: 'description'},
            {name: 'role'},
            {name: 'email'},
            {name: 'pager'}
        ]
    });

    /**
     * @class Zenoss.administrators.Store
     * @extends Zenoss.DirectStore
     * Store for our device administrators grid
     **/
    Ext.define("Zenoss.administrators.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.administrators.Model',
                initialSortColumn: "id",
                directFn: Zenoss.remote.DeviceManagementRouter.getAdminRoles,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.administrators.Grid", {
        alias: ['widget.AdministratorsGrid'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};

            Zenoss.Security.onPermissionsChange(function() {
                disableButtons(this);
            }, this);

            Ext.applyIf(config, {
                stateId: config.id || 'admins_grid',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                stateful: true,
                title: "Administrators",
                tbar:[
                    {
                    xtype: 'button',
                    iconCls: 'add',
                    tooltip: _t('Add a User'),
                    requiredPermission: 'Manage Device', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                    ref: 'addButton',
                        handler: function() {
                            var grid = Ext.getCmp("adminsGrid");
                            adminsDialog(grid);
                        }
                    },{
                        xtype: 'button',
                        iconCls: 'delete',
                        tooltip: _t('Remove selected User from the Device Administrators panel'),
                        requiredPermission: 'Maintenance Windows Edit', // change the line below as well
                        disabled: Zenoss.Security.doesNotHavePermission('Maintenance Windows Edit'),
                        handler: function() {
                            var grid = Ext.getCmp("adminsGrid"),
                                data,
                                selected = grid.getSelectionModel().getSelection();
                            if (Ext.isEmpty(selected)) {
                                return;
                            }

                            data = selected[0].data;
                            new Zenoss.dialog.SimpleMessageDialog({
                                title: _t('Remove Admin from Device'),
                                message: Ext.String.format(_t("Are you sure you want to remove this admin?")),
                                buttons: [{
                                    xtype: 'DialogButton',
                                    text: _t('OK'),
                                    handler: function() {
                                        if (grid.uid) {
                                            Zenoss.remote.DeviceManagementRouter.removeAdmin({uid:grid.uid, id:data.id}, function(response){
                                                if (response.success) {
                                                    grid.reset();
                                                }
                                            });
                                        }
                                    }
                                }, {
                                    xtype: 'DialogButton',
                                    text: _t('Cancel')
                                }]
                            }).show();
                        }
                    },
                    {
                    xtype: 'button',
                    iconCls: 'set',
                    tooltip: _t('Edit users on the advanced user account edit page'),
                    requiredPermission: 'Manage Device', // change the line below as well
                    disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                    ref: 'editAdminButton',
                        handler: function() {
                            Ext.getCmp("adminsGrid").editpage();
                        }
                    },{
                        xtype: 'panel',
                        tooltip: _t('Edit selected Admin Role'),
                        layout: 'hbox',
                        bodyStyle: 'background:transparent;',
                        items:[
                            {
                                xtype: 'label',
                                margin: '5px 3px 0 5px',
                                text: _t('Change Role:')
                            },
                            {
                            xtype: 'combo',
                            disabled: true,
                            id: 'changeroleCombo',
                            editable: false,
                            queryMode:'local',
                            listConfig: {
                                maxWidth:155
                            },
                            listeners: {
                                select: function(combo){
                                    var grid = Ext.getCmp("adminsGrid"),
                                        gridrow = grid.getSelectionModel().getSelection(),
                                        griddata = gridrow[0].data,
                                        params = {
                                            uid:    grid.uid,
                                            name:   griddata.id,
                                            role:   combo.value
                                        };
                                    Zenoss.remote.DeviceManagementRouter.updateAdminRole({params:params}, function(response){
                                        if (response.success) {
                                            grid.reset();
                                        }
                                    });
                                }
                            },
                            store: ['none']
                            }
                        ]
                    }
                ],
                store: Ext.create('Zenoss.administrators.Store', {}),
                columns: [{
                        header: _t("Name"),
                        id: 'admin_name',
                        dataIndex: 'id',
                        width: 150,
                        filter: false,
                        sortable: true,
                        renderer: function(e){
                            var gotoWin = window.location.protocol + '//' + window.location.host;
                            gotoWin += '/zport/dmd/ZenUsers/' + e;
                            return '<a href="'+gotoWin+'" title="Edit the user '+e+' on the user accounts page">'+e+'</a>';
                        }
                    },{
                        id: 'admin_role',
                        dataIndex: 'role',
                        header: _t('Role'),
                        width: 90,
                        filter: false,
                        sortable: true
                    },{
                        id: 'admin_email',
                        dataIndex: 'email',
                        header: _t('Email'),
                        flex: 1,
                        filter: false,
                        sortable: true
                    },{
                        id: 'admin_pager',
                        dataIndex: 'pager',
                        header: _t('Pager'),
                        width: 120,
                        filter: false,
                        sortable: true
                    }]
            });
            this.callParent(arguments);
            this.on('select', this.onRowSelect, this);
        },
        setContext: function(uid) {
            this.uid = uid;
            this.callParent(arguments);
            Zenoss.devicemanagement.setRolesCombo(this, uid, Ext.getCmp('changeroleCombo'));
        },
        onRowSelect: function(model, selectedRow){
            if (Zenoss.Security.hasPermission('Manage Device')) {
                var combo = Ext.getCmp('changeroleCombo');
                combo.setDisabled(false);
                var index = combo.getStore().find('field1', selectedRow.data.role);
                combo.setValue(combo.store.getAt(index));
            }
        },
        reset: function(){
            this.refresh();
            Zenoss.devicemanagement.setComboSleep(Ext.getCmp('changeroleCombo'));
        },
        editpage: function(){
            var location,
                selected = this.getSelectionModel().getSelection();

            if (Ext.isEmpty(selected)) {
                location = '/zport/dmd/ZenUsers/manageUserFolder';
            }else{
                location = '/zport/dmd/ZenUsers/' + encodeURIComponent(selected[0].data.id);
            }

            var hostString = window.location.protocol + '//' + window.location.host;
            window.location = hostString + location;
            // take them to the accounts page under advanced
        }
    });


})();
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2013, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/


(function(){

    var setComboFromData = function(response, combo){
            var data = [];
            for(var i=0;i < response.data.length; i++){
                data.push([response.data[i]]);
            }
            combo.store.loadData(data);
            combo.setValue(combo.store.getAt(-1));
            combo.setValue("Select a Configuration Property");
    };

    var checkComboIndex = function(){
        var combo = Ext.getCmp('propsCombo');
        var v = combo.getValue();
        var record = combo.findRecord(combo.valueField || combo.displayField, v);
        var index = combo.store.indexOf(record);
        // disable the edit box if the index is -1
        if(index === -1){
            Ext.getCmp('overriddenobjbase_grid1').disableButtons(true);
        }else{
            Ext.getCmp('overriddenobjbase_grid1').disableButtons(false);
        }
        return index;
    };

    var loadBaseGrid = function(uid){
        var combo = Ext.getCmp('propsCombo');
        Zenoss.remote.DeviceRouter.getOverriddenObjectsParent({uid:uid, propname:combo.value}, function(response){
            if (response.success) {
                Ext.getCmp('overriddenobjbase_grid1').getStore().loadData(response.data);
            }
        });
    };
    var loadOverriddenGrid = function(uid){
        var combo = Ext.getCmp('propsCombo'), relName = 'devices';
        // switch this based on where the grid is located or embeded

        var refreshit = function(data){
            Ext.getCmp('overriddenobjover_grid2').getStore().loadData(data);
        };

        /* If it finds /dmd/Events in the UID, then this is being used on the
         * Events Classes page and needs to get its overridden objects from
         * a different router method. Otherwise, we use the default devices method
         */
        if(uid.indexOf('/dmd/Events') !== -1) {
            relName = 'instances';
        }
        // default method for devices:
        Zenoss.remote.DeviceRouter.getOverriddenObjectsList({uid:uid, propname:combo.value, relName: relName}, function(response){
            if (response.success) {
                refreshit(response.data);
            }
        });

    };
    var onComboChange = function(uid){
        if(checkComboIndex() === -1) {
            return;
        }
        loadOverriddenGrid(uid);
        loadBaseGrid(uid);
    };

    function showEditCustPropertyDialog(data, grid){
        var path = data.devicelink;
        var pathMsg = '<div style="padding-top:10px;">';
        pathMsg += _t(" Object Path:");
        pathMsg += '<div style="color:#aaa;padding-top:3px;">';
        pathMsg += '<div class="x-grid-cell-inner" style="color:#fff;background:#111;margin:2px 7px 2px 0;padding:5px;"> '+path+' </div>';
        pathMsg += '</div></div>';

        var lbltemplate = '<div style="margin:5px 0 15px 0;"><b>{0}</b> <span style="display:inline-block;padding-left:5px;color:#aaccaa">{1}</span></div>';
        var items = [
                {
                    xtype: 'label',
                    width: 500,
                    height: 50,
                    html: Ext.String.format(lbltemplate, _t("Name:"), Ext.getCmp('propsCombo').getValue())
                },{
                    xtype: 'label',
                    width: 500,
                    height: 50,
                    html: Ext.String.format(lbltemplate, _t("Type:"),data.proptype)
                },{
                    xtype: 'label',
                    width: 300,
                    height: 90,
                    html: pathMsg
                }
            ];

        var pkg = {
            'items': items,
            'uid': path,
            'type': data.proptype,
            'value': data.props,
            'name': Ext.getCmp('propsCombo').getValue(),
            'grid': grid,
            'dialogId': 'editOverriddenDialog',
            'minHeight': 200,
            'width': 500,
            'options': 0
        };
        Zenoss.zproperties.showEditPropertyDialog(pkg);
    }


    /****************************************************** BASE OBJECT GRID ***************************/

    Ext.define("Zenoss.OverriddenObj_base.Grid", {
        alias: ['widget.overriddenobjbase_grid'],
        extend:"Zenoss.ContextGridPanel",
        constructor: function(config) {
            config = config || {};

            Zenoss.Security.onPermissionsChange(function() {
                this.disableButtons(Zenoss.Security.doesNotHavePermission('zProperties Edit'));
            }, this);

            Ext.applyIf(config, {
                stateId: config.id || 'overridden_grid2',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                stateful: true,
                tbar:[
                    {
                    xtype: 'button',
                    iconCls: 'customize',
                    tooltip: _t('Edit selected Configuration Property value'),
                    disabled: Zenoss.Security.doesNotHavePermission('zProperties Edit'),
                    ref: 'customizeButton',
                        handler: function() {
                            var grid1 = Ext.getCmp('overriddenobjbase_grid1');
                            var grid2 = Ext.getCmp('overriddenobjover_grid2');
                            var data, grid, selected;
                            if( Ext.isEmpty(grid1.getSelectionModel().getSelection()) ){
                                // nothing selected in grid1
                                if( Ext.isEmpty(grid2.getSelectionModel().getSelection()) ){
                                    // nothing selected in grid2 either, abort
                                    return;
                                }else{
                                    // nothing selected in grid1, but we do have grid2 selection. Use it.
                                    selected = grid2.getSelectionModel().getSelection();
                                    grid = grid2;
                                }
                            }else{
                                // found a selection in grid1. Use it.
                                selected = grid1.getSelectionModel().getSelection();
                                grid = grid1;
                            }
                            data = selected[0].data;
                            showEditCustPropertyDialog(data, grid);
                        }
                    },
                    {
                    xtype: 'combo',
                    width:350,
                    fieldLabel: 'Configuration Properties',
                    labelWidth: 135,
                    editable: false,
                    id: 'propsCombo',
                    value: 'Select a Configuration Property',
                    queryMode:'local',
                    listConfig: {
                        maxWidth:300
                    },
                    store: ['none']
                    }
                ],
                store: Ext.create('Ext.data.ArrayStore', {
                      fields: [
                          {name: 'devicelink'},
                          {name: 'props'},
                          {name: 'proptype'}
                      ]
                   }),
                columns: [{
                        dataIndex: 'devicelink',
                        header: _t('Object'),
                        width: 340,
                        filter:false,
                        sortable: true,
                        renderer: function(e){
                            return Zenoss.render.DeviceClass(e, e.replace(/^\/zport\/dmd/, ''));
                        }
                    },{
                        dataIndex: 'props',
                        header: _t('Value'),
                        flex: 1,
                        sortable: true,
                        filter: false
                    },{
                        dataIndex: 'proptype',
                        hidden:true
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
            this.on('select', this.onSelectRow, this);
        },
        setContext: function(uid) {
            this.uid = uid;
            var combo = Ext.getCmp('propsCombo');
            // load the grid's store
            this.callParent(arguments);
            Zenoss.remote.DeviceRouter.getOverriddenZprops({uid:uid}, function(response){
                if (response.success) {
                    setComboFromData(response, combo);
                    combo.on('change', function(){
                        onComboChange(uid);
                    });
                }
            });
            Zenoss.remote.DeviceRouter.getOverriddenObjectsParent({uid:uid, propname:''}, function(response){
                if (response.success) {
                    Ext.getCmp('overriddenobjbase_grid1').getStore().loadData(response.data);
                }
            });
            checkComboIndex();
        },
        refresh: function(){
            var uid = Ext.getCmp('overriddenobjbase_grid1').uid;
            loadBaseGrid(uid);
        },
        onSelectRow: function(){
            var otherGrid = Ext.getCmp('overriddenobjover_grid2');
            otherGrid.getSelectionModel().deselectAll();
        },
        onRowDblClick: function() {
            var data,
                selected = this.getSelectionModel().getSelection();
            if(!selected){
                return;
            }
            data = selected[0].data;
            showEditCustPropertyDialog(data, this);
        },
        disableButtons: function(bool) {
            var btns = this.query("button");
            Ext.each(btns, function(btn){
                btn.setDisabled(bool);
            });
        }
    });


    /****************************************************** OVERRIDDEN OBJECT GRID **************************/


    Ext.define("Zenoss.OverriddenObj_over.Grid", {
        alias: ['widget.overriddenobjover_grid'],
        extend:"Zenoss.ContextGridPanel",
        constructor: function(config) {
            config = config || {};

            Ext.applyIf(config, {
                stateId: config.id || 'overridden_grid2',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                stateful: true,
                store: Ext.create('Ext.data.ArrayStore', {
                    sorters: [
                          {property : 'devicelink', direction: 'ASC'}
                      ],
                      fields: [
                          {name: 'objtype'},
                          {name: 'name'},
                          {name: 'devicelink'},
                          {name: 'props'},
                          {name: 'proptype'}
                      ]
                   }),
                columns: [{
                        header: _t("Overriding Object"),
                        dataIndex: 'devicelink',
                        width: 340,
                        sortable: true,
                        renderer: function(val, meta, record) {
                            if (record.get('objtype') == 'devices') {
                                return Zenoss.render.Device(val, record.get('name'));
                            } else {
                                return Zenoss.render.DeviceClass(val.substring(8));
                            }
                        }
                    },{
                        dataIndex: 'props',
                        header: _t('Value'),
                        flex: 1,
                        sortable: true
                    },{
                        dataIndex: 'proptype',
                        hidden:true
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
            this.on('select', this.onSelectRow, this);
        },
        setContext: function(uid) {
            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        },
        refresh: function(){
            var uid = Ext.getCmp('overriddenobjbase_grid1').uid;
            loadOverriddenGrid(uid);
        },
        onSelectRow: function(){
            var otherGrid = Ext.getCmp('overriddenobjbase_grid1');
            otherGrid.getSelectionModel().deselectAll();
        },
        onRowDblClick: function() {
            var data,
                selected = this.getSelectionModel().getSelection();
            if(!selected){
                return;
            }
            data = selected[0].data;
            showEditCustPropertyDialog(data, this);
        }
    });

    Ext.define("Zenoss.form.overriddenobjects", {
        alias:['widget.overriddenobjects'],
        extend:"Ext.Panel",
        constructor: function(config) {
            config = config || {};
            this.grid1Id = "overriddenobjbase_grid1";
            this.grid2Id = "overriddenobjover_grid2";
            Ext.applyIf(config, {
                layout: 'border',
                defaults: {
                    split: true
                },
                items: [{
                    id: this.grid1Id,
                    xtype: "overriddenobjbase_grid",
                    region: 'north',
                    height: '20%',
                    ref: 'overriddenGrid1',
                    displayFilters: config.displayFilters
                },
                {
                    id: this.grid2Id,
                    xtype: "overriddenobjover_grid",
                    region: 'center',
                    ref: 'overriddenGrid2',
                    displayFilters: config.displayFilters
                }]
            });
            this.callParent(arguments);
        },
        setContext: function(uid) {
            Ext.getCmp(this.grid1Id).setContext(uid);
            Ext.getCmp(this.grid2Id).setContext(uid);
        }
    });

})();
(function(){

    /**
     * Classes for determining when the page is completely loaded. Since this
     * is set per page encapsulate the logic here.
     *
     **/
    Ext.define("Zenoss.stats.ResultsWindow", {
        extend: "Zenoss.dialog.BaseDialog",
        constructor: function(config) {
            var finalTime = config.finalTime,
                loadingTime = config.loadingTime;


            Ext.applyIf(config, {
                title: _t('Page Load Time'),
                defaults: {
                    xtype: 'displayfield'
                },
                items: [{
                    value: Ext.String.format(_t("{0} seconds until the page is ready to use."), loadingTime)
                }, {
                    value: Ext.String.format(_t("{0} seconds in layout and rendering on the client."), (finalTime - Zenoss._pageStartTime).toFixed(2))
                }, {
                    value: Ext.String.format(_t("{0} seconds since ExtJs start time."), (finalTime - (Ext._startTime/1000)).toFixed(2))
                },{
                    value: Ext.String.format(_t("{0} seconds spent in Ajax requests."), (config.ajaxTime).toFixed(2))
                }]

            });
            this.callParent([config]);
        }


    });

    Ext.define("Zenoss.stats.Infrastructure", {
        pageName: 'Infrastructure',
        constructor: function() {
            this.addHooks();
        },
        addHooks: function() {
            // the Infrastructure page is loaded when the both the trees
            // and grid stores are loaded and rendered. Since
            // we don't really load the store until after the Devices tree is rendered
            // we can assume the page is fully loaded when the Device grid has finished loading
            // its store.
            Ext.getCmp('device_grid').getStore().on('load', this.checkReady, this, {single:true});
        },
        getTiming: function() {
            var perf = window.performance || {};
            var fn = perf.now || perf.mozNow || perf.webkitNow || perf.msNow || perf.oNow;
            return fn ? fn.bind(perf) : Ext.emptyFn;
        },
        checkReady: function() {
            // convert from milliseconds to seconds
            var finalTime = new Date().getTime() / 1000.0,
                loadingTime = this.getTiming()(),
                ajaxTime = Ext.Array.sum(Ext.pluck(completedRequests, 'time'));
            // perf data is unavailable, display nothing
            if (!loadingTime) {
                return;
            }

            loadingTime = (loadingTime / 1000.00).toFixed(2);
            Zenoss.remote.DetailNavRouter.recordPageLoadTime({
                page: this.pageName,
                time: loadingTime
            });
            // do not listen to ajax requests anymore
            Ext.Ajax.un('beforerequest', beforeAjaxRequest);
            Ext.Ajax.un('requestcomplete', afterAjaxRequest);
            Ext.getCmp('footer_bar').add(['-', {
                xtype: 'button',
                text: Ext.String.format(_t("{0} seconds"), loadingTime),
                handler: function(){
                    Ext.create('Zenoss.stats.ResultsWindow', {
                        finalTime: finalTime,
                        loadingTime: loadingTime,
                        ajaxTime: ajaxTime
                    }).show();
                }
            }]);
        }


    });

    Ext.define("Zenoss.stats.Events", {
        extend: "Zenoss.stats.Infrastructure",
        pageName: 'EventConsole',
        addHooks: function() {
            Ext.getCmp('events_grid').getStore().on('load', this.checkReady, this, {single:true});
        }

    });

    Ext.define("Zenoss.stats.DeviceDetail", {
        extend: "Zenoss.stats.Infrastructure",
        pageName: 'DeviceDetails',
        addHooks: function() {
            var detailnav = Ext.getCmp('deviceDetailNav');
            detailnav.on('componenttreeloaded', this.checkReady, this, {single:true});
        }

    });

    var openRequests = {}, completedRequests=[];

    /**
     * Uniquely identify each ajax request so that
     * we can match it up when the results come back from the server.
     **/
    function getTransactionId(transaction) {
        if (!transaction) {
            return null;
        }

        if (Ext.isArray(transaction)) {
            var ids = Ext.pluck(transaction, "id");
            return ids.join(" ");
        }
        return transaction.id;
    }

    /**
     * Log the start time and Url of each ajax request that has a transaction
     * associated with it.
     **/
    function beforeAjaxRequest(conn, options) {
        var url = options.url, transactionId = getTransactionId(options.transaction);
        if (transactionId) {
            openRequests[transactionId] = {
                url: url,
                starttime: new Date().getTime()
            };
        }
    }

    /**
     * Log the end time so we can get the total time spent in ajax requests.
     **/
    function afterAjaxRequest(conn, response, options) {
        var url = options.url, endtime = new Date().getTime(), transactionId = getTransactionId(options.transaction);
        if (openRequests[transactionId]) {
            completedRequests.push({
                url: url,
                time: (endtime - openRequests[transactionId].starttime) / 1000.00
            });
            delete openRequests[transactionId];
        }
    }

    Ext.onReady(function() {
        if (Zenoss.settings.showPageStatistics) {
            Ext.Ajax.on('beforerequest', beforeAjaxRequest);
            Ext.Ajax.on('requestcomplete', afterAjaxRequest);
        }
    });
}());/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2013, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/

Ext.onReady(function(){
    Ext.ns('Zenoss.eventclasses');

   Ext.define('Zenoss.sequencegrid.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'uid',
        fields: [
            {name: 'id'},
            {name: 'uid'},
            {name: 'eventClass'},
            {name: 'sequence'},
            {name: 'eventClassKey'},
            {name: 'eval'}
        ]
    });

    Ext.define("Zenoss.sequencegrid.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.sequencegrid.Model',
                initialSortColumn: "id",
                directFn: Zenoss.remote.EventClassesRouter.getSequence,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.sequencegrid.SequenceGrid", {
        alias: ['widget.sequencegrid'],
        extend:"Zenoss.BaseGridPanel",
        constructor: function(config) {
            config = config || {};
            var me = this;
            Ext.applyIf(config, {
                stateId: 'mapping_sequence_grid',
                id: 'mapping_sequence_grid',
                sm: Ext.create('Zenoss.SingleRowSelectionModel', {}),
                stateful: true,
                store: Ext.create('Zenoss.sequencegrid.Store', {}),
                enableDragDrop: true,
                ddGroup: 'sequencegriddd',
                viewConfig: {
                    forcefit: true,
                    markDirty:false,
                    plugins: {
                        ptype: 'gridviewdragdrop',
                        dragGroup: 'sequencegriddd',
                        dropGroup: 'sequencegriddd'
                    },
                    listeners: {
                        /**
                         * Updates the graph order when the user drags and drops them
                         **/
                        drop: function() {
                            var records = me.store.getRange();
                            var uids = Ext.pluck(Ext.pluck(records, 'data'), 'uid');
                            Zenoss.remote.EventClassesRouter.resequence({'uids':uids}, function(response){
                                if (response.success) {
                                    // redraw the sequences cells with the new numbers:
                                    Zenoss.message.info(_t('Saved new sequence.'));
                                    me.getSelectionModel().deselectAll();
                                    for (var i = 0; records.length > i; i++){
                                        records[i].set("sequence", i);
                                        if(records[i].get("uid") === Ext.getCmp('mappingDialog').contextUid){
                                            Ext.getCmp('sequence_text_id').setText('Sequence: '+i);
                                        }
                                    }
                                }
                            });
                        }
                    }
                },
                columns: [
                    {
                        id: 'uid_seq_id',
                        dataIndex: 'uid',
                        hidden: true

                    },{
                        header: _t('Seq'),
                        id: 'seq_seq_id',
                        dataIndex: 'sequence',
                        width: 30,
                        sortable: false
                    },{
                        header: _t('ID'),
                        id: 'map_seq_id',
                        dataIndex: 'id',
                        renderer: function(value, metaData, record){
                            if(record.get("uid") === Ext.getCmp('mappingDialog').contextUid){
                                return '<b>* '+value+'</b>';
                            }
                            return value;
                        },
                        flex: 1,
                        sortable: false
                    },{
                        header: _t("Event Class"),
                        id: 'class_seq_id',
                        dataIndex: 'eventClass',
                        width: 200,
                        sortable: false
                    },{
                        header: _t("EventClass Key"),
                        id: 'key_seq_id',
                        dataIndex: 'eventClassKey',
                        flex: 1,
                        sortable: false
                    },{
                        header: _t("Evaluation"),
                        id: 'eval_seq_id',
                        dataIndex: 'eval',
                        flex: 1,
                        sortable: false
                    }
                    ]
            });
            this.callParent(arguments);
        },
        setContext: function(uid) {
            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        }
    });


// ----------------------------------------------------------------- DIALOGS
    Zenoss.eventclasses.mappingDialog = function(grid, data) {
        if(!Ext.isDefined(data)) {
            data = "";
        }
        var addhandler, config, dialog, newEntry;
        newEntry = (data === "");
        var xtraData = {};
        addhandler = function() {
            var c = dialog.getForm().getForm().getValues();
            var save_xform = Ext.getCmp('xform_mapping_panel').getLastInChainValue() !== false ? Ext.getCmp('xform_mapping_panel').getLastInChainValue() : xtraData.transform;
            var params = {
                evclass:                Zenoss.env.contextUid,
                uid:                    xtraData.uid,
                instanceName:           c.instanceName,
                newName:                c.name, // if they change the name of this mapping
                eventClassKey:          c.eventclasskey,
                example:                Ext.getCmp('example_panel').getValue(),
                explanation:            Ext.getCmp('explanation_panel').getValue(),
                regex:                  Ext.getCmp('regex_panel').getValue(),
                rule:                   Ext.getCmp('rule_panel').getValue(),
                resolution:             Ext.getCmp('resolution_panel').getValue(),
                transform:              save_xform
            };
            var isTrans = (save_xform !== "");
            if(newEntry){
                Zenoss.remote.EventClassesRouter.addNewInstance({'params':params}, function(response){
                    if (response.success) {
                        // get the uid from the created instance so we can
                        // save the properties
                        params.uid = response.data.uid;
                        Zenoss.remote.EventClassesRouter.editInstance({'params':params}, function(response){
                            if (response.success) {
                                Ext.getCmp('classes').setTransIcon(isTrans);
                            }
                        });
                    }
                });
            }else{
                Zenoss.remote.EventClassesRouter.editInstance({'params':params}, function(response){
                    if (response.success) {
                        if(grid){
                            Ext.getCmp('classes').setTransIcon(isTrans);
                        }
                    }
                });
            }
        };

        // form config
        config = {
            submitHandler: addhandler,
            height:Ext.getBody().getViewSize().height,
            width:Ext.getBody().getViewSize().width*0.8, //80%
            id: 'mappingDialog',
            contextUid: null,
            title: _t("Add New Event Class Mapping"),
            listeners: {
                'hide': function(){
                    if (grid) {
                        grid.refresh();
                    }
                },
                'close':function(){
                    if (grid) {
                        grid.refresh();
                    }
                },
                'afterrender': function(e){
                    if(!newEntry){
                        // this window will be used to EDIT the values instead of create from scratch
                        // grab extra data from server to populate code boxes:

                       Zenoss.remote.EventClassesRouter.getInstanceData({'uid':data.uid}, function(response){
                            if(response.success){
                                xtraData = response.data[0];
                                Ext.getCmp('mappingDialog').contextUid = xtraData.uid;
                                var titleloc = xtraData.uid.split('/instances').join('').substring(10);
                                e.setTitle(_t("Edit Event Class Mapping")+" for: "+titleloc);
                                var fields = e.getForm().getForm().getFields();
                                Ext.getCmp('sequence_text_id').setText('Sequence: '+xtraData.sequence);
                                fields.findBy(
                                    function(record){
                                        switch(record.getName()){
                                            case "name"             : record.setValue(xtraData.id);  break;
                                            case "instanceName"     : record.setValue(xtraData.id);  break;
                                            case "eventclasskey"    : record.setValue(xtraData.eventClassKey);  break;
                                            case "rule_panel"       :
                                                record.setValue(xtraData.rule);
                                                // collapse or expand this panel depending on "" content
                                                if(xtraData.rule !== "") {
                                                    record.ownerCt.toggleCollapse();
                                                }
                                                break;
                                            case "regex_panel"      :
                                                record.setValue(xtraData.regex);
                                                // collapse or expand this panel depending on "" content
                                                if(xtraData.regex !== "") {
                                                    record.ownerCt.toggleCollapse();
                                                }
                                                break;
                                            case "example_panel"    : record.setValue(xtraData.example); break;
                                            //case "transform_panel"  : record.setValue(xtraData.transform); break;
                                            case "explanation_panel": record.setValue(xtraData.evaluation); break;
                                            case "resolution_panel" : record.setValue(xtraData.resolution); break;
                                        }
                                    }
                                );
                            }
                        });
                    }
                }
            },
            items: [
                {
                    xtype: 'label',
                    name: 'sequence',
                    id: 'sequence_text_id',
                    style: 'color:white;padding:5px 5px 3px 5px;position:absolute;top:-16px;left:600px;',
                    margin: '13 0 0 0',
                    width:100
                },{
                    xtype: 'tabpanel',
                    id: 'blackTabs',
                    listeners: {
                        'afterrender': function(p){
                            if(data.whichPanel === 'sequence'){
                                p.setActiveTab(3);
                            }
                        }
                    },
                    bodyStyle: {
                        padding: '10 0 10 0'
                    },
                        items: [
                            {
                                title: 'Matching',
                                items:[
                                {
                                    xtype: 'panel',
                                    layout: 'hbox',
                                    margin: '0 0 30px 0',
                                    items: [
                                        {
                                            xtype: 'textfield',
                                            name: 'name',
                                            fieldLabel: _t('Instance Name'),
                                            margin: '0 10px 0 0',
                                            width:320,
                                            regex: Zenoss.env.textMasks.allowedNameTextDash,
                                            regexText: Zenoss.env.textMasks.allowedNameTextFeedbackDash,
                                            allowBlank: false
                                        },{
                                            xtype: 'hidden',
                                            name: 'instanceName'
                                        },{
                                            xtype: 'textfield',
                                            name: 'eventclasskey',
                                            margin: '0 40px 0 0',
                                            fieldLabel: _t('Event Class Key'),
                                            regex: Zenoss.env.textMasks.allowedNameTextDashDot,
                                            regexText: Zenoss.env.textMasks.allowedNameTextFeedbackDashDot,
                                            width:320
                                        }
                                    ]

                                },{
                                    xtype: 'panel',
                                    items:[
                                        {
                                            xtype: 'minieditorpanel',
                                            title: 'Example',
                                            id: 'example_panel',
                                            name: 'example_panel',
                                            margin: '0 20 20 0'
                                        },{
                                            xtype: 'minieditorpanel',
                                            name: 'rule_panel',
                                            id: 'rule_panel',
                                            margin: '0 20 20 0',
                                            title: 'Rule (Single line Python Expression)',
                                            tools:[{
                                                type: 'save',
                                                tooltip: 'Test this Rule',
                                                handler:  function(event, html, button){
                                                    var value = button.ownerCt.items.items[0].getValue();
                                                    Zenoss.remote.EventClassesRouter.testRule({'rule':value}, function(response){
                                                        if(response.success){
                                                            Zenoss.message.info(_t('Rule compiles without error'));
                                                        }
                                                    });
                                                }
                                            }],
                                            collapsed: true,
                                            listeners:{
                                                expand: function(){
                                                    var regex_panel = Ext.getCmp('regex_panel');
                                                    if(regex_panel.getValue() !== ""){
                                                        Ext.getCmp('rule_panel').collapse();
                                                        new Zenoss.dialog.SimpleMessageDialog({
                                                            title: _t('Regex panel is not empty'),
                                                            message: _t("Please clear the regex panel to continue. You cannot have both a rule and a regex at the same time."),
                                                            buttons: [{
                                                                xtype: 'DialogButton',
                                                                text: _t('OK')
                                                            }]
                                                        }).show();
                                                    }else{
                                                        regex_panel.collapse();
                                                    }
                                                }
                                            }
                                        },
                                        {
                                            xtype: 'minieditorpanel',
                                            name: 'regex_panel',
                                            id: 'regex_panel',
                                            title: 'Regex (Can only have a rule, or regex, not both)',
                                            tools:[{
                                                type: 'save',
                                                tooltip: 'Test this Regex',
                                                handler:  function(event, html, button){
                                                    var value = button.ownerCt.items.items[0].getValue();
                                                    var ex = Ext.getCmp('example_panel').items.items[0].getValue();
                                                    Zenoss.remote.EventClassesRouter.testRegex({'regex':value, 'example':ex}, function(response){
                                                        if(response.success){
                                                            Zenoss.message.info(_t('Regex works with example'));
                                                        }
                                                    });
                                                }
                                            }],
                                            margin: '0 20 20 0',
                                            collapsed: true,
                                            listeners:{
                                                expand: function(){
                                                    var rule_panel = Ext.getCmp('rule_panel');
                                                    if(rule_panel.getValue() !== ""){
                                                        Ext.getCmp('regex_panel').collapse();
                                                        new Zenoss.dialog.SimpleMessageDialog({
                                                            title: _t('Rule panel is not empty'),
                                                            message: _t("Please clear the rule panel to continue. You cannot have both a rule and a regex at the same time."),
                                                            buttons: [{
                                                                xtype: 'DialogButton',
                                                                text: _t('OK')
                                                                }]
                                                        }).show();
                                                    }else{
                                                        rule_panel.collapse();
                                                    }
                                                }
                                            }
                                        },{
                                            xtype: 'minieditorpanel',
                                            title: 'Explanation',
                                            name: 'explanation_panel',
                                            id: 'explanation_panel',
                                            margin: '0 20 20 0'
                                        },{
                                            xtype: 'minieditorpanel',
                                            title: 'Resolution',
                                            name: 'resolution_panel',
                                            id: 'resolution_panel',
                                            margin: '0 20 20 0'
                                        }
                                    ]
                                } ]
                              },{
                                title:'Transforms',
                                hidden: newEntry,
                                items:[
                                    {
                                        xtype: 'xformmasterpanel',
                                        id: 'xform_mapping_panel',
                                        autoScroll: false,
                                        listeners: {
                                            beforerender: function(g){
                                                g.setContext(data.uid);
                                            }
                                        }
                                    }
                                ]
                              },{
                                title:'Configuration Properties',
                                hidden: newEntry,
                                items:[
                                    {
                                        xtype: 'configpropertypanel',
                                        style: 'background: #fff',
                                        listeners: {
                                            beforerender: function(g){
                                                g.setHeight(Ext.getCmp('mappingDialog').height-150);
                                                g.setContext(data.uid);
                                            }
                                        }
                                    }
                                ]
                              },{
                                title: 'Sequence',
                                hidden: newEntry,
                                items:[
                                    {
                                        xtype: 'sequencegrid',
                                        listeners: {
                                            beforerender: function(g){
                                                g.setContext(data.uid);
                                            }
                                        }
                                    }
                                ]
                              }
                            ]

                }
            ],
            // explicitly do not allow enter to submit the dialog
            keys: {}
        };


        if (Zenoss.Security.hasPermission('Manage Device')) {
            dialog = new Zenoss.SmartFormDialog(config);
            dialog.show();
        }else{ return false; }
    };    // end edit mapping dialog

   Ext.define('Zenoss.classesgrid.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: [
            {name: 'hasTransform'},
            {name: 'id'},
            {name: 'uid'},
            {name: 'eventClassKey'},
            {name: 'eval'}
        ]
    });

    Ext.define("Zenoss.classesgrid.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.classesgrid.Model',
                initialSortColumn: "id",
                directFn: Zenoss.remote.EventClassesRouter.getInstances,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.eventclasses.ClassesGrid", {
        alias: ['widget.classesgrid'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};
            var me = this;
            Ext.applyIf(config, {
                stateId: 'classes_mapping_grid',
                id: 'classes_mapping_grid',
                stateful: false,
                multiSelect: true,
                tbar:[
                    {
                        xtype: 'largetoolbar',
                        id: 'mapping_toolbar',
                        itemId: 'mapping_toolbar',
                        height:30,
                        disabled: true,
                        items: [
                            {
                                xtype: 'button',
                                iconCls: 'add',
                                hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                                tooltip: _t('Add new Event Class Mapping Instance'),
                                handler: function() {
                                    var grid = Ext.getCmp("classesgrid_id");
                                    Zenoss.eventclasses.mappingDialog(grid);
                                }
                            },{
                                xtype: 'button',
                                iconCls: 'delete',
                                hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                                tooltip: _t('Delete selected items'),
                                handler: function() {
                                    var grid = Ext.getCmp("classesgrid_id"),
                                        data = [],
                                        selected = grid.getSelectionModel().getSelection();
                                    if (Ext.isEmpty(selected)) {
                                        return;
                                    }
                                    for (var i=0; selected.length > i; i++){
                                        data.push({'context':selected[i].data.uid.split('/instances/')[0], 'id':selected[i].data.id});
                                    }
                                    new Zenoss.dialog.SimpleMessageDialog({
                                        title: _t('Delete Instance'),
                                        message: _t("Are you sure you want to delete the selected instances?"),
                                        buttons: [{
                                            xtype: 'DialogButton',
                                            text: _t('OK'),
                                            handler: function() {
                                                Zenoss.remote.EventClassesRouter.removeInstance({'instances':data}, function(response){
                                                    if (response.success) {
                                                        grid.refresh();
                                                        var bar = Ext.getCmp('mapping_toolbar');
                                                        bar.items.items[2].setDisabled(false);
                                                        bar.items.items[3].setDisabled(false);
                                                    }
                                                });
                                            }
                                        }, {
                                            xtype: 'DialogButton',
                                            text: _t('Cancel')
                                        }]
                                    }).show();
                                }
                            },{
                                xtype: 'button',
                                iconCls: 'customize',
                                tooltip: _t('View and/or Edit selected Event Class Mapping Instance'),
                                handler: function() {
                                    var grid = Ext.getCmp("classesgrid_id"),
                                        data,
                                        selected = grid.getSelectionModel().getSelection();

                                    if (Ext.isEmpty(selected)) {
                                        return;
                                    }
                                    // single selection
                                    data = selected[0].data;
                                    data.whichPanel = 'default';
                                    Zenoss.eventclasses.mappingDialog(grid, data);
                                }
                            },{
                                xtype: 'button',
                                iconCls: 'set',
                                hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                                tooltip: _t('Use a drag-drop grid to resequence the weights of Instance Maps'),
                                handler: function() {
                                    var grid = Ext.getCmp("classesgrid_id");
                                    var data, selected = grid.getSelectionModel().getSelection();
                                    if (!selected[0]) {
                                        return;
                                    }
                                    data = selected[0].data;
                                    data.whichPanel = 'sequence';
                                    Zenoss.eventclasses.mappingDialog(grid, data);
                                }
                            }, {
                                iconCls: 'adddevice',
                                xtype: 'button',
                                tooptip: _t('Add To ZenPack'),
                                hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                                handler: function() {
                                    var addtozenpack = new Zenoss.AddToZenPackWindow(),
                                        records = me.getSelectionModel().getSelection();
                                    if (records.length) {
                                        var targets = Ext.Array.pluck(Ext.Array.pluck(records, "data"), "uid");
                                        addtozenpack.setTarget(targets);
                                        addtozenpack.show();
                                    }
                                }
                            }]
                        }
                ],
                store: Ext.create('Zenoss.classesgrid.Store', {}),
                listeners: {
                    afterrender: function(e){
                        e.getStore().on('load', function(){
                            Ext.getCmp('mapping_toolbar').setDisabled(false);
                        });
                    },
                    select: function(e){
                            var bar = Ext.getCmp('mapping_toolbar');
                        if(e.getCount() > 1){
                            bar.items.items[2].setDisabled(true);
                            bar.items.items[3].setDisabled(true);
                        }else{
                            bar.items.items[2].setDisabled(false);
                            bar.items.items[3].setDisabled(false);
                        }
                    }
                },
                columns: [
                    {
                        header: _t('xForm'),
                        id: 'xForm',
                        dataIndex: 'hasTransform',
                        width:45,
                        sortable: true,
                        renderer: function(value){
                            var cls = (Ext.isIE) ? 'ie_xforms_grid': 'xforms_grid';
                            if(value){
                                return '<span class="'+cls+'"><img src="/++resource++zenui/img/xtheme-zenoss/icon/is-transform.png" />&nbsp;</span>';
                            }
                            return '<span class="'+cls+'"><img src="/++resource++zenui/img/xtheme-zenoss/icon/no-transform.png" />&nbsp;</span>';
                        },
                        filter:false
                    },{
                        header: _t('ID'),
                        id: 'map_id',
                        dataIndex: 'id',
                        flex: 1,
                        sortable: true,
                        filter: true
                    },{
                        id: 'uid_id',
                        dataIndex: 'uid',
                        hidden: true
                    },{
                        header: _t("EventClass Key"),
                        id: 'key_id',
                        dataIndex: 'eventClassKey',
                        width: 200,
                        sortable: true,
                        filter: true
                    },{
                        header: _t("Explanation"),
                        id: 'eval_id',
                        dataIndex: 'eval',
                        flex: 1,
                        sortable: true,
                        filter: false
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
        },
        setContext: function(uid) {
            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        },
        onRowDblClick: function() {
            var data,
                selected = this.getSelectionModel().getSelection();
            if (!selected) {
                return;
            }
            data = selected[0].data;
            data.whichPanel = 'default';
            Zenoss.eventclasses.mappingDialog(this, data);
        }
    });


    Ext.define('Zenoss.eventclasses.CodeEditorField', {
        extend: 'Ext.form.field.TextArea',
        alias: 'widget.codeeditorfield',
        cls: 'codemirror-field',
        originalValue: "",
        initComponent: function() {
            var me = this;
            Ext.applyIf(me, {
                listeners: {
                    render: {
                        fn: me.onCodeeditorfieldRender,
                        scope: me
                    }
                }
            });
            this.callParent(arguments);
        },
        onCodeeditorfieldRender: function(abstractcomponent) {
            var me = this;
            var element = document.getElementById(abstractcomponent.getInputId());
            this.editor = CodeMirror.fromTextArea(element, {'lineNumbers':true});
            this.editor.on('cursorActivity', function(){
                if (me.getValue() !== me.originalValue){
                    me.ownerCt.getDockedItems()[0].addCls('edited_feedback');
                }else{
                    me.ownerCt.getDockedItems()[0].removeCls('edited_feedback');
                }
            });
        },
        focus: function() {
            this.editor.focus();
        },
        onFocus: function() {
            this.fireEvent('focus', this);
        },
        destroy: function() {
            this.editor.toTextArea();
            this.callParent(arguments);
        },
        getValue: function() {
            this.editor.save();
            return this.callParent(arguments);
        },
        setValue: function(value) {
            if (this.editor) {
                this.editor.setValue(value);
                this.originalValue = value;
                this.ownerCt.getDockedItems()[0].removeCls('edited_feedback');
            }
            return this.callParent(arguments);
        }
    });

    Ext.define('Zenoss.eventclasses.MiniEditorPanel', {
        extend: 'Ext.panel.Panel',
        alias: 'widget.minieditorpanel',
        layout: {
            type: 'fit'
        },
        collapsible: true,
        titleCollapse: true,
        resizable: {
            handles: 's',
            pinned: true
        },
        height:200,
        style: 'background:white',
        initComponent: function() {
            var me = this;
            Ext.applyIf(me, {
                items: [
                    {
                        xtype: 'codeeditorfield',
                        name: me.name
                    }
                ]
            });

            this.callParent(arguments);
        },
        focus: function() {
            this.down('codeeditorfield').focus();
        },
        getValue: function() {
            return this.down('codeeditorfield').getValue();
        },
        setValue: function(value) {
            this.down('codeeditorfield').setValue(value);
        },
        reset: function() {
            this.down('codeeditorfield').setValue('');
        }
    });

    Ext.define('Zenoss.eventclass.XformMasterPanel', {
        extend: 'Ext.panel.Panel',
        alias: 'widget.xformmasterpanel',
        overflowY: 'scroll',
        initComponent: function() {
            this.callParent(arguments);
        },
        getLastInChainValue: function(){
            var items = this.items.items;
            if(items[items.length-1]){
                return items[items.length-1].getValue();
            }else{
                return false;
            }
        },
        setContext: function(uid){
            var me = this;

            Zenoss.remote.EventClassesRouter.getTransformTree({'uid':uid}, function(response){
                if(response.success){
                    var data = response.data;
                    me.removeAll(true);
                    if(data.length > 1){
                        me.add({
                            xtype: 'label',
                            cls: 'transform-headers',
                            text: 'Inherited Transforms: (All Transforms are multi-line Python code)'
                        });
                    }

                    for(var i=0; data.length-1 > i; i++){
                        // add the inheritance chain:
                        me.add({
                                xtype: 'xformeditorpanel',
                                name: 'maintransformseditor'+i,
                                collapsed: true,
                                cls: 'inherited_transforms',
                                listeners:{
                                    afterrender: function(p){
                                        p.setValue(data[i]);
                                    }
                                }
                        });
                    }
                    me.add({
                        xtype: 'label',
                        cls: 'transform-headers',
                        text: 'Current Transform:'
                    });
                    me.add({
                        xtype: 'panel',
                        region:'left',
                        hidden: true,
                        margin: '0 0 5 5',
                        layout: {
                            type: 'hbox',
                            align: 'middle',
                            pack: 'left'
                        },
                        listeners : {
                            beforerender: function(el) {
                                if(Zenoss.Security.doesNotHavePermission('Manage DMD') === false) {
                                    Zenoss.remote.EventClassesRouter.isTransformEnabled({'uid':uid}, function(response){
                                        if(response.success){
                                            if(response.data === false){
                                                el.setVisible(true);
                                            }
                                        }
                                    });
                                }
                            }
                        },
                        items: [{
                            xtype: 'label',
                            text: 'Transformation is currently disabled due to several failed tries to apply it.'
                        },{
                            xtype: 'button',
                            text: 'Re-activate transform',
                            margin: '0 0 0 5',
                            handler : function(el) {
                                if(!el.disabled){
                                    Zenoss.remote.EventClassesRouter.setTransformEnabled({'uid':uid, 'enabled': true}, function(response){
                                         if(response.success){
                                             if(response.data === true){
                                                 Zenoss.message.info(_t('State changed. Transform is now enabled.'));
                                                 var panel = el.findParentByType('panel');
                                                 panel.setVisible(false)
                                             }
                                             else{
                                                 Zenoss.message.info(_t('State not changed. Transform is still disabled. Try again.'));
                                             }
                                         }
                                    });
                                }
                            }
                        }]
                    });
                    // add the current context transform
                    me.add({
                            xtype: 'xformeditorpanel',
                            name: 'maintransformseditor'+(data.length-1),
                            cls: 'main_transform',
                            listeners:{
                                afterrender: function(p){
                                    p.setValue(data[data.length-1]);
                                }
                            }
                    });
                }
            });
        }
    });

    Ext.define('Zenoss.eventclasses.XformEditorPanel', {
        extend: 'Ext.panel.Panel',
        alias: 'widget.xformeditorpanel',
        collapsible: true,
        titleCollapse: true,
        uid: null,
        resizable: {
            handles: 's',
            pinned: true
        },
        autoHeight: true,
        layout: 'fit',
        initComponent: function() {
            var me = this;
            Ext.applyIf(me, {
                tools:[{
                    type: 'save',
                    hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    tooltip: _t('Save and compile this Transform only'),
                    handler:  function(){
                        var value = me.items.items[0].getValue();
                        var uid = me.uid;
                        Zenoss.remote.EventClassesRouter.setTransform({'uid':uid, 'transform':value}, function(response){
                            if(response.success){
                                Zenoss.message.info(_t('One Transform compiled and saved'));
                                me.items.items[0].setValue(value);
                                var isTrans = (value !== "");
                                Ext.getCmp('classes').setTransIcon(isTrans);
                            }
                        });
                    }
                },{
                    type: 'refresh',
                    tooltip: _t('Reset this Transform back to the last save'),
                    hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    handler:  function(){
                        var uid = me.uid;
                        Zenoss.remote.EventClassesRouter.getTransform({'uid':uid}, function(response){
                            if(response.success){
                                var str = me.getDefaultString(response.data)
                                me.items.items[0].setValue(str);
                            }
                        });
                    }
                },{
                    type: 'expand',
                    tooltip: _t('Expand all edit boxes on this page'),
                    handler:  function(){
                        // find all boxes on the page:
                        var items = this.ownerCt.ownerCt.ownerCt.items.items;
                        for (var i=0; items.length > i; i++){
                            // swollow when something doesn't have ability to 'expand'
                            try{ items[i].expand(); }catch(e){}
                        }
                    }
                }],
                items: [
                    {
                        xtype: 'codeeditorfield',
                        name: me.name
                    }
                ]
            });

            this.callParent(arguments);
        },
        focus: function() {
            this.down('codeeditorfield').focus();
        },
        getValue: function() {
            return this.down('codeeditorfield').getValue();
        },
        setValue: function(value) {
            this.setTitle(_t("Transform for")+": "+value.transid);
            this.uid = "/zport/dmd"+value.transid;
            var str = this.getDefaultString(value.trans)
            this.down('codeeditorfield').setValue(str);
        },
        reset: function() {
            this.down('codeeditorfield').setValue('');
        },
        getDefaultString: function(str){
            if (str == '') {
               return '\n'.repeat(10);
            }
            return str;
        }
    });

});
/*****************************************************************************
 *
 * Copyright (C) Zenoss, Inc. 2013, all rights reserved.
 *
 * This content is made available according to terms specified in
 * License.zenoss under the directory where your Zenoss product is installed.
 *
 ****************************************************************************/

Ext.onReady(function(){
    Ext.ns('Zenoss.manufacturers');

// ----------------------------------------------------------------- DIALOGS

    Zenoss.manufacturers.productsDialog = function(grid, data) {
        var baseid = "";
        var originalManufacturer = "";
        if(typeof(data) === "undefined"){
            data = "";
            baseid = grid.uid ? grid.uid : "";
        }else{
            baseid = data.uid.split("/products/")[0];
        }
        var addhandler, config, dialog, newEntry;
        newEntry = (data === "");
        var xtraData = {};
        addhandler = function() {
            var c = dialog.getForm().getForm().getValues();
            var params = {
                uid:                    baseid,
                prodname:               c.name,
                oldname:                c.productname, // if they change the name of this product
                partno:                 c.partNo,
                type:                   c.prodtype,
                prodkeys:               c.keys_panel,
                description:            Ext.getCmp('desc_panel').getValue()
            };
            if (newEntry) {
                Zenoss.remote.ManufacturersRouter.addNewProduct({'params':params}, function(response){
                    if (response.success) {
                        if (grid) {
                            grid.refresh();
                        }
                    }
                });
            } else {
                var mancombo = Ext.getCmp('mansetting');
                Zenoss.remote.ManufacturersRouter.editProduct({'params':params}, function(response){
                    if (response.success) {
                        if (grid) {
                            if(originalManufacturer === mancombo.getValue()){
                                grid.refresh();
                            }
                        }
                    }
                });
                var moveTarget = "/zport/dmd/Manufacturers/"+mancombo.getValue();
                if(originalManufacturer !== mancombo.getValue()){
                    params = {
                        'moveFrom': "/zport/dmd/Manufacturers/"+originalManufacturer,
                        'moveTarget': moveTarget,
                        'ids': [c.name]
                    };
                    Zenoss.remote.ManufacturersRouter.moveProduct(params, function(response){
                        if(response.success) {
                            var tree = Ext.getCmp('manufacturers_tree');
                            tree.refresh();
                            tree.getStore().on('load', function(){
                                var nodeId = moveTarget;
                                    var node = tree.getRootNode().findChild("uid", nodeId, true);
                                tree.getView().select(node);
                            }, this, {single:true});
                        }
                    });
                }

            }
        };

        // form config
        config = {
            submitHandler: addhandler,
            height:Ext.getBody().getViewSize().height,
            width:Ext.getBody().getViewSize().width*0.8, //80%
            id: 'productsDialog',
            contextUid: null,
            title: _t("Add New Product"),
            listeners: {
                'afterrender': function(e){
                    if(!newEntry){
                        // this window will be used to EDIT the values instead of create from scratch
                        // grab extra data from server to populate code boxes:
                       Zenoss.remote.ManufacturersRouter.getProductData({'uid':baseid, 'prodname':data.id}, function(response){
                            if(response.success){
                                xtraData = response.data[0];
                                var instancegrid = Ext.getCmp('instancegrid_id');
                                instancegrid.store.setBaseParam('id', data.id);
                                instancegrid.setContext(baseid);
                                var combo = Ext.getCmp('prodtype');
                                if(data.type === "Hardware"){
                                    combo.store.filter([{
                                        filterFn: function(record) {
                                            return record.get('name') === 'Hardware';
                                        }
                                    }]);
                                }else{
                                    combo.store.filter([{
                                        filterFn: function(record) {
                                            return record.get('name') !== 'Hardware';
                                        }
                                    }]);
                                }
                                Ext.getCmp('productsDialog').contextUid = xtraData.uid;
                                e.setTitle(Ext.String.format(_t("Edit Product Info for: {0}"), data.id));
                                var fields = e.getForm().getForm().getFields();
                                fields.findBy(
                                    function(record){
                                        switch(record.getName()){
                                            case "name"             : record.setValue(xtraData.name);  break;
                                            case "productname"      : record.setValue(xtraData.name);  break;
                                            case "partNo"           : record.setValue(xtraData.partno);  break;
                                            case "keys_panel"       : record.setValue(xtraData.prodKeys.toString());  break;
                                            case "desc_panel"       : record.setValue(xtraData.desc);  break;
                                            case "prodtype"         : record.setValue(xtraData.type); break;
                                        }
                                    }
                                );
                            }

                            /*  get list of manufacturers and
                                add them to the combo box
                            */
                                var mancombo = Ext.getCmp('mansetting');
                                originalManufacturer = data.uid.split("/products/")[0].split("/Manufacturers/")[1];
                                var mandata = [];
                                var tree = Ext.getCmp('manufacturers_tree');
                                if (tree){ // if no tree, then dialog is being called from another context
                                    var children = tree.items.items[0].node.childNodes;
                                    for (var i = 0; i < children.length; i++){
                                        mandata.push([children[i].data.text.text]);
                                    }
                                    mancombo.store.loadData(mandata);
                                }else{
                                    Zenoss.remote.ManufacturersRouter.returnTree({'id':'zport/dmd/Manufacturers'}, function(response){
                                        for (var i = 0; i < response.length; i++){
                                            mandata.push([response[i].text.text]);
                                        }
                                        mancombo.store.loadData(mandata);
                                    });
                                }
                                mancombo.setValue(originalManufacturer);
                        });
                    }else{
                            var tree = Ext.getCmp('manufacturers_tree');
                            Ext.getCmp('mansetting').setValue(tree.getSelectionModel().getSelectedNode().data.text.text);
                    }
                }
            },
            items: [
                {
                    xtype: 'tabpanel',
                    id: 'blackTabs',
                    listeners: {
                        'afterrender': function(p){
                            if(data.whichPanel === 'configprops'){
                                p.setActiveTab(1);
                            }
                        }
                    },
                    bodyStyle: {
                        padding: '10 0 10 0'
                    },
                    items: [
                        {
                            title: _t('Product Details'),
                            items:[
                                {
                                    xtype: 'panel',
                                    layout: 'hbox',
                                    margin: '0 0 30px 0',
                                    items: [
                                        {
                                            xtype: 'textfield',
                                            name: 'name',
                                            fieldLabel: _t('Product Name'),
                                            margin: '0 10px 0 0',
                                            width:320,
                                            regex: Zenoss.env.textMasks.allowedDescText,
                                            regexText: Zenoss.env.textMasks.allowedDescTextFeedback,
                                            allowBlank: false
                                        },{
                                            xtype: 'hidden',
                                            name: 'productname'
                                        },{
                                            xtype: 'textfield',
                                            name: 'partNo',
                                            margin: '0 20px 0 0',
                                            fieldLabel: _t('Part #'),
                                            regex: Zenoss.env.textMasks.allowedDescText,
                                            regexText: Zenoss.env.textMasks.allowedDescTextFeedback,
                                            width:250
                                        },{
                                            xtype: 'combo',
                                            id: 'prodtype',
                                            name:'prodtype',
                                            displayField: 'name',
                                            editable: false,
                                            typeAhead: false,
                                            allowBlank: false,
                                            fieldLabel: _t('Type'),
                                            store:  Ext.create('Ext.data.ArrayStore', {
                                                 model: 'Zenoss.model.Name',
                                                 data: [[
                                                    _t('Hardware')
                                                ],[
                                                    _t('Software')
                                                ],[
                                                    _t('Operating System')
                                                ]]
                                             })
                                        },{
                                            xtype: 'combo',
                                            id: 'mansetting',
                                            name:'mansetting',
                                            displayField: 'name',
                                            editable: false,
                                            typeAhead: false,
                                            width: 200,
                                            fieldLabel: _t('Manufacturer'),
                                            queryMode: 'local',
                                            store: [ 'none' ]
                                        }
                                    ]

                                },{
                                    xtype: 'panel',
                                    items:[
                                        {
                                            xtype: 'textfield',
                                            fieldLabel: _t('Product Keys (comma delimited)'),
                                            id: 'keys_panel',
                                            name: 'keys_panel',
                                            width: '98.5%',
                                            margin: '0 20 20 0'
                                        },{
                                            xtype: 'minieditorpanel',
                                            name: 'desc_panel',
                                            id: 'desc_panel',
                                            height:100,
                                            width:'98.5%',
                                            margin: '0 20 20 0',
                                            title: _t('Description')
                                        },{
                                            xtype: 'instancepanel',
                                            hidden: newEntry,
                                            title: _t('Instances of this product'),
                                            id: 'instancegrid_id',
                                            width:'98.5%'
                                        }
                                    ]// panel items
                              }
                              ] // product details items
                           },{
                                title: _t('Configuration Properties'),
                                hidden: newEntry,
                                items:[
                                    {
                                        xtype: 'configpropertypanel',
                                        style: 'background: #fff',
                                        listeners: {
                                            beforerender: function(g){
                                                g.setHeight(Ext.getCmp('productsDialog').height-150);
                                                g.setContext(data.uid);
                                            }
                                        }
                                    }
                                ]
                            }
                       ] // tab panel items
                }// tab panel
            ], // config items
            // explicitly do not allow enter to submit the dialog
            keys: {}
        };
        if (Zenoss.Security.hasPermission('Manage Device')) {
            dialog = new Zenoss.SmartFormDialog(config);
            dialog.show();
        }else{ return false; }
    };    // end edit product dialog

   Ext.define('Zenoss.productsgrid.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: [
            {name: 'count'},
            {name: 'id'},
            {name: 'uid'},
            {name: 'key'},
            {name: 'type'}
        ]
    });

    Ext.define("Zenoss.productsgrid.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.productsgrid.Model',
                initialSortColumn: "id",
                directFn: Zenoss.remote.ManufacturersRouter.getProductsByManufacturer,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.manufacturers.ProductsGrid", {
        alias: ['widget.productsgrid'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};

            Ext.applyIf(config, {
                stateId: 'products_grid',
                id: 'products_grid',
                stateful: false,
                loadMask:true,
                multiSelect: true,
                tbar:[
                    {
                        xtype: 'largetoolbar',
                        id: 'products_toolbar',
                        itemId: 'products_toolbar',
                        height:30,
                        disabled: true,
                        items: [
                            {
                                xtype: 'button',
                                iconCls: 'add',
                                hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                                tooltip: _t('Add a new product to this manufacturer'),
                                handler: function() {
                                    var grid = Ext.getCmp("productsgrid_id");
                                    Zenoss.manufacturers.productsDialog(grid);
                                }
                            },{
                                xtype: 'button',
                                iconCls: 'delete',
                                hidden: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                                tooltip: _t('Delete selected items'),
                                handler: function() {
                                    var grid = Ext.getCmp("productsgrid_id"),
                                        data = [],
                                        selected = grid.getSelectionModel().getSelection();
                                    if (Ext.isEmpty(selected)) {
                                        return;
                                    }
                                    for (var i=0; selected.length > i; i++){
                                        data.push({'context':selected[i].data.uid.split('/products/')[0], 'id':selected[i].data.id});
                                    }
                                    new Zenoss.dialog.SimpleMessageDialog({
                                        title: _t('Delete Product'),
                                        message: _t("Are you sure you want to delete the selected products?"),
                                        buttons: [{
                                            xtype: 'DialogButton',
                                            text: _t('OK'),
                                            handler: function() {
                                                Zenoss.remote.ManufacturersRouter.removeProducts({'products':data}, function(response){
                                                    if (response.succesws) {
                                                        grid.refresh();
                                                    }
                                                });
                                            }
                                        }, {
                                            xtype: 'DialogButton',
                                            text: _t('Cancel')
                                        }]
                                    }).show();
                                }
                            },{
                                xtype: 'button',
                                iconCls: 'customize',
                                tooltip: _t('View and/or Edit selected'),
                                handler: function() {
                                    var grid = Ext.getCmp("productsgrid_id"),
                                        data,
                                        selected = grid.getSelectionModel().getSelection();

                                    if (Ext.isEmpty(selected)) {
                                        return;
                                    }
                                    // single selection
                                    data = selected[0].data;
                                    data['whichPanel'] = 'default';
                                    Zenoss.manufacturers.productsDialog(grid, data);
                                }
                            }]
                        }
                ],
                store: Ext.create('Zenoss.productsgrid.Store', {}),
                listeners: {
                    afterrender: function(e){
                        e.getStore().on('load', function(){
                            Ext.getCmp('products_toolbar').setDisabled(false);
                        });
                    },
                    select: function(e, record){
                        var bar = Ext.getCmp('products_toolbar');
                        if(e.getCount() > 1){
                            bar.items.items[2].setDisabled(true);
                        }else{
                            bar.items.items[2].setDisabled(false);
                        }
                        var token = Ext.History.getToken().split(":");
                        var newToken = Ext.String.format("{0}:{1}:{2}",
                                                         token[0],
                                                         token[1],
                                                         record.get('uid')
                                                         );
                        Ext.History.add(newToken);
                    }
                },
                columns: [
                    {
                        header: _t('Name'),
                        id: 'prod_id',
                        dataIndex: 'id',
                        flex: 1,
                        sortable: true
                    },{
                        id: 'uid_id',
                        dataIndex: 'uid',
                        hidden: true
                    },{
                        header: _t("Type"),
                        id: 'type_id',
                        dataIndex: 'type',
                        width: 200,
                        sortable: true,
                        filter: []
                    },{
                        header: _t("Product Keys"),
                        id: 'keys_id',
                        dataIndex: 'key',
                        flex: 1,
                        sortable: true,
                        renderer: function(e){
                            return Ext.htmlEncode(e.toString().replace(",",",  "));
                        },
                        filter: []
                    },{
                        header: _t('Count'),
                        id: 'count_id',
                        dataIndex: 'count',
                        width:70,
                        sortable: true,
                        filter: []
                    }]
            });
            this.callParent(arguments);
            this.on('itemdblclick', this.onRowDblClick, this);
        },
        selectByToken: function(id) {
            // decode the id
            var uid = Ext.Object.fromQueryString("uid=" + id).uid;
            var idx = this.getStore().findExact('uid', uid), record, view = this.getView();

            if (idx !== -1) {
                record = this.getStore().getAt(idx);
                this.getSelectionModel().select(record);
                view.focusRow(idx);
            }
        },
        setContext: function(uid) {
            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        },
        onRowDblClick: function() {
            var data,
                selected = this.getSelectionModel().getSelection();
            if (!selected) {
                return;
            }
            data = selected[0].data;
            data['whichPanel'] = 'default';
            Zenoss.manufacturers.productsDialog(this, data);
        }
    });




   Ext.define('Zenoss.instancegrid.Model',  {
        extend: 'Ext.data.Model',
        idProperty: 'device',
        fields: [
            {name: 'id'},
            {name: 'device'}
        ]
    });

    Ext.define("Zenoss.instancegrid.Store", {
        extend: "Zenoss.NonPaginatedStore",
        constructor: function(config) {
            config = config || {};
            Ext.applyIf(config, {
                model: 'Zenoss.instancegrid.Model',
                initialSortColumn: "id",
                directFn: Zenoss.remote.ManufacturersRouter.getProductInstances,
                root: 'data'
            });
            this.callParent(arguments);
        }
    });

    Ext.define("Zenoss.manufacturers.InstanceGrid", {
        alias: ['widget.instancepanel'],
        extend:"Zenoss.FilterGridPanel",
        constructor: function(config) {
            config = config || {};

            Ext.applyIf(config, {
                stateId: 'instance_grid',
                id: 'instance_grid',
                stateful: true,
                loadMask:true,
                store: Ext.create('Zenoss.instancegrid.Store', {}),
                columns: [
                    {
                        header: _t("Device"),
                        id: 'device_id',
                        dataIndex: 'device',
                        flex: 1,
                        sortable: true,
                        renderer: function(name, row, record) {
                            return Zenoss.render.link(null,record.raw.uid, name);
                        }
                    },{
                        header: _t('Name'),
                        id: 'instance_id',
                        dataIndex: 'id',
                        flex: 1,
                        sortable: true,
                        filter: []
                    }]
            });
            this.callParent(arguments);
        },
        setContext: function(uid) {
            this.uid = uid;
            // load the grid's store
            this.callParent(arguments);
        }
    });






    Ext.define('Zenoss.manufacturers.CodeEditorField', {
        extend: 'Ext.form.field.TextArea',
        alias: 'widget.codefield',
        cls: 'codemirror-field',
        originalValue: "",
        initComponent: function() {
            var me = this;
            Ext.applyIf(me, {
                listeners: {
                    render: {
                        fn: me.onCodeeditorfieldRender,
                        scope: me
                    }
                }
            });
            this.callParent(arguments);
        },
        onCodeeditorfieldRender: function(abstractcomponent) {
            var me = this;
            var element = document.getElementById(abstractcomponent.getInputId());
            this.editor = CodeMirror.fromTextArea(element, {'lineNumbers':true});
            this.editor.on('cursorActivity', function(){
                if (me.getValue() !== me.originalValue){
                    me.ownerCt.getDockedItems()[0].addCls('edited_feedback');
                }else{
                    me.ownerCt.getDockedItems()[0].removeCls('edited_feedback');
                }
            });
        },
        focus: function() {
            this.editor.focus();
        },
        onFocus: function() {
            this.fireEvent('focus', this);
        },
        destroy: function() {
            this.editor.toTextArea();
            this.callParent(arguments);
        },
        getValue: function() {
            this.editor.save();
            return this.callParent(arguments);
        },
        setValue: function(value) {
            if (this.editor) {
                this.editor.setValue(value);
                this.originalValue = value;
                this.ownerCt.getDockedItems()[0].removeCls('edited_feedback');
            }
            return this.callParent(arguments);
        }
    });

    Ext.define('Zenoss.manufacturers.MiniEditorPanel', {
        extend: 'Ext.panel.Panel',
        alias: 'widget.minieditor',
        layout: {
            type: 'fit'
        },
        collapsible: true,
        titleCollapse: true,
        resizable: {
            handles: 's',
            pinned: true
        },
        height:200,
        style: 'background:white',
        initComponent: function() {
            var me = this;
            Ext.applyIf(me, {
                items: [
                    {
                        xtype: 'codefield',
                        name: me.name,
                        value: me.value
                    }
                ]
            });

            this.callParent(arguments);
        },
        focus: function() {
            this.down('codefield').focus();
        },
        getValue: function() {
            return this.down('codefield').getValue();
        },
        setValue: function(value) {
            this.down('codefield').setValue(value);
        },
        reset: function() {
            this.down('codefield').setValue('');
        }
    });




});
(function() {
    var router = Zenoss.remote.JobsRouter;
    /**
     * @class Zenoss.JobLog
     * @extends Ext.panel.Panel
     * @constructor
     *
     */
    Ext.define('Zenoss.JobLog', {
        extend: 'Zenoss.dialog.BaseDialog',
        alias: 'widget.joblog',
        constructor: function(config) {
            if (!config.job) {
                throw new Exception("You must specify a jobid for the joblog dialog");
            }
            Ext.applyIf(config, {
                height: 600,
                closeAction: 'destroy',
                width: 800,
                //maximized: true,
                title: Ext.String.format("Log for Add Device: {0}", config.deviceName),
                autoScroll: true,
                items: [{
                    xtype: 'panel',
                    ref: 'logfilecontents',
                    bodyStyle: 'padding:8px;background:#fff;overflow:scroll',
                    autoScroll: true
                }],
                buttons: [{
                    xtype: 'DialogButton',
                    ui: 'dialog-dark',
                    text: _t('Close'),
                    handler: Ext.bind(this.cancelUpdateTask, this)
                }, {
                    xtype: 'button',
                    ui: 'dialog-dark',
                    text: _t('Download Log File'),
                    handler: Ext.bind(function(){
                        window.location = "joblog?job=" + this.jobid;
                    }, this)
                }]
            });
            this.jobid = config.job.uuid;
            this.callParent([config]);
        },
        initComponent: function() {
            this.poll();
            this.on('close', this.cancelUpdateTask, this);
            this.callParent(arguments);
        },
        refresh: function() {
            if (this.job == null) return;
            router.detail({jobid:this.jobid}, function(r){
                var msg = _t("[!] Log file too large for job log screen (viewing last 100 lines). To view full log use link above.");
                var html = "<b style=\'color:black\'>Log file: <a href='joblog?job=" + this.jobid + "'>" + r.logfile + "</a></b><br/><br/>";
                if(r.maxLimit){
                    html += "<b style='color:red;padding-bottom:8px;display:block;'>"+msg+"</b>";
                }
                for (var i=0; i < r.content.length; i++) {
                    var color = i%2 ? '#b58900' : '#657B83';
                    if (r.content[i].indexOf("ERROR zen.") != -1) {
                        color = "red";
                    }
                    html += "<pre style='font-family:Monaco,monospaced;font-size:12px;color:" +
                        color + ";'>" + Ext.htmlEncode(r.content[i]) + '</pre>';
                }
                // this window has been closed or destroyed, cancel the update task
                if (!this.logfilecontents) {
                    return this.cancelUpdateTask();
                }
                this.logfilecontents.update(html);
                var d = this.body.dom;
                d.scrollTop = d.scrollHeight - d.offsetHeight + 16;
                // check to see if we are finished, no need to keep polling if we are
                if (html.indexOf("Finished with result") != -1) {
                    this.cancelUpdateTask();
                }
            }, this);
        },
        cancelUpdateTask: function() {
            if (this.updateTask) {
                this.updateTask.cancel();
                delete this.updateTask;
            }
        },
        poll: function() {
            if (!this.updateTask) {
                this.updateTask = new Ext.util.DelayedTask(this.poll, this);
            }
            this.refresh();
            var status = this.job.status;
            switch (status) {
              case "STARTED":
              case "PENDING":
                this.updateTask.delay(3000);
                break;
              case "ABORTED":
                // Might have updates but only a couple, so let's not
                // spam the server with requests on a dead job
                this.updateTask.delay(5000);
                break;
            }
        }

    });

}());
